Package: spatialwarnings
Type: Package
Title: Spatial Early Warning Signals of Ecosystem Degradation
Version: 3.1.1
Authors@R: c(person(given = "Alain",
                    family = "Danet",
                    role = "aut"),
             person(given = "Alexandre",
                    family = "Genin",
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0002-3333-1338"), 
                    email = "alexandre.genin@sete.cnrs.fr"),
             person(given = "Vishwesha",
                    family = "Guttal",
                    role = "aut"),
             person(given = "Sonia",
                    family = "Kefi",
                    role = "aut"),
             person(given = "Sabiha",
                    family = "Majumder",
                    role = "aut"),
             person(given = "Sumithra",
                    family = "Sankaran",
                    role = "aut"),
             person(given = "Florian",
                    family = "Schneider",
                    role = "aut"))
License: MIT + file LICENSE
Maintainer: Alexandre Genin <alexandre.genin@sete.cnrs.fr>
Description: Tools to compute and assess significance of early-warnings signals (EWS) of ecosystem degradation. EWS are spatial metrics derived from raster data -- e.g. spatial autocorrelation -- that increase before an ecosystem undergoes a non-linear transition (Genin et al. (2018) <doi:10.1111/2041-210X.13058>).
URL: https://github.com/spatial-ews/spatialwarnings
BugReports: https://github.com/spatial-ews/spatialwarnings/issues
Depends: R (>= 3.3.0), future
Imports: Rcpp, ggplot2, plyr, stats, utils, future.apply, gsl,
        segmented
Suggests: moments, poweRlaw, reshape2, testthat, covr, acss, acss.data,
        mgcv, gstat, sp, raster
LinkingTo: Rcpp, RcppArmadillo
LazyData: true
RoxygenNote: 7.3.2.9000
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2025-08-18 13:11:32 UTC; alex
Author: Alain Danet [aut],
  Alexandre Genin [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3333-1338>),
  Vishwesha Guttal [aut],
  Sonia Kefi [aut],
  Sabiha Majumder [aut],
  Sumithra Sankaran [aut],
  Florian Schneider [aut]
Repository: CRAN
Date/Publication: 2025-08-18 15:40:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 11:33:08 UTC; unix
Archs: spatialwarnings.so.dSYM

Package: spatialwarnings
Type: Package
Title: Spatial Early Warning Signals of Ecosystem Degradation
Version: 3.1.2
Authors@R: c(person(given = "Alain",
                    family = "Danet",
                    role = "aut"),
             person(given = "Alexandre",
                    family = "Genin",
                    role = c("aut", "cre"),
                    comment = c(ORCID = "0000-0002-3333-1338"), 
                    email = "alexandre.genin@inrae.fr"),
             person(given = "Vishwesha",
                    family = "Guttal",
                    role = "aut"),
             person(given = "Sonia",
                    family = "Kefi",
                    role = "aut"),
             person(given = "Sabiha",
                    family = "Majumder",
                    role = "aut"),
             person(given = "Sumithra",
                    family = "Sankaran",
                    role = "aut"),
             person(given = "Florian",
                    family = "Schneider",
                    role = "aut"))
License: MIT + file LICENSE
Maintainer: Alexandre Genin <alexandre.genin@inrae.fr>
Description: Tools to compute and assess significance of early-warnings signals (EWS) of ecosystem degradation. EWS are spatial metrics derived from raster data -- e.g. spatial autocorrelation -- that increase before an ecosystem undergoes a non-linear transition (Genin et al. (2018) <doi:10.1111/2041-210X.13058>).
URL: https://github.com/spatial-ews/spatialwarnings
BugReports: https://github.com/spatial-ews/spatialwarnings/issues
Depends: R (>= 3.5.0), future
Imports: Rcpp, ggplot2, plyr, stats, utils, future.apply, gsl,
        segmented
Suggests: moments, poweRlaw, reshape2, testthat, covr, acss, acss.data,
        mgcv, gstat, sp, raster
LinkingTo: Rcpp, RcppArmadillo
LazyData: true
RoxygenNote: 7.3.2.9000
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-07-23 10:55:16 UTC; alex
Author: Alain Danet [aut],
  Alexandre Genin [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3333-1338>),
  Vishwesha Guttal [aut],
  Sonia Kefi [aut],
  Sabiha Majumder [aut],
  Sumithra Sankaran [aut],
  Florian Schneider [aut]
Repository: CRAN
Date/Publication: 2026-07-23 15:50:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,patchdistr_sews_list)
S3method(as.data.frame,patchdistr_sews_single)
S3method(as.data.frame,patchdistr_sews_test_list)
S3method(as.data.frame,patchdistr_sews_test_single)
S3method(as.data.frame,simple_sews_list)
S3method(as.data.frame,simple_sews_single)
S3method(as.data.frame,simple_sews_test_list)
S3method(as.data.frame,simple_sews_test_single)
S3method(convert_to_matrix,factor)
S3method(convert_to_matrix,list)
S3method(convert_to_matrix,matrix)
S3method(display_matrix,default)
S3method(display_matrix,list)
S3method(display_matrix,matrix)
S3method(display_matrix,sews_result_list)
S3method(display_matrix,sews_result_single)
S3method(display_size_info,list)
S3method(display_size_info,sews_result_list)
S3method(display_size_info,sews_result_single)
S3method(extract_spectrum,spectral_sews_list)
S3method(extract_spectrum,spectral_sews_single)
S3method(extract_variogram,variogram_sews_list)
S3method(extract_variogram,variogram_sews_single)
S3method(indictest,flowlength_sews_list)
S3method(indictest,flowlength_sews_single)
S3method(indictest,patchdistr_sews_list)
S3method(indictest,patchdistr_sews_single)
S3method(indictest,simple_sews)
S3method(indictest,simple_sews_list)
S3method(indictest,simple_sews_single)
S3method(indictest,spectral_sews_list)
S3method(indictest,spectral_sews_single)
S3method(indictest,variogram_sews_list)
S3method(indictest,variogram_sews_single)
S3method(plot,patchdistr_sews)
S3method(plot,patchdistr_sews_list)
S3method(plot,simple_sews_list)
S3method(plot,simple_sews_test)
S3method(plot,simple_sews_test_list)
S3method(plot,variogram_sews)
S3method(plot,variogram_sews_list)
S3method(plot,variogram_sews_test)
S3method(plot,variogram_sews_test_list)
S3method(plot_distr,patchdistr_sews)
S3method(plot_distr,patchdistr_sews_list)
S3method(plot_distr,patchdistr_sews_single)
S3method(plot_distr,patchdistr_sews_test_list)
S3method(plot_distr,patchdistr_sews_test_single)
S3method(plot_spectrum,spectral_sews_list)
S3method(plot_spectrum,spectral_sews_single)
S3method(plot_spectrum,spectral_sews_test_list)
S3method(plot_spectrum,spectral_sews_test_single)
S3method(plot_variogram,variogram_sews)
S3method(plot_variogram,variogram_sews_list)
S3method(plot_variogram,variogram_sews_single)
S3method(plot_variogram,variogram_sews_test)
S3method(plot_variogram,variogram_sews_test_list)
S3method(plot_variogram,variogram_sews_test_single)
S3method(predict,patchdistr_sews_list)
S3method(predict,patchdistr_sews_single)
S3method(predict,variogram_sews_list)
S3method(predict,variogram_sews_single)
S3method(print,patchdistr_sews_list)
S3method(print,patchdistr_sews_single)
S3method(print,patchdistr_sews_test_list)
S3method(print,patchdistr_sews_test_single)
S3method(print,simple_sews_list)
S3method(print,simple_sews_single)
S3method(print,simple_sews_test_list)
S3method(print,simple_sews_test_single)
S3method(summary,patchdistr_sews_list)
S3method(summary,patchdistr_sews_single)
S3method(summary,patchdistr_sews_test_list)
S3method(summary,patchdistr_sews_test_single)
S3method(summary,simple_sews_list)
S3method(summary,simple_sews_single)
S3method(summary,simple_sews_test_list)
S3method(summary,simple_sews_test_single)
export(LSW_fit)
export(coarse_grain)
export(compute_indicator)
export(convert_to_matrix)
export(create_indicator)
export(dLSW)
export(display_matrix)
export(display_size_info)
export(exp_fit)
export(extract_spectrum)
export(extract_variogram)
export(flowlength_sews)
export(generic_sews)
export(indicator_plrange)
export(indicator_psdtype)
export(indictest)
export(kbdm_sews)
export(label)
export(lnorm_fit)
export(lsw_sews)
export(pLSW)
export(pair_counts)
export(patchdistr_sews)
export(patchsizes)
export(percolation)
export(pl_fit)
export(plot_distr)
export(plot_spectrum)
export(plot_variogram)
export(raw_cg_moran)
export(raw_cg_skewness)
export(raw_cg_variance)
export(raw_clustering)
export(raw_flowlength_uniform)
export(raw_kbdm)
export(raw_lsw_aicw)
export(raw_moran)
export(raw_patch_radii_skewness)
export(raw_plrange)
export(raw_sdr)
export(raw_structvar)
export(raw_variogram_metrics)
export(rspectrum)
export(spectral_sews)
export(tpl_fit)
export(variogram_sews)
export(xmin_estim)
import(ggplot2)
import(plyr)
import(stats)
import(utils)
importFrom(Rcpp,sourceCpp)
importFrom(future,plan)
useDynLib(spatialwarnings)

# 
#
#' @title Custom Spatial Early-Warning signals
#' 
#' @aliases custom_indicator
#' 
#' @description Computation, significance assessment and display of trends 
#'   of a custom, user-defined indicator.
#' 
#' @param fun A function that takes a matrix as input and returns 
#'   a vector of numerical values. If the function returns a named vector, 
#'   then the names will be used in plots and summaries. The function may also
#'   accept extra arguments. 
#' 
#' @param taskname The task name. A character string used used for plots and
#'   textual summaries that describes the indicator (or set of indicators) 
#'   being computed. If a task name cannot be derived from \code{fun}, then 
#'   a default name is used. 
#' 
#' @return 
#' 
#' \code{create_indicator} returns a function that can be used in the same way 
#'   than the other \code{*_sews} functions (e.g. \code{generic_sews}). This 
#'   function as well as \code{compute_indicator} will return
#'   \code{\link[=simple_sews]{simple_sews_*}} objects. 
#' 
#' @details 
#' 
#' \code{spatialwarnings} provides "workflow functions", named \code{*_sews}, 
#'   that assist the user in computing, displaying and assessing the 
#'   significance of indicator values. The functions \code{create_indicator} and 
#'   \code{compute_indicator} provides such workflow for any arbitrary function. 
#'   
#' \code{create_indicator} takes a function `fun` and returns another function 
#'   that can be used as an indicator similar to the \code{*_sews} functions. The 
#'   results of this function can be assessed for significance using 
#'   \code{indictest} and trends can be displayed using 
#'   \code{plot}, \code{summary}, etc. (see Examples). \code{compute_indicator} 
#'   does the same but without needing an intermediate indicator function. 
#' 
#' @seealso \code{\link{simple_sews}}
#' 
#' @examples
#' 
#' # Use the maximum patch size as indicator of degradation
#' maxpatchsize <- function(mat) { 
#'   max(patchsizes(mat))
#' }
#' 
#' # Create the indicator function
#' maxpatch_sews <- create_indicator(maxpatchsize)
#' 
#' # Then work with this function as if it were a function from the *_sews 
#' # family. 
#' mp_indic <- maxpatch_sews(forestgap)
#' summary(mp_indic)
#' 
#' \donttest{ 
#' # Assess significance and display trends
#' mp_test <- indictest(mp_indic, nulln = 49)
#' plot(mp_test)
#' }
#' 
#' 
#' 
#' # Try spatial coefficient of variation as a spatial EWS. This function can 
#' # have arguments. 
#' spatial_cv <- function(mat, subsize) { 
#'   matc <- coarse_grain(mat, subsize)
#'   return( sd(matc) / mean(matc) )
#' }
#' 
#' # Create indicator function
#' cv_sews <- create_indicator(spatial_cv)
#' 
#' # Compute and display trends
#' cv_indic <- cv_sews(serengeti, subsize = 3)
#' plot(cv_indic, along = serengeti.rain)
#' 
#' # We can do the same work in one go using compute_indicator
#' cv_indic2 <- compute_indicator(serengeti, spatial_cv, subsize = 3)
#' plot(cv_indic2, along = serengeti.rain)
#' 
#' \donttest{ 
#' indictest(cv_indic, nulln = 99)
#' }
#'@export
create_indicator <- function(fun, 
                             taskname = as.character(substitute(fun))) { 

  # Test if taskname is derived from an anonymous function
  if ( length(taskname) > 1 ) { 
    warning('A valid name could not be derived from the function passed', 
            'to create_indicator, using a default name')
    taskname <- "Custom indicator"
  }
  
  # Subfunction that works only on a matrix
  get_one_result <- function(mat, ...) { 
    mat <- convert_to_matrix(mat)
    check_mat(mat)
    
    # Handle the arguments passed to the function and store them in the returned
    # object
    fun.args <- as.list(match.call(expand.dots = FALSE))[['...']]
    fun.args <- lapply(fun.args, eval, envir = parent.frame())
    
    # Sometimes the indicator does not take any extra arguments. In that case 
    # ... is NULL, so we need to replace that with an empty list
    if ( is.null(fun.args) ) { 
      fun.args <- list()
    }
    
    result <- list(value     = fun(mat, ...), 
                   orig_data = mat, 
                   fun.args  = fun.args, 
                   taskname = taskname, 
                   indicf = fun)
    
    class(result) <- c('custom_sews_single', 'simple_sews_single',
                       'sews_result_single')
    return(result)
  }
  
  # Actual function produced
  function(mat, ...) { 
    if ( is.list(mat) ) { 
      result <- future_lapply_seed(mat, get_one_result, ...)
      names(result) <- names(mat)
      class(result) <- c('custom_sews_list', 'simple_sews_list',
                         'sews_result_list')
    } else { 
      result <- get_one_result(mat, ...)
    }
    return(result)
  }
  
}


#
#' @rdname create_indicator
#'
#' @param mat A matrix or a list of matrices. 
#' 
#' @param ... Additional arguments to pass to the function \code{fun}
#' 
#'@export
compute_indicator <- function(mat, fun, 
                              taskname = as.character(substitute(fun)), 
                              ...) { 
  indicfun <- create_indicator(fun, taskname = taskname)
  indicfun(mat, ...)
}

# 


# 
context('Test that methods are properly listed') 

test_that("Methods are OK", { 
  
  
  # Test variogram sews
  a <- variogram_sews(serengeti[1:2])
  
  # For a list
  mets <- list_methods(class(a))
  expect_true({ 
    all(c('plot_variogram', 'plot', 'predict', 'extract_variogram',
          'indictest') %in% mets)
  })
  
  # For a single element
  mets <- list_methods(class(a[[1]]))
  expect_true({ 
    all(c('plot_variogram', 'predict', 'extract_variogram') %in% mets)
  })
  expect_true({ 
    ! c('plot') %in% mets
  })
  
  
  
  # Test spectral sews
  a <- spectral_sews(serengeti[1:2], c(0, .2), c(.8, 1))
  mets <- list_methods(class(a))
  expect_true({ 
    all(c('display_matrix', 'plot', 'indictest', 'plot_spectrum') %in% mets)
  })
  mets <- list_methods(class(a[[1]]))
  expect_true({ 
    ! 'plot' %in% mets
  })
  
  
  
  # Test simple sews
  a <- compute_indicator(serengeti[1:2], 
                         function(mat) mean(mat), 
                         taskname = "Test indic")
  mets <- list_methods(class(a))
  expect_true({ 
    all(c('display_matrix', 'plot', 'indictest', 'as.data.frame') %in% mets)
  })
  mets <- list_methods(class(a[[1]]))
  expect_true({ 
    ! 'plot' %in% mets
  })
  
  
  
  # Test patchdistr sews
  a <- patchdistr_sews(serengeti[1:2])
  mets <- list_methods(class(a))
  expect_true({ 
    all(c('display_matrix', 'plot', 'indictest', 'as.data.frame') %in% mets)
  })
  mets <- list_methods(class(a[[1]]))
  expect_true({ 
    ! 'plot' %in% mets
  })
  
})

# 
# 
# 

context("Test that matrices can be displayed")

isgg <- function(o) { 
  inherits(o, c("ggplot", "ggplot2::ggplot"))
}

test_that("display_matrix methods work", { 
  
  # Test the matrix methods
  expect_true({ 
    a <- display_matrix(serengeti[[length(serengeti)]])
    isgg(a)
  })
  expect_true({ 
    a <- display_matrix(coarse_grain(serengeti[[length(serengeti)]], 5))
    isgg(a)
  })
  
  # Test the sews methods
  sets <- suppressWarnings({ # spectral_sews will produce warnings
    list(spectral_sews(serengeti[2:3]), 
         compute_indicator(serengeti[2:3], raw_moran))
  })
  
  for ( a in sets ) { 
    expect_true({ 
      all(c(isgg( display_matrix(a, along = NULL) ), 
            isgg( display_matrix(a, along = letters[2:3]) ), 
            isgg( display_matrix(a[[1]]) )))
    })
    
    expect_true({
      all(c(isgg( display_matrix(indictest(a, nulln = 3), along = NULL) ), 
            isgg( display_matrix(indictest(a, nulln = 3), 
                                 along = letters[2:3]) ), 
            isgg( display_matrix(indictest(a, nulln = 3)[[1]]) )))
    })
  }
  
})

context('Test the creation of custom indicators')

data(forestgap)
data(serengeti)

datasets <- list(forestgap[3:4], 
                 forestgap[1:2])

test_methods <- function(teststring, datalength, obj) { 
  
  ok_print <- any(grepl(teststring, capture.output(print(obj))))
  expect_true(ok_print)
  
  ok_summary <- any(grepl(teststring, capture.output(summary(obj))))
  expect_true(ok_summary)
  
  ok_as_df <- nrow(as.data.frame(obj)) == datalength
  expect_true(ok_as_df)
  
  return(TRUE)
}

test_that('Custom indicators work', { 
  skip_on_cran()
  
  for (dataset in datasets) { 
    # Run a classical workflow and make sure there are no errors
    testindic <- function(mat) { 
      mat[1, 1]
    }
    
    indicator_mp <- create_indicator(testindic)
    a <- indicator_mp(dataset)
    
    # Test methods for custom indics
    # Several matrices
    test_methods("Spatial Early-Warning:", length(dataset), a)
    test_methods("Spatial Early-Warning:", 1, a[[1]])
    
    if ( length(dataset) > 1 ) { 
      # Suppress the warnings related to missing values in geom_path
      suppressWarnings( plot(a) )
    }
    
    indictest(a[[1]], nulln = 9)
    
    b <- indictest(a, nulln = 9)
    test_methods("Spatial Early-Warning:", length(dataset), b)
    test_methods("Spatial Early-Warning:", 1, b[[1]])
    
    if (length(dataset) > 1) { 
      # Suppress the warnings related to missing values in geom_path
      suppressWarnings( plot(b) )
    }
    
  }
  
  # Test that create_indicator and compute_indicator work the same
  indicator_mp <- create_indicator(testindic)
  a <- indicator_mp(dataset)
  expect_true({ 
    all.equal(a, compute_indicator(dataset, fun = testindic))
  })
  
})


test_that('Custom indicators handles anonymous functions correctly', { 
  # Test create_indicator with anonymous function 
  expect_warning(
    anon_fun_indic <- create_indicator(function(mat) mean(mat))
  )
  expect_true({
    anon_fun_indic <- create_indicator(function(mat) mean(mat), 
                                       taskname = "TestTask")
    TRUE
  })
})

# 
# The functions below fit the Lifshitz, Slyozov, Wagner (LSW) distribution to 
# artificially generated patch sizes. Patch size distributions are expected to 
# approach the LSW distribution as cover goes to zero (Siteur et al., 2023).
# 
# Note that the patch sizes represent the patch radii, and not the patch area.
# Patch radius can be estimated by assuming circular patches r=sqrt(a/2pi).
# 
# LSW does not fit well to distributions with a positive skewness, as patch radii 
# greater than 1.5 times the mean patch size have zero probability.
# 
# 
# 
# 
# 
#' @title Indicators based on the LSW distribution
#'
#' @description LSW indicators for systems with density-dependent aggregation 
#'
#' @param mat A logical matrix (\code{TRUE}/\code{FALSE} values) or a list of such
#'   matrices
#' 
#' @param wrap Determines whether patches are considered to wrap around the 
#'   matrix when reaching on of its edges
#' 
#'@details 
#' 
#' In systems where a mobile resource or consumer can be fixed in space by a sessile 
#' species, specific patterns are expected to appear. Such systems can include situations 
#' where nutrients available in the environment are fixed by a sessile species (e.g.
#' seagrasses or corals), or where the behavior of herbivores is altered to restrict 
#' them to certain areas (see full theoretical background in Siteur et al. 2023). 
#' 
#' In those systems, as environmental conditions change and the global density of 
#' the sessile species decreases, its spatial structure is expected to change. The area 
#' of patches of the sessile species (as measured by their radii, which assumes circular
#' patches), is expected to go from a log-normal to a  Lifshitz–Slyozov–Wagner (LSW)
#' distribution. Thus, measuring how close the observed distribution of radii are to 
#' those two candidate distributions can constitute an indicator of ecosystem 
#' degradation. 
#' 
#' This function measures this through the relative support based on AIC for the two 
#' distributions (equal to 1 when the empirical distribution is best-approximated by an 
#' LSW, and 0 when it is a log-normal distribution (\code{\link{dlnorm}}), and the 
#' skewness of the observed patch radii, which should approach a value around -0.92 as
#' conditions worsen.  
#' 
#' @author This code has received contributions from Koen Siteur
#' 
#' @return 
#' 
#' \code{lsw_sews} returns an object of class \code{simple_sews_single}
#'   (a list) if \code{mat} is a single matrix or an object of class 
#'   \code{simple_sews_list} if \code{mat} is a list. You probably want to use some  
#'   of the methods written for these complicated objects instead of extracting 
#'   values directly (they are displayed using \code{print(<object>)}). 
#' 
#' @seealso \code{\link{dLSW}}, \code{\link{dda}},
#'   \code{\link{raw_patch_radii_skewness}}, \code{\link{raw_lsw_aicw}}
#'
#' @references 
#'  
#' Siteur, Koen, Quan-Xing Liu, Vivi Rottschäfer, Tjisse van der Heide, Max Rietkerk, 
#' Arjen Doelman, Christoffer Boström, and Johan van de Koppel. 2023. 
#' "Phase-Separation Physics Underlies New Theory for the Resilience of Patchy 
#' Ecosystems." Proceedings of the National Academy of Sciences 120 (2): e2202683120.
#' https://doi.org/10.1073/pnas.2202683120.
#' 
#  # TODO other relevant papers ? 
#'
#' @examples 
#'
#' data(dda)
#' data(dda.pars)
#' 
#' # Compute all indicators at once (skewness and relative AIC support)
#' indics <- lsw_sews(dda)
#' plot(indics, along = dda.pars[ ,"tau"]) 
#' 
#' # Compute individual indicators 
#' 
#' # Skewness of the distribution of patch radii
#' radii_skewness <- compute_indicator(dda, raw_patch_radii_skewness)
#' plot(radii_skewness, along = dda.pars[ ,"tau"])
#' 
#' # Aic weight of LSW distribution relative to a lognormal distribution. tau here 
#' # represents the density at equilibrium in Siteur et al's model (2023)
#' lsw_aicw <- compute_indicator(dda, raw_lsw_aicw)
#' plot(lsw_aicw, along = 1 - dda.pars[ ,"tau"])
#' 
#'@export
lsw_sews <- function(mat, wrap = FALSE) { 
  compute_indicator(mat, internal_lsw_sews, 
                    wrap = wrap, 
                    taskname = "LSW-based indicators")
}

internal_lsw_sews <- function(mat, wrap) {
  # Convert object to matrix form (or list of matrices) and check if it is
  # suitable for spatialwarnings
  mat <- convert_to_matrix(mat)
  if ( is.list(mat) ) { 
    lapply(mat, check_mat)
  } else { 
    check_mat(mat)
  }
  
  # Compute metrics
  radii <- get_patch_radii(mat, wrap)
  skewness <- cpp_skewness(radii)
  aicw <- get_lsw_aicw(radii)
  
  # For now we just return the numerical values. We will need something a bit more 
  # fancy to have distributions fitting et al. 
  c(cover = mean(mat), 
    skewness = skewness, 
    lsw_prob = aicw)
}

#'@rdname lsw_sews
#'@export
raw_patch_radii_skewness <- function(mat, wrap = FALSE) { 
  radii <- get_patch_radii(mat, wrap)
  c(psd_radii_skewnes = cpp_skewness(radii))
}

#'@rdname lsw_sews
#'@export
raw_lsw_aicw <- function(mat, wrap = FALSE) { 
  # Get patch size distribution. Assume circle patches from the area. 
  radii <- get_patch_radii(mat, wrap)
  aicw <- get_lsw_aicw(radii)
  return( c(lsw_prob = aicw) )
}

get_patch_radii <- function(mat, wrap = FALSE) { 
  # Get patch size distribution. Assume circle patches from the area. 
  psd <- patchsizes(mat, wrap = wrap)
  radii <- sqrt(psd / pi)
}

get_lsw_aicw <- function(radii) {
  fit_lsw <- LSW_fit(radii)
  
  # Fit a lognormal
  fit_lnorm <- fit_lnorm(radii)
  
  # Compute AIC weight of LSW relative to lnorm 
  minaic <- min(fit_lsw[["AIC"]], fit_lnorm[["AIC"]])
  weight_lsw_min   <- exp( - 0.5 * ( fit_lsw[["AIC"]]   - minaic ) ) 
  weight_lnorm_min <- exp( - 0.5 * ( fit_lnorm[["AIC"]] - minaic ) ) 
  
  aicw <- weight_lsw_min / ( weight_lsw_min + weight_lnorm_min )
  
  return(aicw)
}

fit_lnorm <- function(xs) { 
  
  negll <- function(theta) { 
    - sum( dlnorm(xs, meanlog = theta[1], sdlog = theta[2], log = TRUE) )
  }
  
  # TODO: check that this makes sense and I haven't been confused by the logs, also 
  # this is probably useless (MLE = mean of log / sd of log? )
  mle <- optim(par = c(mean(log(xs)), 
                       sd(log(xs))), 
               fn = negll) 
  theta_est <- mle[["par"]]
  
  aic <- 2 - 2 * ( - negll(theta_est) ) 
  list(meanlog = theta_est[1], 
       sdlog   = theta_est[2], 
       AIC     = aic)
}

# The Lifshitz, Slyozov, Wagner (LSW) probability density function. Used for plotting, 
# and for computation of the AIC.
# 
#' @title The Lifshitz-Slyozov-Wagner distribution 
#'
#' @description Density and distribution function for the Lifshitz-Slyozov-Wagner (LSW)
#'   distribution with mean mu. 
#'
#' @param x vector of quantiles 
#'
#' @param mu the mean of the distribution
#'
#' @param log,log.p logical; if \code{TRUE}, probabilities p are given as log(p)
#'
#' @param lower.tail logical; if TRUE (default), probabilities are
#'         \eqn{P[X \le x]} otherwise, \eqn{P[X > x]}
#' 
#' @details 
#' 
#' The LSW distribution is a continuous distribution with density
#' 
#' \deqn{ 
#'   f(x) = \frac{4x^2}{9\mu^3} ( \frac{3\mu}{3\mu + x} )^{7/3} 
#'            ( \frac{3\mu}{3\mu - 2x} )^{11/3} e^{\frac{2x}{2x - 3\mu}} 
#' }
#' 
#' where \eqn{\mu} is the mean of the distribution. 
#' 
#' The functions \code{dLSW} gives the probability density, \code{pLSW} gives the 
#' distribution function. \code{qLSW} and \code{rLSW} are not implemented. You 
#' can use \code{LSW_fit} to fit an LSW distribution to a set of observations. 
#' 
#' The length of the results is determined by the length of \code{x}, and \code{mu} can 
#' only be a single value. 
#' 
#' Please note that this distribution has support only on the interval \code{[0,mu*3/2)}. 
#'   Probabilities outside this interval are returned as 0. 
#'
#' @references 
#
#TODO: add reference to distribution
#
#' Siteur, Koen, Quan-Xing Liu, Vivi Rottschäfer, Tjisse van der Heide, Max Rietkerk, 
#' Arjen Doelman, Christoffer Boström, and Johan van de Koppel. 2023. 
#' "Phase-Separation Physics Underlies New Theory for the Resilience of Patchy 
#' Ecosystems." Proceedings of the National Academy of Sciences 120 (2): e2202683120.
#' https://doi.org/10.1073/pnas.2202683120.
#'
#' @return
#' dLSW gives the density, pLSW gives the distribution function, both as numerical 
#'   vectors determined by the length of \code{x}. 
#'
#' @seealso \code{\link{lsw_sews}}
#'
#' @examples
#' 
#' # Plot the density 
#' x <- seq(0, 10, l = 128) 
#' plot(x, dLSW(x, mu = 3), type = "l", col = "black")
#' lines(x, dLSW(x, mu = 5), type = "l", col = "red")
#' lines(x, dLSW(x, mu = 7), type = "l", col = "blue")
#' legend(x = 0, y = max(dLSW(x, mu = 3)), lty = 1, col = c("black", "red", "blue"), 
#'        legend = paste("mu =", c(3, 5, 7)))
#' 
#'@export
dLSW <- function(x, mu, log = FALSE) { 
  if ( length(mu) > 1 ) { 
    stop("dLSW does not support multiple values for mu")
  }
  
  zero_val <- ifelse(log, -Inf, 0)
  p <- rep(zero_val, length(x))
  x_ok <- which(x<1.5 * mu & x >= 0)
  x <- x[x_ok]
  
  if ( log ) { 
    p[x_ok] <- - 7 * log(x+3*mu) /3+2 * log(x) + 
                  2 * x / (2 * x -  3 * mu) - 
                  ( 11 * log(3 * mu - 2 * x) ) / 3 + 
                  3 * log(mu) + 
                  log(324)
  } else { 
    p[x_ok] <- (4*x^2/(9*mu^3)) * 
                 (3*mu/(3*mu+x))^(7/3) * 
                 (3*mu/(3*mu-2*x))^(11/3) * 
                 exp(2*x/(2*x-3*mu))
  }
  
  p
}

# The LSW distribution, but standard interface
#'@rdname dLSW 
#'
#'@export 
pLSW <- function(x, mu, lower.tail = TRUE, log.p = FALSE) { 
  
  # TODO: is there an analytical expression for the LSW distribution function ? 
  ans <- vapply(x, function(thisr) { 
    integrate(dLSW, lower = 0, upper = thisr, mu = mu, log = FALSE, 
              subdivisions = 512)[["value"]]
  }, numeric(1))
  
  if ( ! lower.tail ) { 
    ans <- 1 - ans
  }
  
  if ( log.p ) { 
    ans <- log(ans)
  }
  
  ans
}

# # The derivative of the log likelihood function to parameter mu. Used to estimate mu
# dLdmu <- function(r, mu) {
#   n <- length(r)
#   -3*n+(7/3)*sum(r/(3*mu+r))-(11/3)*sum(2*r/(3*mu-2*r))+3*mu*sum(2*r/(2*r-3*mu)^2)
# }

# Function to fit the LSW probability density function. Returns the estimate of mu and 
# the AIC for model comparison
#'@rdname dLSW
#'@export
LSW_fit <- function(x) {
  
  negll <- function(mu) { 
    nll <- - dLSW(x, mu, log = TRUE) 
    if ( any( ! is.finite(nll) ) ) { 
      return(Inf)
    }
    sum(nll)
  }
  
  # Here we want to stick to values so that all r < 3mu/2 otherwise the log 
  # likelihood is infinite, which means max(r) < 3mu/2, 
  # i.e  mu > 2*max(r)/3. We find the smallest value that produces a finite negll and 
  # use that as our lower bound for LL minimization later on. 
  boundsearch <- uniroot(f = function(x) is.finite(negll(x)) - 0.5, 
                         lower = min(x), 
                         upper = max(x))
  minbound <- boundsearch[["root"]] + boundsearch[["estim.prec"]]
  
  est <- optim(par = max(x), 
               lower = minbound, 
               upper = max(x), 
               method = "L-BFGS-B", 
               fn = negll)
  
  mu_fit <- est[["par"]]
  AIC_fit <- 2 - 2 * ( - est[["value"]] )
  
  return(list(mu = mu_fit, 
              AIC = AIC_fit))
}

# 
# Test the implementation of the LSW distribution
# 


test_that("LSW distribution works", { 
  
  cLSW <- function(r, mu) {
    x <- seq(0, 1.5, 0.01) * mu
    PDF <- dLSW(x, mu)
    CDF <- cumsum(PDF) / sum(PDF)
    y <- approx(x, CDF, r)[["y"]]
    
    return(y)
  }

  rs <- seq(0, 20, l = 512)
  mus <- seq(1, 10, l = 5)
  
  # Multiple values for mu is not supported
  expect_error({ 
    dLSW(rs, mus)
  })
  
  for ( mu in mus ) { 
    
    # Make sure log of dLSW is correct 
    logds <- log(dLSW(rs, mu))
    dslog <- dLSW(rs, mu, log = TRUE)
    
    # Compare only values where the log(p) is finite
    expect_true({ 
      all( na.omit(abs(logds[is.finite(logds)] - dslog[is.finite(logds)]))  < 1e-10 )
    })
    
    # Make sure pLSW works
    ps <- pLSW(rs, mu)
    psinv <- pLSW(rs, mu, lower.tail = FALSE)
    expect_true({ 
      all( abs(ps - ( 1 - psinv )) < 1e-10)
    })
    
    # 
    pslog <- pLSW(rs, mu, log.p = TRUE)
    diffs <- suppressWarnings({ 
      ifelse(!(ps > 0), 0, log(ps) - pslog)
    })
    expect_true({ 
      all( abs(diffs) < 1e-10)
    })
    
    # Compare with original functions. There is actually a pretty large difference, 
    # as the original one was quite approximate. We remove NAs that are returned 
    # by cLSW, and only compare values that can be compared.
    orig_ps <- cLSW(rs, mu)
    expect_true({ 
      all( na.omit(abs(ps - orig_ps)) < 0.1 )
    })
    
  }
  
})

test_that("LSW fitting recovers correct values", { 
  
  fits <- vapply(dda, function(m) { 
    LSW_fit(patchsizes(m))[["mu"]]
  }, numeric(1))
  
  # As tau increases, we expect mu to increase. This is a very light test that 
  # will only catch gross errors, but we are not able to produce random samples 
  # from LSW distrib, so we cannot do an in-depth test of the fitting. 
  cor <- cor.test(dda.pars[ ,"tau"], log(fits))[["estimate"]]
  expect_true({ 
    cor > 0
  })
  
})

test_that("LSW indicators produce consistent values", { 
  
  
  m <- list(diag(100)*rnorm(10), 
            diag(100)*rnorm(10))
  
  # Does not work with non-logical matrices. Note the call() to suppressWarnings,
  # because this will produce an error in a future(), which makes future_lapply()
  # spit out a warning
  expect_error({ 
    suppressWarnings( lsw_sews(m) )
  })
  
  
  m <- dda[1:2]
  
  lapply(c(TRUE, FALSE), function(wrap) { 
    ic <- as.data.frame(lsw_sews(m, wrap = wrap))
    
    # Check that results are consistent with individual functions
    ic_ref <- unlist(lapply(m, function(mc) { 
      c(mean(mc), raw_patch_radii_skewness(mc, wrap = wrap), raw_lsw_aicw(mc, wrap = wrap)) 
    }))
    
    expect_true(
      all(abs(ic_ref - ic[ ,"value"]) < 1e-8)
    )
  })
  
})

# 
# These functions factorize some code between numerical indicators. Most 
# notably it handles creating the null model testing, etc.
# 
# This function will compute a null distribution of indicator values on input, 
# by reshuffling it nulln times. 
# 
compute_indicator_with_null <- function(input, 
                                        nulln, 
                                        indicf, 
                                        null_method, 
                                        null_control) { 
  
  # Create null_control list with default arguments 
  null_control <- null_control_set_args(input, null_control, null_method)
  
  # Compute the observed value
  value  <- indicf(input)
  result <- list(value = value)
  
  if ( nulln > 2 ) { 
    
    null_results <- generate_nulls(input, indicf, nulln, 
                                   null_method, null_control)
    
    # nulldistr is a list so far => combine it to a matrix, with each null 
    # replicate as a row
    null_results[["nulldistr"]] <- do.call(rbind, null_results[["nulldistr"]]) 
    
    # Compute summary stats
    summ_stats <- 
      list(null_mean = apply(null_results[["nulldistr"]], 2, mean), 
           null_sd   = apply(null_results[["nulldistr"]], 2, sd), 
           null_qsup = apply(null_results[["nulldistr"]], 2, safe_quantile,
                             null_control[["qsup"]]), 
           null_qinf = apply(null_results[["nulldistr"]], 2, safe_quantile,
                             null_control[["qinf"]]), 
           z_score   = apply(rbind(value, null_results[["nulldistr"]]), 2, 
                             function(X) (X[1] - mean(X[-1])) / sd(X[-1])),
           pval      = apply(rbind(value, null_results[["nulldistr"]]), 2, 
                             function(X) 1 - rank(X)[1] / length(X)))
    
    # If null_method is a function, we need to convert it to a sensible string 
    # for display in summary(), etc. 
    if ( is.function(null_method) ) { 
      null_method <- "custom function"
    }
    
    # Add to that list what is returned from null + some stats
    result <- list(nulldistr = null_results[["nulldistr"]], 
                   summary_values = summ_stats, 
                   info = list(null_method = null_method, 
                               nulln = nulln, 
                               get_nullmat = null_results[["get_nullmat"]]))
    
  }
  
  return(result)
}

# Returns a list with the values of indicf obtained from random matrices + 
# a function used to generate the nulls
generate_nulls <- function(input, indicf, nulln, null_method, 
                           null_control) {
                             
  if ( ( !is.function(null_method) ) && 
       ( ! null_method %in% c("perm", "intercept", "smooth") ) ) { 
    stop(paste0("Unknown null model method: ", null_method))
  }
  
  # Nulldistr is null as long as we've not computed it yet
  has_computed_nulldistr <- FALSE
  
  
  
  
  # If the user provided a function to generate null matrices
  if ( is.function(null_method) ) { 
    
    # We warn if the function does not return logical values 
    if ( is.logical(input) && ! is.logical(null_method(input)) ) { 
      warning("The original data is logical (TRUE/FALSE), but the null ", 
              "method does not return logical values")
    }
    
    null_mod <- NULL # No model involved when the function is provided
    get_nullmat <- function() { null_method(input) }
  }
  
  
  
  
  # If the null method is set to permutation
  if ( is.character(null_method) && null_method == "perm" ) { 
    # Compute the index on a randomized matrix. Here we use a dedicated 
    # function that can do the shuffling in-place in C++, instead of relying 
    # on R which copies matrices in memory many times. 
    # 
    # shuffle_and_compute will convert all matrices to numeric matrices 
    # internally. We need to explicitely convert back to logical after 
    # shuffling before computing the indicator. 
    if ( is.logical(input) ) { 
      nulldistr <- shuffle_and_compute(input, function(x) indicf(x>0), 
                                       nulln)
    } else { 
      nulldistr <- shuffle_and_compute(input, indicf, nulln)
    }
    has_computed_nulldistr <- TRUE
    null_mod <- NULL # No model involved when we are permuting
    get_nullmat <- function() { 
      if ( is.logical(input) ) { 
        a <- shuffle_matrix(input) > 0.5 
      } else { 
        a <- shuffle_matrix(input) 
      }
      # Pass attributes to the new, random matrix
      attributes(a) <- attributes(input)
      return(a)
    }
  }
  
  
  
  
  # If the null method is to fit an intercept only 
  if ( is.character(null_method) && null_method == "intercept" ) { 
    
    values <- as.vector(input)
#     sub <- select_subset(length(values), null_control[["model_subset"]], 
#                          min(length(values), 512))
    null_mod <- glm(values ~ 1, family = null_control[["family"]])
    get_nullmat <- create_nullmat_generator(input, 
                                            null_mod, 
                                            null_control[["family"]])
  }
  
  
  
  # If the null method is to fit a smoothing spline to the matrix
  if ( is.character(null_method) && null_method == "smooth" ) { 
    
    if ( ! requireNamespace("mgcv", quietly = TRUE) ) { 
      stop("The 'gam' method requires mgcv. Please install it beforehand.")
    }
    
    # We fit a smoothing isotropic spline over the matrix, with automatic
    # parameter selection for the spline. 
    mat_tab <- data.frame(expand.grid(row = seq.int(nrow(input)), 
                                      col = seq.int(ncol(input))), 
                          value = as.vector(input))
#     sub <- select_subset(nrow(mat_tab), 
#                          null_control[["model_subset"]], 
#                          min(nrow(mat_tab), 512))
#     mat_tab <- mat_tab[sub, ]
    null_mod <- mgcv::gam(value ~ s(row, col, bs = "tp"), 
                          data = mat_tab, 
#                           control = mgcv::gam.control(trace = TRUE), 
                          family = null_control[["family"]])
    get_nullmat <- create_nullmat_generator(input, 
                                            null_mod, 
                                            null_control[["family"]])
  }
  
  
  
  # Compute the actual indicator values in the null distribution (if that's 
  # not already done)
  if ( ! has_computed_nulldistr ) { 
    nulldistr <- lapply(seq.int(nulln), function(n) { 
      indicf(get_nullmat())
    })
  }

  list(nulldistr   = nulldistr, 
       get_nullmat = get_nullmat, 
       null_mod    = null_mod)
}

# We use a safe version of quantile that reports as warnings 
# the appearance of NAs in the null distribution.
safe_quantile <- function(nulldistr, p) { 
  if ( any( is.na(nulldistr) ) ) { 
    warning(paste0('Computation of null values produced NAs (', 
                   sum(is.na(nulldistr)), " out of ", length(nulldistr), "). "))
  }
  quantile(nulldistr, p, na.rm = TRUE)
}

# Returns TRUE if fam is a binomial() object
is.binomial <- function(fam) { 
  ( !is.list(fam) && fam == "binomial") || 
    ( inherits(fam, "family") && fam[["family"]] == "binomial" )
}

# Set default arguments for null methods 
null_control_set_args <- function(mat, arglist, null_method) { 
  
  # Choose a sensible default if family is unset. Note that if null_method can 
  # be a function that the user provided. 
  if ( ! "family" %in% names(arglist) ) { 
    
    if ( is.logical(mat) ) { 
      family <- binomial()
      if ( is.character(null_method) && 
           null_method %in% c("smooth", "intercept") ) { 
        warning("indictest: using a binomial() family with default options")
      }
      
    } else { 
      family <- gaussian()
      if ( is.character(null_method) && 
           null_method %in% c("smooth", "intercept") ) { 
        warning("indictest: using a gaussian() family with default options")
      }
    }
    
  } 
  
  args <- list(family = family, 
               qinf = .05, 
               qsup = .95, 
               model_subset = 0.1) # Add other arguments here
  
  for ( i in seq_along(arglist) ) { 
    if ( names(arglist)[i] %in% names(args) ) { 
      args[[ names(arglist)[i] ]] <- arglist[[i]]
    } else { 
      warning("Unknown null model control argument: ", names(arglist)[i])
    }
  }
  
  return(args)
}

# Returns a function that generates matrices, given a null model and the 
# original matrix
create_nullmat_generator <- function(mat, null_mod, family) { 
  function() { 
    newdat <- expand.grid(row = seq.int(nrow(mat)), 
                          col = seq.int(ncol(mat)))
    
    # When the family is binomial, simulate often returns 1/0 instead of 
    # TRUE/FALSE values so we need to convert it back here. 
    sim <- matrix(stats::simulate(null_mod, newdata = newdat)[ ,1], 
                  nrow = nrow(mat), ncol = ncol(mat)) 
    if ( is.binomial(family) ) { 
      sim <- sim > .5
    }
    # Transfer matrix attributes
    attributes(sim) <- attributes(mat)
    return(sim)
  }
}

# # Select a subset of values in N values We take the maximum of at_least points 
# # and the fraction of the N values asked for
# select_subset <- function(N, subset_frac, at_least) { 
#   if ( N <= at_least ) { 
#     return( rep(TRUE, N) )
#   }
#   
#   keep_every <- min(floor(N / at_least), 
#                     floor(1 / subset_frac))
#   seq.int(N) %% keep_every == 0
# }

# 
# # Here newdata is assumed to have columns and x, y
# simulate_newdat <- function(mod, newdata, family) { 
#   
#   # If we passed a vector of values, convert that to a data.frame so it is 
#   # compatible with predict()
#   if ( is.vector(newdata) && !is.data.frame(newdata) ) { 
#     newdata <- data.frame(blank = newdata)
#   }
#   n <- nrow(newdata)
#   
#   # Extract the string from family object if needed
#   fam <- family$family
#   linv <- family$linkinv 
#   
#   if ( fam == "gaussian" ) { 
#     mu_pred <- predict(mod, newdata, type = "response")
#     resp_pred <- rnorm(n, mu_pred, sigma(mod))
#   } else if ( fam == "binomial" ) { 
#     mu_pred <- predict(mod, newdata, type = "response")
#     resp_pred <- rbinom(n, size = 1, prob = mu_pred)
#   } else if ( fam == "poisson" ) { 
#     lambda_pred <- predict(mod, newdata, type = "response")
#     resp_pred <- rpois(n, lambda = lambda_pred)
#   } else { 
#     stop("Family ", fam, " is not handled by this function")
#   }
#   
#   return(resp_pred)
# }

# 
# 
# 

context("Test that all null model methods work") 

# Here we just run the code to check that it works
test_that("All null model methods work", { 
  
  # We need to increase the max number of iterations otherwise warnings are produced
  options(spatialwarnings.constants.maxit = 10^(8.5))
  options(spatialwarnings.constants.reltol = 1e-8)
  
  # Check that all methods run 
  all_methods <- c("perm", "intercept", "smooth") 
  testmat <- matrix(runif(30*30) > .7, ncol = 30)
  testmat[1:15, ] <- TRUE
  
  dat <- list(testmat, testmat)
  a <- generic_sews(dat)
  b <- patchdistr_sews(dat)
  c <- suppressWarnings( spectral_sews(dat) )
  null_control <- list(family = binomial())
  for ( m in all_methods ) { 
    indictest(a, nulln = 3, null_method = m, 
              null_control = null_control) 
    indictest(b, nulln = 3, null_method = m, 
              null_control = null_control) 
    indictest(c, nulln = 3, null_method = m, 
              null_control = null_control) 
    expect_true(TRUE)
  }
  
  # Check the values returned by the null model 
  ictest <- indictest(a[[2]], 3, null_method = "intercept", 
                      null_control = null_control)
  nullmean <- mean(replicate(99, mean( ictest$get_nullmat() )))
  expect_equal(mean(ictest[["orig_data"]]), nullmean, 
               tol = 0.01)
  
  # Check the values returned by the null model 
  ictest <- indictest(a[[2]], 3, null_method = "smooth", 
                      null_control = null_control)
  nullmean <- mean(replicate(99, mean( ictest$get_nullmat() )))
  expect_equal(mean(ictest[["orig_data"]]), nullmean, 
               tol = 0.01)
  
  # Check that smoothed null model is closer to reality than the intercept
  # model
  ictest <- indictest(a[[2]], 3, null_method = "intercept", 
                      null_control = null_control)
  error_intercept <- replicate(49, { 
    mean( abs(ictest$get_nullmat() - a[[2]][["orig_data"]]) )
  })
  ictest <- indictest(a[[2]], 3, null_method = "smooth", 
                      null_control = null_control)
                      
  error_smooth <- replicate(49, { 
    mean( abs(ictest$get_nullmat() - a[[2]][["orig_data"]]) )
  })
  expect_true({ 
    mean(error_intercept) > mean(error_smooth)
  })
  
  # Check that we warn when the null method is a function and does not return 
  # logical values when the input matrix is logical 
  nullfun <- function(mat) { mat * rnorm(prod(dim(mat))) } 
  expect_warning({ 
    indictest(compute_indicator(serengeti[[1]], raw_cg_moran), 3, 
              null_method = nullfun)
  })
  
  # Check that arguments are properly set 
  expect_warning({ 
    null_control_set_args(serengeti[[1]], list(a = "opl"), "perm")
  })
  expect_true({ 
    a <- null_control_set_args(serengeti[[1]], 
                               list(family = "binomial"), 
                               "intercept")[["family"]]
    is.binomial(a)
  }) 
  expect_true({ 
    a <- null_control_set_args(serengeti[[1]], 
                               list(family = binomial()), 
                               "intercept")[["family"]]
    is.binomial(a)
  }) 
  
  # Model option are honored
  expect_warning({
      null_control_set_args(serengeti[[1]], list(), "intercept")
  }, regexp = "using a binomial")
  expect_warning({
    null_control_set_args(coarse_grain(serengeti[[1]], 5), 
                          list(), "intercept")
  }, regexp = "using a gaussian")
  
  # Set options to their default
  options(spatialwarnings.constants.maxit = NULL)
  options(spatialwarnings.constants.reltol = NULL)
  
  # Test the simulation of new values 
  
#   # Intercept-only models 
#   mu <- 10
#   dat <- rnorm(1000, mu, 2)
#   gmod <- glm(y ~ 1, data = data.frame(y = dat), family = gaussian())
#   newvals <- simulate_newdat(gmod, seq(0, 10000), family = gaussian())
#   expect_equal(mu, mean(newvals), tol = 0.1)
#   
#   lambda <- 10
#   dat <- rpois(1000, lambda)
#   gmod <- glm(y ~ 1, data = data.frame(y = dat), family = poisson())
#   newvals <- simulate_newdat(gmod, seq(0, 10000), family = poisson())
#   expect_equal(lambda, mean(newvals), tol = 0.1)
#   
#   prob <- 0.4
#   dat <- rbinom(1000, size = 1, prob = prob)
#   gmod <- glm(y ~ 1, data = data.frame(y = dat), family =  binomial())
#   newvals <- simulate_newdat(gmod, seq(0, 10000), family = binomial())
#   expect_equal(prob, mean(newvals), tol = 0.1)
#   
#   
#   # Helper function for whats following
#   matricize <- function(tab) { 
#     matrix(tab[ ,3], nrow = max(tab[ ,1]), ncol = max(tab[ ,1]))
#   }
#   
#   
#   # Smooth models: binomial
#   input <- serengeti[[7]]
#   # parameter selection for the spline. 
#   mat_tab <- data.frame(expand.grid(row = seq.int(nrow(input)), 
#                                     col = seq.int(ncol(input))), 
#                         value = as.vector(input))
#   null_mod <- mgcv::gam(value ~ s(row, col, bs = "tp"), 
#                         data = mat_tab, 
#                         family = binomial())
#   ovals <- expand.grid(row = seq.int(nrow(input)), 
#                       col = seq.int(ncol(input)))
#   newvals <- simulate_newdat(null_mod, 
#                             ovals, 
#                             family = binomial())
#   ovals[ ,"y"] <- newvals
#   ovals[ ,"yref"] <- simulate(null_mod)[, 1]
#   ggplot(ovals, aes(x = row, y = col)) + 
#     geom_raster(aes(fill = y > 0.5))
#   
#   m    <- coarse_grain(matricize(ovals[ ,c("row", "col", "y")]),    20)
#   mref <- coarse_grain(matricize(ovals[ ,c("row", "col", "yref")]), 20)
#   expect_true({ 
#     mean( abs(m - mref) ) < 0.05
#   })
#   
#   if ( exists("VISUAL_TESTS") && VISUAL_TESTS ) { 
#     display_matrix(matricize(ovals[ ,c("row", "col", "y")])) + 
#       labs(title = "produced")
#     dev.new()
#     display_matrix(matricize(ovals[ ,c("row", "col", "yref")])) + 
#       labs(title = "ref")
#   
#   }
#   
#   
#   # Smooth models: gaussian
#   input <- matrix(ifelse(serengeti[[7]][], 
#                          rnorm(prod(dim(serengeti[[7]])), mean = 20 , sd = 8), 
#                          rnorm(prod(dim(serengeti[[7]])), mean = 100, sd = 8)), 
#                   nrow = nrow(serengeti[[7]]), 
#                   ncol = ncol(serengeti[[7]]))
#   # parameter selection for the spline. 
#   mat_tab <- data.frame(expand.grid(row = seq.int(nrow(input)), 
#                                     col = seq.int(ncol(input))), 
#                         value = as.vector(input))
#   null_mod <- mgcv::gam(value ~ s(row, col, bs = "tp"), 
#                         data = mat_tab, 
#                         family = gaussian())
#   ovals <- expand.grid(row = seq.int(nrow(input)), 
#                        col = seq.int(ncol(input)))
#   newvals <- simulate_newdat(null_mod, 
#                              ovals, 
#                              family = gaussian())
#   ovals[ ,"y"] <- newvals
#   ovals[ ,"yref"] <- simulate(null_mod)[, 1]
#   
#   m    <- coarse_grain(matricize(ovals[ ,c("row", "col", "y")]),    20)
#   mref <- coarse_grain(matricize(ovals[ ,c("row", "col", "yref")]), 20)
#   expect_true({ 
#     cor(as.vector(m), as.vector(mref)) > 0.95
#   })
#   expect_equal(var(ovals$y), var(ovals$yref), tol = 4)
#   
#   if ( exists("VISUAL_TESTS") && VISUAL_TESTS ) { 
#     display_matrix(matricize(ovals[ ,c("row", "col", "y")])) + 
#       labs(title = "produced")
#     dev.new()
#     display_matrix(matricize(ovals[ ,c("row", "col", "yref")])) + 
#       labs(title = "ref")
#   
#   }
#   
#   
#   
#   
#   # Smooth models: poisson
#   input <- matrix(ifelse(serengeti[[7]][], 
#                          rpois(prod(dim(serengeti[[7]])), lambda = 20 ), 
#                          rpois(prod(dim(serengeti[[7]])), lambda = 100)), 
#                   nrow = nrow(serengeti[[7]]), 
#                   ncol = ncol(serengeti[[7]]))
#   # parameter selection for the spline. 
#   mat_tab <- data.frame(expand.grid(row = seq.int(nrow(input)), 
#                                     col = seq.int(ncol(input))), 
#                         value = as.vector(input))
#   null_mod <- mgcv::gam(value ~ s(row, col, bs = "tp"), 
#                         data = mat_tab, 
#                         family = gaussian())
#   ovals <- expand.grid(row = seq.int(nrow(input)), 
#                        col = seq.int(ncol(input)))
#   newvals <- simulate_newdat(null_mod, 
#                              ovals, 
#                              family = gaussian())
#   ovals[ ,"y"] <- newvals
#   ovals[ ,"yref"] <- simulate(null_mod)[, 1]
#   
#   m    <- coarse_grain(matricize(ovals[ ,c("row", "col", "y")]),    20)
#   mref <- coarse_grain(matricize(ovals[ ,c("row", "col", "yref")]), 20)
#   expect_true({ 
#     cor(as.vector(m), as.vector(mref)) > 0.95
#   })
#   expect_equal(var(ovals$y), var(ovals$yref), tol = 4)
#   
#   if ( exists("VISUAL_TESTS") && VISUAL_TESTS ) { 
#     display_matrix(matricize(ovals[ ,c("row", "col", "y")])) + 
#       labs(title = "produced")
#     dev.new()
#     display_matrix(matricize(ovals[ ,c("row", "col", "yref")])) + 
#       labs(title = "ref")
#   }
  
  
  
  
  
  
})

# 
# 
# img <- serengeti[[length(serengeti)]]
# img_coarse <- coarse_grain(img, 1)
# img_tab <- data.frame(expand.grid(row = seq.int(nrow(img_coarse)), 
#                                   col = seq.int(ncol(img_coarse))), 
#                       value = as.vector(img_coarse))
# 
# 
# # Test if trend 
# mod <- mgcv::gam(value ~ s(row, col, bs = "tp"), data = img_tab, 
#                  family = binomial())
# img_tab[ ,"pred"] <- predict(mod, type = "response")
# img_tab[ ,"sim"]  <- simulate(mod, type = "response") > .5
# 
# 
# 
# plot1 <- ggplot(img_tab) + 
#   geom_raster(aes(x = col, y = row, fill = value)) + 
#   scale_fill_brewer(palette = "Spectral") + 
#   coord_fixed() + 
#   theme_minimal() + 
#   labs(x = "x", y = "y", title = "Observed matrix")
# 
# plot2 <- ggplot(img_tab) + 
#   geom_raster(aes(x = col, y = row, fill = 1 - pred)) + 
#   scale_fill_distiller(palette = "Spectral", 
#                        name = "Cover", 
#                        direction = -1) + 
#   coord_fixed() + 
#   theme_minimal() + 
#   labs(x = "x", y = "y", title = "Fitted model")
# 
# plot3 <- ggplot(img_tab) + 
#   geom_raster(aes(x = col, y = row, fill = sim)) + 
#   scale_fill_brewer(palette = "Spectral") + 
#   coord_fixed() + 
#   theme_minimal() + 
#   labs(x = "x", y = "y", title = "One null matrix")
# 
# 
# 
# library(patchwork)
# plot1 + plot2 + plot3 + 
#   plot_layout(nrow = 1)
# 
# 
# # Create a null matrix 
# library(mgcv)
# library(memoise)
# fit_model <- memoise(function(data) { 
#   mgcv::gam(value ~ s(row, col, bs = "tp"), data = data, 
#             family = binomial())
# })
# fnull <- function(mat) { 
#   mat_tab <- data.frame(expand.grid(row = seq.int(nrow(mat)), 
#                                     col = seq.int(ncol(mat))), 
#                         value = as.vector(mat))
#   mod <- fit_model(data = mat_tab) 
#   mat[ , ] <- simulate(mod) > .5 
#   return(mat)
# }
# 
# indic <- compute_indicator(serengeti, raw_moran)
# test  <- indictest(indic, null_method = fnull, nulln = 99)
# plot(test, along = serengeti.rain)

#
# This file contains the indicator function based on the power-law range
#   (how much of the data behaves like a power law?)
#
#' @title Power-law range indicator
#' 
#' @description Compute the power-law range of a matrix 
#' 
#' @param mat A logical matrix, or a list of logical matrices 
#'
#' @param merge Controls whether the patch-size distributions of the input 
#'   matrices are merged together before computing the power-law range. Setting 
#'   this value to \code{TRUE} makes the function return a single value even 
#'   if multiple matrices are given as input. 
#'
#' @param xmin_bounds A vector of two integer values, defining a range in which 
#'   to search for the best xmin (see Details). 
#' 
#' @details 
#' 
#' Some ecosystems show typical changes in their patch-size 
#' distribution as they become more and more degraded. In particular, an 
#' increase in the truncation of the patch-size distribution (PSD) is expected 
#' to occur. The power-law range (PLR) measures the truncation of the PSD 
#' in a single value (see also \code{\link{patchdistr_sews}} for more details). 
#' 
#' To compute the PLR, power-laws are fitted with a variable 
#' minimum patch size (\eqn{x_{min}}) and the one with the lowest Kolmogorov-Smirnov
#' distance to the empirical distribution is retained. PLR is then computed 
#' using this best-fitting \eqn{x_{min}} as: 
#' 
#' \deqn{\frac{log(x_{max}) - log(x_{min})}{log(x_{max}) - log(x_{smallest})}}{ (log(xmax) - log(xmin))/(log(xmax) - log(xsmallest))}
#' 
#' where \eqn{x_{max}}{x_max} is the maximum observed patch size, and 
#' \eqn{x_{smallest}}{x_smallest} is the minimum observed patch size. 
#' 
#' @references 
#' 
#' Clauset, A., Shalizi, C. R., & Newman, M. E. (2009). 
#'   Power-law distributions in empirical data. SIAM review, 51(4), 661-703.
#' 
#' Berdugo, M., Kefi, S., Soliveres, S. & Maestre, F.T. (2017). Plant spatial 
#' patterns identify alternative ecosystem multifunctionality states in 
#' global drylands. Nature in Ecology and Evolution.
#' 
#' @return A \code{data.frame} with columns \code{minsize}, \code{maxsize} which are the
#'   observed minimum and maximum patch sizes, the estimated \eqn{x_{min}} as column
#'   \code{xmin} and the value of the power-law range as \code{plrange}. If multiple
#'   matrices were provided, then a list of data.frames is returned
#' 
#' @seealso \code{\link{patchdistr_sews}}
#' 
#' @examples
#' \donttest{
#' forestgap.plr <- indicator_plrange(forestgap) 
#' do.call(rbind, forestgap.plr) # convert results to data.frame
#' 
#' # Restrict to small xmins 
#' forestgap.plr2 <- indicator_plrange(forestgap, xmin_bounds = c(1, 10)) 
#' do.call(rbind, forestgap.plr2) 
#' }
#'@export
indicator_plrange <- function(mat,
                              merge = FALSE,
                              xmin_bounds = NULL) {
  
  if ( !merge && is.list(mat) ) {
      # Returns a list of lists
    return( lapply(mat, indicator_plrange, merge, xmin_bounds) )
  }
  
  psd <- patchsizes(mat, merge = merge)
  
  if ( is.null(xmin_bounds) ) {
    xmin_bounds <- range(psd)
  }
  
  # If there are not enough patches to work with -> return NA
  if ( length(unique(psd)) <= 2 ) { 
    warning('Not enough different patch sizes to estimate xmin: returning NA')
    result <- data.frame(min(psd), max(psd), NA, NA)
  } else {
    # Compute xmin and range
    plrange_result <- plrange(psd, xmin_bounds) # returns xmin also
    result <- data.frame(min(psd), max(psd), plrange_result)
  }
  
  names(result) <- c("minsize", "maxsize", "xmin_est", "plrange")
  return(result)
}
#
# This file contains the indicator function based on the power-law range
#   (how much of the data behaves like a power law?)
#
#' @title Power-law range indicator
#' 
#' @description Compute the power-law range of a matrix 
#' 
#' @param mat A logical matrix, or a list of logical matrices 
#'
#' @param xmin_bounds A vector of two integer values, defining a range in which 
#'   to search for the best xmin (see Details). 
#' 
#' 
#' @details 
#' 
#' Some ecosystems show typical changes in their patch-size 
#' distribution as they become more and more degraded. In particular, an 
#' increase in the truncation of the patch-size distribution (PSD) is expected 
#' to occur. The power-law range (PLR) measures the truncation of the PSD 
#' in a single value (see also \code{\link{patchdistr_sews}} for more details). 
#' 
#' To compute the PLR, power-laws are fitted with a variable 
#' minimum patch size (xmin) and the one with the lowest Kolmogorov-Smirnov
#' distance to the empirical distribution is retained. PLR is then computed 
#' using this best-fitting xmin: 
#' 
#' \deqn{\frac{log(x_{max}) - log(x_{min})}{log(x_{max}) - log(x_{smallest})}}{ (log(xmax) - log(xmin))/(log(xmax) - log(xsmallest))}
#' 
#' where \eqn{x_{max}}{x_max} is the maximum observed patch size, and 
#' \eqn{x_{smallest}}{x_smallest} is the minimum observed patch size. 
#' 
#' @references 
#' 
#' Clauset, A., Shalizi, C. R., & Newman, M. E. (2009). 
#'   Power-law distributions in empirical data. SIAM review, 51(4), 661-703.
#' 
#' Berdugo, M., Kefi, S., Soliveres, S. & Maestre, F.T. (2017). Plant spatial 
#' patterns identify alternative ecosystem multifunctionality states in 
#' global drylands. Nature in Ecology and Evolution.
#' 
#' @return A named vector containing the power-law range value
#' 
#' @seealso \code{\link{patchdistr_sews}}
#' 
#' @examples
#' \donttest{
#' forestgap.plr <- raw_plrange(forestgap[[2]]) 
#' 
#' # Restrict to small xmins 
#' forestgap.plr2 <- indicator_plrange(forestgap[[2]], xmin_bounds = c(1, 10)) 
#' }
#'@export
raw_plrange <- function(mat, xmin_bounds = NULL) { 
  
  psd <- patchsizes(mat)
  
  if ( is.null(xmin_bounds) ) {
    xmin_bounds <- range(psd)
  }
  plrange_result <- plrange(psd, xmin_bounds)
  
  return( c(plrange = plrange_result[ ,"plrange"]) )
}

plrange <- function(psd, xmin_bounds) {
  
  if ( any(psd == 0) ) { 
    warning(paste0("You have zeros in your patch size distribution, plrange ", 
                   "will be NaN"))
  }
  
  # If psd is empty, then return NA
  if ( length(unique(psd)) <= 1) {
    return( data.frame(xmin_est = NA_real_, plrange = NA_real_) )
  }
  
  xsmallest <- min(psd)
  xmax <- max(psd)
  xmin <- xmin_estim(psd, bounds = xmin_bounds)
  
  if ( is.na(xmin) ) { # finding xmin failed
    result <- data.frame(NA_real_, NA_real_)
  } else {
    result <- data.frame(xmin, 1 - (log10(xmin) - log10(xsmallest)) /
                                (log10(xmax) - log10(xsmallest)))
  }
  
  names(result) <- c('xmin_est', 'plrange')
  
  return(result)
}

#
# This is the main page for the package documentation.
#
#' @title Early Spatial-Warnings of Ecosystem Degradation
#' 
#' @description 
#' 
#' Many dynamical systems such as ecosystems exhibit non-linear responses to 
#' changes in their external drivers, resulting in possible wide state shifts 
#' with strong ecological or economical consequences. This often happens when 
#' a system exhibit a change in its stability properties as a threshold is 
#' crossed, \emph{e.g.} going from multiple stable states to a single stable 
#' state. For a few decades, much research has been dedicated to finding a 
#' way to anticipate these tipping points in ecological systems. This has led 
#' to the development of indicators, which are metrics based on spatial
#' structure, that could reflect the proximity of an ecosystem to a tipping
#' point. 
#' 
#' This package implements many of these indicators, or 
#' \emph{early-warning signals} (EWS), on spatial raster data. High-level 
#' functions and methods provide familiar workflows to compute several related
#' indicators at once, and display their variations along environmental gradients 
#' or time-series. Lower-level functions are also available to integrate 
#' early-warning signals into different workflows. 
#' 
#' Main, "workflow" functions provided by this package: 
#' 
#' "Workflow" functions, which may compute several indicators at once:
#' \itemize{ 
#'   \item \code{\link{generic_sews}}: Generic spatial EWS
#'   \item \code{\link{spectral_sews}}: Spectrum-based EWS
#'   \item \code{\link{patchdistr_sews}}: EWS based on patch-size distributions
#'   \item \code{\link{kbdm_sews}}: Kolmogorov complexity
#'   \item \code{\link{flowlength_sews}}: Flow length 
#'   \item \code{\link{variogram_sews}}: Variogram-based indicators
#'   \item \code{\link{lsw_sews}}: Indicators based on the Lifshitz-Slyozov-Wagner distribution
#' }
#' 
#' Individual indicator functions, which may be used to compute raw indicator values 
#' 
#' \itemize{ 
#'   \item \code{\link{raw_cg_moran}}: lag-1 spatial autocorrelation 
#'   \item \code{\link{raw_cg_variance}}: Spatial variance
#'   \item \code{\link{raw_cg_skewness}}: Spatial skewness
#'   \item \code{\link{raw_sdr}}: Spectral density ratio (SDR)
#'   \item \code{\link{indicator_psdtype}}: Patch-size distribution shape 
#'   \item \code{\link{indicator_plrange}}, 
#'           \code{\link{raw_plrange}} : Power-law range
#'   \item \code{\link{raw_kbdm}}: Kolmogorov complexity
#'   \item \code{\link{raw_flowlength_uniform}}: Flow Length
#'   \item \code{\link{raw_structvar}}: Structural variance
#'   \item \code{\link{raw_lsw_aicw}}: Support for LSW distribution relative to lognormal
#'   \item \code{\link{raw_patch_radii_skewness}}: Skewness of patch radii
#' }
#' 
#' The package homepage is available at 
#' \href{https://github.com/spatial-ews/spatialwarnings}{Github}
#' and a  
#' \href{https://alex.lecairn.org/spatialwarnings-faq.html}{user guide} is 
#' also available. Do not hesitate to get in touch if you want to make changes to the 
#' package or add another indicator. 
#' 
#' @aliases spatialwarnings-package
#' 
#' @keywords internal
"_PACKAGE"

# 
# 

# 
# 
# 
# Testing method 
#'@export
indictest.variogram_sews_list <- function(x, 
                                          nulln = 999, 
                                          null_method = 'perm', 
                                          null_control = NULL, 
                                          ...) { 
  
  results <- future_lapply_seed(x, indictest.variogram_sews_single, 
                                nulln, null_method, null_control, ...)
  
  # Add matrixn column with correct matrixn number
  for ( nb in seq_along(results) ) { 
    results[[nb]][['matrixn']] <- nb
  }
  
  class(results) <- c('variogram_sews_test_list', 
                      'variogram_sews_list', 
                      'variogram_sews_test', 
                      'simple_sews_test_list', 
                      'sews_result_list')
  return(results)
}
#'@export
indictest.variogram_sews_single <- function(x, 
                                            nulln = 999, 
                                            null_method = 'perm', 
                                            null_control = NULL, 
                                            ...) { 
  
  # This function will produce a vector, with the 4 first values holding 
  # the summarized metrics, and the last ones the values of the variogram. 
  metric_compute <- function(mat) { 
    a <- with(x[["pars"]], fit_variogram(mat, model, nmax, nbins, cutoff))
    unlist(with(a, c(compute_vario_metrics(pars), vario[ ,"gamma"])))
  }
  
  test_values <- compute_indicator_with_null(x[['orig_data']], 
                                             nulln = nulln, 
                                             indicf = metric_compute, 
                                             null_method = null_method, 
                                             null_control = null_control)
  
  # Format results. The first four values are parameters, the rest is the 
  # variogram.
  x <- append(x, llply(test_values[["summary_values"]], function(o) o[1:4]))
  
  # Import variogram data in original object
  vario <- lapply(test_values[["summary_values"]], function(o) o[-(1:4)])
  vario <- data.frame(x[["variogram"]], as.data.frame(vario))
  row.names(vario) <- as.character(seq.int(nrow(vario)))
  x[["variogram"]] <- vario
  
  # Import information
  x <- append(x, test_values[["info"]])
  
  class(x) <- c('variogram_sews_test_single', 
                'variogram_sews_single', 
                'simple_sews_test_single', 
                'sews_result_single')
  return(x)
}

#' @rdname variogram_sews_plot
#' 
#' @param what The trendline to be displayed. Defaults to the indicator's 
#'   values ("value") but other metrics can be displayed. Accepted values are 
#'   "value", "pval", "difference" (obs - null mean), or "z_score" 
#'   ( (obs - null mean) / (null sd) ).
#' 
#' @param display_null Chooses whether a grey ribbon should be added to reflect
#'   the null distribution. Note that it can not be displayed when the trend 
#'   line reflects something else than the indicator values (when \code{what} 
#'   is not set to "value").
#' 
#' @param ... Other arguments are ignored.
#'@export
plot.variogram_sews_test <- function(x, 
                                along = NULL, 
                                what = "value", 
                                display_null = TRUE, 
                                ...) { 
  NextMethod("plot")
}
#'@export
plot.variogram_sews_test_list <- function(x, along = NULL, 
                                          what = "value", 
                                          display_null = TRUE, 
                                          ...) { 
  plot.simple_sews_test_list(x, along, what, display_null, ...)
}

# We need to put that into the plot help page for variogram-based indics
#'@rdname variogram_sews_plot
#'@method plot_variogram variogram_sews_test
#'@export
plot_variogram.variogram_sews_test <- function(x, 
                                along = NULL, 
                                what = "value", 
                                display_null = TRUE, 
                                ...) { 
  NextMethod("plot")
}
#'@export
plot_variogram.variogram_sews_test_list <- function(x, along = NULL, ...) { 
  ggobj <- plot_variogram.variogram_sews_list(x, along = along)
  
  # Extract null values and display them 
  variodf <- extract_variogram(x)
  if ( ! is.null(along) ) { 
    variodf[ ,"along"] <- along[variodf[ ,"matrixn"]]
  } else { 
    variodf[ ,"along"] <- variodf[ ,"matrixn"]
  }
  
  ggobj$layers <- c(geom_ribbon(aes_string(x = "dist", ymin = "null_qinf", 
                                           ymax = "null_qsup"), 
                                data = variodf, 
                                fill = 'grey',
                                group = 1, 
                                alpha = .8), 
                    geom_line(aes_string(x = "dist", y = "null_mean"), 
                              data = variodf, 
                              color = 'black', alpha = .1), 
                    ggobj$layers)
  return(ggobj)
}

#'@export
plot_variogram.variogram_sews_test_single <- function(x, ...) { 
  ggobj <- plot_variogram.variogram_sews_single(x)
  # Extract null values and display them 
  variodf <- extract_variogram(x)
  
  ggobj$layers <- c(geom_ribbon(aes_string(x = "dist", ymin = "null_qinf", 
                                           ymax = "null_qsup"), 
                                data = variodf, 
                                fill = 'grey',
                                group = 1, 
                                alpha = .8), 
                    geom_line(aes_string(x = "dist", y = "null_mean"), 
                              data = variodf, 
                              color = 'black', alpha = .1), 
                    ggobj$layers)
  return(ggobj)
}

#
# These tags are required to use functions in other packages
# 
#' @import ggplot2
#' @import plyr
#' @import stats
#' @import utils
# 
# These tags are required by Rcpp
# 
#' @useDynLib spatialwarnings
#' @importFrom Rcpp sourceCpp
# 
# 

#
# These tags are here to explicitely show what is imported
#
#' @import stats
#' @import utils

# Spatialwarnings will load the `future` package at boot, but does not use 
# directly its functions. Here we import plan() explicitely to make sure 
# check() does not complain that we depend on a package we don't use. 
#
#'@importFrom future plan 
future::plan

# 
# This file contains code related to patch size distribution fitting. These 
#   functions can fit Power-law (pl), Truncated Power-law (tpl), Lognormal 
#   (lnorm) and Exponential (exp) distributions using maximum likelihood, as 
#   per Clauset et al. 's (2007) recommandations.
# 
# In addition, it provides the estimation of xmin using the ks-distance for 
# power-laws (same, following Clauset et al. 2007)

# Optimisation global options
ITERLIM <- 10000
GRADTOL <- 1e-10 
STEPTOL <- 1e-10
STEPMAX <- 5

ifnotfinite <- function(x, otherwise = .Machine$double.xmax) { 
  ifelse(is.finite(x), x, sign(x) * otherwise)
}

warnbound <- function() { 
  getOption("spatialwarnings.debug.fit_warn_on_bound", default = FALSE)
}

warnNA <- function() { 
  getOption("spatialwarnings.debug.fit_warn_on_NA", default = FALSE)
}

# This is a safe version of nlm that returns a sensible result (NaNs) when 
# the algorithm fails to converge. This can happen quite often when looking 
# for pathological cases (e.g. fitting distribution based on few points in the 
# tails, etc.). 
optim_safe <- function(f, pars0, 
                       lower = rep(-Inf, length(pars0)), 
                       upper = rep(Inf,  length(pars0)), 
                       fit_on_logscale = FALSE, ...) { 
  
  
  if ( fit_on_logscale ) { 
    optimf <- function(pars) { 
      f(exp(pars))
    }
    pars0 <- log(pars0)
    lower <- suppressWarnings(ifnotfinite(log(lower)))
    upper <- suppressWarnings(ifnotfinite(log(upper)))
  } else { 
    optimf <- f
  }
  
  optiresult <- try({ 
    optim(pars0, optimf, 
          control = list(maxit = ITERLIM), 
          lower = lower, upper = upper, 
          method = "L-BFGS-B", ...)
  }, silent = TRUE)
  
  # Code results above 3 means a true problem, below 3 the 
  # solution is either exact or approximate. 
  # Sometimes L-BFGS-B gets stuck in a very flat area just because our initial 
  # guess was very good, and L-BFGS-B does not like that (it reports 
  # abnormal termination of line search). Or L-BFGS-B will try the bounds of 
  # the parameter space, which will return error. So here we try to use BFGS 
  # which should report success if our initial guess was not too bad. 
  if ( inherits(optiresult, "try-error") || optiresult[["convergence"]] > 3 ) { 
    optiresult_bfgs <- try({ 
      optim(pars0, optimf, 
            control = list(maxit = ITERLIM), 
            method = "BFGS", ...)
    }, silent = TRUE)
    
    # If success, go with BFGS results
    if ( inherits(optiresult_bfgs, "try-error") ) { 
      optiresult <- optiresult_bfgs
    }
  }
  
  # If we could not reach a proper solution, report an error
  if ( inherits(optiresult, "try-error") ) { 
    optiresult <- list(value  = NaN, 
                       par = rep(NaN, length(pars0)), 
                       convergence = 128) 
    return(optiresult)
  }

  if ( optiresult[["convergence"]] > 3 ) { 
    msg <- paste0('optim returned an error (error code:',
                  optiresult[["convergence"]], ").\n",
                  "Make sure the results are reasonable using plot_distr")
    if ( optiresult[["convergence"]] %in% c(51, 52) ) {
      msg <- paste0(msg, "\nError message from optim(): ", optiresult[["message"]])
    }
    warning(msg)
  }
  
  # Convert the estimated pars back from log scale 
  if ( fit_on_logscale ) { 
    optiresult[["par"]] <- exp(optiresult[["par"]])
  }
  
  return(optiresult)
}

# Bounds on parameters, these should be large and no observed distribution should 
# have values beyond them
# Power-laws lambdas
PLMIN <- 1 + sqrt(.Machine$double.eps)
PLMAX <- 10
# Exponential rates
EXPMIN <- sqrt(.Machine$double.eps) # A very close value to, but not, zero
EXPMAX <- 10
# Bounds for truncated power-laws
TPL_EXPOMIN <- -1 # Taken from Clauset's code
TPL_EXPOMAX <- 10
TPL_RATEMIN <- sqrt(.Machine$double.eps)
TPL_RATEMAX <- 10

# Riemann zeta function with xmin taken into account :
# sum( 1/k^-expo ) for i=xmin to i = inf
# This is vectorized over xmins so that we do not sum things several times. 
zeta_w_xmin <- function(expo, xmins) { 
  perm <- order(xmins)
  xmins <- xmins[order(xmins)]
  
  # Compute zeta value
  zetaval <- gsl::zeta(expo)
  
  # Initialize
  output <- rep(NaN, length(xmins))
  current_k <- xmins[1]
  output[perm[1]] <- zetaval - sum_all_one_over_k(from = 1, to = xmins[1], expo)
  
  # If there is only one value, we bail now
  if ( length(xmins) <= 1) { 
    return(output)
  }
  
  for ( i in 2:length(xmins)) { 
    next_k <- xmins[i]
    if (next_k > current_k) { 
      output[perm[i]] <- output[perm[i-1]] - 
                           sum_all_one_over_k(from = current_k, 
                                              to = next_k, expo)
      current_k <- next_k
    } else { 
      output[perm[i]] <- output[perm[i-1]]
    }
  }
  
  return(output)
}




# PL fitting 
# ---------------------------------------

# Normalizing constant for pl with xmin
displnorm <- function(expo, xmin) { 
  # Adjust constant for threshold (note that this has no effect if xmin == 1, 
  #   as expected)
  const <- gsl::zeta(expo)
  const - sum_all_one_over_k(from = 1, to = xmin, expo)
}

# PL: P(x=k)
dpl <- function(x, expo, xmin = 1, log = FALSE) { 
  const <- displnorm(expo, xmin)
  
  # Compute values
  if ( ! log ) { 
    ans <- (1/const) * x^(-expo)
    ans[x < xmin] <- NaN
  } else { 
    if ( const < 0 ) { 
      # Const can be negative as nlm finds its way: the check makes sure 
      # no warning is produced by the log. 
      ans <- NaN
    } else { 
      ans <- -expo * log(x) - log(const)
    }
  }
  
  return(ans)
}

# PL: P(x>=k) 
ippl <- function(x, expo, xmin = 1) { 
  const <- displnorm(expo, xmin)
  
  is_below_xmin <- x < xmin
  
  ps <- zeta_w_xmin(expo, x[!is_below_xmin]) / const 
  
  # Values below threshold are NA'ed
  ans <- NaN*x
  ans[!is_below_xmin] <- ps
  
  return(ans)
}

# PL: Log likelihood
pl_ll <- function(dat, expo, xmin) { 
  sum( dpl(dat, expo, xmin, log = TRUE) )  
}


#' @title Distribution-fitting functions 
#'
#' @description These functions fit parametric distributions to a set of
#'   discrete values. 
#'
#' @param dat The set of values to which the distribution are fit 
#'
#' @param xmin The minimum possible value to consider when fitting the
#'   distribution
#'
#' @return A list containing at list the following components: 
#'
#' \describe{ 
#'    \item{\code{type}}{The type of distribution fitted (as a character string)}
#'    \item{\code{method}}{The method used for the fit - here, maximum likelihood, 'll'}
#'    \item{\code{ll}}{The log likelihood at the estimated parameter values}
#'    \item{\code{xmin}}{The value of xmin used for the fit}
#'    \item{\code{npars}}{The number of parameters of the distribution}
#'  }
#' 
#' Additionally, this list may have one or more of the following elements depending on 
#'   the type of distribution that has been fitted: 
#'   \describe{ 
#'     \item{\code{plexpo}}{The exponent of the power-law}
#'     \item{\code{cutoff}}{The rate of truncation, for truncated power law and 
#'        exponential fits}
#'     \item{\code{meanlog}}{The mean of the lognormal distribution}
#'     \item{\code{sdlog}}{The s.d. of the lognormal distribution}
#'   }
#' 
#' @details These functions will fit distributions to a set of values using 
#'   maximum-likelihood estimation. In the context of the 'spatialwarnings' 
#'   package, they are most-often used to fit parametric distributions on patch
#'   size distributions. As a result, these functions assume that the data 
#'   contains only integer, strictly positive values. The type of distribution
#'   depends on the prefix of the function: 'pl' for power-law, 'tpl' for
#'   truncated power-law, 'lnorm' for lognormal and 'exp' for an exponential
#'   distribution. 
#' 
#' In the context of distribution-fitting, 'xmin' represents the minimum value 
#'   that a distribution can take. It is often used to represent the minimum 
#'   scale at which a power-law model is appropriate (Clauset et al. 2009), and 
#'   can be estimated on an empirical distribution using
#'   \code{\link{xmin_estim}}. Again, please note that the fitting procedure 
#'   assumes here that xmin is equal or grater than one.
#' 
#' Please note that a best effort is made to have the fit converge, but 
#'   it may sometimes fail when the parameters are far from their usual 
#'   range, and numerical issues may occur. It is good practice to make 
#'   sure the fits are sensible when convergence warnings are reported.
#' 
#' For reference, the shape of the distributions is as follow: 
#' 
#' \describe{
#'   \item{power-law}{\eqn{x^{-a}}{x^(-a)} where a is the power-law exponent}
#'   \item{exponential}{\eqn{exp(-bx)}{exp(-bx)} where b is the truncation rate
#'           of the exponential}
#'   \item{truncated power-law}{\eqn{x^{-a}exp(-bx)}{x^(-a)exp(-bx)} where a
#'     and b are the exponent of the power law and the rate of truncation}
#' }
#' 
#' The lognormal form follows the \link[=dlnorm]{standard definition}.
#' 
#' The following global options can be used to change the behavior of fitting functions 
#'   and/or produce more verbose output: 
#' \describe{ 
#'   \item{spatialwarnings.constants.reltol}{the relative tolerance to use to compute 
#'     the power-law normalizing constant
#'     \deqn{sum_{k=1}^{\infty} x^{ak}e^{-bk}}{sum( x^(ak)exp(-bk)) for k in 1:Inf}. 
#'     Increase to increase the precision of this constant, which can be useful in some 
#'     cases, typically with large sample sizes. Default is 1e-8.}
#'   \item{spatialwarnings.constants.maxit}{the maximum number of iterations to compute 
#'     the normalizing constant of a truncated power-law. Increase if you get a warning 
#'     that the relative tolerance level (defined above) was not reached. Default is 1e8}
#'   \item{spatialwarnings.debug.fit_warn_on_bound}{logical value. Warn if the fit is 
#'     at the boundary of the valid range for distribution parameter}
#'   \item{spatialwarnings.debug.fit_warn_on_NA}{logical value. Warn if the returned fit 
#'     has \code{NA}/\code{NaN} parameters}
#' }
#' 
#' @seealso \code{\link{patchdistr_sews}}, \code{\link{xmin_estim}}
#' 
#' @references
#' 
#' Clauset, Aaron, Cosma Rohilla Shalizi, and M. E. J. Newman. 2009. “Power-Law
#' Distributions in Empirical Data.” SIAM Review 51 (4): 661–703. 
#' https://doi.org/10.1137/070710111.
#' 
#' @examples 
#' 
#' # Fit an exponential model to patch size distribution 
#' exp_fit(patchsizes(forestgap[[8]]))
#'  
#' # Use the estimated parameters as an indicator function
#' \donttest{
#' 
#' get_truncation <- function(mat) { 
#'    c(exp_cutoff = exp_fit(patchsizes(mat))$cutoff)
#' }
#' trunc_indic <- compute_indicator(forestgap, get_truncation)
#' plot(trunc_indic)
#' plot(indictest(trunc_indic, nulln = 19))
#' 
#' }
#' 
#'@export
pl_fit <- function(dat, xmin = 1) { 
  
  # Cut data to specified range
  dat <- dat[dat >= xmin] 
  
  # Start with the approximation given in Clauset's 
  npts <- length(dat)
  expo_estim <- 1 + npts / (sum(log(dat)) - npts*log(xmin-.5))
  
  negll <- function(expo) {
    result <- - pl_ll(dat, expo, xmin) 
    if ( is.infinite(result) ) { 
      return(NaN)
    } else { 
      return(result)
    }
  }
  
  
  est <- optim_safe(negll, expo_estim, 
                    lower = PLMIN, upper = PLMAX)
  
  result <- list(type = 'pl',
                 method = 'll', 
                 plexpo = est[["par"]],
                 ll = - est[['value']],
                 xmin = xmin,
                 npars = 1)
  
  if ( warnNA() && is.na(result[["plexpo"]]) ) { 
    warning("Fitting of PL returned NA")
    return(result)
  }
  
  if ( warnbound() && any(abs(result[["plexpo"]] - c(PLMIN, PLMAX)) < 1e-8) ) { 
    warning("Estimated PL exponent is on the boundary of the valid range")
  }
  
  return(result)
}

#' @title Estimate the minimum patch size of a power-law distribution 
#' 
#' @description When fitting a power-law to a discrete distribution, it might 
#'   be worth discarding points below a certain threshold (xmin) to improve 
#'   the fit. This function estimates the optimal xmin based on the 
#'   Kolmogorov-Smirnoff distance between the fit and the empirical 
#'   distribution, as suggested by Clauset et al. (2009). 
#' 
#' @param dat A vector of integer values
#' 
#' @param bounds A vector of two values representing the bounds in which 
#'   the best xmin is searched
#' 
#' @return The estimated xmin as an integer value 
#' 
#' @details The function returns NA if \code{dat} has only three unique values 
#'   or if the power-law fit failed. 
#' 
#' @seealso \code{\link{patchdistr_sews}}
#' 
#' @references 
#' 
#' Clauset, A., Shalizi, C. R., & Newman, M. E. (2009). 
#'   Power-law distributions in empirical data. SIAM review, 51(4), 661-703.
#' 
#' @examples 
#' 
#' \donttest{ 
#' psd <- patchsizes(forestgap[[5]])
#' xmin_estim(psd)
#' }
#'@export
xmin_estim <- function(dat, bounds = range(dat)) { 
  
  # Create a vector of possible values for xmin
  xmins <- sort(unique(dat))
  
  # We need at least 3 values for a pl fit, so the last value of xmin 
  # needs to have three points after it
  if ( length(xmins) <= 3 ) { 
    return(NaN)
  }
  
  # We build a vector of possible xmins. The last three values are stripped 
  #   away as they won't allow enough data for a fit
  xmins <- head(xmins, length(xmins)-3)
  xmins <- xmins[xmins >= min(bounds) & xmins <= max(bounds)]
  
  # Compute all ks-distances
  kss <- adply(xmins, 1, get_ks_dist, dat = dat)[ ,2]
  
  if ( all(is.nan(kss)) ) { 
    return(NaN)
  }
  
  # Note that sometimes the fit fails, especially when xmin is around the 
  #   distribution tail -> we need to remove some NAs here
  xmin <- xmins[!is.na(kss) & kss == min(kss, na.rm = TRUE)]
  
  # Note that xmin can be NaN
  return(xmin)
}

get_ks_dist <- function(xmin, dat) { 
  # Crop dat to values above xmin and compute cdf
  dat <- dat[dat >= xmin]
  # Compute empirical (inverse) cdf values
  udat <- unique(dat)
  cdf_empirical <- rep(NA, length(dat))
  for ( val in udat ) { 
    cdf_empirical[dat == val] <- mean(dat >= val)
  }
  
  # Fit and retrieve cdf. Note: here we suppress the warnings because finding 
  # xmin requires removing patches below a threshold, which often leads to fit 
  # being done on pathological cases like few unique patch sizes
  fit <- suppressWarnings({ 
    pl_fit(dat, xmin = xmin)
  })
  
  if ( is.na(fit[['plexpo']]) ) { 
    # Note: a warning was already produced in this case as it means that the 
    # fit failed to converge: we do not produce one here again. 
    return(NaN)
  }
  
  cdf_fitted <- ippl(dat, fit[["plexpo"]], fit[["xmin"]])

#   # debug
#   plot(data.frame(dat, rbinom(length(dat), 1, .5)), type = 'n')
#   plot(log10(data.frame(dat, cdf_empirical))) 
#   points(log10(data.frame(dat, cdf_fitted)), col = 'red')
#   browser()
#   zeta.fit(dat, xmin)$exponent
#   fit$expo
  
  # We return the ks distance
  maxks <- max(abs(cdf_empirical - cdf_fitted))
#   cat(xmin, ",", max(dat), "->", maxks,  "\n" )
  return( maxks )
}




# EXP fitting 
# ---------------------------------------
# pexp/dexp is already implemented in R

# EXP: P(x = k)
ddisexp <- function(dat, rate, xmin = 1, log = FALSE) { 
  # sum(P = k) for k = 1 to inf
  if ( log ) { 
    
    const <- log(1 - exp(-rate)) + rate * xmin
    return( ifelse(dat < xmin, NaN, const - rate * dat) )
  
  } else { 
    
    const <- (1 - exp(-rate)) * exp(rate*xmin) 
    return( ifelse(dat < xmin, NaN, const * exp(-rate * dat)) )
    
  }
} 

# EXP: P(x>=k)
# Imported and cleaned up from powerRlaw (def_disexp.R)
ipdisexp <- function(x, rate, xmin) {
  # p >= k
  p <- pexp(x + .5, rate, lower.tail = FALSE) 
  # p >= 1 
  const <-  1 - pexp(xmin + .5, rate) 
  return(p/const)
}

exp_ll <- function(dat, rate, xmin) { 
  sum( ddisexp(dat, rate, xmin, log = TRUE)) 
}

#'@rdname pl_fit
#'@export
exp_fit <- function(dat, xmin = 1) { 
  
  dat <- dat[dat>=xmin]
  
  rate0 <- 1 / mean(dat)
  
  negll <- function(rate) {
    - exp_ll(dat, rate, xmin)
  }
  
  est <- optim_safe(negll, rate0, 
                    fit_on_logscale = TRUE, 
                    lower = EXPMIN, 
                    upper = EXPMAX)
  
  result <- list(type = 'exp',
                 method = 'll', 
                 cutoff = est[['par']], 
                 ll = - est[["value"]],
                 npars = 1)
  
  if ( warnbound() && any(abs(result[["cutoff"]] - c(EXPMIN, EXPMAX)) < 1e-8) ) { 
    warning("Estimated EXP exponent is on the boundary of the valid range")
  }
  
  return(result)
}





# LNORM fitting 
# ---------------------------------------

# LNORM: P(X=k)
ddislnorm <- function(x, meanlog, sdlog, xmin, log = FALSE) { 
  
  p_over_thresh <- plnorm(xmin - .5, meanlog, sdlog, lower.tail = FALSE)
   
  p_equals_k <- plnorm(x-.5, meanlog, sdlog, lower.tail = FALSE) - 
                  plnorm(x+.5, meanlog, sdlog, lower.tail = FALSE)
  
  if ( !log ) { 
    return( ifelse(x<xmin, NaN, p_equals_k / p_over_thresh) )
  } else { 
    return( ifelse(x<xmin, NaN, log(p_equals_k) - log(p_over_thresh)) )
  }
}

# LNORM: P(X>=k)
ipdislnorm <- function(x, meanlog, sdlog, xmin) { 
  px_supto_k <- plnorm(x - .5, meanlog, sdlog, lower.tail = FALSE)
  px_supto_xmin <- plnorm(xmin - .5, meanlog, sdlog, lower.tail = FALSE)
  ifelse(x<xmin, NaN, px_supto_k / px_supto_xmin)
}

# LNORM: LL
lnorm_ll <- function(x, meanlog, sdlog, xmin) { 
  x <- x[x>=xmin]
  sum( ddislnorm(x, meanlog, sdlog, xmin, log = TRUE) ) 
}

# LNORM: fit
#'@rdname pl_fit
#'@export
lnorm_fit <- function(dat, xmin = 1) { 
  
  # Pars[1] holds mean of log-transformed data
  # Pars[2] holds sd 
  pars0 <- c( mean(log(dat)), sd(log(dat)) )
  
  negll <- function(pars) { 
    ll <- - lnorm_ll(dat, pars[1], pars[2], xmin) 
    if ( is.finite(ll) ) ll else 1e10
  }
  
  est <- optim_safe(negll, pars0)
  
  result <- list(type = 'lnorm',
                 method = 'll', 
                 meanlog = est[['par']][1], 
                 sdlog = est[['par']][2], 
                 ll = - est[["value"]], 
                 npars = 2)
  return(result)
}




# TPL fitting 
# ---------------------------------------

tplnorm <- function(expo, rate, xmin) { 
  
  maxit <- getOption("spatialwarnings.constants.maxit", default = 1e8L)
  reltol <- getOption("spatialwarnings.constants.reltol", default = 1e-8)
  
  a <- tplinfsum(expo, rate, xmin, maxit, reltol)
  
  a
}

# P(x=k)
dtpl <- function(x, expo, rate, xmin, log = FALSE) { 
  
  const <- tplnorm(expo, rate, xmin)
  if ( ! log ) { 
    ps <- x^(-expo) * exp(- x * rate) / const
  } else { 
    ps <- - expo * log(x) - rate * x - log(const)
  }
  
  return( ifelse(x < xmin, NaN, ps) )
}

# P(x>=k)
iptpl <- function(x, expo, rate, xmin) { 
  const <- tplnorm(expo, rate, xmin)
  
  # tplsum is vectorized over x
  p_inf_to_k <- tplsum(expo, rate, x, xmin) / const
  
  return( 1 - p_inf_to_k ) 
}

tpl_ll <- function(x, expo, rate, xmin, approximate = FALSE) { 
  x <- x[x>=xmin]
  
  ll <- sum( dtpl(x, expo, rate, xmin, log = TRUE) )
  
  if ( !is.finite(ll) ) { 
    ll <- sign(ll) * .Machine$double.xmax
  }
  return( ll )
} 

#'@rdname pl_fit
#'@export
tpl_fit <- function(dat, xmin = 1) { 
  
  negll <- function(pars) { 
    - tpl_ll(dat, pars[1], pars[2], xmin) 
  }
  
  # Initialize and find minimum
  expo0 <- pl_fit(dat, xmin)[['plexpo']] 
  
  # Do a line search over the cutoff to find a minimum, starting from zero 
  # up to 100
  is <- seq(0, 100, length = 128)
  is <- 10^seq(-7, 2, l = 128)
  lls <- unlist(lapply(is, function(i) { 
    negll(c(expo0, i))
  }))
  llmin <- min(lls[abs(lls) != .Machine$double.xmax & is.finite(lls)])
  expmrate0 <- is[which(lls == llmin)]
  
  # If multiple cutoffs produce the same ll, then use the lowest one 
  if ( length(expmrate0) > 1 ) { 
    expmrate0 <- expmrate0[1]
  }
  
  # Debug thing lls
#   plot( log(spatialwarnings:::cumpsd(dat)) )
#   vals <- ippl(dat, expo0, xmin = xmin)
#   points(log(dat), log(vals), col = "red")
#   vals <- iptpl(dat, expo0, rate = 0, xmin = xmin)
#   points(log(dat), log(vals), col = "darkgreen")
#   vals <- iptpl(dat, expo0, rate = expmrate0, xmin = xmin)
#   points(log(dat), log(vals), col = "blue")
  
  pars0 <- c(expo0, expmrate0)
  est <- optim_safe(negll, pars0, 
                    lower = c(TPL_EXPOMIN, TPL_RATEMIN), 
                    upper = c(TPL_EXPOMAX, TPL_RATEMAX), 
                    fit_on_logscale = FALSE)
  
  # For very small cutoffs, we may want to fit on log scale to get a descent estimate, 
  # as otherwise there is too little variation in the cutoff for BFGS
  if ( any(is.na(est[["par"]])) ) { 
    est <- optim_safe(negll, pars0, 
                      lower = c(TPL_EXPOMIN, TPL_RATEMIN), 
                      upper = c(TPL_EXPOMAX, TPL_RATEMAX), 
                      fit_on_logscale = TRUE)
  }
  
  result <- list(type = 'tpl',
                 method = 'll', 
                 plexpo = est[['par']][1], 
                 cutoff = est[['par']][2], 
                 ll = - est[["value"]],
                 npars = 2)
  
  if ( warnNA() && is.na(result[["plexpo"]]) ) { 
    warning("Fitting of TPL returned NA/NaN")
  }
  
  if ( is.na(result[["plexpo"]]) ) { 
    return(result)
  }
  
  if ( warnbound() && 
       any(abs(result[["plexpo"]] - c(TPL_EXPOMIN, TPL_EXPOMAX)) < 1e-8) ) { 
    warning("Estimated TPL exponent is on the boundary of the valid range")
  }
  
  # We do not warn for reaching minimum exponent because this is quite common when 
  # fitting truncated power laws to have a very small exponent
  if ( warnbound() && 
       any(abs(result[["cutoff"]] - c(TPL_RATEMIN, TPL_RATEMAX)) < 1e-8) ) { 
    warning("Estimated TPL cutoff is on the boundary of the valid range")
  }
  
  return(result)
}

context('Test that xmin estimations are correct')

if ( exists('EXTENDED_TESTS') && EXTENDED_TESTS ) { 
  
  # Change dir if running tests manually
  if ( file.exists('./tests/testthat') ) { 
    library(testthat)
    setwd('./tests/testthat') 
  }
  
  # Setup pli from Clauzet et al's
  for ( s in dir('./pli-R-v0.0.3-2007-07-25', 
                full.names = TRUE, pattern = '*.R') ) { 
    source(s)
  }
  
  # Compile auxiliary binaries
  system("cd ./pli-R-v0.0.3-2007-07-25/zeta-function/ && make")
  system("cd ./pli-R-v0.0.3-2007-07-25/exponential-integral/ && make")
  system("cd ./pli-R-v0.0.3-2007-07-25/ && \
            gcc discpowerexp.c -lm -o discpowerexp && \
            chmod +x discpowerexp")
  
  test_that("xmins estimation is correct", { 
    
    parms <- expand.grid(expo = c(1.5, 1.3, 1.2), 
                         rate = c(0.001, 0.005, 0.01, 0.1, 0.15, 0.2, 0.25, 0.3, 
                                  0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1))
    
    estim_xmin <- function(df) { 
      
      pldat <- round(rpowerexp(10000, 1, df[ ,'expo'], df[, 'rate']))
      
      est_xmin <- suppressWarnings( spatialwarnings:::xmin_estim(pldat) )
      # est_xmin <- spatialwarnings:::xmin_estim(pldat) 
      
      # Create pl object and estimate its xmin
      pl_obj <- poweRlaw::displ$new(pldat)
      est_xmin_plpkg <- poweRlaw::estimate_xmin(pl_obj)[["xmin"]]
      
      if ( ! is.na(est_xmin) && 
           ! is.na(est_xmin_plpkg) && 
           length(unique(pldat)) >= 5 ) { 
        # Note: In some pathological cases (few unique patches), there can be 
        # a small difference in xmin, so we use an acceptable error here. 
        fit_is_ok <- ( abs(est_xmin - est_xmin_plpkg) <= 1 ) 
        
        if ( est_xmin != est_xmin_plpkg ) { 
          cat(" Ours: ", est_xmin, ' -> poweRlaw\'s: ', est_xmin_plpkg, " [", 
              length(unique(pldat)), " unique patches]", "\n", sep = "")
        }
        
        # In this case, inspect the fit provided by the poweRlaw package
        if ( exists('GRAPHICAL') && GRAPHICAL && est_xmin != est_xmin_plpkg ) { 
          dev.new()
          par(mfrow = c(1, 2))
          plot(log10(cumpsd(pldat[pldat >= est_xmin])))
          our_fit <- pl_fit(pldat, xmin = est_xmin)
          xs <- unique(round(seq(min(pldat), max(pldat), length.out = 100)))
          lines(log10(xs), 
                log10(ippl(xs, our_fit$plexpo, xmin = est_xmin)))
          title("OUR FIT")
          
          plot(log10(cumpsd(pldat[pldat >= est_xmin_plpkg])))
          plpkg_expo <- poweRlaw::estimate_xmin(pl_obj)$pars
          xs <- unique(round(seq(min(pldat), max(pldat), length.out = 100)))
          lines(log10(xs), 
                log10(ippl(xs, plpkg_expo, xmin = est_xmin_plpkg)))
          title("PWL FIT")
          browser()
        }
      }
      
      data.frame(df, est_xmin = est_xmin, fit_is_ok = fit_is_ok)
    }
    
    xmin_ests <- ddply(parms, ~ expo + rate, estim_xmin)
    
    # Check that fits are overwhelmingly good. Sometimes the poweRlaw code will
    # produce different fits, so we need to be able to accomodate some errors here.
    # We put five as a maximum tolerable number of mismatches (out of 45)
    number_of_ok_fits <- sum(xmin_ests[ ,"fit_is_ok"])
    cat(paste0("Est. xmins OK: ", number_of_ok_fits, "/", nrow(parms), "\n"))
    expect_true({ 
      ( nrow(parms) - number_of_ok_fits ) <= 5
    })
    
  })
  
  # Remove auxiliary binaries now that tests are done
  system("cd ./pli-R-v0.0.3-2007-07-25/zeta-function/ && rm zeta_func zeta_func.o")
  system("cd ./pli-R-v0.0.3-2007-07-25/exponential-integral/ && rm exp_int exp_int.o")
  system("cd ./pli-R-v0.0.3-2007-07-25/ && rm discpowerexp")
  
} else { 
  message('Skipping xmin estimation testing')
}

#
#
# This code tests the psd-fitting functions
#

library(plyr)
library(ggplot2)

context('Test the fitting of distributions')

test_that("dtpl handles the log argument correctly", {
  testpoints <- 10^(seq(0, 5, length.out = 64))
  expect_equal(exp(dtpl(testpoints, 2, 1.0, 1, log = TRUE)),
               dtpl(testpoints, 2, 1.0, 1, log = FALSE),
               tol = 1e-8)
})

if ( exists("EXTENDED_TESTS") && EXTENDED_TESTS ) {

  test_that('PL fitting works', {

    # Change dir if running tests manually
    if ( file.exists('./tests/testthat') ) {
      library(testthat)
      setwd('./tests/testthat')
    }

    # Setup pli from Clauzet et al
    for ( s in dir('./pli-R-v0.0.3-2007-07-25',
                  full.names = TRUE, pattern = '*.R') ) {
      source(s)
    }

    # Compile auxiliary binaries
    system("cd ./pli-R-v0.0.3-2007-07-25/zeta-function/ && make")
    system("cd ./pli-R-v0.0.3-2007-07-25/exponential-integral/ && make")
    system("cd ./pli-R-v0.0.3-2007-07-25/ && \
              gcc discpowerexp.c -lm -o discpowerexp && \
              chmod +x discpowerexp")

    expos <- c(1.2, 2.5)
    xmins <- c(1,  10)
    for ( expo in expos ) {
      for ( xmin in xmins ) {
        if ( requireNamespace("poweRlaw", quietly = TRUE) ) {
          dat <- unique(round(seq.int(1, 1000, length.out = 100)))
          pldat <- poweRlaw::rpldis(1000, xmin = xmin, alpha = expo)
          # Squeeze tail a bit for speed
          pldat <- pldat[pldat < 1e5]

          # left: pli <-> right: spw
          # dpl <-> dzeta
          expect_equal(dzeta(dat,, exponent = expo, threshold = xmin),
                        dpl(dat, expo, xmin = xmin))

          # ppl <-> pzeta with higher tail
          expect_equal(pzeta(dat, exponent = expo, threshold = xmin, lower.tail = FALSE),
                        ippl(dat, expo, xmin = xmin))

          # ppl_ll <-> zeta.loglike
          expect_equal(zeta.loglike(pldat, exponent = expo, threshold = xmin),
                      pl_ll(pldat, expo, xmin = xmin))

          # pl_fit <-> zeta.fit
          our_expo <- pl_fit(pldat, xmin = xmin)[['plexpo']]
          clauset_expo <- zeta.fit(pldat, threshold = xmin)[['exponent']]
          powerlaw_expo <- poweRlaw::estimate_pars( poweRlaw::displ$new(pldat) )[["pars"]]
          expect_equal(clauset_expo, our_expo, tol = 1e-3)

          # Look at fit
          plot(log10(cumpsd(pldat[pldat>=xmin])))
          xs <- c(min(pldat[pldat>=xmin]), max(pldat[pldat>=xmin]))
          lines(log10(xs), log10(ippl(xs, clauset_expo, xmin = xmin)),
                lwd = 2, col = 'blue')
          lines(log10(xs), log10(ippl(xs, powerlaw_expo, xmin = xmin)),
                lwd = 2, col = 'green')
          lines(log10(xs), log10(ippl(xs, our_expo, xmin = xmin)), col = 'red')
          title('PLFIT')
        }
      }
    }

  })

  test_that('EXP fitting works', {

    rates <- c(1.1, 1.5)
    xmins <- c(1, 3)
    for (xmin in xmins) {
      for ( rate in rates ) {
          dat <- seq.int(1000)
          expdat <- ceiling(rexp(1000, rate = rate))

          expect_equal(ddiscexp(dat, rate, threshold = xmin),
                       ddisexp(dat, rate, xmin = xmin))

          ipdisexp(dat, rate, xmin = xmin)

          fit <- exp_fit(expdat, xmin = xmin)
          expect_equal(fit[['cutoff']],
                       discexp.fit(expdat, threshold = xmin)[["lambda"]],
                       tol = 1e-3)

          # Look at fit
          plot(log10(cumpsd(expdat[expdat >= xmin])))
          xs <- seq(min(expdat), max(expdat), length.out = 100)
          lines(log10(xs),
                log10(ipdisexp(xs, fit[["cutoff"]], xmin = xmin)),
                col = 'red')
          title('EXPFIT')
      }
    }

  })

  test_that('LNORM fitting works', {

    meanlogs <- c(1, 10)
    sdlogs <- c(1, 3)
    xmins <- c(1, 30)
    for (xmin in xmins) {
      for ( meanlog in meanlogs ) {
        for ( sdlog in sdlogs ) {

          xmax <- 1000
          dat <- seq.int(xmax)
          lnormdat <- ceiling(rlnorm(xmax, meanlog, sdlog))
  #         cat('meanlog: ', meanlog, ', sdlog: ', sdlog, ', xmin: ', xmin, "\n")

          # Test distr functions
          expect_equal(ddislnorm(dat, meanlog, sdlog, xmin),
                        dlnorm.tail.disc(dat, meanlog, sdlog, threshold = xmin))

          # plnorm.tail.disc returns negative values (?!)
  #         plnorm.tail.disc(dat, meanlog, sdlog, threshold = xmin)
  #         ipdislnorm(dat, meanlog, sdlog, xmin)

          # Note that dlnorm.disc assumes xmin = 0, so we cannot test it
          #   with variable xmin.
          our_dlnorm <- ddislnorm(dat, meanlog, sdlog, xmin = 0)
          clauset_dlnorm <- dlnorm.disc(dat, meanlog, sdlog)
          expect_equal(our_dlnorm,
                        clauset_dlnorm)

          # Test obtained fits
          our_fit <- suppressWarnings( lnorm_fit(lnormdat, xmin = xmin) )
          clauset_fit <- suppressWarnings( fit.lnorm.disc(lnormdat, threshold = xmin) )
  #         expect_equal(our_fit[['meanlog']], clauset_fit[['meanlog']], tol = 1e-2)
  #         expect_equal(our_fit[['sdlog']], clauset_fit[['sdlog']], tol = 1e-2)

          # Look at fit
          plot(log10(cumpsd(lnormdat[lnormdat >= xmin])))
          xs <- seq(min(lnormdat), max(lnormdat), length.out = 10000)
          suppressWarnings(
            lines(log10(xs),
                  log10(ipdislnorm(xs,
                                   our_fit[['meanlog']],
                                   our_fit[['sdlog']],
                                   xmin = xmin)), col = 'red')
          )
          suppressWarnings(
            lines(log10(xs),
                  log10(ipdislnorm(xs,
                                   fit.lnorm.disc(lnormdat, threshold = xmin)[["meanlog"]],
                                  fit.lnorm.disc(lnormdat, threshold = xmin)[["sdlog"]],
                                  xmin = xmin)),
                                  col = "blue")
          )
          title('LNORMFIT')

        }
      }
    }

  })

  # For TPL xmax must not be too low (>3?)
  test_that('TPL fitting works', {

    rates <- c(0.05, 1.1)
    expos <- c(1.1, 1.3)
    xmins <- c(1, 10)
    for (xmin in xmins) {
      for ( rate in rates ) {
        for ( expo in expos ) {

          tpldat <- round(rpowerexp(1000, 1, expo, rate))
          tpldat <- tpldat[tpldat < 1e5]
          dat <- seq(0, 1000)

          # Normalizing coeff
          # Here, we check that the binary called by the Clauset code actually
          # returns something.
          clauset_result <- suppressWarnings({
            discpowerexp.norm(xmin, expo, rate)
          })

          if ( length(clauset_result) > 0 ) {
            expect_equal(clauset_result, tplnorm(expo, rate, xmin),
                          tolerance = 1e-4)
          }

          # P(X=x)
          # Note: we explicitely convert output from pli to numeric as if
          # the returned values are all NAs then R thinks it's logical
          dtpl_result <- dtpl(dat, expo, rate, xmin)
          ddpxp_result <- suppressWarnings (
              as.numeric( ddiscpowerexp(dat, expo, rate, threshold = xmin) )
            )
          if ( ! all( is.na(dtpl_result) ) ) {
            expect_equal(dtpl_result, ddpxp_result,
                         tolerance = 1e-4)
          }

          # Here, we check that the binary called by the Clauset code actually
          # returns something before performing the test.
          clauset_result <- suppressWarnings(
              discpowerexp.loglike(tpldat, expo, rate, threshold = xmin)
            )
          if ( length(clauset_result) > 0 ) {
            expect_equal(tpl_ll(tpldat, expo, rate, xmin),
                         clauset_result,
                         tolerance = 1e-4)
          }

          # We suppress warnings, because we are concerned with the correctness of
          # the result only.
          suppressWarnings({
            our_fit <- tpl_fit(tpldat, xmin = xmin)
          })

          # The fit with pli can fail, so we check only the results
          # when it succeeds. Sometimes the returned fits are on the boundary
          # (expo ~= 1), so we do not test in those cases.
          clauset_fit <- tryNULL(discpowerexp.fit(tpldat, threshold = xmin))

          if ( ! is.null(clauset_fit) &&
               abs(clauset_fit$exponent - (-1)) > 1e-4 &&
               ! is.na(our_fit$ll) ) {
            if ( our_fit$ll < clauset_fit$loglike ) {
              if ( abs(our_fit$ll - clauset_fit$loglike) > 1e-3 ) {
                message("We found a better fit than the reference code")
              }
            } else {
              # If the difference is not floating-point error
              if ( abs(our_fit$ll - clauset_fit$loglike) > 1e-3 ) {
                message("Reference code found a better fit")
                fail()
              }
              # These tests will fail
  #               expect_equal(our_fit$expo, clauset_fit$exponent, tol = 5e-2)
  #               expect_equal(our_fit$rate, clauset_fit$rate, tol = 5e-2)
            }
          }
          if ( !is.null(clauset_fit) ) {
            # Look at fit
            plot(log10(cumpsd(tpldat[tpldat >= xmin])))
            xs <- unique( round( seq(min(tpldat), max(tpldat),
                                      length.out = 100) ))
            lines(log10(xs),
                  log10(iptpl(xs,
                              clauset_fit[["exponent"]],
                              clauset_fit[["rate"]], xmin)), col = 'blue',
                  lwd = 2)
            lines(log10(xs),
                  log10(iptpl(xs, our_fit[["plexpo"]],
                              our_fit[["cutoff"]], xmin)),
                  col = 'red')
            title('TPLFIT')
          }
        }
      }
    }

  })


  test_that("PL estimations work with xmins", {

    expos <- c(1.5, 2)
    for (expo in expos) {
      for (xmin in c(1, 5)) {
        x <- seq.int(1000)

        pldat <- poweRlaw::rpldis(1000, xmin, expo)
        pldat <- pldat[pldat < 1e5] # squeeze tail for speed

        # Test dpl with xmin != 1
        expect_equal(dzeta(x, xmin, expo),
                     dpl(x, expo, xmin))

        # Test ippl
        expect_equal(pzeta(x, xmin, expo, lower.tail = FALSE),
                     ippl(x, expo, xmin))

        # Test likelihood func
        expect_equal(zeta.loglike(pldat, xmin, expo),
                     pl_ll(pldat, expo, xmin))

        # Test equality of fits
        expect_equal(pl_fit(pldat, xmin = xmin)[["plexpo"]],
                     zeta.fit(pldat, xmin)[["exponent"]],
                     tol = 1e-3)

        # Test the estimation of xmin
        expect_is(xmin_estim(pldat), "numeric")
      }
    }

  })

  # Remove auxiliary binaries now that tests are done
  system("cd ./pli-R-v0.0.3-2007-07-25/zeta-function/ && rm zeta_func zeta_func.o")
  system("cd ./pli-R-v0.0.3-2007-07-25/exponential-integral/ && rm exp_int exp_int.o")
  system("cd ./pli-R-v0.0.3-2007-07-25/ && rm discpowerexp")

}


test_that("Warnings in TPL constant are properly reported", {

  # Not enough iterations to reach reltol
  options(spatialwarnings.constants.reltol = 1e-8)
  options(spatialwarnings.constants.maxit = 10)
  expect_warning({
    tplnorm(2.41, 1e-7, 300)
  })

  # Increase number of iterations, sufficient to reach reltol
  options(spatialwarnings.constants.reltol = 1e-8)
  options(spatialwarnings.constants.maxit = 1e8)
  expect_true({
    tplnorm(2.41, 1e-7, 300)
    TRUE # no warning
  })

  # Increase reltol, not enough iterations
  options(spatialwarnings.constants.reltol = 1e-20)
  options(spatialwarnings.constants.maxit = 1e8)
  expect_warning({
    tplnorm(2.41, 1e-7, 300)
  })

  options(spatialwarnings.constants.reltol = NULL)
  options(spatialwarnings.constants.maxit = NULL)

})

# 
# Methods for a simple indicator, i.e. an indicator that produces on or more 
# numeric values for each matrix. 
# 



# as.df methods
# ---------------
#'@method as.data.frame simple_sews_single
#'@export
as.data.frame.simple_sews_single <- function(x, ..., wide = FALSE) { 
  as.data.frame.simple_sews_list( list(x), wide = wide )
}
#'@method as.data.frame simple_sews_list
#'@export
as.data.frame.simple_sews_list <- function(x, ..., wide = FALSE) { 
  
  # Find or create the indicator names
  indicnames <- ifNULLthen(names(x[[1]][['value']]), 
                           paste0("indic_", seq_along(x[[1]][['value']])))
  
  if ( wide ) { 
    output <- Map(function(n, o) { 
        a <- as.data.frame(matrix(o[['value']], nrow = 1))
        names(a) <- indicnames
        data.frame(matrixn = n, a)
      }, seq_along(x), x)
  } else { 
    output <- Map(function(n, o) { 
        data.frame(matrixn = n, indic = indicnames, value = o[['value']])
      }, seq_along(x), x)
  }
  output <- do.call(rbind, output)
  row.names(output) <- NULL
  output
}



# Print methods
# ---------------
#'@method print simple_sews_single
#'@export
print.simple_sews_single <- function(x, ...) { 
  summary.simple_sews_single(x, ...)
}
#'@method print simple_sews_list
#'@export
print.simple_sews_list <- function(x, ...) { 
  summary.simple_sews_list(x, ...)
}



# Summary methods
# ---------------
#'@method summary simple_sews_single
#'@export
summary.simple_sews_single <- function(object, 
                                       indicname = object[["taskname"]], 
                                       ...) { 
  cat('Spatial Early-Warning:', indicname, '\n') 
  cat('\n')
  display_size_info(object)
  cat('\n')
  
  # Format output table
  output <- as.data.frame(object, wide = TRUE)
  names(output)[1] <- c('Mat. #')
  
  print.data.frame(output, row.names = FALSE, digits = DIGITS)
  
  cat('\n')
  cat("The following methods are available: \n")
  cat(list_methods(class(object)), "\n")
}
#'@method summary simple_sews_list
#'@export
summary.simple_sews_list <- function(object, 
                                     indicname = object[[1]][["taskname"]], 
                                     ...) { 
  
  cat('Spatial Early-Warning:', indicname, '\n') 
  cat('\n')
  display_size_info(object)
  cat('\n')
  
  # Format output table
  output <- as.data.frame.simple_sews_list(object, wide = TRUE)
  names(output)[1] <- c('Mat. #')
  
  print.data.frame(output, row.names = FALSE, digits = DIGITS)
  
  cat('\n')
  cat("The following methods are available: \n")
  cat(list_methods(class(object)), "\n")
  
  invisible(output)
}


# Plot methods 
# ------------
# /!\ the doc for this function is documented in ./R/simple_indic_indictest.R
#'@rdname simple_sews_methods
#'@method plot simple_sews_list
#'@export
plot.simple_sews_list <- function(x, along = NULL, ...) { 
  plot.simple_sews_test_list(x, along = along, display_null = FALSE)
}

#' A list of binary matrices and their associated parameters 
#'
#' @rdname forestgap
#' 
#' @format A list of logical matrices which are the end results of simulations 
#'   from Kubo's Forest Gap model along a gradient of increasing values of 
#'   stress (see references). 
#' 
#' @source Generated using the implementation of Kubo's model in caspr 0.2.0 
#'   \url{https://github.com/fdschneider/caspr}. 
#' 
#' @references
#' 
#' Kubo, T., Iwasa, Y., & Furumoto, N. (1996). Forest spatial dynamics with gap
#'   expansion: Total gap area and gap size distribution. Journal of Theoretical
#'   Biology, 180(3), 229-246. \doi{10.1006/jtbi.1996.0099}
#' 
"forestgap"

#' @rdname forestgap
#' 
#' @format The parameters used for the simulations, as a data frame. 
#' 
#' @details Kubo's forest gap model has three parameters, \eqn{\alpha}{alpha} 
#'   that controls the reproductive rate of trees, \eqn{d}{d} controls the 
#'   non-spatialized mortality and \eqn{\delta}{delta} the increased mortality 
#'   due to the presence of a neighboring gap. 
#' 
"forestgap.pars"



#' @rdname arizona
#' 
#' @title Aerial views of vegetation from Arizona, USA
#' 
#' @format A list of logical matrices which were obtained through the 
#'   classification of aerial images of vegetation taken in Arizona (USA). 
#' 
#' @source Derived from the images provided in the Supplementary Material of 
#'   Rodriguez et al. (2017). 
#' 
#' @references
#' 
#' Rodriguez, F., A. G. Mayor, M. Rietkerk, and S. Bautista. 2017. A null model 
#'   for assessing the cover-independent role of bare soil connectivity as 
#'   indicator of dryland functioning and dynamics. Ecological Indicators.
#' 
"arizona"



#' @rdname serengeti
#' 
#' @title Serengeti dataset
#' 
#' @description Vegetation data along a rainfall gradient in Serengeti national 
#'   park. 
#' 
#' @details 
#' 
#'  The data-set consists of a rectangular area of size 7.5 km x 90 km. 
#'  These data are represented as a list of matrices. Each matrix is a moving 
#'  window of 7.5 km x 7.5 km which moves my 2.5 km along the length of the 
#'  rectangular data-set. 
#'   
#'  Each entry in the matrix is vegetation data at a resolution of 30m as 
#'  classified into binary units with FALSE (grass) and TRUE (forest).
#'  The rainfall data provided here is the average rainfall (mm/yr) of a 
#'  moving window of dimension 7.5km which moves my 2.5 km along the length 
#'  of the rectangular data-set.   
#' 
#' @format A list of logical matrices 
#' 
#' @source Extracted from Eby's et al (2017) supplementary material 
#'   \url{https://github.com/tee-lab/spacetime-csd/}
#' 
#' @references 
#' 
#' Eby, S., Agrawal, A., Majumder, S., Dobson, A.P. & Guttal, V. (2017). 
#' Alternative stable states and spatial indicators of critical slowing down 
#' along a spatial gradient in a savanna ecosystem: Global Ecology 
#' and Biogeography, 26, 638-649
#' 
#' Reed, D. N., Anderson, T. M., Dempewolf, J., Metzger, K., & Serneels, S. (2009). 
#' The spatial distribution of vegetation types in the Serengeti ecosystem: 
#' the influence of rainfall and topographic relief on vegetation patch 
#' characteristics. Journal of Biogeography, 36(4), 770-782.
# 
# The strings below have no effect except forcing R to read this file. 
"serengeti" 

#' @rdname serengeti
#' 
#' @format The annual rainfall corresponding to the matrices in the serengeti 
#'   dataset, in the corresponding order. 
#' 
"serengeti.rain"



# A list of binary matrices for the density-dependent aggregation model 
#' @rdname dda
#' 
#' @title Density-dependent aggregation model
#' 
#' @description \code{dda} is a list of matrices representing results from the 
#'   density-dependent aggregation model (Siteur et al. 2023) in \code{dda}.
#'   \code{dda.pars} is a data frame with the model parameters. Each row of
#'   \code{dda.pars} corresponds to a matrix in \code{dda}, in the same order. All
#'   parameters were maintained constant, except for tau (see model
#'   definition in Siteur et al. 2023).
#' 
#' @source Kindly provided by Koen Siteur
#' 
#' @examples 
#' 
#' ddasews <- lsw_sews(dda)
#' plot(ddasews, along = dda.pars[ ,"tau"]) # tau is the changing parameter
#' 
#' display_matrix(dda[[4]]) 
#' 
#' @references 
#' 
#'   Siteur, Koen, Quan-Xing Liu, Vivi Rottschäfer, Tjisse van der Heide, Max Rietkerk,
#'   Arjen Doelman, Christoffer Boström, and Johan van de Koppel. 2023. "Phase-Separation
#'    Physics Underlies New Theory for the Resilience of Patchy Ecosystems." Proceedings 
#'    of the National Academy of Sciences 120 (2): e2202683120. 
#'    https://doi.org/10.1073/pnas.2202683120.
"dda"

#' @rdname dda
#' 
"dda.pars"

# 
# This file contains miscellaneous documentation for various objects. 
# 

#' @name simple_sews
#' 
#' @title \code{simple_sews} objects 
#' 
#' @description This help page describes the structure of \code{simple_sews_*} 
#'   objects, such as those defined by the classes \code{simple_sews_single},
#'   \code{simple_sews_list}
#' 
#' @details 
#' 
#' The \code{\link{spatialwarnings}} uses S3 objects (lists) internally to store 
#'   indicator values, along with the necessary data to plot and display
#'   results. It is not recommended to extract data directly from these objects,
#'   as they are subject to change with different releases of the package. The 
#'   preferred method is to use dedicated generic functions such as 
#'   \code{plot()} or \code{as.data.frame()} to display or export the results.
#'   Nonetheless, we document the structure of these objects here for reference. 
#' 
#' \code{simple_sews} objects are returned by all indicator functions that 
#'   return numeric values. This includes for example \code{\link{generic_sews}}, 
#'   \code{\link{flowlength_sews}}, \code{\link{compute_indicator}} but *not*
#'   \code{\link{patchdistr_sews}} or \code{\link{spectral_sews}}, which provide
#'   indicators that depend on non-numeric values (e.g. patch-size distribution
#'   type), or need to store more information than just a single numerical value
#'   (e.g. the spectrum of the input matrix). 
#' 
#' \code{simple_sews} objects come in multiple variants:
#'   \code{simple_sews_single} is the result of an indicator function applied to 
#'   a single matrix, and \code{simple_sews_test_single} is the result of 
#'   \code{\link{indictest}} applied to a \code{simple_sews_single} object. Both 
#'   these objects have list equivalents, \code{simple_sews_list} and 
#'   \code{simple_sews_test_list} which are simply a collection of their 
#'   'single' equivalent. These 'list' objects are used to store the results 
#'   of computations when working with multiple matrices. 
#' 
#' A \code{simple_sews_single} object is a list with the following components
#'   \describe{
#'     \item{\code{itemvalue}}{the indicator values. A vector of length one if there is 
#'       only one numeric value returned by the indicator function (e.g.
#'       \code{\link{flowlength_sews}}, or with a length above one otherwise}
#'     \item{\code{orig_data}}{the original matrix on which the indicator was computed}
#'     \item{\code{fun.args}}{the argument used in the call to the indicator function 
#'       (the function that given a matrix, returns the spatial metrics of
#'       interest)}
#'     \item{\code{taskname}}{a character string describing the current indicator(s) 
#'       being computed}
#'     \item{\code{indicf}}{the indicator function, which given the matrix, returns the
#'       spatial metric(s) of interest}
#'   }
#' 
#'   \code{simple_sews_test_single} have all of the above components, plus 
#' the following: 
#' 
#' \describe{ 
#'   \item{\code{nulldistr}}{the null distribution of values, with nulln rows and 
#'     as many columns as the number of values returned by the indicator 
#'     function}
#'   \item{\code{null_mean}}{the mean indicator values in the null distribution}
#'   \item{\code{null_sd}}{the standard deviation of the null distribution}
#'   \item{\code{null_qsup}}{the upper quantile of the null distribution, by default 
#'     the 95% upper quantile, but see \code{\link{indictest}} for a way to 
#'     adjust this}
#'   \item{\code{null_qinf}}{the upper quantile of the null distribution, by default 
#'     the 05% upper quantile, but see \code{\link{indictest}} for a way to 
#'     adjust this}
#'   \item{\code{z_score}}{the z_score of the observed value relative to the null 
#'     distribution, i.e. (value - null_mean) / null_sd}
#'   \item{\code{pval}}{the p-value of the indicator, i.e. the proporation of values 
#'     of the null distribution that fall below the observed indicator value}
#'   \item{\code{null_method}}{the method used to produce the null matrices. See 
#'     \code{\link{indictest}} for details}
#'   \item{\code{nulln}}{the number of null matrices used}
#'   \item{\code{get_nullmat}}{a function that can be called to obtain a randomized 
#'     matrix}
#'   \item{\code{matrixn}}{the number of the matrix, can be above one if the
#'     computations have been run on a list of matrices, or non-existent if 
#'     only one matrix was used}
#' }
#' 
#' @seealso \code{\link{custom_indicator}} 
#'
#' @aliases simple_sews_object
#'
NULL

#
# This contains a function to compute clustering on matrices
#

#' @title Clustering of pairs
#'
#' @description Compute the number of pairs of neighbor cells in a landscape and derive 
#'   from it clustering indices
#'
#' @param mat A matrix, usually with discrete values (logical, integers, etc.)
#'
#' @param wrap Whether space should be considered to wrap at the edges
#'
#' @param use_8_nb Set to \code{TRUE} to use an 8-way neighborhood. The default is set
#'   to FALSE, which uses a 4-way neighborhood
#'
#' @param prop Return the counts for all pairs in the matrix (FALSE), or
#'   the proportions for each pair (out of all possible pairs)
#' 
#' @details
#'
#'   The clustering of pairs is defined as the density of pairs, i.e.
#'     the proportion of all neighboring pairs of cells that share the same state, 
#'     divided by the null expectation given a random, homogeneous spatial structure.
#'
#'   For example, let's consider a matrix with two states, 'a' and 'b'.
#'     \code{raw_clustering} will count all pairs of cells 'a-a' or 'b-b', and
#'     divide this by the total number of pairs. This proportion is then again
#'     divided by the probability of obtaining these proportion of pairs under
#'     the assumption of no spatial structure (random mixing of cells in the
#'     matrix). 
#'
#'   Clustering is equal to one when there is no spatial structure. It is
#'     above one when two states are found next to each other (i.e. cluster)
#'     more than expected by chance. Values below one means that those two
#'     states tend to be neighbors less frequently than expected by chance. 
#'
#'   If you are only interested in the proportion of pairs for each combination
#'     of states, you can use the function \code{pair_counts}, which returns
#'     a matrix with as many rows and columns as there are states in the matrix, and 
#'     contains the counts for all possible pairs of cells found in the matrix 
#'     (or their relative proportions).
#'
#' @return
#'
#'   A vector with the requested clutering values for \code{raw_clustering},
#'     whose names are equal to each state (unique value) found in the
#'     original matrix, preceded by 'clust_' (to make sure names are
#'     compatible with other functions).
#'
#'  \code{pair_counts} returns the counts of pairs of states in the matrix,
#'     or the proportion of each pair, depending on the value of \code{prop}
#'
#' @examples
#'
#' # The clustering of a random matrix is close to one
#' ls <- 100 # lattice size
#' mm <- matrix(sample(c("sp1", "sp2", "sp3", "sp4"), size = ls^2, replace = TRUE),
#'              nrow = ls, ncol = ls)
#' clust <- raw_clustering(mm, wrap = TRUE, use_8_nb = TRUE)
#' print(clust)
#' 
#' # Compute clustering along the gradient for the serengeti dataset
#' \donttest{
#' 
#' data(forestgap)
#' clust_indic <- compute_indicator(serengeti, raw_clustering,
#'                                  wrap = TRUE, use_8_nb = FALSE)
#' # The interesting one is the clustering of state 0 (FALSE in the original matrix),
#' # which corresponds to grassland pixels, which get more and more clustered with 
#' # increasing rainfall (see also ?generic_sews for how that compares with generic 
#' # indicators)
#' plot(clust_indic, along = serengeti.rain)
#' 
#' # Add null trend
#' clust_test <- indictest(clust_indic, nulln = 19)
#' plot(clust_test, along = serengeti.rain)
#' 
#' # Show the proportion of each pairs of states in the matrix... 
#' pair_counts(serengeti[[5]])
#' 
#' # ... or the total count
#' pair_counts(serengeti[[5]], prop = FALSE)
#' 
#' }
#' 
#'@export
raw_clustering <- function(mat,
                           wrap = TRUE,
                           use_8_nb = FALSE) {

  mat_fmt <- format_mat_for_pair_count(mat)
  mat <- mat_fmt[[1]]
  ilevels <- mat_fmt[[2]]

  ncells <- prod(dim(mat))

  # Count the number of neighboring pairs of each state
  total_pairs <- pair_counts(mat, wrap = wrap, use_8_nb = use_8_nb,
                             prop = TRUE)
  pair_props <- diag(total_pairs)

  state_props <- sapply(seq_along(ilevels)-1, function(lvl) {
    mean(mat == lvl)
  })

  # N1 = number of cells in state 1
  # n = total number of cells
  # P1 = proportion of cells in state 1 = N1/n
  # P(two time N1) = ( N1 / n * (N1-1) / (n-1) ) =
  #   = P1 * ( P1 * n - 1 ) / ( n - 1)
  pair_nulls <- state_props * ( state_props * ncells - 1 ) / ( ncells - 1 )
  clust_pairs <- pair_props / pair_nulls

  names(clust_pairs) <- paste0("clust_", ilevels)

  return(clust_pairs)
}


#'@export
#'@rdname raw_clustering
pair_counts <- function(mat,
                        wrap = TRUE,
                        use_8_nb = FALSE,
                        prop = TRUE) {
  mat_fmt <- format_mat_for_pair_count(mat)
  mat <- mat_fmt[[1]]
  ilevels <- mat_fmt[[2]]

  pairs <- pair_counts_internal(mat, length(ilevels), wrap, use_8_nb)
  rownames(pairs) <- colnames(pairs) <- ilevels

  if ( prop ) {
    total_pairs <- total_pair_count(mat, wrap, use_8_nb)
    pairs <- pairs / total_pairs
  }

  # Make a semi-triangular matrix
  for ( row in seq(1, nrow(pairs)-1) ) {
    for ( col in seq(row+1, ncol(pairs)) ) {
      pairs[col, row] <- pairs[col, row] + pairs[row, col]
      pairs[row, col] <- NA
    }
  }
#   pairs[lower.tri(pairs)] <- pairs[upper.tri(pairs)] + pairs[lower.tri(pairs)]
#   pairs[upper.tri(pairs)] <- NA

  return(pairs)
}


# Format a matrix before feeding it to pair-counting cpp functions
format_mat_for_pair_count <- function(mat) {

   if ( ! is.matrix(mat) ) {
     msg <- paste0("clustering functions only accept a single matrix as input. ",
                   "Use compute_indicator(<list>, fun) to use several matrices ",
                   "at once")
     stop(msg)
  }

  if ( is.logical(mat) ) {
    dims <- dim(mat)
    mat <- factor(c("FALSE", "TRUE")[mat+1], levels = c("TRUE", "FALSE"))
    dim(mat) <- dims
  }

  # If the matrix has floats, convert it to factor. We don't know the full
  # range of levels in mat, so just use as.factor()
  if ( is.numeric(mat) || is.character(mat) ) {
    dims <- dim(mat)
    mat <- as.factor(mat)
    dim(mat) <- dims
  }

  # mat is always a factor at this point
  lvls <- levels(mat)

  if ( is.factor(mat) ) {
    mat <- matrix(as.integer(mat) - 1, nrow = nrow(mat), ncol = ncol(mat))
  }

  return( list(mat, lvls) )
}

# Get the number of pairs for a given matrix size and options. There is
# more information about pair counting in clustering.cpp
total_pair_count <- function(mat, wrap, use_8_nb) {
  nr <- nrow(mat)
  nc <- ncol(mat)
  ncells <- nr * nc
  total_pairs <- ifelse(use_8_nb,
                        4 * ncells,
                        2 * ncells)
  if ( ! wrap ) {
    if ( use_8_nb ) {
      total_pairs <- total_pairs - ( 2 + nr + nc + (nr-1)*2 + (nc-1)*2 )
    } else {
      total_pairs <- total_pairs - (nr + nc) # ok
    }
  }

  return(total_pairs)
}

#
# This tests the computation of clustering
#

test_that("Neighbor-counting is OK", {

  # Check that total number of pairs matches counting done by hand
  expect_true( total_pair_count(diag(2), TRUE, TRUE) == 16  )
  expect_true( total_pair_count(diag(2), FALSE, TRUE) == 6  )
  expect_true( total_pair_count(diag(2), FALSE, FALSE) == 4 )
  expect_true( total_pair_count(diag(2), TRUE, FALSE) == 8  )
  
  # Check that total number of pairs matches a manual sum after the fact
  for ( wrap in c(TRUE, FALSE) ) { 
    for ( use_8_nb in c(TRUE, FALSE) ) { 
      a <- pair_counts(diag(10), wrap = wrap, use_8_nb = use_8_nb)
      b <- pair_counts(diag(10), prop = FALSE, wrap = wrap, use_8_nb = use_8_nb) 
      b <- b / sum(b, na.rm = TRUE)
      expect_true({ 
        all( abs(a[!is.na(a)] - b[!is.na(b)]) < 1e-8 )
      })
    }
  }
})

test_that("Counting gives the same results as hand-counting", {
  egmat <- matrix(
    c("a", "b", "c",
      "a", "a", "b",
      "c", "a", "c"),
    byrow = TRUE, nrow = 3, ncol = 3
  )

  # 4 neihbors
  test1 <- pair_counts(egmat, prop = FALSE, wrap = FALSE)
  expect_true({
    all(
      c(test1["a", "a"] == 3,
        test1["b", "b"] == 0,
        test1["c", "c"] == 0,
        test1["b", "a"] == 3,
        test1["c", "a"] == 3,
        test1["c", "b"] == 3,
        test1["b", "b"] == 0)
    )
  })

  test1 <- pair_counts(egmat, prop = FALSE, wrap = TRUE)
  expect_true({
    all(
      c(test1["a", "a"] == 3,
        test1["b", "b"] == 0,
        test1["c", "c"] == 2,
        test1["b", "a"] == 5,
        test1["c", "a"] == 5,
        test1["c", "b"] == 3)
    )
  })

  test1 <- pair_counts(egmat, prop = FALSE, wrap = TRUE, use_8_nb = TRUE)
  expect_true({
    all(
      c(test1["a", "a"] == 6,
        test1["b", "b"] == 1,
        test1["c", "c"] == 3,
        test1["b", "a"] == 8,
        test1["c", "a"] == 12,
        test1["c", "b"] == 6)
    )
  })
})

test_that("Computation of clustering are OK", {

  mat <- matrix(c(1, 0, 0,
                  0, 0, 0,
                  0, 0, 0),
                ncol = 3, byrow = TRUE)


  # With 4 nbs and wrapping, we have 14 pairs of 0-0, 0 of 1-1
  clusts <- clustering_core(mat, ns = 2, wrap = TRUE, use_8_nb = FALSE)
  expect_true({
    all(c(clusts[ ,1] == c(14, 0),
          clusts[ ,2] == c(8, 1) )) # counts of single cells
  })

  # With 8 nbs and wrapping, we have 21 pairs of 0-0, 0 of 1-1
  clusts <- clustering_core(mat, ns = 2, wrap = TRUE, use_8_nb = TRUE)
  expect_true({
    all(c(clusts[ ,1] == c(28, 0),
          clusts[ ,2] == c(8, 1) )) # counts of single cells
  })

  # With 4 nbs and no wrapping, we have ten pairs of 0-0, 0 of 1-1
  clusts <- clustering_core(mat, ns = 2, wrap = FALSE, use_8_nb = FALSE)
  expect_true({
    all(c(clusts[ ,1] == c(10, 0),
          clusts[ ,2] == c(8, 1) )) # counts of single cells
  })

  # With 8 nbs and no wrapping, we have 13 pairs of 0-0, 0 of 1-1
  clusts <- clustering_core(mat, ns = 2, wrap = FALSE, use_8_nb = TRUE)
  expect_true({
    all(c(clusts[ ,1] == c(17, 0),
          clusts[ ,2] == c(8, 1) )) # counts of single cells
  })



  # Make sure clustering of random matrices is close to one
  if ( exists("EXTENDED_TESTS") && EXTENDED_TESTS ) {
    clusts <- replicate(199, {
      m <- matrix(sample(letters[1:4], size = 100^2, replace = TRUE),
                  nrow = 100, ncol = 100)
      raw_clustering(m)
    })
    expect_true({
      all( abs(apply(clusts, 1, mean) - 1) < 0.10 )
    })
  }

})

test_that("Counting of pairs makes sense", {
  n <- 99
  counts <- lapply(seq.int(n), function(n) { 
    m <- matrix(sample(letters[1:4], size = 100^2, replace = TRUE),
                nrow = 100, ncol = 100)
    pair_counts(m, prop = TRUE)
  })
  
  means <- Reduce(`+`, counts) / n
  
  # Sum should be 1
  expect_true({ 
    abs( sum(means, na.rm = TRUE) - 1 ) < 0.00001 
  })
  
  # Diags should be equal
  expect_true({ 
    all(abs(diff(diag(means))) < 0.1)
  })
  
  # Lower tri values should be equal
  expect_true({ 
    all( abs(diff(means[lower.tri(means)])) < 0.01)
  })

  expect_true({
    all(abs(
      c(means["a", "a"] + means["b", "a"]/2 + means["c", "a"]/2 + means["d", "a"]/2,
        means["b", "a"]/2 + means["b", "b"] + means["c", "b"] / 2 + means["d", "b"]/2) -
        .25
    ) < 1e-2)
  })

})

test_that("A buggy matrix is now correctly handled", {
  a <- readRDS("./buggy_matrix_for_pairs.rds")
  counts <- pair_counts(a)
  expect_true({
    abs(counts["a", "b"] - 0) < 1e-8
  })
})

# 
# 

# Function that transforms a set of values into stars for display
pval_stars <- function(value, NA_ret = NA) { 
  
  if ( length(value) > 1 ) { 
    stars <- sapply(value, pval_stars, NA_ret = NA_ret) 
    attributes(stars) <- attributes(value)
    return(stars)
  }
  
  if ( is.na(value) ) { 
    return(NA_ret) 
  }
  
  if (value < 0.001) { 
    return('***' )
  } else if (value < 0.01) { 
    return('** '  )
  } else if (value < 0.05) { 
    return('*  '   )
  } else if (value < 0.1) { 
    return('.  '   )
  } else { 
    return('   ')
  }
}

# Format pvalues to reflect the number of null values instead of printing a 
# 0 when it is below what's detectable. 
format_pvalues <- function(X, nreps) { 
  ifelse(X == 0, 
         paste0("<", formatC(1/nreps, 
                             width = 3, 
                             digits = 0, 
                             format = "e")), 
         formatC(X, format = "f", digits = 3))
}

# Print a line with matrix size information
#' @title Display size information of a sews_result object
#'
#' @param x A \code{sews_result} object (as provided by **_sews functions, such
#'   as \code{generic_sews()}
#'
#' @param ... Other arguments are ignored.
#'
#' @details This function is used to display information about the size of matrices
#'   used by various functions in spatialwarnings. It is mostly destined to internal
#'   use.
#'
#'@export
display_size_info <- function(x, ...) {
  UseMethod('display_size_info')
}

# ... for sews_result_single class
#'@rdname display_size_info
#'@exportS3Method
display_size_info.sews_result_single <- function(x, ...) {
  display_size_info(list(x))
}

# ... for sews_result_list class
#'@rdname display_size_info
#'@exportS3Method
display_size_info.sews_result_list <- function(x, ...) {
  sizes <- sapply(x, function(x) dim(x[["orig_data"]]))
  sizes <- apply(sizes, 1, function(X) length(unique(X)) == 1)
  has_different_sizes <- ! any(sizes)
  if (has_different_sizes) {
    size_text_report <- "variable sizes"
  } else {
    size_text_report <- paste0("size: ", nrow(x[[1]][["orig_data"]]), 'x',
                               ncol(x[[1]][["orig_data"]]))
  }
  cat(' ',
      length(x), ' ',
      ifelse(length(x)>1, 'matrices', 'matrix'), ' ',
      "(", size_text_report,')\n', sep = '')
}

# ...for generic lists
#'@rdname display_size_info
#'@exportS3Method
display_size_info.list <- function(x, ...) {
  display_size_info.sews_result_list(x, ...)
}

ifNULLthen <- function(a, b) { 
  if (is.null(a)) b else a 
}

list_methods <- function(class, 
                         exclude = c("print", "display_size_info", 
                                     "summary")) { 
  all_methods <- lapply(class, function(class) { 
    tab <- attr(methods(class = class), "info")
#     tab[tab[ ,"from"] == "spatialwarnings", "generic"]
    tab[ ,"generic"]
  })
  all_methods <- sort(unique(unlist(all_methods)))
  
  # For _test_ objects, we do not report indictest
  if ( any(grepl("_sews_test_", class)) ) { 
    all_methods <- all_methods[!grepl("indictest", all_methods)]
  }
  
  # Exclude some reported methods 
  all_methods <- all_methods[! all_methods %in% exclude]
  
  return(all_methods)
}

# Wrapper around future_lapply to set future.seed = TRUE so that random number 
# generation in child processes is correct. 
future_lapply_seed <- function(...) { 
  future.apply::future_lapply(..., future.globals = FALSE, future.seed = TRUE)
}

# The goal of this file is to test the computation of the sum:
#
#   x^(-a)exp(-bx) for x from 0 to infinity
#
# which is done by tplinfsum in spatialwarnings

if ( exists("EXTENDED_TESTS") && EXTENDED_TESTS ) {

  # very small to 10, log-separated
  res <- 8
  grid <- expand.grid(a = seq(-.5, 10, l = res),
                      b = seq(.Machine$double.eps, 10, l = res))

  reltol <- 1e-8
  maxit <- 1e7
  chunksize <- 1e6

  results <- plyr::ldply(seq.int(nrow(grid)), function(gridi) {

    a <- grid[gridi, "a"]
    b <- grid[gridi, "b"]
    exps <- pows <- prod <- 0

    i <- inext <- 1
    while ( i < maxit ) {
      inext <- min(i + chunksize + 1, maxit)
      xs <- seq(i, inext, by = 1)

      exps <- exps + sum( exp(-b * xs ) )
      pows <- pows + sum( xs^(-a)       )
      prod <- prod + sum( exp(-b * xs ) * xs^(-a) )
      i <- inext
    }

    # This can produce warnings that the max number of iterations has been reached.
    # Ignore those.
    suppressWarnings({
      ref <- tplinfsum(a, b, xmin = 1, maxit = maxit, reltol = reltol)
    })

    data.frame(grid[gridi, ],
              imax = inext,
              maxit = maxit,
              expsum = exps,
              powsum = pows,
              totsum = prod,
              ref = ref)
  }, .progress = "none")

  with(results, plot(log10(totsum), log10( abs(totsum - ref) / totsum )))

  ggplot(results, aes(x = a, y = b)) +
    geom_raster(aes(fill = log10(totsum - ref))) +
    scale_fill_distiller(palette = "Spectral")

}

# 
# 
# This file contains methods related to objects of the patchdistr task
# 



# Plot methods
# --------------------------------------------------
# 
#' @rdname patchdistr_sews_plot
#' @name patchdistr_sews_plot
#' 
#' @title Early-warning signals based on patch size distributions
#' 
#' @description Plot early-warning signals based on patch size distributions 
#' 
#' @param x An object as produced by \code{\link{patchdistr_sews}}, or 
#'   the result of \code{\link{indictest}} called on such object
#' 
#' @param along A vector providing values along which the indicator trends 
#'   will be plotted. If \code{NULL} then the values are plotted sequentially 
#'   in their original order. 
#' 
#' @param ... Ignored 
#' 
#' @details 
#'   
#'   The \code{plot} function will produce a figure summarizing the changes
#'   in patch size distributions along a set of values. The figure has two 
#'   panels: 
#'   \itemize{ 
#'      \item the upper panel shows the percolation status of empty 
#'        (\code{FALSE}) and occupied cells (\code{TRUE}), and shows the mean 
#'        value (proportion of \code{TRUE} values). The background shows 
#'        the proportion of each type of distribution for each unique values 
#'        of the \code{along} vector. 
#'        
#'      \item the bottom panel displays the power-law range
#'   }
#'  
#'  The \code{plot_distr} function displays each distribution in an 
#'    individual facet, with an overlay of the best distribution fit and a blue 
#'    bar showing the power-law range. If appropriate, a grey ribbon is used to
#'    display the expected distribution given the null expectation (i.e. when 
#'    \code{plot_distr} is called on the results of \code{indictest()}. This 
#'    function can produce quite crowded graphs, but it displays in full the
#'    shape of the distributions, and can be useful e.g. to assess the quality 
#'    of the fits. 
#' 
#' @seealso \code{\link{patchdistr_sews}}
#' 
#' @examples
#' 
#' \donttest{ 
#' data(forestgap)
#' psd_indic <- patchdistr_sews(forestgap)
#' 
#' plot(psd_indic, along = forestgap.pars[ ,"d"]) 
#' 
#' # When along is non-numeric, bars are used for display
#' plot(psd_indic, along = as.factor(forestgap.pars[ ,"d"]))
#' 
#' # Display individual distributions
#' plot_distr(psd_indic, along = forestgap.pars[ ,"d"])
#' 
#' # We can display the distributions along with the null expectation after 
#' # indictest() is run
#' psd_test <- indictest(psd_indic, nulln = 19)
#' plot_distr(psd_test, along = forestgap.pars[ ,"d"])
#' }
#'
#'@method plot patchdistr_sews
#'@export
plot.patchdistr_sews <- function(x, along = NULL, ...) { 
  if ( 'patchdistr_sews_single' %in% class(x) ) { 
    stop('I cannot plot a trend with only one value')
  }
  
  plot.patchdistr_sews_list(x, along)
}

# Note: plot will display how characteristics change along a gradient. To 
#   have a plot of distributions, use plot_distr
# 
#'@method plot patchdistr_sews_list
#'@export
plot.patchdistr_sews_list <- function(x, along = NULL, ...) { 
  
  if ( !is.null(along) && (length(along) != length(x)) ) { 
    stop('The along values are unfit for plotting (size mismatch)')
  }
  
  obj_table <- as.data.frame(x)
  
  # Subset table for best fits
  obj_table <- obj_table[is.na(obj_table[ ,'best']) | obj_table[ ,'best'], ]
  
  if ( all(is.na(obj_table[ ,"best"])) && 
       all(is.na(obj_table[ ,"plrange"])) ) { 
    warning("There is no non-missing data to plot in the provided object")
    return( invisible(NULL) )
  }
  
  # If along is provided, then add it to the table
  xtitle <- deparse(substitute(along))
  if ( is.null(along) ) { 
    along <- as.factor(obj_table[ ,'matrixn'])
    xtitle <- "Matrix number"
  }
  
  obj_table[ ,"along"] <- along
  
  # Now we summarise the obj_table
  alltypes <- na.omit(unique(obj_table[ ,"type"]))
  summ <- ddply(obj_table, 'along',
                function(df) { 
                  type_freqs <- sapply(alltypes, 
                         function(current_type) { 
                           sum(!is.na(df[ ,'type']) & df[ 'type'] == current_type)
                         })
                  type_freqs <- type_freqs / ifelse(sum(type_freqs) > 0, 
                                                    sum(type_freqs), 1)

                  data.frame(type = alltypes, type_freq = type_freqs, 
                             percolation = mean(df[ ,'percolation']), 
                             percolation_empty = mean(df[ ,'percolation_empty']), 
                             cover = mean(df[ ,"cover"]), 
                             # We remove NAs as sometime the power-law range cannot 
                             # be computed (not enough points)
                             plrange = mean(df[ ,'plrange'], na.rm = TRUE))
                  })
  
  # Make a data.frame with dummy facets 
  classif_df <- data.frame(plot_type = "PSD Classif.", summ)
  classif_df[ ,'plrange'] <- NA
  
  plrange_df <- ddply(summ, "along", function(df) { 
                       data.frame(plot_type = "PL-range", 
                                  plrange = df[1, "plrange"])
                      })
  
  summ <- rbind.fill(classif_df, plrange_df)
  
  # Construct base plot
  plot <- ggplot(summ) + 
    theme_spwarnings() + 
    fillscale_spwarnings(name = "PSD type") + 
    scale_linetype_discrete(name = "Cover") + 
    facet_grid( plot_type ~ ., switch = "y") + 
    ylab('') + 
    xlab(xtitle)
  
  if ( ! is.numeric(along) ) { 
    # Note that it is quite tricky to make ggplot produce a line over factors: 
    # `group=1` seems to do the trick. 
    plot <- plot + 
      geom_bar(aes_q(x = ~along, y = ~type_freq, fill = ~type), 
               position = "stack", stat = "identity") + 
      stat_summary(aes_q(x = ~along, y = ~percolation, 
                         linetype = "Perc. (full)", group = 1), 
                   fun = mean, geom = "line") + 
      stat_summary(aes_q(x = ~along, y = ~percolation_empty, 
                         linetype = "Perc. (empty)", group = 1), 
                   fun = mean, geom = "line") + 
      stat_summary(aes_q(x = ~along, y = ~cover, 
                         linetype = "Mean cover", group = 1), 
                   fun = mean, geom = "line") 
  
  } else { 
    plot <- plot + 
      geom_area(aes_q(x = ~along, y = ~type_freq, fill = ~type), 
                position = "stack") + 
      geom_line(aes_q(x = ~along, y = ~percolation, linetype = "Perc. (full)"))  + 
      geom_line(aes_q(x = ~along, y = ~percolation_empty, linetype = "Perc. (empty)"), 
                color = 'black') +
      geom_line(aes_q(x = ~along, y = ~cover, linetype = "Mean cover"), 
                color = 'black') 
      
  }
  
  # Add plrange to the graph
  plot <- plot + 
      geom_point(aes_q(x = ~along, y = ~plrange, group = 1), 
                stat = "identity") + 
      geom_line(aes_q(x = ~along, y = ~plrange, group = 1), 
                stat = "identity") 
  
  return(plot)
}


#' @rdname patchdistr_sews_plot
#' 
# // along arg is already documented in plot() method
#' 
#' @param best_only Plot only the best fit the empirical (inverse cumulative) 
#'   patch-size distribution with an overlay of the estimated fits. 
#' 
#' @param plrange Plot the power-law range 
#'
#'@export
plot_distr <- function(x, along = NULL, best_only = TRUE, plrange = TRUE) { 
  UseMethod('plot_distr')
}

#'@export
plot_distr.patchdistr_sews <- function(x, along = NULL, best_only = TRUE, 
                                        plrange = TRUE) { 
  NextMethod('plot_distr')
}

#'@export
plot_distr.patchdistr_sews_single <- function(x, 
                                              along = NULL, 
                                              best_only = TRUE, 
                                              plrange = TRUE) { 
  
  if ( !is.null(along) ) { 
    warning('Ignoring along parameter when plotting only one patch-size', 
            'distribution')
  }
  
  # Get plottable data.frames
  values <- predict(x, best_only = best_only, xmin_rescale = TRUE)  
  
  # Create base plot 
  plot <- ggplot() + 
    scale_x_continuous(trans = "log10") +
    scale_y_continuous(trans = "log10") + 
    theme_spwarnings() +
    labs(x = 'Patch size',
         y = 'Observed frequency (P>=x)',
         color = 'Fitted type')
  
  # If plrange exists, then add it to the plot 
  plrange_dat <- x[['plrange']]
  if ( plrange && !is.na(plrange_dat[ ,"plrange"]) ) { 
    plrange_dat[ ,"xmax"] <- max(values[["obs"]][ ,"patchsize"])
    plot <- plot + 
      geom_segment(aes_q(x = ~xmin_est, y = 1, 
                         xend = ~xmax,  yend = 1), 
                   data = plrange_dat, 
                   arrow = arrow(ends = "both", type = "open", 
                                 length = unit(0.05, "inches")), 
                   color = "blue")
  }
  
  # Add observed values
  plot <- plot + 
    geom_point(aes_string(x = "patchsize", y = "y"), data = values[['obs']]) 
  
  # It can happen that no distribution have been fitted. Check for that 
  # before plotting otherwise it produces an error. 
  if ( any(!is.na(values[['pred']][ ,"y"])) ) { 
    
    # Get data and remove NAs (where no fit has been carried out
    pred_values <- values[["pred"]]
    pred_values <- pred_values[!is.na(pred_values[ ,"y"]), ]
    
    # Use only the y values of the fit that falls within the range of the observed 
    # values. Otherwise fit curves can go very low and we don't see a thing on the plot 
    pred_values <- pred_values[pred_values[ ,"y"] >= min(values[["obs"]]) & 
                               pred_values[ ,"y"] <= max(values[["obs"]]), ]
    
    if ( nrow(pred_values) > 0 ) { 
      plot <- plot + 
        geom_line(aes_string(x = "patchsize", y = "y", color = "type"), 
                  data = pred_values)
    }
    
  } else { 
    warning('No distribution has been fitted to the observed patch size distribution')
  }
  
  return(plot)
}

#'@export
plot_distr.patchdistr_sews_list <- function(x, 
                                            along = NULL, 
                                            best_only = TRUE, 
                                            plrange = TRUE) { 
  
  if ( !is.null(along) && (length(along) != length(x)) ) { 
    stop('The along values are unfit for plotting (size mismatch)')
  }
  
  # Get plottable data.frames
  values <- predict(x, best_only = best_only, xmin_rescale = TRUE)
  
  values[['obs']][ ,"along"]  <- values[['obs']][ ,'matrixn']
  values[['pred']][ ,"along"] <- values[['pred']][ ,'matrixn']
  # Modify matrixn column if necessary and reorder values
  if ( ! is.null(along) ) { 
    values[['obs']][ ,'along']  <- along[values[["obs"]][ ,'matrixn']]
    values[['pred']][ ,'along'] <- along[values[["pred"]][ ,'matrixn']]
  }
  
  plot <- ggplot() + 
    scale_y_log10() +
    scale_x_log10() + 
    facet_wrap( ~ along ) + 
    xlab('Patch size') + 
    ylab('Frequency (P>=x)') + 
    theme_spwarnings()
  
  if ( plrange ) { 
    # Add plrange to the plot. We need to extract info 
    # from the observed psd so that we can place the segment on the plot. 
    plrange_dat <- unique( as.data.frame(x)[ ,c("matrixn", 'xmin_est')] )
    plrange_dat[ ,"along"] <- plrange_dat[ ,"matrixn"]
    if ( ! is.null(along) ) { 
      plrange_dat[ ,"along"] <- along[plrange_dat[ ,"matrixn"]]
    }
    
    patches_minmax <- ddply(values[['obs']], "matrixn", 
                          function(df) { 
                            data.frame(xmax = max(df[ ,"patchsize"]))
                          })
    plrange_dat <- join(plrange_dat, patches_minmax, type = "left", 
                        match = "first", by = "matrixn")
    
    plot <- plot + 
      geom_segment(aes_q(x = ~xmin_est, y = 1, 
                         xend = ~xmax,  yend = 1), 
                    data = plrange_dat, 
                    arrow = arrow(ends = "both", type = "open", 
                                  length = unit(0.05, "inches")), 
                    color = "blue") 
  
  }
  
  # Add observed values
  plot <- plot + 
    geom_point(aes_string(x = "patchsize", y = "y"), 
               data = values[['obs']]) 
  
  # It can happen that no distribution have been fitted. Check for that 
  # before plotting otherwise it produces an error. 
  if ( any(!is.na(values[['pred']][ ,"y"])) ) { 
    # Get data and remove NAs (where no fit has been carried out
    pred_values <- values[["pred"]]
    pred_values <- pred_values[! is.na(pred_values[ ,"type"]) & 
                               ! is.na(pred_values[ ,"y"]), ]
    
    # Use only the y values of the fit that falls within the range of the observed 
    # values. Otherwise fit curves can go very low and we don't see a thing on the plot 
    pred_values <- pred_values[pred_values[ ,"y"] >= min(values[["obs"]]) & 
                               pred_values[ ,"y"] <= max(values[["obs"]]), ]
    if ( any(is.na(pred_values[ ,"type"])) ) browser()
    
    if ( nrow(pred_values) > 0 ) { 
      plot <- plot + 
        geom_line(aes_string(x = "patchsize", y = "y", color = "type"), 
                  data = pred_values)
    }
    
    plot <- plot + 
              geom_line(aes_string(x = "patchsize", 
                                   y = "y", 
                                   color = "type", 
                                   group = "paste(type, matrixn)"), 
                        data = pred_values)

  } else { 
    warning('No distribution has been fitted to the observed patch size distributions')
  }
  
  return(plot)
}





# Predict methods 
# --------------------------------------------------
#' @rdname patchdistr_sews_predict
#' @name patchdistr_sews_predict
#' 
#' @title predict method for patchdistr_sews objects
#' 
#' @description Export the observed and fitted patch size distributions 
#' 
#' @param object An \code{\link{patchdistr_sews}} object 
#' 
#' @param newdata A vector of patch sizes at which the fit is returned (default 
#'   to 200 regularly-spaced values). 
#' 
#' @param ... Additional arguments (ignored)
#' 
#' @param best_only Return values for only the best fit of each element (matrix)
#'   in \code{object}, or return the values for all fitted distribution. 
#' 
#' @param xmin_rescale If the xmin value used for fits is above one, then setting this
#'   to \code{TRUE} will rescale the predicted probabilities so that they align on 
#'   the cumulative distribution of the observed patch sizes 
#' 
#' @return A list with component \code{obs}, a \code{data.frame} containing the observed 
#'   distribution values and \code{pred}, a \code{data.frame} containing the fitted 
#'   values. Both \code{data.frame}s have columns \code{matrixn}, the number of 
#'   the matrix for which values are given, \code{type}, the fitted type of distribution, 
#'   as well as \code{patchsize} and \code{y}, the patch size and value of the
#'   inverse cumulative distribution function (i.e. \eqn{1 - P(x<k)}). 
#' 
#' @details The function \code{\link{patchdistr_sews}} fits competing 
#'   distribution models to the observed patch size distributions. This 
#'   functions is able to export the observed values and the fitted values 
#'   altogether. 
#' 
#' @examples 
#' 
#' \donttest{ 
#' patch_indics <- patchdistr_sews(forestgap)
#' 
#' predict(patch_indics)
#' 
#' }
#' 
#' @seealso \code{\link{patchdistr_sews}}
#' 
#'@export
predict.patchdistr_sews_single <- function(object, ..., 
                                           newdata = NULL,
                                           best_only = FALSE, 
                                           xmin_rescale = FALSE) { 
  
  # Get observed values
  vals_obs <- cumpsd(object[["psd_obs"]])
  
  # Shapes table
  shptbl <- object[['psd_type']]
  xmin_used <- shptbl[1, "xmin_fit"] # should be the same for all distribs
  stopifnot(length(unique(shptbl[ , "xmin_fit"])) == 1) # enforce it
  
  # Bail if no fit carried out. Note that we need to set classes in the 
  # output pred df otherwise coercion when merging with existing results 
  # goes wrong (e.g. factor converted to integer).
  if ( all(is.na(shptbl[ ,'type'])) ) { 
    result <- list(obs = vals_obs, 
                   pred = data.frame(type = NA_character_, 
                                     patchsize = NA_integer_, 
                                     y = NA_real_))
    return(result)
  }
  
  # Create x vector of values. We predict only values above xmin, as the distribution 
  # is undefined below that. 
  if ( is.null(newdata) ) { 
    newdata <- unique( round(10^(seq(log10(xmin_used), 
                                     log10(max(object[["psd_obs"]])), 
                                     length.out = 256))) )
  }
  
  if ( best_only ) { 
    shptbl <- shptbl[shptbl[ ,'best'], ]
  }
  
  # Get values
  vals_pred <- data.frame()
  for ( type in shptbl[ ,"type"] ) { 
    type_yvals <- switch(type, 
                         pl  = ippl(newdata, 
                                    shptbl[type, "plexpo"], 
                                    shptbl[type, "xmin_fit"]),
                         tpl = iptpl(newdata, 
                                     shptbl[type, "plexpo"], 
                                     shptbl[type, "cutoff"], 
                                     shptbl[type, 'xmin_fit']),
                         exp = ipdisexp(newdata,  
                                        shptbl[type, "cutoff"], 
                                        shptbl[type, "xmin_fit"]),
                         lnorm = ipdislnorm(newdata, 
                                            shptbl[type, "meanlog"], 
                                            shptbl[type, "sdlog"],  
                                            shptbl[type, "xmin_fit"]))
    
    vals_pred <- rbind(vals_pred, 
                       data.frame(type = type, 
                                  patchsize = newdata, 
                                  y = type_yvals))
  }
  
  # If xmin > 1, we need to rescale the values by a factor so that the probability 
  # matches the observed data in the cumulative distribution representation. This is 
  # mainly used when plotting the distributions. 
  if ( xmin_rescale ) { 
    # We use approx because there is no guarantee that the xmin used falls on an 
    # observed patch size
    scale_factor <- approx(vals_obs[ ,"patchsize"], 
                           vals_obs[ ,"y"], 
                           xmin_used)[["y"]]
    
    vals_pred[ ,"y"] <- vals_pred[ ,"y"] * scale_factor
  }
  
  # Return data
  return( list(obs = vals_obs, 
               pred = vals_pred) )
}

#'@export
predict.patchdistr_sews_list <- function(object, ..., 
                                         newdata = NULL, 
                                         best_only = FALSE, 
                                         xmin_rescale = FALSE) { 
  
  dat <- lapply(object, predict.patchdistr_sews_single, 
                newdata = newdata, best_only = best_only, 
                xmin_rescale = xmin_rescale)
  
  # Add id but handle when psd is empty
  add_id <- function(n, x) { 
    if (nrow(x) > 0) { 
      x <- data.frame(matrixn = n, x) 
    } else {
      x <- data.frame(matrixn = n, patchsize = NA_integer_)
    } 
  }
  
  dat <- Map(function(n, x) { 
               x[['obs']]  <- add_id(n, x[["obs"]])
               x[['pred']] <- add_id(n, x[["pred"]])
               x
             }, seq.int(length(dat)), dat)
  
  # Convert to data.frame
  dat <- list(obs  = plyr::ldply(dat, function(x) x[['obs']]), 
              pred = plyr::ldply(dat, function(x) x[['pred']]))
  
  return(dat)
}




# As data.frame methods
# --------------------------------------------------

#'@export
as.data.frame.patchdistr_sews_single <- function(x, ...) { 
  ans <- data.frame(x[['psd_type']], x[['plrange']],
                    # Add the name explicitely for these as they are single values
                    percolation = x[['percolation']], 
                    percolation_empty = x[['percolation_empty']], 
                    cover = x[['cover']], 
                    npatches = x[['npatches']],
                    unique_patches = x[['unique_patches']], 
                    stringsAsFactors = FALSE)
  # We convert the psd type to a character vector for simplicity
  ans[ ,'type'] <- as.character(ans[ ,'type'])
  ans
}

#'@export
as.data.frame.patchdistr_sews_list <- function(x, ...) { 
  
  # Format data
  results <- lapply(x, as.data.frame.patchdistr_sews_single)
  results <- Map(function(n, df) { 
                   data.frame(matrixn = n, df) 
                 }, seq.int(length(results)), results)
  
  # Bind it altogether and return df 
  do.call(rbind, results)
}




# Summary methods
# --------------------------------------------------

prepare_summary_table <- function(x, ...) { 
  
  dat <- as.data.frame(x)
  # Select lines that either have no fit or select the best fit
  dat <- dat[is.na(dat[ ,"best"]) | dat[ ,"best"], ]
  
  # Add formated unique patch column 
  dat[ ,'pretty_patches'] <- with(dat, 
                                  paste0(npatches, "(", unique_patches, ")"))
  # Add formated plrange column
  dat[ ,'plrange'] <- with(dat, 
                           ifelse(is.na(plrange), "", 
                                  paste0(round(plrange * 100), "%")))
  
  # Add formated distribution type column
  dat[ ,'type'] <- ifelse(is.na(dat[ ,'type']), " ", dat[ ,'type'])
  
  # Set the column names to extract
  cols <- c('pretty_patches', 'cover', 'percolation', 'percolation_empty',
            'type', 'plrange')
  pretty_names <- c('N(uniq.)', 'Cover', 'Percl.Full', 'Percl.Empt', 'Type', 'PLR')
  
  # If there is a matrixn column (for when several matrices were 
  #   used at once), then add it to these columns
  if ( "matrixn" %in% names(dat) ) { 
    cols <- c("matrixn", cols)
    pretty_names <- c('Mat. #', pretty_names)
  }
  
  # If there is a pvalue columne, then add it to the output table at the end, 
  # along with the stars
  if ( "plrpval" %in% names(dat) ) { 
    dat[ ,"pvalstars"] <- pval_stars(dat[ ,"plrpval"])
    dat[ ,'pvalstars'] <- ifelse(is.na(dat[ ,'plrpval']), " ", 
                                 as.character(dat[ ,'pvalstars']))
    dat[ ,'plrpval'] <- ifelse(is.na(dat[ ,'plrpval']), " ", 
                               as.character(dat[ ,'plrpval']))
    
    cols <- c(cols, "plrpval", "pvalstars")
    pretty_names <- c(pretty_names, "P<null (PLR)", "")
  }
  
  # Extract data and rename cols
  dat <- dat[, cols]
  names(dat) <- pretty_names
  
  return(dat)
}

#'@export
summary.patchdistr_sews_single <- function(object, ...) { 
  summary.patchdistr_sews_list(object, ...)
}
#'@export
summary.patchdistr_sews_list <- function(object, ...) { 
  dat <- prepare_summary_table(object)
  
  cat('Spatial Early-Warning: Patch-based indicators\n') 
  cat('\n')
  print.data.frame(dat, row.names = FALSE, digits = DIGITS)
  cat('\n')
  cat("The following methods are available: \n")
  cat(list_methods(class(object)), "\n")
  
  invisible(dat)
}



# Print methods
# --------------------------------------------------
# Note this method works for both single and lists objects so we only 
#   define it for patchdistr_sews (and not their *_list *_single 
#   equivalents)

#'@export
print.patchdistr_sews_single <- function(x, ...) { 
  summary.patchdistr_sews_single(x, ...)
}
#'@export
print.patchdistr_sews_list <- function(x, ...) { 
  summary.patchdistr_sews_list(x, ...)
}



# Helper function 
# ---------------
# Get the inverse cumulative distribution of a psd (P(x >= k)). x here are the 
# values at which to evaluate the inverse cumulative psd. 
cumpsd <- function(dat, x = sort(unique(dat))) { 
  N <- length(dat)
  y <- sapply(x, function(k) { sum(dat >= k) / N })
  return( data.frame(patchsize = x, y = y) )
}

# 
# This file defines a generic function to convert matrices. spatialwarningsGis
# will add methods to this generic function so that foreign gis formats can be 
# directly used in spatialwarnings. 
# 
#'
#'
#' @title Convert an object to a matrix
#' 
#' @description This function is mainly for internal use by the 
#'   \code{\link{spatialwarnings}} package to convert objects before they are 
#'   processed by \code{*_sews} functions. 
#' 
#' @param object An object (typically, a matrix or a list of matrices)
#' 
#' @param ... Additional arguments (currently ignored)
#' 
#' @details 
#'   This generic function is here so that other packages can extend it. 
#' For example, \href{https://github.com/spatial-ews/spatialwarningsGis}{spatialwarningsGis} will provide methods so that GIS objects can be handled 
#' (e.g. \code{RasterLayer} from package \code{raster}).
#' 
#' @examples 
#' 
#' # this does nothing
#' convert_to_matrix(serengeti[2:3]) 
#' 
#'@export
convert_to_matrix <- function(object, ...) { 
  UseMethod("convert_to_matrix")
}

#'@export
convert_to_matrix.matrix <- function(object, ...) { 
  object
}

#'@export 
convert_to_matrix.factor <- function(object, ...) { 
  if ( is.matrix(object) ) { 
    convert_to_matrix.matrix(object) 
  } else { 
    stop("This factor object has no dimensions. I cannot convert it to a matrix.")
  }
}

#'@export
convert_to_matrix.list <- function(object, ...) { 
  lapply(object, convert_to_matrix) 
}

# A function that computes the "simple" flowlength as described in Rodriguez 
#   et al. 
# 
#' @title Flowlength connectivity indicator (uniform topography)
#' 
#' @description Measures the connectivity of runoff-source areas as determined 
#'   by vegetation patterns and (uniform) topography
#' 
#' @details 
#'   
#' This function computes Flowlength, a simple metric that measures the 
#'   potential hydrological connectivity of runoff-source areas 
#'   (e.g., bare soil) considering vegetation cover, vegetation patterns and 
#'   topography. Flowlength is defined as the average length of all the 
#'   potential runoff pathways in the target area. Thus, a higher value of 
#'   the index indicates a higher hydrologic connectivity of runoff source 
#'   areas. This function is designed for an idealized uniform hillslope 
#'   (e.g., with constant slope angle, the direction of maximum slope being 
#'   from the top to the bottom of the input matrices). 
#' 
#' The deviations of Flowlength from its expected values under random or 
#'   aggregated-pattern null models can be used as an indicator of imminent 
#'   transition to a degraded state (Rodriguez et al. 2017) in the context 
#'   of arid drylands. An increased deviation of flowlength compared to its 
#'   null values is expected as a possible transition gets closer. This deviation
#'   can be computed using any null model provided by \code{spatialwarnings} (see 
#'   \code{\link{indictest}} for more details), but a specific null model is 
#'   provided for Flowlength based on a much-faster analytical approximation, 
#'   using the argument \code{null_method = "approx_rand"} when calling 
#'   \code{indictest} (see examples below). 
#' 
#' In general, Flowlength can be used as indicator of dryland functional status 
#'   by assessing potential water and soil losses in patchy landscapes 
#'   (Mayor et al. 2008, Moreno-de las Heras et al. 2012, Mayor et al. 2013, 
#'   Okin et al. 2015). Finally, the combination of observed and expected 
#'   Flowlength under null models for random or aggregated vegetation cover 
#'   can be used for assessing the cover-independent role of bare-
#'   soil connectivity (Rodriguez et al. 2018).
#'   
#' @references
#' 
#' Rodriguez, F., A. G. Mayor, M. Rietkerk, and S. Bautista. 2017. A null model 
#'   for assessing the cover-independent role of bare soil connectivity as 
#'   indicator of dryland functioning and dynamics. Ecological Indicators.
#' 
#' Mayor, A.G., Bautista, S., Small, E.E., Dixon, M., Bellot, J., 2008. 
#'   Measurement of the connectivity of runoff source areas as determined by 
#'   vegetation pattern and topography: a tool for assessing potential water 
#'   and soil losses in drylands. Water Resour. Res. 44, W10423.
#' 
#' Mayor, A.G., Kefi, S., Bautista, S., Rodriguez, F., Carteni, F., Rietkerk, 
#'   M., 2013. Feedbacks between vegetation pattern and resource loss 
#'   dramatically decrease ecosystem resilience and restoration potential in 
#'   a simple dryland model. Landsc. Ecol. 28, 931-942.
#' 
#' Moreno-de las Heras, M., Saco, P.M., Willgoose, G.R., Tongway, D.J., 2012. 
#'   Variations in hydrological connectivity of Australian semiarid landscapes 
#'   indicate abrupt changes in rainfall-use efficiency of vegetation. 
#'   J. Geophys. Res. 117, G03009.
#' 
#' Okin, G.S., Moreno-de las Heras, M., Saco, P.M., Throop, H.L., Vivoni, E.R., 
#'   Parsons, A.J., Wainwright, J., Peters, D.P.C., 2015. Connectivity in 
#'   dryland landscapes: shifting concepts of spatial interactions. 
#'   Front. Ecol. Environ. 13 (1), 20-27.
#' 
#' @param mat The input matrix (must be a logical matrix)
#' 
#' @param slope The slope of the area documented by the matrix (in degree). 
#' 
#' @param cell_size The horizontal size of a cell in the matrix (as viewed 
#'   from above). 
#' 
#' @return A `simple_sews` object containing the flow length value, among 
#'   other things, see \code{\link{simple_sews_object}} for more information.
#' 
#' @seealso 
#'   \code{\link{raw_flowlength_uniform}}, 
#'   \code{\link{indictest}} to test the significance of indicator values. 
#' 
#' @examples 
#' 
#' \donttest{ 
#' fl_result <- flowlength_sews(arizona, slope = 20, cell_size = 1)
#' 
#' # Compute the Z-score (standardized deviation to null distribution) and plot 
#' #   its variations along the gradient. This Z-score is suggested by 
#' #   Rodriguez et al. (2017) as an indicator of degradation. 
#' fl_test <- indictest(fl_result, nulln = 19)
#' plot(fl_test, what = "z_score")
#' 
#' # Use the analytical approximation suggested in Rodriguez et al. (2017), 
#' # instead of permuting the original values in the matrix (much faster)
#' fl_test <- indictest(fl_result, null_method = "approx_rand")
#' plot(fl_test, what = "z_score")
#' 
#' }
#' 
#'@export
flowlength_sews <- function(mat,        # Input matrix
                            slope = 20, # Slope (in degrees)
                            cell_size = 1) { # Cell size
  
  fl_result <- compute_indicator(mat, raw_flowlength_uniform, 
                                 slope = slope, 
                                 cell_size = cell_size, 
                                 taskname = "Flow length (uniform topography)")
  
  # Make sure we adjust the class to add flowlength-related information. This is 
  # used later on so that indictest() will use specific methods written for the 
  # flowlength. 
  if ( inherits(fl_result, "sews_result_list") ) { 
    class(fl_result) <- c("flowlength_sews_list", class(fl_result))
    for ( i in seq_along(fl_result) ) { 
      class(fl_result[[i]]) <- c("flowlength_sews_single", class(fl_result[[i]]))
    }
  } else { 
    class(fl_result) <- c("flowlength_sews_single", class(fl_result))
  }
  
  return(fl_result)
}

#'@export
indictest.flowlength_sews_list <- function(x, 
                                           nulln = 999, 
                                           null_method = "perm", 
                                           null_control = NULL, 
                                           ...) { 
  
  results <- future_lapply_seed(x, indictest.flowlength_sews_single, 
                                nulln, null_method, null_control, 
                                ...)
  
  # Add matrixn value with correct number
  for ( nb in seq_along(results) ) { 
    results[[nb]][['matrixn']] <- nb
  }
  
  class(results) <- c('flowlength_sews_test_list', 
                      'simple_sews_test_list', 'sews_test', 
                      'sews_result_list')
  
  return(results)
}

#'@export
indictest.flowlength_sews_single <- function(x, 
                                             nulln = 999, 
                                             null_method = "perm", 
                                             null_control = NULL, 
                                             ...) { 
  
  # Read null_control values we may use to compute null expectation for FL
  qinf <- ifelse(is.null(null_control[["qinf"]]), .05, null_control[["qinf"]]) 
  qsup <- ifelse(is.null(null_control[["qsup"]]), .95, null_control[["qsup"]])
  
  # These methods use an analytical approximation to compute the expected values
  # of flowlength given an homogeneous matrix, or (TODO) one that is somewhat
  # aggregated (see Rodriguez et al., Ecological Indicators).
  if ( ( ! is.function(null_method) ) && ( null_method == "approx_rand" ) ) { 
    
    rho <- mean(x[["orig_data"]])
    L <- nrow(x[["orig_data"]])
    slope <- x[["fun.args"]][["slope"]]
    cell_size <- x[["fun.args"]][["cell_size"]]
    ds <- cell_size / cos(slope * pi / 180)
    
    # Eq. 1 in the above paper (with ds = 1)
    E_fl <- ds * (1 - rho)*(rho * L - (1 - rho)*(1 - (1 - rho)^L)) / (rho^2*L)
    E_fl2 <- ds^2*( (1 - rho)*(rho^2*(1 - rho)*(L+1)^2 + rho^2*L - 6 * (1 - rho) + 
                        (1 - rho)^(L+1)*(rho^2*(2*L^2 - 1) + 6 * rho * L + 6)) ) / 
                        (rho^4 * L^2)
    V_fl <- E_fl2 - E_fl^2
    
    # NOTE: the values above are for a column vector, we need to compute 
    # the variance for the average of FL for all columns, which is different
    V_fl <- V_fl / ( ncol(x[["orig_data"]]) )
    
    # Now add the values in the resulting test object 
    # "nulldistr"   "null_mean"   "null_sd"     "null_qsup"   "null_qinf"  
    # "z_score"     "pval"        "null_method" "nulln"       "get_nullmat"
    ans <- x
    
    # These are undefined because we never computed a null distribution 
    # ans[["nulldistr"]] <- NULL
    # ans[["nulln"]] <- NULL
    # ans[["get_nullmat"]] <- NULL
    
    ans[["null_mean"]] <- E_fl
    ans[["null_sd"]] <- sqrt(V_fl) 
    ans[["null_method"]] <- null_method
    ans[["null_qsup"]] <- qnorm(qsup, mean = E_fl, sd = sqrt(V_fl))
    ans[["null_qinf"]] <- qnorm(qinf, mean = E_fl, sd = sqrt(V_fl))
    # P-value close to zero when indicator is above the null distribution
    ans[["pval"]] <- pnorm(x[["value"]], mean = E_fl, sd = sqrt(V_fl), 
                           lower.tail = FALSE)
    ans[["z_score"]] <- ( x[["value"]] - E_fl ) / sqrt(V_fl)
    
    class(ans) <- c("flowlength_sews_test_single", 
                    'simple_sews_test_single', 'sews_test', 
                    'sews_result_single')
    
    return(ans)
  }
  
  # Catch-all: we revert to the default, usual way of doing it 
  ans <- indictest.simple_sews_single(x, nulln, null_method, null_control)
  class(ans) <- c("flowlength_sews_test_single", 
                  'simple_sews_test_single', 'sews_test', 
                  'sews_result_single')
  
  return(ans)
}

#
# A function that computes the "simple" flowlength as described in Rodriguez 
#   et al. 
# 
#' @title Flow length (uniform slope)
#' 
#' @description Compute a simple approximation of the flow length assuming a 
#'   constant slope 
#' 
#' @details 
#'   
#'   This function computes the Flowlength of a given matrix, using a 
#'     uniform approximation (the slope is constant across the whole matrix, 
#'     with maximum slope being from the top of the matrix to its bottom), 
#'     as per Rodriguez et al. (2017). See \code{\link{flowlength_sews}} for 
#'     more details. 
#'   
#' @references
#' 
#' Rodriguez, F., A. G. Mayor, M. Rietkerk, and S. Bautista. 2017. A null model 
#'   for assessing the cover-independent role of bare soil connectivity as 
#'   indicator of dryland functioning and dynamics. Ecological Indicators.
#' 
#' @param mat The input matrix (must be a logical matrix)
#' 
#' @param slope The slope of the area documented by the matrix (in degrees). 
#' 
#' @param cell_size The horizontal size of a cell in the matrix (as viewed 
#'   from above). 
#' 
#' @return A named vector of length 1 containing the flow length numerical 
#'   value 
#' 
#' @seealso \code{\link{flowlength_sews}}
#' @seealso 
#'   \code{\link{indictest}}, to test the significance of indicator values. 
#' 
#' @examples 
#' 
#' \donttest{ 
#' raw_flowlength_uniform(arizona[[1]], slope = 20, cell_size = 1)
#' }
#' 
#' 
#'@export
raw_flowlength_uniform <- function(mat,        # Input matrix
                                   slope, # Slope (in degrees)
                                   cell_size) { # Cell size
  
  if ( is.vector(mat) ) { 
    mat <- matrix(mat, ncol = 1, nrow = length(mat))
  }
  
  # Mat must be a logical matrix 
  if ( ! is.matrix(mat) ) { 
    stop('The supplied object is not a matrix')
  }
  
  # Mat must be a logical matrix 
  if ( ! is.logical(mat) ) { 
    stop('The supplied matrix must be logical to compute the flow length')
  }
  
  # Take the negative of mat, so that ones represent empty cells
  nmat <- ! mat 
  
  # Cell size along the slope 
  p_slope <- cell_size / cos(slope * pi / 180)
  
  # Compute flow length without taking slope into account (C++ function)
  flmean <- fl_internal(nmat)
  
  # Adjust value for slope
  fl <- flmean * p_slope
  
  return(c(flowlength = fl))
}

# 
# This tests the code for the flowlength
# 
# 

context('Test the computation of flow length')

# Test raw values. rho here represents cover. We test that what is computed 
# from the code for a random matrix matches the theoretical expectations. See 
# Rodriguez et al. 2017 (Eco. Ind.)
test_that("Flowlength computations are OK", { 
  all_fls <- plyr::adply(seq(0.01, 1, length.out = 4), 1, function(rho) { 
    
    # Set other parameters randomly
    L <- 100
    cols <- 1 # Relationships hold for a column vector only (especially variance)
    cell_size <- 0.5
    slope <- runif(1, 10, 20) 
    
    # Compute flow lengths
    ds <- cell_size / cos(slope * pi/180)
    fls <- replicate(499, { 
      dat <- matrix(runif(L*cols), ncol = cols, nrow = L) < rho
      raw_flowlength_uniform(dat, cell_size = cell_size, slope = slope)
    })
    
    mean_fl <- mean(fls)
    var_fl <- var(fls)
    # Theoretical values
    E_fl <- ds * (1 - rho)*(rho * L - (1 - rho)*(1 - (1 - rho)^L)) / (rho^2*L)
    E_fl2 <- ds^2*( (1 - rho)*(rho^2*(1 - rho)*(L+1)^2 + rho^2*L - 6 * (1 - rho) + 
                        (1 - rho)^(L+1)*(rho^2*(2*L^2 - 1) + 6 * rho * L + 6)) ) / 
                        (rho^4 * L^2)
    V_fl <- E_fl2 - E_fl^2
    
    data.frame(rho = rho, 
               obsmean = mean_fl, 
               obsvar  = var_fl, 
               themean = E_fl, 
               thevar  = V_fl, 
               themax  = ds*(L+1)/2)
  }, .progress = "none", .id = NULL)

  # Obs. mean and Theoretical mean should be equal 
  expect_true( with(all_fls, t.test(x = obsmean, y = themean)$p.value) > 0.5 )
  expect_true( with(all_fls, t.test(x = obsvar, y = thevar)$p.value) > 0.5 )

  # Obs mean and theoretical mean should be very close
  with( subset(all_fls, themean > 0 & thevar > 0), { 
    expect_true({ 
      all(coef(lm(obsmean ~ themean)) < c(0.05, 1.10))
    })
    expect_true({ 
      all(coef(lm(obsvar ~ thevar)) < c(0.05, 1.10))
    })
  })

  # library(ggplot2)
  # library(tidyr)
  # 
  # ggplot( gather(subset(all_fls), var, value, obsmean, themean) ) + 
  #   geom_line(aes(x = rho, y = value, color = var))
  #   
  # ggplot( gather(subset(all_fls), var, value, obsvar, thevar) ) + 
  #   geom_line(aes(x = rho, y = value, color = var))
  # 
  # ggplot( subset(all_fls) ) + 
  #   geom_point(aes(x = obsmean, y = themean)) + 
  #   geom_abline(slope = 1, intercept = 0)
  # 
  # ggplot( subset(all_fls)) + 
  #   geom_point(aes(x = obsvar, y = thevar)) + 
  #   geom_abline(slope = 1, intercept = 0)

  # Test that full/empty columns have correct values
  zero_column <- matrix(0, ncol = 1, nrow = 10) > 0
  ds <- 1 / cos(20*pi/180)
  expect <- ds * (10+1)/2
  expect_true({
    abs(expect - raw_flowlength_uniform(zero_column, slope = 20, cell_size = 1)) < 0.001
  })
  expect_true({ 
    abs(0 - raw_flowlength_uniform(!zero_column, slope = 20, cell_size = 1)) < 0.001
  })
  
  
  # Test that we reproduce results from images. Note that we base these 
  # tests on images that have been already binarized in matlab/octave
  if ( exists('EXTENDED_TESTS') && EXTENDED_TESTS ) { 
    imagedir <- './rodriguez2018/images'
    allimgs <- dir(imagedir, full.names = TRUE, pattern = "*.csv")
    all_images <- lapply(allimgs, 
                        function(i) { 
                          a <- as.matrix(read.csv(i, header = FALSE)) > 0 
                          dimnames(a) <- NULL
                          a
                        })
    fls <- flowlength_sews(all_images, slope = 0.6, cell_size = 0.223)
    ref <- data.frame(covers = c(0.42, 0.37, 0.17, 0.12, 0.19, 0.13, 
                                0.48, 0.38, 0.34, 0.218, 0.114, 0.08), 
                      fl = c(1.1231, 1.2624, 5.1928, 9.0719, 4.6266, 8.6047, 
                            1.5144, 2.5934, 3.3909, 4.945, 11.631, 15.6279))
    
    fl.df <- as.data.frame( indictest(fls, nulln = 9) )
    
    compare <- data.frame(fl.df, ref)
    expect_true({ 
      with(compare, all( abs(value - fl) < 0.01))
    })
    
    # Check approximation
    rodri_results <- read.table("./rodriguez2018/fl_approx_values.txt", 
                                header = TRUE)
    rodri_results <- rodri_results[order(rodri_results[ ,"im"]), ]
    cell_size <- 0.223
    slope <- 0.6
    
    ourvalues <- as.data.frame( indictest(fls, null_method = "approx_rand") )
    
    # Check that the observed value, the null mean and sd are close to each other
    expect_equal(fl.df[ ,"value"], 
                 ourvalues[ ,"value"])
    expect_equal(fl.df[ ,"null_mean"], 
                 ourvalues[ ,"null_mean"], 
                 tolerance = 1/100)
    expect_equal(fl.df[ ,"null_sd"], 
                 ourvalues[ ,"null_sd"], 
                 tolerance = 1/100)
  }
})

# 
# The code below is to check the FL approximation is correct 
# 
# 
# test_that("Paco's approximation is correctly implemented (Rodriguez 2018)", { 
#   
# 
# plan(multiprocess)
# a <- future.apply::future_lapply(seq.int(256), function(n) { 
# # a <- lapply(seq.int(8), function(n) { 
#   cat(".")
#   # Generate random matrix 
#   ldply(seq(0.01, 1, length = 24), function(p) { 
#     nr <- round(runif(1, 100, 200))
#     m <- matrix(runif(nr*nr) < p, nrow = nr)
#     fl <- flowlength_sews(m)
#     testperm    <- indictest(fl, nulln = 99, null_method = "perm")
#     testapprox  <- indictest(fl, null_method = "approx_rand")
#     
#     a <- rbind(data.frame(null_method = "perm", as.data.frame(testperm)), 
#                data.frame(null_method = "approx_rand", as.data.frame(testapprox)))
#     data.frame(n = n, nr = nr, p = mean(m), a)
#   })
# })
# a <- rbind.fill(a)
# ac <- reshape2::dcast(a, nr + n + p ~ null_method, value.var = "null_sd")
# 
# ggplot(ac) + 
#   geom_abline(intercept = 0, slope = 1, color = "red") + 
#   geom_point(aes(x = perm, 
#                  y = approx_rand), 
#              alpha = .8) + 
#   scale_x_continuous(trans = "log10") + 
#   scale_y_continuous(trans = "log10") 
# 
# ggplot(a) + 
#   geom_line(aes(x = p, y = null_sd, color = null_method)) 
# 
# mean(with(ac, perm/approx_rand), na.rm = TRUE)
# 
# # For which covers is the approximation OK ? 
# ggplot(ac) + 
#   geom_abline(intercept = 0, slope = 0, color = "red") + 
#   geom_point(aes(x = p, y = perm - approx_rand)) 
# 
# setwd("./tests/testthat")
# rodri_results <- read.table("./rodriguez2018/fl_approx_values.txt", 
#                             header = TRUE)
# our_results <- ldply(dir("./rodriguez2018/images", 
#                          pattern = "*.csv", full = TRUE), 
#                      function(im) { 
# a <- as.matrix(read.csv(im, header = FALSE) > 0 )
# #   b <- as.data.frame( indictest(flowlength_sews(a), nulln = 99, 
# #                                 null_method = "perm") )
# b <- as.data.frame( indictest(flowlength_sews(a), null_method = "approx_rand") )
# b
#   
# })

#'
#' @title Indicator based on Kolmogorov Complexity 
#' 
#' @description 
#' 
#'   Computes the Kolmogorov Complexity on a set of matrices, 
#'     using the Block Decomposition Method. 
#'
#' @details 
#' 
#    The Kolmogorov complexity of a given matrix has been suggested to 
#'     be a useful indicator to anticipate transitions in model ecological 
#'     systems (Dakos and Soler-Toscano, 2017). When close to the transition 
#'     critical point, the complexity is expected to decrease. 
#'   
#'   The Kolmogorov complexity cannot be computed directly for large strings 
#'     (i.e. matrices). However, the complexity of smaller submatrices can be 
#'     estimated, then combined to obtain an approximation of the complexity 
#'     of the whole matrix. This method, the Block Decomposition Method is 
#'     implemented in this indicator following Dakos and Soler-Toscano (2017). 
#' 
#' @return 
#' 
#'   \code{kbdm_sews} returns an object of class \code{simple_sews_single} 
#'     (a list) if mat is a single matrix, and an object of class 
#'     \code{simple_sews_list} if mat is a list of matrices. These objects can 
#'     be used with generic methods indictest (to test significance) or plot 
#'     (to display trends), see also the examples below. 
#'   
#' @references 
#'   
#'   Dakos, V., and F. Soler-Toscano. 2017. Measuring complexity to infer 
#'   changes in the dynamics of ecological systems under stress. Ecological 
#'   Complexity 32:144-155. 
#'   
#' @param mat A logical matrix (\code{TRUE}/\code{FALSE} values) or a list of logical 
#'   matrices
#' 
#' @param subsize A submatrix size to carry out the Block Decomposition Method
#'   (must be between 1 and 3)
#' 
#' @seealso \code{\link{raw_kbdm}}, \code{\link[acss]{acss}}, 
#'   \code{\link{indictest}}, to test the significance of indicator values. 
#' 
#' @examples 
#' 
#' \donttest{ 
#' 
#' kbdm_result <- kbdm_sews(serengeti, subsize = 3)
#' plot(kbdm_result, along = serengeti.rain)
#' 
#' kbdm_test <- indictest(kbdm_result, nulln = 49)
#' plot(kbdm_test, along = serengeti.rain)
#' 
#' # Plot deviation to null expectation
#' plot(kbdm_test, along = serengeti.rain, what = "z_score") 
#' 
#' }
#' 
#' 
#'@export 
kbdm_sews <- function(mat, subsize = 3) { 
    compute_indicator(mat, raw_kbdm, subsize = subsize, 
                      taskname = "Kbdm Complexity")
}



# This function takes a matrix and a returns a single value.
#' 
#' @title Kolmogorov complexity of a matrix 
#' 
#' @description Compute the Kolmogorov complexity of a matrix using the 
#'   Block Decomposition Method (requires the \code{acss} package).
#' 
#' @details 
#' 
#'   The Kolmogorov complexity cannot be computed directly for large strings 
#'     (i.e. matrices). However, the complexity of smaller submatrices can be 
#'     estimated, then combined to obtain an approximation of the complexity 
#'     of the whole matrix. This method, the Block Decomposition Method is 
#'     implemented in this function. See also \code{\link{kbdm_sews}} 
#'     for more details. 
#' 
#' @return The KBDM numeric value as a named vector
#' 
#' @param mat A logical matrix (with TRUE/FALSE values)
#' 
#' @param subsize A submatrix size to carry out the Block Decomposition Method
#'   (must be between 1 and 3)
#' 
#' @seealso \code{\link{kbdm_sews}}, \code{\link[acss]{acss}}
#' 
#' @examples 
#' 
#' \donttest{ 
#' raw_kbdm(forestgap[[1]], subsize = 3)
#' }
#' 
#'@export
raw_kbdm <- function(mat, subsize) {
  
  if ( ! requireNamespace("acss") ) { 
    stop(paste0('Computation of kbdm requires the acss package. You can install it ',
                'using the command install.packages("acss")'))
  }
  
  # Split matrix
  xs <- seq(1, nrow(mat), by = subsize)
  ys <- seq(1, ncol(mat), by = subsize)
  allblockns <- expand.grid(seq.int(length(xs)-1),
                            seq.int(length(ys)-1))
  all_substr <- Map(function(xblockn, yblockn) {
                      dict <- as.vector(mat[xs[xblockn]:(xs[xblockn+1]-1),
                                            ys[yblockn]:(ys[yblockn+1]-1)]) 
                      dict <- as.integer(dict) 
                      dict <- paste(dict, collapse = "")
                    },
                    allblockns[ ,1], allblockns[ ,2]) 
  all_substr <- unlist(all_substr)
  
  # Summarize the substrings
  counts <- table(all_substr)
  counts <- data.frame(string = names(counts),
                       multip = as.vector(counts),
                       kctm = acss::acss(names(counts), alphabet = 2)[ ,1])
  
  # Compute Kbdm
  return( c(kbdm = with(counts, sum(log2(multip) + kctm))) )
}

# 
# Test for the Kolmogorov complexity
# 
if ( requireNamespace("acss", quietly = TRUE) ) { 

context('Test that the computations for Kolmogorov complexity are correct')

test_that("Kbdm results are correct", { 
  
  skip_on_cran()

  # We can't reproduce exactly the results because we cannot compute 
  # Kbdm using 4x4 submatrices in acss::acss. We just compute the test on 
  # the scale-free patterns which do not show as much noise as the 
  # irregular/turing patterns. 
  dat <- read.csv("./kbdm_data/scale-free/extracted_data.csv", 
                  header = FALSE)

  allfiles <- dir("./kbdm_data/scale-free/", 
                  pattern = "^.*.txt$", full.names = TRUE)
  matrices <- lapply(allfiles, 
                    function(f) { 
                          a <- as.matrix(read.table(f, header = FALSE))
                          attr(a, "dimnames") <- NULL
                          a > mean(a) # binarize as done in Dakos 2017
                          })

  kbdm_vals <- as.data.frame(kbdm_sews(matrices))

  xvals <- sapply(allfiles, function(f) { 

      a <- strsplit(f, split = c("CA_b"))[[1]][2]
      a <- gsub(".txt", "", a)
      a <- as.numeric(a)
      a
    }, USE.NAMES = FALSE)

  # We cannot compare the absolute values, but we compare the ranks 
  ref <- rank(dat[ ,2])
  val <- rank(kbdm_vals[order(xvals),"value"])

  plot(ref, val)
  abline(0, 1, col = "red")
  linmod <- lm(y ~ x, data = data.frame(x = ref, y = val))
  lines(ref, predict(linmod))

  # Slope of relationship should be close to zero
  expect_true({
    abs(1 - coef(linmod)[2]) < .1
  })

  # Both values should have a max that is very close 
  expect_true({ 
    abs(xvals[ref == max(ref)] - xvals[val == max(val)]) < .1
  })

  
})

}

# 
# This file contains a function that identifies patch size distribution types 
#   by fitting all types using ML, then retaining the best one using AIC 
#   (note that it does not perform LR tests across distribution types to 
#   assess significance (yet?))
# 

#' @title Change in patch-size distributions types
#' 
#' @description This functions fits different patch size distributions 
#'  types (power-law, log-normal, exponential and truncated power-law) to 
#'  the patches contained in a matrix. The distributions are returned with 
#'  their corresponding AIC, BIC and AICc to select the best fit.
#'  
#' @param x A logical (\code{TRUE}/\code{FALSE} values) matrix or a list of these. 
#' 
#' @param xmin The xmin to be used to fit the patch size distributions. Use 
#'   the special values "estimate" to use an estimated xmin for each fit
#' 
#' @param merge The default behavior is to produce indicators values for each 
#'   matrix. If this parameter is set to TRUE then the patch size distributions 
#'   are pooled together for fitting. 
#' 
#' @param fit_lnorm Fit also a log-normal distribution 
#' 
#' @param xmin_bounds Restrict the possible xmins in this range (defaults to 
#'   the whole range of observed patch sizes)
#' 
#' @param best_by The criterion used to select the best distribution type 
#'   (one of \code{"AIC"}, \code{"BIC"} or \code{"AICc"}). 
#' 
#' @param wrap Determines whether patches are considered to wrap around the 
#'  matrix when reaching the side 
#' 
#' @param nbmask Either "moore" for 8-way neighborhood, "von_neumann" for four-way 
#'   neighborhood (default), or a 3x3 matrix describing which neighbors to 
#'   consider around a cell. See \code{\link{patchsizes}} for details on how to specify 
#'   such neighborhoods.
#' 
#' @return A data.frame (or a list of these if x is a list) with the 
#'   following columns:
#'     \itemize{
#'       \item \code{method} the method used for fitting (currently: only 
#'          log-likelihood is implemented, "ll")
#'       \item \code{type} the type of distribution
#'       \item npars the number of parameters of the distribution type
#'       \item AIC, `AICc` and `BIC` the values for Akaike Information 
#'         Criterion (or the corrected for small samples equivalent AICc), 
#'         and Bayesion Information Criterion (BIC)
#'       \item \code{best} A logical vector indicating which distribution is the 
#'         best fit 
#'       \item \code{plexpo}, \code{cutoff}, \code{meanlog}, \code{sdlog} the estimates
#'          for distribution parameters (see \code{\link{pl_fit}})
#'       \item \code{percolation} A logical value indicating whether there is 
#'         \code{\link{percolation}} in the system. 
#'     }
#' 
#' @details 
#' 
#' Patterned ecosystems can exhibit a change in their spatial structure as they 
#' become more and more stressed. It has been suggested that this should be 
#' reflected in changes in the observed patch size distributions (PSD). 
#' The following sequence is expected to occur (Kefi et al. 2011) as patterned 
#' ecosystems become more and more degraded:
#' 
#'   - Percolation of vegetation patches occurs (a patch has a width or height 
#'   equal to the size of the system)
#'   
#'   - The patch-size distribution follows a power-law
#'   
#'   - The patch-size distribution deviates from a power-law as larger patches 
#'   break down
#'   
#'   - The patch-size distribution is closer to an exponential 
#'   distribution
#' 
#' This indicator fits the observed patch size distribution based on 
#' maximum-likelihood (following Clauset et al. 2009 recommendations), then 
#' select the best model using AIC, BIC (default) or AICc. 
#' 
#' @seealso \code{\link{patchdistr_sews}}
#' 
#' @references
#' 
#' Kefi, S., Rietkerk, M., Roy, M., Franc, A., de Ruiter, P.C. & Pascual, M.
#' (2011). Robust scaling in ecosystems and the meltdown of patch size
#' distributions before extinction: Patch size distributions towards 
#' extinction. Ecology Letters, 14, 29-35.
#' 
#' Kefi, S., Rietkerk, M., Alados, C.L., Pueyo, Y., Papanastasis, V.P., ElAich,
#' A., et al. (2007). Spatial vegetation patterns and imminent desertification
#' in Mediterranean arid ecosystems. Nature, 449, 213-217.
#' 
#' Clauset, A., Shalizi, C. R., & Newman, M. E. (2009). 
#'   Power-law distributions in empirical data. SIAM review, 51(4), 661-703.
#' 
#' @seealso \code{\link{patchdistr_sews}}
#' 
#' @examples
#' 
#' data(forestgap)
#' 
#' # One logical matrix only
#' indicator_psdtype(forestgap[[1]])
#' 
#' # A list of these matrices
#' \donttest{ 
#' indicator_psdtype(forestgap)
#' }
#'
#'@export 
indicator_psdtype <- function(x, 
                              xmin = 1, 
                              merge = FALSE, 
                              fit_lnorm = FALSE,
                              xmin_bounds = NULL, 
                              best_by = "AIC", 
                              wrap = FALSE, 
                              nbmask = "von_neumann") { 
  
  if ( !merge && is.list(x) ) { 
    return( lapply(x, indicator_psdtype, xmin, merge, fit_lnorm, xmin_bounds, 
                   best_by, wrap, nbmask) )
  } 
  
  # Here we do not test if x is not a matrix. This happens when merge = TRUE, 
  # when we pool patch size distributions together 
  if ( ! is.list(x) ) { 
    check_mat(x)
  }
  
  # Estimate power-law range and set xmin to the estimated value if set to 
  # auto. 
  if ( xmin == "estimate" ) { 
    # Set bounds to search for xmin
    if ( length(psd) > 0 && is.null(xmin_bounds) ) { 
      xmin_bounds <- range(psd)
    }
    xmin <- plrange(psd)['xmin_est']
  }
  
  # Compute psd
  psd <- patchsizes(x, merge = merge, wrap = wrap, nbmask = nbmask)
  
  # Compute percolation point. If the user requested a merge of all 
  #   patch size distributions, then we return the proportion of matrices 
  #   with percolation. 
  if ( is.list(x) ) { 
    percol <- lapply(x, percolation, nbmask = nbmask)
    percol <- mean(unlist(percol))
    percol_empty <- lapply(x, function(x) percolation(!x, nbmask = nbmask))
    percol_empty <- mean(unlist(percol_empty))
  } else { 
    percol <- percolation(x, nbmask = nbmask)
    percol_empty <- percolation(!x, nbmask = nbmask)
  } 
  
  psdtype_result <- psdtype(psd, xmin, best_by, fit_lnorm)
  
  result <- data.frame(psdtype_result, 
                       percolation = percol, 
                       percolation_empty = percol_empty)
  return(result)
}

psdtype <- function(psd, xmin, best_by, fit_lnorm) { 
  
  table_names <- c('method', 'type', 'npars', 'AIC', 'AICc', 'BIC', 'best', 
                   'plexpo', 'cutoff', 'xmin_fit')
  if ( fit_lnorm ) { 
    table_names <- c(table_names, 'meanlog', 'sdlog')
  }
  
  # Chop off psd to what is above xmin 
  psd <- psd[psd >= xmin]
  
  # We return NA if there are not enough patches to work with 
  if ( length(unique(psd)) <= 3 ) { 
    NA_result <- as.data.frame(as.list(rep(NA, length(table_names))))
    names(NA_result) <- table_names
    return(NA_result)
  }
  
  # Fit a model for everyone. Note that we store the results of the variables 
  # here so we reuse the previous expo and rate for tpl fitting. 
  plfit  <- pl_fit(psd, xmin)
  expfit <- exp_fit(psd, xmin)
  tplfit <- tpl_fit(psd, xmin)
  
  models <- list(pl = plfit, tpl = tplfit, exp = expfit)
  
  if (fit_lnorm) { 
    lnormfit <- lnorm_fit(psd, xmin)
    models <- c(models, list(lnorm = lnormfit))
  }
  
  models <- lapply(models, as.data.frame)
  models <- do.call(rbind.fill, models)
  row.names(models) <- models[ ,'type']
  
  # Compute AICs
  models[ ,'AIC']  <- get_AIC(models[ ,'ll'],  models[ ,'npars'])
  models[ ,'AICc'] <- get_AICc(models[ ,'ll'], models[ ,'npars'], length(psd))
  models[ ,'BIC']  <- get_BIC(models[ ,'ll'],  models[ ,'npars'], length(psd))
  
  # We need to remove NA's here as sometimes one of the fits fails and its "best"
  # column is NA. We do not consider failed fit as good solutions. 
  models[ ,'best'] <- models[ ,best_by] == min(models[ ,best_by], na.rm = TRUE)
  models[ ,'best'] <- models[ ,'best'] & ! is.na(models[ ,'best']) 
  
  # Add an xmin column 
  models[ ,"xmin_fit"] <- xmin
  
  # Reorganize columns 
  models <- models[ ,table_names]
  
  return(models)
}

get_AIC <- function(ll, k) { 
  2*k - 2*ll 
}

get_AICc <- function(ll, k, n) { 
  2*k - 2*ll + (2*k*(k+1))/(n-k-1)
}

get_BIC <- function(ll, k, n) { 
  2*k*log(n) - 2*ll 
}

# 
# 
# This file contains the indictest functions for spectral sews
# 

# 
# Indictest functions for spectral_sews objects.
#'@export
indictest.spectral_sews_list <- function(x, 
                                         nulln = 999, 
                                         null_method = 'perm', 
                                         null_control = NULL, 
                                         ...) { 
  
  # Compute a distribution of null values for SDR
  results <- future_lapply_seed(x, indictest.spectral_sews_single, 
                                nulln, null_method, null_control, ...)
  
  # Format and return output
  class(results) <- c('spectral_sews_test_list', 
                      'spectral_sews_list', 
                      'simple_sews_test_list', 
                      'simple_sews_list', 
                      'sews_result_list')
  
  return(results)
}

#' @export
indictest.spectral_sews_single <- function(x, 
                                           nulln = 999, 
                                           null_method = 'perm', 
                                           null_control = NULL, 
                                           ...) { 
  
  # Build closure passed to compute_indicator_with_null that uses the correct
  #   high and low ranges, and is compatible with the use of matrixn(). 
  sdr_indicf <- function(mat) { 
    spectrum <- rspectrum(mat)
    
    c(sdr = indicator_sdr_do_ratio(spectrum, x[['low_range']], 
                                   x[['high_range']]), 
      spectrum = spectrum[ ,'rspec'])
  }
  
  # Compute null values for SDR
  test_values <- compute_indicator_with_null(x[['orig_data']], 
                                             nulln = nulln, 
                                             indicf = sdr_indicf, 
                                             null_method = null_method, 
                                             null_control = null_control)
  
  # Format results. We import summary stats for SDR results 
  x <- append(x, lapply(test_values[["summary_values"]], function(o) o[1]))
  
  # We extract spectrum values 
  spec <- llply(test_values[["summary_values"]], function(o) o[-1])
  spec <- data.frame(x[["spectrum"]], as.data.frame(spec))
  row.names(spec) <- as.character(seq.int(nrow(spec)))
  x[["spectrum"]] <- spec
  
  # We import information about the null computation in x 
  x <- append(x, test_values[["info"]])
  
  class(x) <- c('spectral_sews_test_single', 
                'spectral_sews_single', 
                'simple_sews_test_single', 
                'simple_sews_single', 
                'sews_result_single')
  
  return(x)
}


# Methods to extract the spectrum from spectral_sews objects
# ---------------------------------------------------------

#' @title Extract the r-spectrum from objects 
#'
#' @description Extract the r-spectrum from objects produced by
#'   \code{spectral_sews}. 
#'
#' @param x An object produced by \code{spectral_sews} or the result of the 
#'   \code{indictest} function called on such object
#' 
#' @param ... Other arguments are ignored 
#' 
#' @return The empirical r-spectrum as a \code{data.frame}
#' 
#' @seealso \code{\link{spectral_sews}}, \code{\link{rspectrum}}
#' 
#' @examples 
#' 
#' # Extract the r-spectrum after computing indicators
#' indics <- spectral_sews(serengeti[2:3])
#' extract_spectrum(indics) 
#' 
#'@export
extract_spectrum <- function(x, ...) { 
  UseMethod("extract_spectrum")
}
#'@export
extract_spectrum.spectral_sews_list <- function(x, ...) { 
  values <- Map(function(n, o) data.frame(matrixn = n, o[["spectrum"]]), 
                seq_along(x), x)
  do.call(rbind, values)
}
#'@export
extract_spectrum.spectral_sews_single <- function(x, ...) { 
  x[["spectrum"]]
}

#' @title Display the r-spectrum of a \code{spectral_sews} object
#' 
#' @description Display the r-spectrum (or multiple spectra) that are contained
#'   in an object returned by \code{\link{spectral_sews}} object (or the result 
#'   of \code{\link{indictest}} applied on such object). 
#' 
#' @param x An object produced by \code{\link{spectral_sews}} or the result 
#'   returned by \code{\link{indictest}} applied on such object
#' 
#' @param along A vector providing numerical or categorical values along 
#'   which the indicator trends will be plotted. If \code{NULL}, then the
#'   indicator values are plotted sequentially in their original order. 
#' 
#' @param log Whether to use a log scale or a linear scale on the y axis
#' 
#' @param display_null Whether to display null information. This argument is 
#'   ignored if \code{x} has not been produced through \code{\link{indictest}} 
#'   (and thus does not contain data regarding the null model)
#' 
#' @param ... Other arguments are ignored 
#' 
#' @seealso \code{\link{rspectrum}}, \code{\link{spectral_sews}}, 
#'   \code{\link{extract_spectrum}}
#' 
#'@export
plot_spectrum <- function(x, 
                          along = NULL, 
                          log = TRUE, 
                          display_null = TRUE, 
                          ...) { 
  UseMethod("plot_spectrum")
}

# Method for indictest output
#'@export
plot_spectrum.spectral_sews_test_list <- function(x, 
                                                  along = NULL, 
                                                  log = TRUE, 
                                                  display_null = TRUE, 
                                                  ...) { 
  
  ggobj <- plot_spectrum.spectral_sews_list(x, along, log)
  
  # Add layers with null model information. We use this method so that the 
  # ribbon appears below the line corresponding to observed values. 
  if ( display_null ) { 
    ggobj$layers <- c(geom_line(aes_string(x = "dist", y = "null_mean"), 
                                color = 'black', alpha = .1), 
                      geom_ribbon(aes_string(x = "dist", ymin = "null_qinf", 
                                            ymax = "null_qsup"), 
                                  fill = 'grey',
                                  group = 1, 
                                  alpha = .8), 
                      ggobj$layers)
  }
    
  return(ggobj) 
}

#'@export
plot_spectrum.spectral_sews_test_single <- function(x, 
                                                  along = NULL, 
                                                  log = TRUE, 
                                                  display_null = TRUE, 
                                                  ...) { 
  
  # Get base plot 
  ggobj <- plot_spectrum.spectral_sews_single(x, along, log)
  
  # Add layers with null model information. We use this method so that the 
  # ribbon appears below the line corresponding to observed values. 
  if ( display_null ) { 
    ggobj$layers <- c(geom_line(aes_string(x = "dist", y = "null_mean"), 
                                color = 'black', alpha = .1), 
                      geom_ribbon(aes_string(x = "dist", ymin = "null_qinf", 
                                            ymax = "null_qsup"), 
                                  fill = 'grey',
                                  group = 1, 
                                  alpha = .8), 
                      ggobj$layers)
  }
  
  return(ggobj)
}

# Method for spectral_sews output (list object)
#'@export
plot_spectrum.spectral_sews_list <- function(x, 
                                             along = NULL, 
                                             log = TRUE, 
                                             display_null = TRUE, 
                                             ...) { 
  
  tab <- extract_spectrum(x)
  check_suitable_for_plots(tab, along)
  
  if ( ! is.null(along) ) { 
    tab[ ,"along"] <- along[tab[ ,"matrixn"]]
  } else { 
    tab[ ,"along"] <- tab[ ,"matrixn"]
  }
  
  p <- ggplot(tab, aes_string(x = "dist", y = "rspec")) + 
    geom_line() + 
    theme_spwarnings() + 
    facet_wrap( ~ along ) + 
    labs(x = "Distance", y = "r-spectrum")
  
  if (log) { 
    p <- p + scale_y_continuous(trans = "log10")
  }
  return(p)
}

# Method for spectral_sews output (single object)
#'@export
plot_spectrum.spectral_sews_single <- function(x, 
                                               along = NULL, 
                                               log = TRUE, 
                                               display_null = TRUE, 
                                               ...) { 
  p <- ggplot(extract_spectrum(x), 
         aes_string(x = "dist", y = "rspec")) + 
    geom_line() + 
    theme_spwarnings() + 
    labs(x = "Distance", y = "r-spectrum")
  
  if (log) { 
    p <- p + scale_y_continuous(trans = "log10")
  }
  
  return(p)
}

#
#' @title Early-warning signals based on patch size distributions
#' 
#' @description Compute early-warning signals based on patch size distributions 
#' 
#' @param mat A logical matrix (\code{TRUE}/\code{FALSE} values) or a list of these
#' 
#' @param merge The default behavior is to produce indicators values for each 
#'   matrix. If this parameter is set to TRUE then the patch size distributions 
#'   are pooled together before fitting, yielding only one final indicator 
#'   value for the set of input matrices (argument \code{mat}).
#' 
#' @param fit_lnorm When patch size distributions are compared, should we 
#'   consider lognormal type ? (see details)
#' 
#' @param best_by The criterion to use to select the best fit (one of "AIC", 
#'   "BIC" or "AICc")
#' 
#' @param xmin The \eqn{x_{min}} to be used to fit the patch size distributions. Use 
#'   the special value "estimate" to compute first the \eqn{x_{min}} that produces 
#'   the best power-law fit, then use this estimated value to fit all 
#'   distributions. 
#' 
#' @param xmin_bounds Bounds when estimating \eqn{x_{min}} for power-law distributions
#' 
#' @param wrap Determines whether patches are considered to wrap around the 
#'  matrix when reaching the side 
#' 
#' @param nbmask Either "moore" for 8-way neighborhood, "von_neumann" for four-way 
#'   neighborhood (default), or a 3x3 matrix describing which neighbors to 
#'   consider around a cell. See \code{\link{patchsizes}} for details on how to specify 
#'   such neighborhoods.
#' 
#' @return A list object of class 'psdfit' containing among other things 
#'   - the observed patch size distribution data
#'   - the model outputs for the candidate distribution fits
#'   - the power-law range values 
#'   - the percolation values (if several matrices were provided and 
#'   `merge` was TRUE, then the average percolation value is returned)
#' 
#' @details 
#' 
#' Patterned ecosystems can exhibit a change in their spatial structure as they 
#' become more and more stressed. It has been suggested that this should be 
#' reflected in changes in the observed patch size distributions (PSD). 
#' The following sequence is expected to occur (Kefi et al. 2011) as patterned 
#' ecosystems become more and more degraded:
#' 
#'   - Percolation of vegetation patches occurs (a patch has a width or height 
#'   equal to the size of the system)
#'   
#'   - The patch-size distribution follows a power-law
#'   
#'   - The patch-size distribution deviates from a power-law as larger patches 
#'   break down
#'   
#'   - The patch-size distribution is closer to an exponential 
#'   distribution
#' 
#' Additionally, it has been suggested that these changes in patch size 
#' distribution shape should be reflected in the power-law range (PLR). This 
#' function carries out all the required computations and helps display 
#' the results in a convenient form. 
#' 
#' The fitting of PSDs is based on maximum-likelihood following Clauset et al.'s 
#' procedure. The best discrete distribution is estimated among these 
#' candidates: a power-law \eqn{x^\lambda}, an exponential 
#' \eqn{exp(\alpha x)}, a truncated power-law and \eqn{x^\lambda exp(\alpha x)},
#' and optionally, a log-normal. Each distribution parameter is estimated 
#' using maximum-likelihood, with a minimum patch size (xmin) fixed to one. 
#' The best distribution is selected based on BIC by default. In raw results, 
#' \code{plexpo} refers to the power-law exponent (\eqn{\lambda} in the previous 
#' equations) and \code{cutoff} referes to the exponential decay rate
#' \eqn{\alpha}. 
#' 
#' To compute the Power-law range (PLR), power-laws are fitted with a variable 
#' minimum patch size (xmin) and the one with the lowest Kolmogorov-Smirnov
#' distance to the empirical distribution is retained. PLR is then computed 
#' using this best-fitting xmin: 
#' 
#' \deqn{\frac{log(x_{max}) - log(x_{min})}{log(x_{max}) - log(x_{smallest})}}{ (log(xmax) - log(xmin))/(log(xmax) - log(xsmallest))}
#' 
#' Results can be displayed using the text-based \code{summary} and \code{print}, 
#' but graphical options are also available to plot the trends (\code{plot}) and 
#' the fitted distributions (\code{\link{plot_distr}}). Plotting functions are 
#' documented in a \link[=patchdistr_sews_plot]{separate page}. Observed and 
#' fitted distributions can be produced using the \code{predict} function, 
#' as documented on \link[=patchdistr_sews_predict]{this page}. 
#' 
#' @seealso \code{\link{patchsizes}}, \code{\link{plot_distr}}, 
#'   \code{\link[=predict.patchdistr_sews_single]{predict}}, 
#'   \code{\link[=plot.patchdistr_sews]{plot}}, 
#' 
#' @seealso 
#'   \code{\link{indictest}}, to test the significance of indicator values. 
#'     
#' @references 
#' 
#' Kefi, S., Rietkerk, M., Alados, C. L., Pueyo, Y., Papanastasis, 
#'   V. P., ElAich, A., & De Ruiter, P. C. (2007). Spatial vegetation patterns 
#'   and imminent desertification in Mediterranean arid ecosystems. 
#'   Nature, 449(7159), 213-217.
#' 
#' Kefi, S., Rietkerk, M., Roy, M., Franc, A., de Ruiter, P.C. & Pascual, M. 
#'   (2011). Robust scaling in ecosystems and the meltdown of patch size 
#'   distributions before extinction: Patch size distributions towards 
#'   extinction. Ecology Letters, 14, 29-35.
#' 
#' Berdugo, M, Sonia Kefi, Santiago Soliveres, and Fernando T. Maestre. (2017) 
#'   Plant Spatial Patterns Identify Alternative Ecosystem Multifunctionality 
#'   States in Global Drylands. Nature in Ecology and Evolution, no. 1.
#' 
#' Clauset, A., Shalizi, C. R., & Newman, M. E. (2009). 
#'   Power-law distributions in empirical data. SIAM review, 51(4), 661-703.
#' 
#' @examples
#' 
#' data(forestgap)
#' 
#' \donttest{
#' psd_indic <- patchdistr_sews(forestgap)
#' 
#' summary(psd_indic)
#' plot(psd_indic)
#' 
#' # Plots can be modified using ggplot2 directives 
#' if ( require(ggplot2) ) { 
#'   plot(psd_indic) + 
#'     theme_minimal()
#' }
#' 
#' # Export results to a data.frame
#' psd_indic_export <- as.data.frame(psd_indic) 
#' head(psd_indic_export)
#' 
#' }
#' @export
patchdistr_sews <- function(mat, 
                            merge = FALSE,
                            fit_lnorm = FALSE,
                            best_by = "BIC", 
                            xmin = 1, # a number, or "estimate" option
                            xmin_bounds = NULL, 
                            wrap = FALSE, 
                            nbmask = "von_neumann") {
  
  # If input is a list -> apply on each element
  if ( ( ! merge ) && is.list(mat)) { 
    results <- future_lapply_seed(mat, patchdistr_sews, merge,
                                  fit_lnorm, best_by, xmin,
                                  xmin_bounds, wrap, nbmask)
    class(results) <- c('patchdistr_sews_list', 'sews_result_list')
    return(results)
  } 
  
  # Convert object to matrix form (or list of matrices) and check if it is
  # suitable for spatialwarnings
  mat <- convert_to_matrix(mat)
  if ( is.list(mat) ) { 
    lapply(mat, check_mat)
  } else { 
    check_mat(mat)
  }
  
  # Get patch size distribution
  psd <- patchsizes(mat, merge = merge, wrap = wrap, nbmask = nbmask)
  
  # Set bounds to search for xmin
  if ( length(psd) > 0 && is.null(xmin_bounds) ) { 
    xmin_bounds <- range(psd)
  }
  
  # Estimate power-law range and set xmin to its estimated value if set to 
  # auto. 
  plr_est <- plrange(psd, xmin_bounds)
  if ( xmin == "estimate" ) { 
    xmin <- plr_est[ ,'xmin_est'] # plrange returns a data frame
  }
  
  # Compute percolation 
  if ( is.list(mat) ) { 
    percol <- lapply(mat, percolation, nbmask = nbmask)
    percol <- mean(unlist(percol))
    percol_empty <- lapply(mat, function(mat) percolation(!mat, nbmask = nbmask))
    percol_empty <- mean(unlist(percol_empty))
  } else { 
    percol <- percolation(mat, nbmask = nbmask)
    percol_empty <- percolation(!mat, nbmask = nbmask)
  } 
  
  # Compute the mean cover 
  if ( is.list(mat) ) { 
    meancover <- mean(laply(mat, mean))
  } else { 
    meancover <- mean(mat)
  }
  
  # Return object 
  result <- list(psd_obs = sort(psd), 
                 psd_type = psdtype(psd, xmin, best_by, fit_lnorm),
                 percolation = percol,
                 percolation_empty = percol_empty,
                 cover = meancover,
                 plrange = plr_est, 
                 npatches = length(psd),
                 unique_patches = length(unique(psd)), 
                 orig_data = mat)
  class(result) <- c('patchdistr_sews_single', 'sews_result_single')
  
  return(result)
}

# 
# Misc tests with working with patch sizes
# 

test_that("Labelling functions & friends work correctly", { 
  
  n <- 512

  # Fix seed to one that does not produce warning in patchdistr_sews()
  oseed <- .Random.seed
  set.seed(123)
  m <- matrix(rnorm(n*n) > 0.3, nrow = n, ncol = n)
  .Random.seed <- oseed

  # Make sure args are dispatched correctly
  X <- patchsizes(list(m, m), nbmask = "moore", wrap = TRUE)
  expect_true({ 
    all(X[[1]] == X[[2]])
  })
  
  
  # Make sure the type of neighborhood is passed even when using high-level 
  # functions. We remove warnings because we fit on random data that can sometimes
  options(spatialwarnings.constants.maxit = 1e9L)
  x1 <- patchdistr_sews(m)
  x2 <- patchdistr_sews(m, nbmask = "moore")
  expect_true({ 
    x1[["npatches"]] > x2[["npatches"]]
  })
  options(spatialwarnings.constants.maxit = NULL)
  
  m <- matrix(c(1, 0, 0, 
                0, 1, 1, 
                0, 0, 0) > 0, 
              ncol = 3, nrow = 3, byrow = TRUE)
  
  expect_true({ 
    length(patchsizes(m)) == 2 && all(patchsizes(m) == c(1, 2))
  })
  
  expect_true({ 
    length(patchsizes(m, nbmask = "moore")) == 1 && patchsizes(m, nbmask = "moore") == 3
  })
  
  m <- matrix(c(1, 0, 0, 
                0, 0, 1, 
                0, 0, 0) > 0, 
              ncol = 3, nrow = 3, byrow = TRUE)
  
  expect_true({ 
    a <- patchdistr_sews(m, nbmask = "von_neumann", wrap = TRUE)
    a[["npatches"]] == 2
  })
  
  expect_true({ 
    a <- patchdistr_sews(m, nbmask = "moore", wrap = TRUE)
    a[["npatches"]] == 1
  })
  
  
  
})

# 
# Make sure we honor the choice of xmin when predicting values after fitting distributions
# 

test_that("xmin is used when predicting values", { 
  
  # These tests take a lot of time, so skip them on CRAN
  skip_on_cran()

  # This can produce warnings, but it is not what we want to test here so we suppress
  # them

  # Test for single-element run of patchdistr_sews
  suppressWarnings({
    a <- patchdistr_sews(serengeti[[12]], xmin = 1)
  })
  preds <- predict(a)[["pred"]]
  expect_true({
    abs(max(preds[ ,"y"], na.rm = TRUE) - 1) < 1e-8
  })

  # Test for single-element run of patchdistr_sews, now with xmin
  suppressWarnings({
    a <- patchdistr_sews(serengeti[[12]], xmin = 10)
  })
  preds <- predict(a, xmin_rescale = TRUE)[["pred"]]
  # should be rescaled so that different from 1
  expect_true({
    abs(max(preds[ ,"y"], na.rm = TRUE) - 1) > 0.1
  })

  # Test for multiple-matrix run of patchdistr_sews
  suppressWarnings({ 
    a <- patchdistr_sews(serengeti[11:12], xmin = 1)
  })
  preds <- predict(a)[["pred"]]
  expect_true({
    abs(max(preds[ ,"y"], na.rm = TRUE) - 1) < 1e-8
  })
  
  # Test for multiple-matrix run of patchdistr_sews
  suppressWarnings({ 
    a <- patchdistr_sews(serengeti[11:12], xmin = 10)
  })
  preds <- predict(a, xmin_rescale = TRUE)[["pred"]]
  expect_true({
    abs(max(preds[ ,"y"], na.rm = TRUE) - 1) > 0.1
  })
  
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

clustering_core <- function(m, nstates, wrap, use_8_nb) {
    .Call('_spatialwarnings_clustering_core', PACKAGE = 'spatialwarnings', m, nstates, wrap, use_8_nb)
}

pair_counts_internal <- function(m, nstates, wrap, use_8_nb) {
    .Call('_spatialwarnings_pair_counts_internal', PACKAGE = 'spatialwarnings', m, nstates, wrap, use_8_nb)
}

coarse_grain_cpp <- function(mat, subsize) {
    .Call('_spatialwarnings_coarse_grain_cpp', PACKAGE = 'spatialwarnings', mat, subsize)
}

fl_internal <- function(m) {
    .Call('_spatialwarnings_fl_internal', PACKAGE = 'spatialwarnings', m)
}

label_cpp <- function(mat, nbmask, wrap) {
    .Call('_spatialwarnings_label_cpp', PACKAGE = 'spatialwarnings', mat, nbmask, wrap)
}

#' 
#' @title Spatial correlation at lag 1
#'
#' @description This function computes the Moran's I index of spatial 
#'   correlation at lag 1.
#' 
#' @param mat A matrix containing logical, numeric or integer values
#' 
#' @return The Moran's I numeric value as a numeric number.
#' 
#' @details This function returns the spatial correlation as measured by 
#'   the Moran's I index. If the variance of the matrix is zero, then 
#'   \code{NaN} is returned. This function assumes a 4-way neighborhood (a.k.a. 
#'   von-Neumann neighborhood), and does not wrap around at the sides of the matrix.
#' 
#' @seealso \code{\link{generic_sews}} 
#' 
#' @examples
#' 
#' # Spatial correlation of white noise is close to zero
#' rmat <- matrix(runif(1000) > .5, ncol = 100)
#' raw_moran(rmat) 
#' 
#' # Spatial correlation of a half-ones / half-zeros matrix is close to one. 
#' # This would produce close but inaccurate results in version <3.0.2
#' m <- cbind(matrix(1, nrow = 100, ncol = 50), 
#'            matrix(0, nrow = 100, ncol = 50))
#' 
#' raw_moran(m)
#' 
#'@export
raw_moran <- function(mat) {
    .Call('_spatialwarnings_raw_moran', PACKAGE = 'spatialwarnings', mat)
}

tplsum <- function(expo, rate, xs, xmin) {
    .Call('_spatialwarnings_tplsum', PACKAGE = 'spatialwarnings', expo, rate, xs, xmin)
}

tplinfsum <- function(expo, rate, xmin, maxit, reltol) {
    .Call('_spatialwarnings_tplinfsum', PACKAGE = 'spatialwarnings', expo, rate, xmin, maxit, reltol)
}

shuffle_matrix <- function(mat) {
    .Call('_spatialwarnings_shuffle_matrix', PACKAGE = 'spatialwarnings', mat)
}

shuffle_and_compute <- function(mat, indic, nrep) {
    .Call('_spatialwarnings_shuffle_and_compute', PACKAGE = 'spatialwarnings', mat, indic, nrep)
}

#' @title r-spectrum 
#' 
#' @description Compute the r-spectrum of a matrix 
#' 
#' @param mat A matrix with logical or numeric values 
#' 
#' @return A data.frame with two columns: \code{dist}, the wave number and 
#'   \code{rspec}, the normalized value of the r-spectrum
#' 
#' @details This functions returns a \code{data.frame} with \code{NA}s in the
#'   \code{rspec} column if the input matrix has zero variance. Note that if the matrix 
#'   is not square, then only the largest square matrix fitting in the upper left
#'   corner is used. 
#' 
#' @seealso \code{\link{spectral_sews}}
#' 
#' @examples 
#' 
#' # Spectrum of white noise
#' rmat <- matrix(runif(100*100) > .5, ncol = 100)
#' spec <- rspectrum(rmat) 
#' plot(spec, type = "l")
#' 
#' # Add some spatial correlation and compare the two spectra
#' rmat.cor <- rmat
#' for (i in seq(1, nrow(rmat)-1)) { 
#'   for (j in seq(1, nrow(rmat)-1)) { 
#'     rmat.cor[i,j] <- mean(rmat[(i-1):(i+1), (j-1):(j+1)])
#'   }
#' }
#' spec.cor <- rspectrum(rmat.cor)
#' plot(spec.cor, type = "n")
#' lines(spec, col = "black")
#' lines(spec.cor, col = "blue")
#' 
#' @export
rspectrum <- function(mat) {
    .Call('_spatialwarnings_rspectrum', PACKAGE = 'spatialwarnings', mat)
}

cpp_skewness <- function(X) {
    .Call('_spatialwarnings_cpp_skewness', PACKAGE = 'spatialwarnings', X)
}

variogram_internal_cpp <- function(mat, nmax, bins, cutoff) {
    .Call('_spatialwarnings_variogram_internal_cpp', PACKAGE = 'spatialwarnings', mat, nmax, bins, cutoff)
}

sum_all_one_over_k <- function(from, to, expo) {
    .Call('_spatialwarnings_sum_all_one_over_k', PACKAGE = 'spatialwarnings', from, to, expo)
}

context('Test the computation of moran correlation')

test_that("Moran correlation is computed correctly", { 
  
  within_bounds <- function(x) { 
    expect_true( all(x >= -1 & x <= 1) )
  }
  
  # We work on small matrices so that the handling of sides has a big effect
  n <- 5*5
  nreps <- 9999
  
  # Correlation of random matrices is near-zero 
  morans <- replicate(nreps, { 
    raw_moran(matrix(rnorm(n), ncol = sqrt(n)))
  })
  within_bounds(morans)
  # Check that the mean value is equal to the expected value
  expect_true( abs(mean(morans) - ( -1 / ( n - 1 ) )) < 1e-2)
  
  # Correlation of checkerboard is near -1 
  # If n is even, then make it odd, so that the testing matrices are really checkerboards
  ncheck <- ifelse(n %% 2 == 0, (sqrt(n)+1)^2, n)
  c1 <- matrix((1+seq.int(ncheck)) %% 2, ncol = sqrt(ncheck))
  c2 <- matrix(seq.int(ncheck) %% 2, ncol = sqrt(ncheck))
  expect_true( abs(raw_moran(c1) - -1) < 1e-10 )
  expect_true( abs(raw_moran(c2) - -1) < 1e-10 )
  within_bounds(raw_moran(c1))
  within_bounds(raw_moran(c2))
  
  # Correlation of left/right matrix is near +1
  nm <- 200*200
  m1 <- cbind(matrix(1, nrow = sqrt(nm), ncol = floor(sqrt(nm) / 2)), 
              matrix(0, nrow = sqrt(nm), ncol = ceiling(sqrt(nm) / 2)))
  expect_true( abs( raw_moran(m1) - 1 ) < 0.01 )
  within_bounds(raw_moran(m1))
  
  if ( exists("EXTENDED_TESTS") && EXTENDED_TESTS ) { 
    # Test our implementation against the one in package raster
    # Note that raster produces warnings that do not affect us
    if ( suppressWarnings( requireNamespace("raster", quietly = TRUE) ) ) { 
      c <- seq(0.01, 0.99, l = 21)
      w <- matrix(c(0, 1, 0, 1, 0, 1, 0, 1, 0), byrow = TRUE, ncol = 3)
      
      for ( i in seq_along(c) ) { 
        m <- matrix(runif(n) < c[i], ncol = sqrt(n))
        diffidx <- abs( raw_moran(m) - raster::Moran(raster::raster(m), w) )
        if ( ! is.nan(diffidx) ) { 
          expect_true( diffidx < 1e-10 )
        }
      }
      expect_true({ 
        abs(raw_moran(c1) - raster::Moran(raster::raster(c1), w)) < 1e-10
      })
      expect_true({ 
        abs(raw_moran(c2) - raster::Moran(raster::raster(c2), w)) < 1e-10
      })
      
    }
  }
  
})

# 
# 
# 
context("Test that skewness computation is correct") 

test_that("Skewness computation is OK", { 
  if ( requireNamespace("moments", quietly = TRUE) ) { 
    X <- runif(1000) 
    expect_equal(moments::skewness(X), 
                 cpp_skewness(X))
  }
})

# 
# 
# Test the computation of spectra (r-spectrum so far)
# 

context("Test the correct computation of r-spectrum")

test_that("the cpp implementation of the spectrum computations is correct", { 
  
  # Redefine the old functions
  rspectrum_old <- function(mat, step) { 

  nr <- nrow(mat)
  nc <- ncol(mat)
  
  n0x <- floor(nc/2) + 1
  n0y <- floor(nr/2) + 1
  
  # Create distance and angle matrices
  f1 <- t(replicate(nr,seq(1,nc))) - n0x
  f2 <- replicate(nc,seq(1,nr)) - n0y
  DIST <- sqrt(f1^2 + f2^2)
  
  # Calculate DFT
  mi <- 1
  ma <- min(c(n0x,n0y))
  DISTMASK <- DIST>=mi & DIST <= ma
  
  tmp <- fft(mat)
  class(tmp) <- "matrix"
  tmpshift <- myfftshift(tmp)
  tmpshift[n0x, n0y] <- 0
  aspectr2D <- abs(tmpshift)^2 / (n0x*n0y)^4
  
  sig2 <- sum(aspectr2D[DISTMASK]) #Normalisation
  
  aspectr2D <- aspectr2D/sig2 #Normalisation
  
  # Now calculate r-spectrum
  STEP <- 1
  ray <- seq(mi,ma,STEP)
  rspectr <- numeric(length(ray))
  for (i in 1:length(ray))
  {
    m <- DIST >= ray[i] - STEP/2 & DIST < ray[i] + STEP/2
    rspectr[i] <- mean(aspectr2D[m])
  }

  out <- data.frame(dist  = ray, rspec = rspectr)
  
  return(out)
  }
  
  myfftshift <- function(X) {
    nr=dim(X)[1]
    nc=dim(X)[2]
    shiftX = X
    if (nr != nc)
      print("Not a square matrix X")
    else
    {
      n=nc
      shift = floor(n/2)
      for (i in 1:n)
        for (j in 1:n)
        {
          a = (i+shift)%%n
          b = (j+shift)%%n
          if (a==0) a = n
          if (b==0) b = n
          shiftX[a,b] = X[i,j]
        }
    }
    return(shiftX)
  }
  
  for ( i in c(1, 2) ) {
    testmat <- serengeti[[i]]
    
    # Test if there is more than one value in the matrix, because the behavior 
    # is to return NA whereas the old code just compute things 
    if ( length(unique(as.vector(testmat))) > 1 ) { 
    
    ospec <- rspectrum_old(testmat)
    expect_equal(ospec, 
                 rspectrum(testmat), 
                 tolerance = 1/1000) 
    }
  }
  
  # Test on odd-sized matrix 
  s <- 101
  modd <- matrix(runif(s^2) < 0.5, ncol = s)
  expect_equal(rspectrum(modd), rspectrum_old(modd), 
               tolerance = 1/1000)
  
})

test_that("rspectrum/sdr show realistic values", { 
  
  
  # The expected value of sdr on random matrices is one 
  sdr_random <- mean(replicate(299, { 
    n <- 100
    m <- matrix(rnorm(n^2), nrow = n, ncol = n)
    raw_sdr(m, c(0, 0.2), c(0, 1))
  }))
  
  expect_true({ 
    abs(sdr_random - 1 ) < 0.1
  })
  
  # When there is reddening, the sdr increases 
  sdr_red <- mean(replicate(299, { 
    n <- 100
    m <- matrix(rnorm(n^2), nrow = n, ncol = n)
    m[2:n, 2:n] <- (m[2:n, 2:n] + m[1:(n-1), 1:(n-1)]) / 2
    raw_sdr(m, c(0, 0.2), c(0, 1))
  }))
  
  expect_true({ 
    sdr_red > 1 
  })
  
})

# A simple R code to calculate early warning signals of critical transitions in space.

#Load libraries
library(moments)

#Define functions

#Function to calculate Moran's I coefficient OR Spatial autocorrelation between nearest neighbors.
#Can also be calculated using spatial statistics softwares.
corr_m <- function(input)
{
  m <- mean(as.vector(input))
  v <- var(as.vector(input))
  n <- (nrow(input)-1)
  
  moranI <- 0
  for (i in 2:n){
    for (j in 2:n) {
      moranI <- moranI + (input[i,j]-m)*(input[i,j-1]+input[i,j+1]+input[i-1,j]+input[i+1,j]-4*m)	
    }
  }
  moranI <- moranI/(4*v*(n-2)*(n-2))
  moranI
}

# Read data from the file. 
# Format: Each spatial snapshop is of size nxn matrix. There are t such matries, appended one after other.

fulldata <- read.table(file='CA_all.txt',header=FALSE,sep='\t')
data <- data.matrix(fulldata)

numrows<-nrow(data) 
numcol<-ncol(data)
nsnaps<-numrows/numcol #number of snapshots in the data set.

#To store moments of spatial variable as a function of time
mean_spatialvar <- mat.or.vec(nsnaps,1)
var_spatialvar <- mat.or.vec(nsnaps,1)
cvar_spatialvar <- mat.or.vec(nsnaps,1)
skew_spatialvar <- mat.or.vec(nsnaps,1)
corr_spatialvar <-mat.or.vec(nsnaps,1)
lmdft_spatialvar <- mat.or.vec(nsnaps,1)

mean_reduced <- mat.or.vec(nsnaps,1)
var_reduced <- mat.or.vec(nsnaps,1)
cvar_reduced <- mat.or.vec(nsnaps,1)
skew_reduced <- mat.or.vec(nsnaps,1)
corr_reduced <-mat.or.vec(nsnaps,1)
lmdft_reduced <- mat.or.vec(nsnaps,1)

coarse=10;
n=numcol/coarse;
reddata <- mat.or.vec(n,n);

#Calculate early warning signals as a function of time.
for (snap in 1:nsnaps)
{
	rowstart <- (snap-1)*numcol+1
	rowend <- snap*numcol
	datasnap <- data[rowstart:rowend, 1:numcol]
	
	mean_spatialvar[snap] <- mean(as.vector(datasnap))
	var_spatialvar[snap] <- var(as.vector(datasnap))
	cvar_spatialvar[snap] <- var_spatialvar[snap]/mean_spatialvar[snap];
    skew_spatialvar[snap] <- skewness(as.vector(datasnap))
    corr_spatialvar[snap] <- corr_m(datasnap)
     
    #TO do: DFT.
    dft <- fft(datasnap)
    lmdft <- Mod(dft)
    lmdft_spatialvar[snap] <- mean(lmdft[2:25,2:25])
    
    #Calculate reduced matrix by coarse graining over coarsexcoarse submatrix.
	for (i in 1:n)
	{
		for (j in 1:n)
		{
			submatrix = datasnap[((i-1)*coarse+1):(i*coarse),((j-1)*coarse+1):(j*coarse)];
			reddata[i,j] = mean(as.vector(submatrix));
		}
	}
    
	mean_reduced[snap] <- mean(as.vector(reddata))
	var_reduced[snap] <- var(as.vector(reddata))
	cvar_reduced[snap] <- var_reduced[snap]/mean_reduced[snap]
    skew_reduced[snap] <- skewness(as.vector(reddata))
    corr_reduced[snap] <- corr_m(reddata)
    dft <- fft(reddata)
    lmdft <- Mod(dft)
    lmdft_reduced[snap] <- mean(lmdft[2:n/2,2:n/2])
}




# [ADDED BY ALEX]
# There is only DFT-related code beyond this point with plotting stuff.
#   We do not want this for automated tests. 
# ------------------------------------------------------




# par(mfrow=c(3,2),mar=c(5, 4.2, 4, 2) + 0.1,cex.axis=1.6,cex.lab=1.6)
# 
# params <- c(0.39, 0.40, 0.46, 0.475, 0.49, 0.50, 0.51, 0.53, 0.60, 0.65)
# 
# plot(params,mean_spatialvar)
# plot(params,var_spatialvar)
# plot(params,cvar_spatialvar)
# plot(params,skew_spatialvar)
# plot(params,corr_spatialvar)
# plot(params,lmdft_spatialvar)
# 
# dev.new()
# par(mfrow=c(3,2),mar=c(5, 4.2, 4, 2) + 0.1,cex.axis=1.6,cex.lab=1.6)
# params <- c(0.39, 0.40, 0.46, 0.475, 0.49, 0.50, 0.51, 0.53, 0.60, 0.65)
# 
# plot(params,mean_reduced)
# plot(params,var_reduced)
# plot(params,cvar_reduced)
# plot(params,skew_reduced)
# plot(params,corr_reduced)
# plot(params,lmdft_reduced)
# 
# #---------------------------#
# #Plot DFT using a two dimensional color plot.
# 
# Xax <- (1:100)
# Yax <- (1:100)
# half <- 1+(100/2)
# axhalf <- (2:half)
# 
# data <- read.table(file='CA_b0.39.txt',header=FALSE,sep='\t')
# data39 <- data.matrix(data)
# 
# data <- read.table(file='CA_b0.40.txt',header=FALSE,sep='\t')
# data40 <- data.matrix(data)
# 
# data <- read.table(file='CA_b0.46.txt',header=FALSE,sep='\t')
# data46 <- data.matrix(data)
# 
# data <- read.table(file='CA_b0.475.txt',header=FALSE,sep='\t')
# data475 <- data.matrix(data)
# 
# data <- read.table(file='CA_b0.49.txt',header=FALSE,sep='\t')
# data49 <- data.matrix(data)
# 
# data <- read.table(file='CA_b0.50.txt',header=FALSE,sep='\t')
# data50 <- data.matrix(data)
# 
# data <- read.table(file='CA_b0.51.txt',header=FALSE,sep='\t')
# data51 <- data.matrix(data)
# 
# data <- read.table(file='CA_b0.53.txt',header=FALSE,sep='\t')
# data53 <- data.matrix(data)
# 
# data <- read.table(file='CA_b0.60.txt',header=FALSE,sep='\t')
# data60 <- data.matrix(data)
# 
# data <- read.table(file='CA_b0.65.txt',header=FALSE,sep='\t')
# data65 <- data.matrix(data)
# 
# dft <- fft(data39);
# mdft39 <- Mod(dft[2:half,2:half])
# lmdft39 <- log(mdft39)
# 
# dft <- fft(data40);
# mdft40 <- Mod(dft[2:half,2:half])
# lmdft40 <- log(mdft40)
# 
# dft <- fft(data46);
# mdft46 <- Mod(dft[2:half,2:half])
# lmdft46 <- log(mdft46)
# 
# dft <- fft(data475);
# mdft475 <- Mod(dft[2:half,2:half])
# lmdft475 <- log(mdft475)
# 
# dft <- fft(data49);
# mdft49 <- Mod(dft[2:half,2:half])
# lmdft49 <- log(mdft49)
# 
# dft <- fft(data50);
# mdft50 <- Mod(dft[2:half,2:half])
# lmdft50 <- log(mdft50)
# 
# dft <- fft(data51);
# mdft51 <- Mod(dft[2:half,2:half])
# lmdft51 <- log(mdft51)
# 
# dft <- fft(data53);
# mdft53 <- Mod(dft[2:half,2:half])
# lmdft53 <- log(mdft53)
# 
# dft <- fft(data60);
# mdft60 <- Mod(dft[2:half,2:half])
# lmdft60 <- log(mdft60)
# 
# dft <- fft(data65);
# mdft65 <- Mod(dft[2:half,2:half])
# lmdft65 <- log(mdft65)
# 
# par(mfrow=c(3,3),mar=c(5, 4.2, 4, 2) + 0.1,cex.axis=1.6,cex.lab=1.6)
# 
# zrange <- range(c(data39,data40, data46, data475, data49, data50,data51,data53, data60,data65))
# 
# image(Xax,Yax,data39,zlim=zrange,col=heat.colors(50),xlab='X',ylab='Y',
# main='b=0.39')
# 
# image(Xax,Yax,data40,zlim=zrange,col=heat.colors(50),xlab='X',ylab='Y',
# main='b=0.40')
# 
# image.plot(Xax,Yax,data46,zlim=zrange,col=heat.colors(50),xlab='X',ylab='Y',
# main='b=0.46')
# 
# image(Xax,Yax,data475,zlim=zrange,col=heat.colors(50),xlab='X',ylab='Y',
# main='b=0.475')
# 
# image(Xax,Yax,data49,zlim=zrange,col=heat.colors(50),xlab='X',ylab='Y',
# main='b=0.49')
# 
# image.plot(Xax,Yax,data51,zlim=zrange,col=heat.colors(50),xlab='X',ylab='Y',
# main='b=0.51')
# 
# image(Xax,Yax,data53,zlim=zrange,col=heat.colors(50),xlab='X',ylab='Y',
# main='b=0.53')
# 
# image(Xax,Yax,data60,zlim=zrange,col=heat.colors(50),xlab='X',ylab='Y',
# main='b=0.60')
# 
# image.plot(Xax,Yax,data65,zlim=zrange,col=heat.colors(50),xlab='X',ylab='Y',
# main='b=0.65')
# 
# #Plot DFT
# par(mfrow=c(3,3),mar=c(5, 4, 4, 2) + 0.1,cex.axis=1.6,cex.lab=1.6)
# 
# zrange <- range(c(mdft39,mdft49,mdft51,mdft53,mdft65,mdft40))
# 
# image(axhalf,axhalf,mdft39,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='',
# main='mdft: b=0.39')
# 
# image(axhalf,axhalf,mdft40,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='mdft: b=0.40')
# 
# image.plot(axhalf,axhalf,lmdft46,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='mdft: b=0.46')
# 
# image(axhalf,axhalf,mdft475,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='mdft: b=0.475')
# 
# image(axhalf,axhalf,mdft49,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='mdft: b=0.49')
# 
# image.plot(axhalf,axhalf,mdft50,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='',
# main='mdft: b=0.50')
# 
# image(axhalf,axhalf,mdft51,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='',
# main='mdft: b=0.51')
# 
# image(axhalf,axhalf,mdft60,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='mdft: b=0.60')
# 
# image.plot(axhalf,axhalf,mdft65,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='mdft: b=0.65')
# 
# #Plot mDFT with log for better clarity on patterns.
# 
# par(mfrow=c(3,3),mar=c(5, 4, 4, 2) + 0.1,cex.axis=1.6,cex.lab=1.6)
# 
# zrange <- range(c(lmdft39,lmdft49,lmdft51,lmdft53))
# 
# image(axhalf,axhalf,lmdft39,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='',
# main='log(mdft): b=0.39')
# 
# image(axhalf,axhalf,lmdft40,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='log(mdft): b=0.40')
# 
# image.plot(axhalf,axhalf,lmdft46,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='log(mdft): b=0.46')
# 
# image(axhalf,axhalf,lmdft475,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='log(mdft): b=0.475')
# 
# image(axhalf,axhalf,lmdft49,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='log(mdft): b=0.49')
# 
# image.plot(axhalf,axhalf,lmdft50,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='',
# main='log(mdft): b=0.50')
# 
# image(axhalf,axhalf,lmdft51,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='',
# main='log(mdft): b=0.51')
# 
# image(axhalf,axhalf,lmdft60,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='log(mdft): b=0.60')
# 
# image.plot(axhalf,axhalf,lmdft65,zlim=zrange,col=heat.colors(50),xlab='X Freq',ylab='Y Freq',
# main='log(mdft): b=0.65')

# 
# 
# Textual output options 
DIGITS <- 2

# This is going to run when the user loads the package
# 
.onAttach <- function(libname, pkgname){
  
  parallelism_message <- "Use plan(multisession) to set up parallel processing"
  
  package_ver <- utils::packageDescription("spatialwarnings", 
                                           fields = "Version")
  
  devel_message <- ""
  if ( grepl("99$", package_ver, perl = TRUE) ) { 
    devel_message <- " (development version, use at your own risk!)"
  }
  
  packageStartupMessage({ 
    paste0("This is spatialwarnings ", 
           package_ver, devel_message, 
           "\n", parallelism_message)
    }, appendLF = TRUE)
  
}



# A function returning a theme that gives visual homogenity to all the plots 
# produced by the spatialwarnings package. 
theme_spwarnings <- function() { 
  lightgray <- "#DCDAD5"
  lightgray_darker <- "#B6B5B1"
  lightgray_lighter <- "#F1EFE9"
  theme_bw() + 
    theme(panel.grid.major  = element_line(color = lightgray_darker, 
                                           linetype = 2, 
                                           size = .3), 
          panel.grid.minor  = element_blank(), 
          strip.background = element_rect(fill = NA, color = NA), 
          panel.border = element_rect(color = lightgray))
}

# Line color scale
linescale_spwarnings <- function() { 
  scale_color_manual(values = c('#EAD710', '#EA4510', '#6E10EA', '#10EA12'))
}

fillscale_spwarnings <- function(...) { 
#   scale_fill_manual(values = c('#F2C84A', "#F24A4A", "#734AF2", "#4AF24B"))
  scale_fill_manual(..., values = c("#F7DC8D", "#F78D8D", "#A78DF7","#8DF78E"))
}

# 
# This file contains a function to vizualise matrices as those we use in
#   spatialwarnings 
# 

#' @title Plot a matrix 
#' 
#' @description Display a matrix or a list of matrices in a plot 
#' 
#' @param object A matrix, a list of matrices, an object produced by 
#'   \code{*_sews} functions or \code{indictest()}
#' 
#' @param palette A color palette to use in the plot. It can be any color 
#'   palette understood by \link[ggplot2]{scale_fill_brewer}. 
#' 
#' @param along A vector of values used in facet headers. If \code{object} is 
#'   a matrix, this is ignored. 
#' 
#' @param ... Other arguments are ignored. 
#' 
#' @details This function will plot a matrix using ggplot2, using the provided 
#'   palette. Its use-case is very much like \code{image()}, but its produces 
#'   nicer plots by default (\code{image()} is much faster, however). 
#' 
#' @return A ggplot2 object, which is printed when this function is used 
#'   interactively. 
#' 
#' @examples 
#' 
#' # display_matrix works with single matrices or lists of matrices 
#' display_matrix(serengeti[2:3])
#' 
#' # display_matrix is compatible with "*_sews" objects
#' indics <- compute_indicator(serengeti[2:3], raw_moran) 
#' display_matrix(indics)
#' 
#'@export
display_matrix <- function(object, palette = "RdYlBu", along = NULL, ...) { 
  UseMethod("display_matrix")
}

#'@export
display_matrix.matrix <- function(object, palette = "RdYlBu", 
                                  along = NULL, ...) { 
  check_mat(object, NAerror = FALSE) 
  
  if ( is.numeric(object) ) { 
    fillscale <- scale_fill_distiller(palette = palette)
  } else { 
    fillscale <- scale_fill_brewer(palette = palette)
  }
  
  ggplot(tabularize(object), aes_string(x = "col", y = "row")) + 
    geom_raster(aes_string(fill = "value")) + 
    fillscale + 
    scale_y_reverse() + 
    theme_spwarnings() + 
    theme(axis.title.x = element_blank(), 
          axis.title.y = element_blank()) + 
    coord_fixed()
}

#'@export 
display_matrix.list <- function(object, palette = "RdYlBu", 
                                along = NULL, ...) { 
                                  
  # Convert and check objects
  object <- lapply(object, convert_to_matrix)
  lapply(object, check_mat, NAerror = FALSE)

  # Convert all matrices to data frames
  all_tabs <- Map(function(n, o) { data.frame(matrixn = n, tabularize(o)) }, 
                  seq_along(object), object)
  all_tabs <- do.call(rbind, all_tabs)
  
  # Add the along vector 
  check_suitable_for_plots(all_tabs, along)
  if ( ! is.null(along) ) { 
    all_tabs[ ,"along"] <- along[all_tabs[ ,"matrixn"]]
  } else { 
    all_tabs[ ,"along"] <- all_tabs[ ,"matrixn"]
  }
  
  # Choose proper color scale 
  if ( is.numeric(all_tabs[ ,"value"]) ) { 
    fillscale <- scale_fill_distiller(palette = palette)
  } else { 
    fillscale <- scale_fill_brewer(palette = palette)
  }
  
  ggplot(all_tabs, aes_string(x = "col", y = "row")) + 
    geom_raster(aes_string(fill = "value")) + 
    facet_wrap( ~ along ) + 
    fillscale + 
    scale_y_reverse() + 
    theme_spwarnings() + 
    theme(axis.title.x = element_blank(), 
          axis.title.y = element_blank()) + 
    coord_fixed()
}

#'@export 
display_matrix.sews_result_list <- function(object, palette = "RdYlBu", 
                                            along = NULL, ...) { 
  
  # Extract original data from object 
  orig_dat <- lapply(object, function(o) o[["orig_data"]])
  display_matrix.list(orig_dat, palette = palette, along = along, ...)
}

#'@export 
display_matrix.sews_result_single <- function(object, palette = "RdYlBu", 
                                              along = NULL, ...) { 
  
  # Extract original data from object 
  display_matrix.matrix(object[["orig_data"]], palette = palette, 
                        along = NULL, ...)
}

#'@export
display_matrix.default <- function(object, palette = "RdYlBu", 
                                   along = NULL, ...) { 
  display_matrix( convert_to_matrix(object) )
}


tabularize <- function(mat) { 
  data.frame(expand.grid(row = seq.int(nrow(mat)), 
                         col = seq.int(ncol(mat))), 
             value = as.vector(mat))
}

# 
# This file contains functions to test significance and plot simple indicators 
# 

#' @export
indictest.simple_sews <- function(x, 
                                  nulln = 999, 
                                  null_method = 'perm', 
                                  null_control = NULL, 
                                  ...) { 
  NextMethod('indictest')
}
#'@export
indictest.simple_sews_single <- function(x, 
                                         nulln = 999, 
                                         null_method = 'perm', 
                                         null_control = NULL, 
                                         ...) { 
  
  # We do not support low numbers of null simulations
  if ( nulln < 3 ) { 
    stop('The number of null simulations should be above 3 to ', 
         'assess significance')
  }
  
  new_indicf <- function(mat) { 
    do.call(x[['indicf']], c(list(mat), x[['fun.args']]))
  }
  
  # Compute a distribution of null values
  null_values <- compute_indicator_with_null(x[["orig_data"]],
                                             nulln = nulln, 
                                             indicf = new_indicf, 
                                             null_method = null_method, 
                                             null_control = null_control)
  
  # Add data into original object
  x[["nulldistr"]] <- null_values[["nulldistr"]]
  x <- append(x, null_values[["summary_values"]])
  x <- append(x, null_values[["info"]])
  
  class(x) <- c('simple_sews_test_single', 'sews_test', 
                'sews_result_single')
  
  return(x)
}
#'@export
indictest.simple_sews_list <- function(x, 
                                       nulln = 999, 
                                       null_method = 'perm', 
                                       null_control = NULL, 
                                       ...) { 
  
  results <- future_lapply_seed(x, indictest.simple_sews_single, 
                                nulln, null_method, null_control, 
                                ...)
  
  # Add matrixn value with correct number
  for ( nb in seq_along(results) ) { 
    results[[nb]][['matrixn']] <- nb
  }
  
  class(results) <- c('simple_sews_test_list', 'sews_test', 
                      'sews_result_list')
  
  return(results)
}

#'@method as.data.frame simple_sews_test_single
#'@export
as.data.frame.simple_sews_test_single <- function(x, ...) { 
  # Find or create the indicator names
  indicnames <- ifNULLthen(names(x[['value']]), 
                           paste0("indic_", seq_along(x[['value']])))
  
  # Names of components to extract from x 
  test_names <- c("value", "null_mean", "null_sd", "null_qsup",
                  "null_qinf", "z_score", "pval")
  
  # We need to explicitely add a `matrixn` column because it will 
  # be used by funs down the stream. 
  ans <- data.frame(matrixn = 1, 
                    indic = indicnames, 
                    as.data.frame.list(x[test_names]))
  
  return(ans)
}
#'@method as.data.frame simple_sews_test_list
#'@export
as.data.frame.simple_sews_test_list <- function(x, ...) { 
  # Find or create the indicator names
  indicnames <- ifNULLthen(names(x[[1]][['value']]), 
                           paste0("indic_", seq_along(x[[1]][['value']])))

  # Extract the relevant components from the object, and put them in a data 
  # frame. 
  test_names <- c("value", "null_mean", "null_sd", "null_qsup",
                  "null_qinf", "z_score", "pval")
  tab <- Map(function(n, x) { 
    data.frame(matrixn = n, 
               indic = indicnames, 
               as.data.frame.list(x[test_names])) 
  }, seq_along(x), x)
  
  tab <- do.call(rbind, tab)
  
  # Reorder cols so that 'matrixn' is first'
  tab <- data.frame(matrixn = tab[ ,'matrixn'], 
                    tab[ ! names(tab) %in% 'matrixn'])
  tab
}


#'@export
summary.simple_sews_test_single <- function(object, 
                                            taskname = object[["taskname"]], 
                                            ...) { 
  object.list <- list(object)
  summary.simple_sews_test_list(object.list, taskname, ...)
}
#'@export
summary.simple_sews_test_list <- function(object, 
                                          taskname = object[[1]][["taskname"]], 
                                          ...) { 
  
  tab <- as.data.frame(object)
  
  # Format pvalues 
  tab[ ,'stars'] <- pval_stars(tab[ ,'pval'])
  
  tab <- tab[ ,c('matrixn', 'indic', 'value', 'pval', 'stars')]
  
  # Format P-values 
  nulln <- object[[1]][["nulln"]]
  tab[ ,"pval"] <- format.pval(tab[ ,"pval"], digits = DIGITS, eps = 1/nulln)
  
  # Reshape the table 
  tab_pretty <- llply(unique(tab[['indic']]), function(current_indic) { 
    a <- tab[tab[ ,"indic"] == current_indic, c('value', 'pval', 'stars')]
    names(a) <- c(as.character(current_indic), 
                  paste0('pval_', current_indic), 
                  paste0('stars_', current_indic))
    a
  })
  tab_pretty <- do.call(data.frame, tab_pretty)
  tab_pretty <- data.frame(matrixn = seq.int(nrow(tab_pretty)), tab_pretty)
  
  # Build the header to print. Note that we set it to the names of the 
  # data.frame. This is handy because then print.data.frame handles all the 
  # spacing for us, but we effectively create a data.frame with duplicated 
  # column names. I don't know how brittle this is, but probably very. 
  header <- names(tab_pretty) 
  header[grepl('pval_', header)] <- 'P<null'
  header[grepl('stars_', header)] <- ''
  header[grepl('matrixn', header)] <- 'Mat #'
  names(tab_pretty) <- header
  
  cat('Spatial Early-Warning:', taskname, '\n') 
  cat("\n")
  display_size_info(object)

  cat('\n')
  print.data.frame(tab_pretty, row.names = FALSE, digits = DIGITS)
  cat('\n')
  cat(" Null model: ", object[[1]][["null_method"]], " (", 
      object[[1]][["nulln"]], " null matrices used)", "\n", sep = "")
  cat(" Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1", '\n')
  cat('\n')
  
  cat("The following methods are available:\n")
  cat(list_methods("simple_sews_test_list"), "\n")
  
  invisible(tab)
}



#'@export
print.simple_sews_test_single <- function(x, ...) { 
  summary.simple_sews_test_single(x, ...)
}
#'@export
print.simple_sews_test_list <- function(x, ...) { 
  summary.simple_sews_test_list(x, ...)
}

#' @title Spatial early-warning signals: display of trends
#' 
#' @param x A \code{simple_sews_test} object (as provided by **_sews functions, such 
#'   as \code{generic_sews()}
#' 
#' @param along A vector providing values over which the indicator trend 
#'   will be plotted. If \code{NULL} then the values are plotted sequentially 
#'   in their original order. 
#' 
#' @details Note that the produced plot is adjusted depending on whether 
#'   \code{along} is numeric or not. 
#' 
#' @param what The trendline to be displayed. Defaults to the indicator's 
#'   values ("value") but other metrics can be displayed. Accepted values are 
#'   "value", "pval", "difference" (obs - null mean), or "z_score" 
#'   ( (obs - null mean) / (null sd) ).
#' 
#' @param display_null Chooses whether a grey ribbon should be added to reflect
#'   the null distribution. Note that it can not be displayed when the trend 
#'   line reflects something else than the indicator values (when \code{what} 
#'   is not set to "value").
#' 
#' @param ... Other arguments are ignored.
#' 
#'@rdname simple_sews_methods
#'@method plot simple_sews_test
#'@export
plot.simple_sews_test <- function(x, 
                                  along = NULL,
                                  what = "value", 
                                  display_null = TRUE, 
                                  ...) { 
  NextMethod('plot')
}

#'@method plot simple_sews_test_list
#'@export
plot.simple_sews_test_list <- function(x, 
                                       along = NULL, 
                                       what = "value", 
                                       display_null = TRUE, 
                                       ...) { 
  
  # Give the user some slack 
  what <- tolower(what)
  what <- gsub("-", "_", what) # common mistake with the underscore in z-score
  
  # Transform into data.frame
  x.df <- as.data.frame(x)
  
  # Add the difference to the table 
  if ( what == "difference" ) { 
    x.df[ ,"difference"] <- x.df[ ,"value"] - x.df[ ,"null_mean"]
  }
  
  # If along is not provided, then use the matrixn number
  set_default_xlab <- FALSE 
  if ( is.null(along) ) { 
    along <- unique(x.df[ ,"matrixn"])
    set_default_xlab <- TRUE 
  }
  
  check_suitable_for_plots(x.df, along)
  
  plot_data <- data.frame(as.data.frame(x.df), 
                          gradient = along[x.df[ ,'matrixn']])
  
  # Create base ggplot2 object
  # Create base plot object 
  plot <- ggplot2::ggplot(plot_data) + 
            linescale_spwarnings() + 
            theme_spwarnings() + 
            facet_wrap(~ indic, scales = "free_y")
  
  # Check if we really want to add a null ribbon
  add_null <- display_null
  if ( display_null && ! "null_mean" %in% colnames(x.df) ) { 
    warning('Null data was specified to be displayed but could not be found ', 
            'in the provided object')
    add_null <- FALSE
  }
  if ( display_null && what != "value" ) { 
    warning(
      'Display quantiles for null values cannot be shown when the indicator value is ',
      'not displayed'
    )
    add_null <- FALSE
  }
  
  if ( add_null ) { 
    null_data <- data.frame(plot_data,
                            null_ymin = x.df[ ,'null_qinf'],
                            null_ymax = x.df[ ,'null_qsup'])
    
    plot <- plot + 
      ggplot2::geom_ribbon(ggplot2::aes_string(x = 'gradient',
                                               ymin = 'null_ymin',
                                               ymax = 'null_ymax'),
                           data = null_data, 
                           fill = 'grey',
                           group = 1, 
                           alpha = .8) + 
      ggplot2::geom_line(ggplot2::aes_string(x = "gradient", 
                                             y = "null_mean"), 
                         group = 1, 
                         color = 'black', alpha = .1) 
  }
  
  # Add the trend on the graph (Note that we add it over the null ribbon)
  plot <- plot + 
    ggplot2::geom_point(ggplot2::aes_string(x = 'gradient', 
                                            y = what)) + 
    ggplot2::geom_line(ggplot2::aes_string(x = 'gradient', 
                                           y = what))
  
  # Set ylab  
  ylabel <- "Value"
  plot <- plot + ylab(ylabel)
  
  # Set xlab
  if ( set_default_xlab ) { 
    plot <- plot + 
      ggplot2::xlab('Matrix number') 
  } else { 
    plot <- plot + ggplot2::xlab(as.character(substitute(along)))
  }
  
  return(plot)
}

#' @title Labelling of unique patches and detection of percolation. 
#' 
#' @description Label each patch with a number in a binary matrix 
#' 
#' @param mat A binary matrix
#' 
#' @param nbmask Either "moore" for 8-way neighborhood, "von_neumann" for four-way 
#'   neighborhood (default), or a 3x3 matrix describing which neighbors to 
#'   consider around a cell. See \code{\link{patchsizes}} for details on how to specify 
#'   such neighborhoods.
#' 
#' @param wrap Whether to wrap around lattice boundaries (`TRUE`/`FALSE`), 
#'   effectively using periodic boundaries.
#'
#' @return A matrix containing ID numbers for each connected patch. Default 
#'   parameters assume 4-cell neighborhood and periodic boundaries. The 
#'   distribution of patch sizes is returned as the attribute "psd" and the 
#'   percolation status as "percolation" (whether a TRUE patch has a width 
#'   or height equal to the size of the matrix). 
#' 
#' @details The \code{label} function "labels" the patches of a binary 
#'   (\code{TRUE}/\code{FALSE}) matrix. It returns a matrix of similar height and width, 
#'   with integer values representing the ID of each unique patch (contiguous
#'    cells). Empty cells are labelled as \code{NA}.
#' 
#' @seealso \code{\link{patchsizes}}, \code{\link{patchdistr_sews}}
#' 
#' @examples 
#' 
#' data(forestgap)
#' 
#' rmat <- matrix(rnorm(100) > .1, ncol = 10)
#' display_matrix(label(rmat))
#' 
#' # With 8-way neighborhood mask and no wrapping around borders
#' display_matrix(label(rmat, "moore", wrap = FALSE))
#' 
#' # On real data: 
#' display_matrix(label(forestgap[[5]], "moore", wrap = FALSE))
#' 
#' @export
label <- function(mat, 
                  nbmask = "von_neumann", 
                  wrap = FALSE) {
  
  nbmask <- parse_neighbor_spec(nbmask)
  
  if ( ! is.logical(mat) ) { 
    stop('Labelling of patches requirese a logical matrix',
         '(TRUE/FALSE values): please convert your data first.')
  }
  
  if ( ! is.matrix(mat) ) { 
    stop('The input object must be a matrix')
  }
  
  # The matrix is full
  if ( all(mat) ) { 
    result <- matrix(1, nrow = nrow(mat), ncol = ncol(mat)) 
    attr(result, "psd") <- prod(dim(mat))
    attr(result, "percolation") <- TRUE
    return(result)
  } 
  
  # The matrix is empty
  if ( !any(mat) ) { 
    result <- matrix(NA, nrow = nrow(mat), ncol = ncol(mat)) 
    attr(result, "psd") <- integer(0)
    attr(result, "percolation") <- FALSE
    return(result)
  }
  
  # The matrix is a row/column vector 
  if ( ncol(mat) == 1 || nrow(mat) == 1 ) { 
    vec <- as.vector(mat)
    result <- cumsum( c(vec[1] > 0, diff(vec)) == 1 ) * vec
    result <- ifelse(result > 0, result, NA)
    # If we wrap, then we need to merge the two patches at the end of the vector
    if ( wrap && !is.na(head(result, 1)) && !is.na(tail(result, 1)) ) { 
      result[ result == tail(result, 1) ] <- head(result, 1)
    }
    
    # PSD is the just the number of times each unique values appears in the 
    # result vector. 
    attr(result, "psd") <- tabulate(result)
    # Adjust dimensions
    dim(result) <- dim(mat)
    
    # If there is a patch, then it necessarily has the width or height 
    # of the matrix, so percolation is present. 
    attr(result, "percolation") <- any(mat)
    return(result)
  }
  
  # Otherwise we scan for patches
  label_cpp(mat, nbmask, wrap)
}

#' @rdname label
#'
#' @description \code{percolation()} detects whether percolation occurs in the
#'   matrix (i.e. a patch has a width or a height equal to the size of the 
#'   matrix)
#' 
#' @export
percolation <- function(mat, nbmask = "von_neumann") { 
  
  # We never wrap for percolation, by definition. 
  patches <- label(mat, nbmask, wrap = FALSE)
  return(attr(patches, "percolation"))
}


#' @title Get patch sizes.
#' 
#' @description Get the distribution of patch sizes from a logical matrix
#' 
#' @param mat A logical matrix or a list of such matrices.
#' 
#' @param merge Controls whether the obtained patch size distributions are to 
#'   be pooled together if \code{mat} is a list of matrices. 
#' 
#' @param nbmask Either "moore" for 8-way neighborhood, "von_neumann" for four-way 
#'   neighborhood (default), or a 3x3 matrix describing which neighbors to 
#'   consider around a cell. See \code{\link{patchsizes}} for details on how to specify 
#'   such neighborhoods.
#' 
#' @param wrap Whether to wrap around lattice boundaries (`TRUE`/`FALSE`), 
#'   effectively using periodic boundaries.
#' 
#' @return If mat is a logical matrix, then the function returns a vector of 
#'   patch sizes. If mat is a list of logical matrices, then it returns 
#'   a list of vectors of patch sizes: this list is flattened if merge is TRUE.
#' 
#' @seealso \code{\link{label}}
#' 
#' @examples
#' 
#' data(forestgap)
#' patchsizes(forestgap[[5]]) # Use a single matrix
#' 
#' # Compute the average patch size of each matrix
#' list_patches <- patchsizes(forestgap) # get the patch size for each matrix
#' print( sapply(list_patches, mean)) # print the average patch size 
#' 
#' # Example with 8-way neighborhood
#' patchsizes(forestgap[[5]], nbmask = "moore")
#' 
#' # Same neighborhood as above, but specified in matrix form 
#' moore_nb <- matrix(c(1, 1, 1, 
#'                      1, 0, 1, 
#'                      1, 1, 1), 
#'                    nrow = 3, ncol = 3, byrow = TRUE)
#' patchsizes(forestgap[[5]], nbmask = moore_nb) 
#' 
#' @export
patchsizes <- function(mat, 
                       merge = FALSE,
                       nbmask = "von_neumann", 
                       wrap = FALSE) { 
  
  if ( is.list(mat)) { 
    result <- lapply(mat, patchsizes, merge, nbmask, wrap) 
    if (merge) { 
      # This always works even if there is only one element
      result <- do.call(c, result)
      names(result) <- NULL
    }
    return(result)
  }
  
  if ( ! is.logical(mat) ) { 
    stop('Computing patch-size distributions requires a logical matrix',
         '(TRUE/FALSE values): please convert your data first.')
  }
  
  # We use the label function -> it returns patch sizes as attributes
  map <- label(mat, nbmask, wrap)
  
  return(attr(map, "psd"))
}


parse_neighbor_spec <- function(spec) { 
  
  moore_nb <- matrix(c(1, 1, 1, 
                       1, 0, 1, 
                       1, 1, 1), 
                     nrow = 3, ncol = 3, byrow = TRUE)
  von_neumann <- matrix(c(0, 1, 0, 
                          1, 0, 1, 
                          0, 1, 0), 
                        nrow = 3, ncol = 3, byrow = TRUE)
  
  if ( is.matrix(spec) ) { 
    
    if( nrow(spec) == 3 && ncol(spec) == 3 ) { 
      return(spec)
    } 
  }
  
  if ( is.numeric(spec) && spec == 4 ) { 
    spec <- "von_neumann"
  }
  
  if ( is.numeric(spec) && spec == 8 ) { 
    spec <- "moore"
  }
  
  if ( is.character(spec) && spec == "moore" ) { 
    return(moore_nb)
  }
  
  if ( is.character(spec) && spec == "von_neumann" ) { 
    return(von_neumann)
  }
  
  stop("Unknown neighborhood specification")
}

 
context('Test that patch labelling handles corner cases')


test_that("Patch counting works and handles weird matrices", { 
    
    # Labelling 
    
    # Fail if not a logical matrix
    expect_error(label(diag(10)))
    
    # Fail if not a matrix
    expect_error(label(seq.int(10)))
    
    # Wrapping 
    a <- diag(5) > 0; a[5,1] <- TRUE
    t <- label(a, wrap = FALSE)
    expect_true((t[1,1] != t[5,1]) != t[5,5])
    t <- label(a, wrap = TRUE)
    expect_true((t[1,1] == t[5,1]) == t[5,5])
    
    # Neighboring mask 
    nbm <- label(diag(5) > 0, nbmask = matrix(c(1,1,1,1,0,1,1,1,1), 
                                              ncol = 3, nrow = 3))
    expect_true(unique(na.omit(as.vector(nbm))) == 1)
    
    # Return things
    expect_true( all(is.na(label(diag(10) == 2))) )
    expect_true( unique(as.vector(label(diag(10) < 2))) == 1 )
    
    # Patch counting 
    testlist <- list(diag(10) > 0, diag(10) > 0 )
    expect_true( all(patchsizes(testlist, merge = TRUE) == 1) )
    expect_error( patchsizes(diag(10)) )
    
    # column/row vector patch counting 
    ex <- matrix(seq.int(5) > 2, ncol = 1)
    expect_true(attr(label(ex), "psd") == 3)
    expect_true(attr(label(t(ex)), "psd") == 3)
    expect_true(all(dim(label(ex)) == dim(ex))) # Check that dims are equal
    
    # Non-square matrix counting (width > height)
    ex <- matrix(c(1, 0, 0, 1, 
                   0, 0, 0, 1, 
                   0, 0, 0, 1), byrow = TRUE, ncol = 4) > 0
    test <- label(ex, wrap = TRUE)
    expect_true(test[1,1] == test[2, 4])
    test <- label(ex, wrap = FALSE)
    expect_true(test[1,1] != test[2, 4])
    expect_true(test[1,4] == test[2, 4])
    
    # Non-square matrix counting (height > width)
    ex <- matrix(c(1, 0, 0, 
                   1, 0, 0, 
                   1, 0, 0, 
                   1, 0, 1), byrow = TRUE, ncol = 3) > 0
    test <- label(ex, wrap = TRUE)
    expect_true(test[1,1] == test[4, 3])
    test <- label(ex, wrap = FALSE)
    expect_true(test[1,1] != test[4, 3])
    expect_true(test[4,1] == test[3, 1])
    
    # Non-square matrix counting 
    ex <- matrix(c(1, 1, 1, 1, 
                   0, 0, 0, 1, 
                   0, 0, 0, 1), byrow = TRUE, ncol = 4) > 0
    test <- label(ex)
    expect_true(attr(test, "percolation"))
    
})

# We add the two next lines so we do not export the functions written in this 
# file, unless explicitely specified.

# A function that checks the arguments passed to the indicator functions.
check_mat <- function(mat, NAerror = TRUE) { 
  
  # Not a matrix or something we can handle ? 
  if ( ! is.matrix(mat) ) {
    stop('I don\'t know what to do with an object of class ', class(mat))
  }
  
  # Has NA values
  if ( NAerror && any( is.na(mat) ) ) { 
    warning('NAs in provided matrix.')
  }
  
  # Has only two unique values but it is not a logical matrix
  if ( length(unique(as.vector(mat))) == 2 && (!is.logical(mat) ) ) { 
    warning("The matrix has only two unique values, but it is not of logical ", 
            "type. Did you mean to use TRUE/FALSE values?")
  }
  
  return(TRUE)
}

# Check whether some variables are suited to make plots : used in task_generic
#   and task_spectral sews
check_suitable_for_plots <- function(obj, 
                                     along) { 
  
  if ( ! 'matrixn' %in% colnames(obj) || 
        ( !is.null(along) && length(along) <= 1 ) ) { 
    warning('Trying to display a trend with only one value')
  }
  
  if ( !is.null(along) && max(obj[ ,'matrixn']) != length(along) ) { 
    stop('External data length (along = ...) does not match ',
         'the number of matrices !')
  }
  
}

# 
# 
# This file contains the workflow function for generic EWS
# 
# 
#' @title Generic Spatial Early-Warning signals
#' 
#' @description Computation, significance assessment and display of spatial 
#'   generic early warning signals (Moran's I, variance and skewness)
#' 
#' @param mat A matrix (quantitative data), a binary matrix (which contains 
#'   \code{TRUE} or \code{FALSE} values) 
#'   or a list of those
#' 
#' @param subsize The subsize used for the coarse-graining phase (see Details)
#'   
#' @param abs_skewness Should the absolute skewness be used instead of its 
#'   raw values ? 
#' 
#' @param moranI_coarse_grain Should the input matrix be coarse-grained before
#'   computing the Moran's I indicator value ?
#' 
#' 
#' @return 
#' 
#' \code{generic_sews} returns an object of class \code{simple_sews_single}
#'   (a list) if \code{mat} is a single matrix or an object of class 
#'   \code{simple_sews_list} if \code{mat} is a list. You probably want to use some  
#'   of the methods written for these complicated objects instead of extracting 
#'   values directly (they are displayed using \code{print(<object>)}).
#' 
#' \code{indictest} returns an object of class \code{generic_test} (a \code{data.frame}). 
#' 
#' \code{plot} methods return \code{ggplot} objects, usually immediately displayed 
#'   when \code{plot} is being called at the R prompt. 
#' 
#' @details 
#' 
#' The Generic Early warning signal are based on the property of a 
#'   dynamical system to "slow down" when approaching a critical point, 
#'   that is, take more time to return to equilibrium after a perturbation. This 
#'   is expected to be reflected in several spatial characteristics: the 
#'   variance, the spatial autocorrelation (at lag-1) and the skewness. This 
#'   function provides a convenient workflow to compute these indicators, 
#'   assess their significance and display the results. 
#' 
#' Before computing the actual indicators, the matrix can be "coarse-grained". 
#'   This process reduces the matrix by averaging the nearby cells using 
#'   a square window defined by the \code{subsize} parameter. This makes spatial  
#'   variance and skewness reflect actual spatial patterns when working with 
#'   binary (\code{TRUE}/\code{FALSE} data), but is optional when using 
#'   continuous data. Keep in mind that it effectively reduces the size of 
#'   the matrix by approximately \code{subsize} on each dimension. 
#'   
#' The significance of generic early-warning signals can be estimated by 
#'   reshuffling the original matrix (function \code{indictest}). Indicators 
#'   are then recomputed on the shuffled matrices and the values obtained are 
#'   used as a null distribution. P-values are obtained based on the rank of 
#'   the observed value in the null distribution. A small P-value means 
#'   that the indicator is significantly above the null values, as expected 
#'   before a critical point. 
#'
#' The \code{plot} method can displays the results graphically. A text summary 
#'   can be obtained using the \code{summary} method. 
#' 
#' @references 
#' 
#'   Kefi, S., Guttal, V., Brock, W.A., Carpenter, S.R., Ellison, A.M., 
#'   Livina, V.N., et al. (2014). Early Warning Signals of Ecological 
#'   Transitions: Methods for Spatial Patterns. PLoS ONE, 9, e92097.
#'   
#'   Dakos, V., van Nes, E. H., Donangelo, R., Fort, H., & 
#'   Scheffer, M. (2010). Spatial correlation as leading indicator of 
#'   catastrophic shifts. Theoretical Ecology, 3(3), 163-174.
#'   
#'   Guttal, V., & Jayaprakash, C. (2008). Spatial variance and spatial 
#'   skewness: leading indicators of regime shifts in spatial ecological 
#'   systems. Theoretical Ecology, 2(1), 3-12. 
#'   
#' @seealso 
#'   \code{\link{indictest}}, to test the significance of indicator values. 
#'   Individual indicators: \code{\link{raw_cg_moran}}
#'     \code{\link{raw_cg_variance}}, \code{\link{raw_cg_skewness}}, 
#'     \code{\link{simple_sews}}
#' 
#' @examples
#' 
#' data(serengeti)
#' gen_indic <- generic_sews(serengeti, subsize = 5, 
#'                            moranI_coarse_grain = TRUE)
#' 
#' # Display results
#' summary(gen_indic)
#' 
#' # Display trends along the varying model parameter
#' plot(gen_indic, along = serengeti.rain)
#' 
#' # Compute significance (long)
#' \donttest{
#' gen_test <- indictest(gen_indic, nulln = 199)
#' 
#' print(gen_test)
#' 
#' # Display the trend, now with a grey ribbon indicating the 5%-95% quantile
#' # range of the null distribution
#' plot(gen_test, along = serengeti.rain)
#' 
#' # Display the effect size compared to null distribution 
#' plot(gen_test, along = serengeti.rain, what = "z_score")
#' 
#' # Note that plot() method returns a ggplot object that can be modified
#' # for convenience
#' if ( require(ggplot2) ) { 
#'   plot(gen_test, along = serengeti.rain) + 
#'     geom_vline(xintercept = 733, color = "red", linetype = "dashed") +
#'     xlab('Annual rainfall') + 
#'     theme_minimal()
#' }
#' 
#' }
#' 
#' @export
generic_sews <- function(mat, 
                         subsize = 4, 
                         abs_skewness = FALSE, 
                         moranI_coarse_grain = FALSE) { 
  compute_indicator(mat, raw_generic_indic, 
                    subsize = subsize, 
                    abs_skewness = abs_skewness, 
                    moranI_coarse_grain = moranI_coarse_grain, 
                    taskname = "Generic indicators")
}


# Build the right indicator function (closure) that take into accounts the 
#   above options. 
raw_generic_indic <- function(mat, 
                              subsize, 
                              abs_skewness, 
                              moranI_coarse_grain) { 
  
  # Warn if the matrix is continuous but we will coarse grain anyway
  if ( is.numeric(mat) && subsize > 1 ) { 
    warning(paste("Input matrix has continous values but will be", 
                  "coarse-grained anyway. Set subsize=1 to disable coarse", 
                  "graining."))
  }
  
  if ( is.logical(mat) && subsize == 1 ) { 
    warning(paste("Input matrix is binary but no coarse-graining will be",
                  "performed."))
  }
  
  # We do coarse-graining only once for the whole matrix
  mat_cg <- coarse_grain(mat, subsize)
  
  # Handle the case where there is a single unique value in the matrix
  if ( length(unique(as.vector(mat_cg))) == 1 ) { 
    return( c(variance = var(as.vector(mat_cg)), 
              skewness = NA_real_, 
              moran    = NA_real_, 
              mean     = mean(mat)) )
  }
  
  skewness_value <- cpp_skewness(mat_cg)
  if (abs_skewness) { 
    skewness_value <- abs(skewness_value)
  }
  
  if (moranI_coarse_grain) { 
    moran_value <- raw_moran(mat_cg) 
  } else { 
    moran_value <- raw_moran(mat)
  }
  
  c(variance = var(as.vector(mat_cg)),
    skewness = skewness_value,
    moran    = moran_value,
    mean     = mean(mat))
}


#' @title Spatial variance indicator
#'
#' @description This functions computes the spatial variance of a matrix. 
#' 
#' @param mat A matrix. Its values can be logical, with \code{FALSE} (empty) 
#'   or \code{TRUE} (occupied) values. The entries can also be continuous 
#'   (like NDVI or EVI data). 
#' 
#' @param subsize  Dimension of the submatrix used to coarse-grain the 
#'   original matrix. This must be an integer less than size of the full 
#'   matrix. Coarse-graining reduces the size of the matrix by a factor 
#'   \code{subsize} in each dimension of the matrix. Variance is calculated 
#'   on the coarse-grained matrix. 
#' 
#' @return The variance of the coarse-grained matrix as a named vector 
#' 
#' @details
#' 
#' Spatial variance is a measure of fluctuations in space. Based on the theory 
#' of critical slowing down, when systems approach critical points
#' they are expected to show increased fluctuations in space. Thus, increasing 
#' spatial variance is proposed as an early warning signal of impending 
#' critical transitions. 
#' 
#' Many high resolution spatial data are classified as FALSE (empty) 
#' or TRUE (occupied). In such cases, spatial variance captures just 
#' the variance in data, but not that of spatial structure. 
#' To resolve the issue, this function employs a method called coarse-graining, 
#' proposed in Kefi et al (2014), and described in detail in 
#' Sankaran et al. (2017). One must specify a subsize above one for 
#' binary valued data sets to obtain meaningful values. 
#' 
#' \code{subsize} has to be an integer. It has to be less than or equal to 
#' half of matrix size (N). \code{subsize} must also be preferably a 
#' divisor of N. If it is not a divisor of N, the remainder rows and columns 
#' are discarded when computing coarse-graining matrices. 
#' 
#' Null model evaluations are also done on coarse-grained matrices. 
#' 
#' @references 
#' 
#' Guttal, V., and Jayaprakash, C. (2009). Spatial variance and 
#' spatial skewness: leading indicators of regime shifts in spatial 
#' ecological systems. Theoretical Ecology, 2(1), 3-12.
#' 
#' Kefi, S., Guttal, V., Brock, W.A., Carpenter, S.R., Ellison, A.M., 
#' Livina, V.N., et al. (2014). Early Warning Signals of Ecological 
#' Transitions: Methods for Spatial Patterns. PLoS ONE, 9, e92097.
#' 
#' Sankaran, S., Majumder, S., Kefi, S., and Guttal, V. (2017). Implication 
#' of being discrete and spatial in detecting early warning signals
#' of regime shifts. Ecological Indicators. 
#' 
#' @seealso \code{\link{generic_sews}}
#' 
#' @examples
#' 
#' \donttest{
#' data(serengeti)
#' raw_cg_variance(serengeti[[1]])
#' compute_indicator(serengeti, fun = raw_cg_variance, subsize = 5)
#' }
#' 
#'@export
raw_cg_variance <- function(mat, subsize = 5) { 
  if ( ! is.matrix(mat) ) { 
    stop("raw_cg_variance only accepts a single matrix as input.")
  }
  
  c(variance = var( as.vector( coarse_grain(mat, subsize) ) ))
}


#' @title Moran's Index at lag of 1
#'
#' @description This functions computes the Moran's spatial correlation index 
#'   (with lag one) on a matrix. 
#'
#' @references 
#'
#' Dakos, V., van Nes, E. H., Donangelo, R., Fort, H., & 
#' Scheffer, M. (2010). Spatial correlation as leading indicator of 
#' catastrophic shifts. Theoretical Ecology, 3(3), 163-174.
#'
#' Legendre, P., & Legendre, L. F. J. (2012). Numerical Ecology.
#' Elsevier Science.
#'
#' @param mat A matrix
#' 
#' @param subsize logical. Dimension of the submatrix used to coarse-grain the 
#'   original matrix (set to 1 for no coarse-graining).
#' 
#' @return The Moran's I index measuring autocorrelation at lag 1 as a 
#'   named vector
#'
#' @seealso \code{\link{generic_sews}}
#' 
#' @examples 
#' 
#' \donttest{
#' data(serengeti)
#' raw_cg_moran(serengeti[[1]], subsize = 1)
#' }
#' 
#'@export
raw_cg_moran <- function(mat, subsize = 1) { 
  if ( ! is.matrix(mat) ) { 
    stop("raw_cg_moran only accepts a single matrix as input.")
  }
  c(moran = raw_moran( coarse_grain(mat, subsize) ))
}



#' @title Skewness indicator
#'
#' @description Compute the spatial skewness of spatial data (a matrix).
#'   
#' @param mat A matrix. The matrix 
#'   values can be logical, with \code{FALSE} (empty) or \code{TRUE} (occupied) 
#'   values. The entries can also be continuous (like NDVI or EVI data). 
#' 
#' @param subsize Dimension of the submatrix used to coarse-grain the 
#'   original matrix. This must be an integer less than size of the full 
#'   matrix. Coarse-graining reduces the size of the matrix by a factor 
#'   \code{subsize} in each dimension of the matrix. Skewness is calculated 
#'   on the coarse-grained matrix. 
#' 
#' @param absolute Should the function return the absolute value or raw value 
#'   of skewness ?
#' 
#' @return The spatial skewness of the matrix as a named vector
#' 
#' @details
#' 
#' Spatial skewness is a measure of fluctuations in space; specifically, it 
#' measures if fluctuations are getting biased (skewed) in one direction. Based 
#' on the theory of critical slowing down, when systems approach critical 
#' points they are expected to show increased fluctuations in space. Thus, 
#' increasing spatial skewness is proposed as an early warning signal of 
#' impending critical transitions. 
#' 
#' Computing spatial skewness is straightforward. However, detecting trends of 
#' skewness that correspond to critical slowing down can be tricky, especially 
#' if data come from discrete classification of state variable.
#' 
#' Many high resolution spatial data are classified as FALSE (empty) 
#' or TRUE (occupied by plant). In such cases, spatial skewness captures just 
#' the skewness in data, but not that of spatial structure. 
#' To resolve the issue, this function employs a method called coarse-graining, 
#' proposed in Kefi et al (2014), and described in detail in 
#' Sankaran et al. (2017). One must specify a subsize above one for 
#' binary valued data sets to obtain meaningful values. 
#' 
#' \code{subsize} has to be an integer. It has to be less than or equal to 
#' half of matrix size (N). \code{subsize} must also be preferably a 
#' divisor of N. If it is not a divisor of N, the remainder rows and columns 
#' are discarded when computing coarse-graining matrices. 
#' 
#' Null model evaluations are also done on coarse-grained matrices. 
#' 
#' @references 
#' 
#' Guttal, V., and Jayaprakash, C. (2009). Spatial variance and 
#' spatial skewness: leading indicators of regime shifts in spatial 
#' ecological systems. Theoretical Ecology, 2(1), 3-12.
#' 
#' Kefi, S., Guttal, V., Brock, W.A., Carpenter, S.R., Ellison, A.M., 
#' Livina, V.N., et al. (2014). Early Warning Signals of Ecological 
#' Transitions: Methods for Spatial Patterns. PLoS ONE, 9, e92097.
#' 
#' Sankaran, S., Majumder, S., Kefi, S., and Guttal, V. (2017). Implication of 
#' being discrete and spatial in detecting early warning signals of regime 
#' shifts. Ecological indicators. 
#' 
#' @seealso \code{\link{generic_sews}}
#' 
#' @examples 
#' 
#' data(serengeti)
#' \donttest{
#' raw_cg_skewness(serengeti[[1]])
#' compute_indicator(serengeti, fun = raw_cg_skewness, subsize = 5)
#' }
#' 
#'@export
raw_cg_skewness <- function(mat, subsize = 5, absolute = TRUE) { 
  if ( ! is.matrix(mat) ) { 
    stop("raw_cg_skewness only accepts a single matrix as input.")
  }
  
  a <- cpp_skewness( coarse_grain(mat, subsize) )
  if (absolute) { 
    a <- abs(a) 
  } 
  c(skewness = a)
}

# 
# This file tests whether the indicators are producing correct values
# 

data(forestgap)


testmat <- forestgap[[3]]
subsizes <- c(2, 4, 7)


context('Test that all indicator functions return correct values')

if ( exists("EXTENDED_TESTS") ) { 
  
  # Test variance indicator
  test_that("Indicator variance returns correct values", { 
    skip_on_cran()

    varf <- function(mat) var(as.vector(mat))
    
    expect_true(abs(varf(testmat) - 
                    raw_cg_variance(testmat, subsize = 1)) < 0.001)
    
    for (subsize in subsizes) { 
      expect_true( abs(varf(coarse_grain(testmat, subsize = subsize)) - 
                       raw_cg_variance(testmat, subsize = subsize)) < 0.001 )
    }
  })




  test_that("Indicator skewness returns correct values", { 
    skip_on_cran()

    skewf <- function(mat) moments::skewness(as.vector(mat))
    
    expect_true(abs(skewf(testmat) - 
                      raw_cg_skewness(testmat, subsize = 1, 
                                      absolute = FALSE)) < 0.001 )
    
    for (subsize in subsizes) { 
      expect_true( abs(skewf(coarse_grain(testmat, subsize = subsize)) - 
                       raw_cg_skewness(testmat, subsize = subsize, 
                                       absolute = FALSE)) < 0.001 ) # adjust for test
    }
  })




  test_that('Indicator Moran returns correct values', { 
    skip_on_cran()

    # Test first the raw function
    
    # Moran's I should be -1 for checkboard pattern
    checkerboard <- matrix(c(0,1), byrow = TRUE, nrow = 1000, ncol = 1001)
    expect_true( abs(raw_cg_moran(checkerboard) - (-1)) < 0.01 )
    
    # Moran's I should be zero for random matrix
    random <- matrix(rbinom(1e6, 1, .5), nrow = 1e3, ncol = 1e3)
    expect_true(abs(raw_cg_moran(random) - 0) < 1e-2)
    
    # Moran's I should be one for perfectly segregated matrix
    split <- cbind(matrix(1, ncol = 500, nrow = 1000), 
                   matrix(0, ncol = 500, nrow = 1000))
    expect_true(abs(raw_cg_moran(split) - 1) < 1e-2)
    
  })
  
  
  
  test_that('Indicator psdtype returns correct values', { 
    skip_on_cran()

    # Test one fit on the whole dataset
    expect_true(with(indicator_psdtype(forestgap, merge = TRUE), 
                     as.character(type[best])) == "tpl") 
    
    # Test individual fits
    if ( exists("EXTENDED_TESTS") ) { 
      a <- sapply(indicator_psdtype(forestgap, best_by = 'BIC'), 
                  with, as.character(type[best]))
      expect_true(all(c(is.na(a[1]), 
                        a[-1] == c("pl", "pl", "pl", "tpl", "tpl", "tpl", 
                                  "tpl", "exp"))))
    }
  })
  
  
  
  test_that('Indicator plrange returns correct values', { 
    skip_on_cran()
    
    # Test that workflow function and individual function return the same thing
    indiv_ic_plrange <- indicator_plrange(forestgap, merge = TRUE)$plrange
    workflow_plrange <- patchdistr_sews(forestgap, merge = TRUE)$plrange$plrange
    
    expect_true( abs(indiv_ic_plrange - 0.8153096) < 0.001 )
    expect_true( abs(workflow_plrange - 0.8153096) < 0.001 )
    expect_true( abs(workflow_plrange - indiv_ic_plrange) < 0.001 )
    
  })




  test_that('Generic indicator task function returns correct values', { 
    skip_on_cran()
    
    # Parameters
    size <- 4
    moran_do_cg <- FALSE
    
    genindic_result <- generic_sews(testmat, 
                                    subsize = size, 
                                    moranI_coarse_grain = moran_do_cg)
    
    # Moran
    expect_true( abs(as.data.frame(genindic_result, wide = TRUE)$moran - 
                     raw_cg_moran(testmat, subsize = 1)) < 0.001 )
    
    # Skewness
    expect_true( abs(as.data.frame(genindic_result, wide = TRUE)$skewness - 
                     raw_cg_skewness(testmat, subsize = size, 
                                     absolute = FALSE)) < 0.001 ) 
    
    # Variances
    expect_true( abs(as.data.frame(genindic_result, wide = TRUE)$variance - 
                     raw_cg_variance(testmat, subsize = size)) < 0.001 )
    
  })
}

# 
# 
# 

context("Test that matrix sizes are being correctly reported")

test_that("Correct sizes are reported", { 
  
  a <- generic_sews(serengeti)
  expect_true( any(grepl("size: 250x250", capture.output(print(a)))) )
  b <- indictest(a, 3)
  expect_true( any(grepl("size: 250x250", capture.output(print(b)))) )
  
  # List of matrices with different sizes
  a <- suppressWarnings({ generic_sews(arizona) })
  expect_true( any(grepl("(variable sizes)", capture.output(print(a)))) )
  b <- indictest(a, 3)
  expect_true( any(grepl("(variable sizes)", capture.output(print(b)))) )
  
})

# 
# This function makes sure parallelism is enabled 
#

context("Test that parallel computing works")

test_that("Parallelism work", { 
  
  if ( exists("EXTENDED_TESTS") && 
       EXTENDED_TESTS && 
       availableCores() > 1 ) { 
    # We increase the dataset size because otherwise the overhead of setting up workers
    # may make the parallel verison slower overall
    a <- generic_sews( c(forestgap, forestgap) )
    
    plan(sequential)
    b.1 <- system.time( indictest(a, 49) ) 
    
    plan(multisession)
    b.2 <- system.time( indictest(a, 49) ) 
    
    expect_true( b.1["elapsed"] > b.2["elapsed"] )
    
    plan(sequential) # restore plan to no-parallelism
  } else { 
    # This is a dummy test so that testhat does not report an empty test
    expect_true(TRUE)
  }
  
})

# 
# 
# This file contains code that tests the generic indicator result against 
#   the published results in PLOS One (Kéfi et al. 2014)
# 

context("Test that results match those in PLOS One")

test_that('results matches those in PLOS One', { 
  
  if ( requireNamespace("moments", quietly = TRUE) ) { 
    
    datdir <- './plosone/' # mind the trailing /
    source(paste0(datdir,'early_warning_generic_R_code.R'), chdir = TRUE)
    
    # This creates a whole bunch of variables, among which : 
    # mean_reduced, var_reduced, cvar_reduced, skew_reduced that contains 
    #   indicator values after a 10x10 coarse-graining
    # 
    # We redo the same our way and compare the results
    
    cgsize <- 10
    
    # Read the data first
    fulldat <- read.table(paste0(datdir, 'CA_all.txt'))
    dataplos <- data.matrix(fulldat)
    nreplicates <- nrow(dataplos) / ncol(dataplos)
    
    # Extract the matrices from the binary data and conver it
    startends <- data.frame(start = (seq.int(nreplicates)-1)*ncol(dataplos)+1,
                            end   = seq.int(nreplicates)*ncol(dataplos))
    
    matrices <- plyr::dlply(startends, ~ start + end, 
                            function(df) { 
                              dataplos[seq.int(df[['start']], df[['end']]), ]
                            })
    matrices <- lapply(matrices, function(x) x == 1) # veg is 1
    
    # Now compute indicators
    test_results  <- generic_sews(matrices, subsize = 10, 
                                  moranI_coarse_grain = TRUE)
    test_reshaped <- ddply(as.data.frame(test_results), ~ matrixn, 
                           function(df) { 
      a <- as.list(df[ ,"value"])
      names(a) <- df[ ,"indic"]
      a <- a[c("mean", "moran", "skewness", "variance")]
      as.data.frame(a)
    })
    test_reshaped <- as.matrix(test_reshaped[ ,-1])
    
    # Now test for concordance
    ref_results <- cbind(mean = mean_reduced,  # ! order matters !
                         corr = corr_reduced, 
                         skew = skew_reduced, 
                         var  = var_reduced)
    
    # Mean skew var are expected to map 1:1
    expect_true(all(abs(ref_results[ ,"mean"] - test_reshaped[ ,"mean"]) < 1e-10))
    expect_true(all(abs(ref_results[ ,"skew"] - test_reshaped[ ,"skewness"]) < 1e-10))
    expect_true(all(abs(ref_results[ ,"var"] - test_reshaped[ ,"variance"]) < 1e-10))
    
    # Moran correlation is computed differently in spw, so resuults do not have to 
    # match exactly 
    expect_true(cor(ref_results[ ,"corr"], test_reshaped[ ,"moran"]) > 0.8)
    
  }
  
})

# 
#' @title Matrix coarse-graining
#' 
#' @description This function averages the spatial data locally. It divides 
#'   the input matrix into submatrices of dimension \code{subsize} and 
#'   averages the spatial data in these submatrices. By doing this, the 
#'   dimension of resultant matrix is reduced by a factor of 
#'   \code{subsize}. 
#' 
#' @details If the data is classified into discrete units, the calculation of 
#' variance and skewness can give spurious results irrelevant to the proximity 
#' to transition. Therefore, discrete data should be 'coarse-grained' before 
#' calculating the spatial early warning signals. However, this can also be 
#' applied to continuous state data. 
#' 
#' @references 
#' 
#'   Sankaran, S., Majumder, S., Kefi, S. and Guttal, V. (2017). Implications 
#'   of being discrete and spatial for detecting early warning signals of 
#'   regime shifts. Ecological Indicators. 
#' 
#' @param mat A matrix
#' 
#' @param subsize Dimension of the submatrix. This has to be a positive 
#' integer smaller than the dimension of input matrix.  
#' 
#' @return A matrix of reduced dimension.
#' 
#' @seealso \code{\link{generic_sews}}
#' 
#' @examples 
#' rmat <- matrix(runif(20*10) > .5, 
#'                ncol = 20, nrow = 10)
#' rmat.cg <- coarse_grain(rmat, subsize = 2)
#' 
#' par(mfrow = c(1, 2))
#' image(rmat)
#' title('Raw matrix') 
#' image(rmat.cg) 
#' title('Coarse-grained matrix')
#' 
#'@export
coarse_grain <- function(mat, subsize) { 
  
  if ( subsize < 1 ) { 
    warning('Cannot coarse-grain a matrix with a subsize argument under 1, ', 
            'returning the matrix unchanged')
    return(mat)
  }
  
  # CG with length == 1 is no CG at all ! 
  if ( subsize == 1) { 
    return(mat)
  }
  
  coarse_grain_cpp(mat, subsize)
  
}
  

# 
# Test whether coarse-graining works
# 

context('Test the coarse-graining function') 

testmat4x4 <- matrix(c(1, 1, 0, 0, 
                       1, 1, 0, 0,
                       0, 0, 1, 1,
                       0, 0, 1, 1), 
                       byrow = TRUE, ncol = 4)

checkerboard <- matrix(c(0,1), byrow = TRUE, nrow = 1000, ncol = 1001)


test_that("Coarse-graining works", { 
  
  expect_equal(coarse_grain(testmat4x4, 2), 
               matrix(c(1, 0, 
                        0, 1), ncol = 2, byrow = TRUE))
  
  expect_equal(unique(as.vector(coarse_grain(checkerboard, subsize = 2))), 
               0.5)
  
  expect_warning(coarse_grain(checkerboard, subsize = 0))
  
  for (size in c(1, 3, 10, 100, 1000)) { 
    for ( subsize in c(1, 2, 3, 4, 5) ) { 
      testdiag <- diag(size)
      ref <- diag(floor(size/subsize)) / subsize
      expect_equal(coarse_grain(testdiag, subsize), ref)
    }
  }
  
})

test_that("CG handles negative values", { 
  expect_true( all(is.numeric( coarse_grain(matrix(runif(100, -10, -5), 100, 100), 
                                        subsize = 4) )) )
})

# 
# 
# This file contains a function that compute, assess and display results from 
#   spectrum-based indicators: reddening of powerspectrum, spectrum density
#   ratio.
# 



#' @title Spectrum-based spatial early-warning signals. 
#' 
#' @description Computation of spatial early warning signals based on spectral
#'   properties.
#' 
#' @param mat The input matrix or a list of matrices. 
#' 
#' @param sdr_low_range The range of values (in proportion) to 
#'   use for the computation of the spectral density ratio.
#'   For example, for the lowest 20\% (default value), set \code{sdr_low_range} to 
#'   \code{c(0, .2)}. See also the Details section. 
#'  
#' @param sdr_high_range The range of values (in proportion) to 
#'   use for the computation of the spectral density ratio. For example, for 
#'   the higher 20\% (default value), set \code{sdr_high_range} to 
#'   \code{c(.8, 1)}. See also the Details section. 
#' 
#' @param quiet Do not display some warnings
#' 
#' @return 
#' 
#' Function \code{spectral_sews} object of class \code{spectral_sews_list} or 
#'   \code{spectral_sews_single} depending on whether the input was a list of 
#'   matrices or a single matrix. 
#' 
#' Function \code{indictest} 
#' 
#' The \code{plot} methods returns a ggplot object (usually displayed 
#' immediately when called interactively). 
#' 
#' @details Spectral early warning signals are based on the fact that some 
#'   dynamical systems can exhibit an change in specific characteristics of their 
#'   spatial structure when approaching a transition. In particular, long-range 
#'   correlations are expected to have an increased importance. This is expected 
#'   to be reflected in the spectrum of the spatial structure as an increase of the
#'   relative importance of lower frequencies over higher frequencies ("reddening" 
#'   of the spatial spectrum).
#'   
#' This task allows computing the radial-spectrum which gives the relative 
#'   importance of each space scale as a function of distance, from 1 to 
#'   \code{N/2} (\code{N} being the minimum between the number of rows and columns). 
#'   If the matrix is not square, then it is cropped to biggest square that 
#'   fits within the left side of the matrix. 
#' 
#' Additionally, it summarizes this spectrum into a Spectral 
#'   Density Ratio (SDR), which is the ratio of low frequencies over 
#'   high frequencies of the r-spectrum. The SDR value is expected to increase
#'   before a transition point.
#' 
#' The significance of spectral early-warning signals can be estimated by 
#'   reshuffling the original matrix (function \code{indictest}). Indicators 
#'   are then recomputed on the shuffled matrices and the values obtained are 
#'   used as a null distribution. P-values are obtained based on the rank of 
#'   the observed value in the null distribution. 
#' 
#' The trend of SDR values can be plotted using the \code{plot()} method. 
#'   Alternatively, the spectrum itself can be plotted (with facets 
#'   if multiple input matrices were used) using the \code{plot_spectrum} 
#'   method.
#' 
#' 
#' @references 
#' 
#'   Kefi, S., Guttal, V., Brock, W.A., Carpenter, S.R., Ellison, A.M., 
#'   Livina, V.N., et al. (2014). Early Warning Signals of Ecological 
#'   Transitions: Methods for Spatial Patterns. PLoS ONE, 9, e92097.
#' 
#' @seealso \code{\link{rspectrum}}, \code{\link{plot_spectrum}}, 
#'   \code{\link{raw_sdr}}, \code{\link{extract_spectrum}}
#' 
#' @seealso 
#'   \code{\link{indictest}}, to test the significance of indicator values. 
#' 
#' @examples
#' \donttest{
#' 
#' data(serengeti) 
#' data(serengeti.rain) 
#' 
#' 
#' spec_indic <- spectral_sews(serengeti, 
#'                             sdr_low_range  = c(0, .2), 
#'                             sdr_high_range = c(.8, 1))
#' 
#' summary(spec_indic)
#' 
#' # Display trends along the varying model parameter
#' plot(spec_indic, along = serengeti.rain)
#' 
#' # Computing spectra many times is expensive, consider setting parallel 
#' # computing using: options(mc.cores = n)
#' 
#' # Assess significance
#' spec_test <- indictest(spec_indic, nulln = 199)
#' summary(spec_test)
#' 
#' # Display the SDR trend, now with a grey ribbon representing 5%-95% 
#' # quantiles of the null distribution
#' plot(spec_test, along = serengeti.rain)
#' 
#' # Add a line highlighting the shift 
#' if (require(ggplot2)) {
#'   plot(spec_test, along = serengeti.rain) + 
#'     geom_vline(xintercept = 590, color = "red", linetype = "dashed")
#' }
#' 
#' 
#' # Display radial-spectra
#' plot_spectrum(spec_indic, along = serengeti.rain)
#' 
#' # Graphics can be modified using ggplot2 functions
#' if (require(ggplot2)) { 
#'   plot_spectrum(spec_indic, along = serengeti.rain) + 
#'     scale_y_log10()
#' }
#' 
#' }
#' @export
spectral_sews <- function(mat, 
                          sdr_low_range  = NULL, 
                          sdr_high_range = NULL, 
                          quiet = FALSE) { 
  
  if ( is.null(sdr_low_range) ) { 
    if ( !quiet ) { 
      warning("Choosing the 20% lowest frequencies for spectral density ratio ",
              "as no range was specified. Use parameter sdr_low_range to ", 
              "choose a different value.")
    }
    sdr_low_range <- c(0, .2)
  }
  
  if ( is.null(sdr_high_range) ) { 
    if ( !quiet ) { 
      warning("Choosing the 20% highest frequencies for spectral density ", 
              "ratio as no range was specified. Use parameter sdr_high_range ", 
              "to choose a different value.")
    }
    sdr_high_range <- c(.8, 1)
  }
  
  # Handle list case
  if ( is.list(mat) ) { 
    results <- lapply(mat, spectral_sews, sdr_low_range, 
                      sdr_high_range, quiet)
    names(results) <- names(mat)
    class(results) <- c('spectral_sews_list',  
                        'spectral_sews', 
                        'simple_sews_list', 
                        'simple_sews', 
                        'sews_result_list')
    
    return(results)
  }
  
  # Convert object to matrix and check if it is suitable for spatialwarnings
  mat <- convert_to_matrix(mat)
  check_mat(mat)
  
  # Now the mat is always a matrix -> process it
  # Check and warn if not square
  if ( nrow(mat) != ncol(mat) ) { 
    if ( !quiet ) { 
      warning('The matrix is not square: only a squared subset of the left ", 
              "part will be used.')
    }
    
    mindim <- min(dim(mat))
    mat <- mat[seq.int(mindim), seq.int(mindim)]
  }
  
  # Compute powerspectrum
  spectrum <- rspectrum(mat)
  
  # Compute SDR
  ranges_absolute <- convert_ranges_to_absolute(mat, sdr_low_range, 
                                                sdr_high_range)
  
  sdr_value <- indicator_sdr_do_ratio(spectrum, 
                                      ranges_absolute[["low"]], 
                                      ranges_absolute[["high"]])
  
  # Return list containing both
  output <- list(value = c(sdr = sdr_value), 
                 spectrum = spectrum, 
                 orig_data = mat, 
                 low_range = ranges_absolute[['low']], 
                 high_range = ranges_absolute[['high']], 
                 taskname = "Spectrum-based indicators")
  class(output) <- c('spectral_sews_single', 
                     'spectral_sews', 
                     'simple_sews_single', 
                     'simple_sews', 
                     'sews_result_single')
  
  return(output)
}

# 
#' @title Spectral Density Ratio (SDR) indicator
#' 
#' @description Compute the ratio of low frequencies over high frequencies
#'   of the r-spectrum. 
#' 
#' @param mat A matrix with continuous values, or a logical matrix 
#'   (\code{TRUE}/\code{FALSE} values). 
#' 
#' @param sdr_low_range The range of values (in proportion) to 
#'   use for the computation of the spectral density ratio.
#'   For example, for the lowest 20\% (default value), set \code{sdr_low_range}
#'   to \code{c(0, .2)}.
#'  
#' @param sdr_high_range The range of values (in proportion) to 
#'   use for the computation of the spectral density ratio. For example, for 
#'   the highest 20\% (default value), set \code{sdr_high_range} to 
#'   \code{c(.8, 1)}. 
#' 
#' @return The SDR values computed on the matrix as a named vector
#' 
#' @details 
#' 
#' SDR measures the increase in long-range correlations before a critical point. 
#'   It is the ratio of the average low frequency value over high frequency 
#'   values. In this implementation, an increase in SDR implies a "reddening" 
#'   of the \link[=rspectrum]{r-spectrum}. See also \code{\link{spectral_sews}} for 
#'   a more complete description. 
#' 
#' Low and high frequencies are averaged in order to compute the SDR. The 
#'   parameters \code{sdr_low_range} and \code{sdr_high_range} control which 
#'   frequencies are selected for averaging. For example 
#'   \code{sdr_low_range = c(0, .2)} (default) uses the lower 20% to compute 
#'   the average of low frequencies. \code{sdr_high_range = c(.8, 1)} uses the 
#'   higher 20% for the average of high frequencies. 
#' 
#' @seealso \code{\link{indictest}}, 
#'   \code{\link{rspectrum}}, \code{\link{plot_spectrum}}, 
#'   \code{\link{spectral_sews}}, \code{\link{extract_spectrum}}
#' 
#' @references 
#' 
#' Carpenter, S.R. & Brock, W.A. (2010). Early warnings of regime shifts in 
#'   spatial dynamics using the discrete Fourier transform. Ecosphere
#' 
#' @examples 
#' 
#' \donttest{ 
#' data(serengeti)
#' serengeti.sdr <- raw_sdr(serengeti[[1]], 
#'                          sdr_low_range = c(0, 0.2), 
#'                          sdr_high_range = c(0.8, 1))
#' compute_indicator(serengeti, raw_sdr)
#' }
#' 
#' @export
raw_sdr <- function(mat, 
                    sdr_low_range  = NULL, 
                    sdr_high_range = NULL) { 
  
  check_mat(mat) # checks if sensible
  
  if ( is.null(sdr_low_range) ) { 
    warning("Choosing the 20% lowest frequencies for spectral density ratio ",
            "as none was specified. Use parameter sdr_low_range to choose ", 
            "a different value.")
    sdr_low_range <- c(0, .2)
  }
  
  if ( is.null(sdr_high_range) ) { 
    warning("Choosing the 20% highest frequencies for spectral density ratio ",
            "as none was specified. Use parameter sdr_high_range to choose ", 
            "a different value.")
    sdr_high_range <- c(.8, 1)
  }
  
  ranges_absolute <- convert_ranges_to_absolute(mat, sdr_low_range, 
                                                sdr_high_range)
  
  c(sdr = indicator_sdr_core(mat, 
                             ranges_absolute[["low"]],
                             ranges_absolute[["high"]]))
}

indicator_sdr_core <- function(mat, low_range, high_range) { 
  
  # Compute r-spectrum
  spectrum <- rspectrum(mat)
  
  # Compute ratio
  return( indicator_sdr_do_ratio(spectrum, low_range, high_range) )
}

indicator_sdr_do_ratio <- function(spectrum, low_range, high_range) { 
  
  # Compute subsets
  low_subset  <- with(spectrum, dist <= max(low_range)  & 
                                  dist >= min(low_range))
  high_subset <- with(spectrum, dist <= max(high_range) & 
                                  dist >= min(high_range))
  
  # If the number of values to estimate means is very low, then warn. 
  if ( sum(low_subset) < 3 || sum(high_subset) < 3 ) { 
    warning('The number of values used to compute the SDR ratio is very low ', 
            'it may be unreliable')
  }
  
  # Return ratio of means
  with(spectrum, mean(rspec[low_subset]) / mean(rspec[high_subset])) 
}

# Convert ranges from proportional values to absolute distance values
convert_ranges_to_absolute <- function(mat, 
                                       sdr_low_range  = NULL, 
                                       sdr_high_range = NULL) { 
    
  maxdist <- 1 + floor(min(dim(mat)) / 2)
  low_range_absolute  <- sdr_low_range * maxdist
  high_range_absolute <- sdr_high_range * maxdist
  
  return( list(low = low_range_absolute, 
               high = high_range_absolute) )
}

#'@export
indictest.patchdistr_sews_single <- function(x, 
                                             nulln = 999, 
                                             null_method = 'perm', 
                                             null_control = NULL, 
                                             ...) { 
  
  null_control <- null_control_set_args(x[["orig_data"]], null_control, 
                                        null_method)
  
  # Obtain null distributions + plrange values. We compute both at the same 
  # time so we do not reshuffle matrices twice. 
  null_results <- generate_nulls(x[["orig_data"]], 
                                 nulln = nulln, 
                                 null_method = null_method, 
                                 null_control = null_control, 
                                 function(nullmat) { 
      list(plrange = raw_plrange(nullmat), 
           psd = patchsizes(nullmat))
  })
  
  # Extract plrange from the above computation all the unique patch sizes, 
  # then compute P-values, etc. 
  # We only report a p-value if the observed plrange is all right. 
  plrange_null_distr <- laply(null_results[["nulldistr"]], 
                              function(o) o[["plrange"]])
  
  if ( is.na(x[["plrange"]][["plrange"]]) ) { 
    plrange_pval <- NA_real_
  } else { 
    # Compute the actual P-value
    plrange_pval <- 
      rank(c(x[["plrange"]][["plrange"]], plrange_null_distr))[1] / (1 + nulln)
    plrange_pval <- 1 - plrange_pval
  }
  
  # Get a vector of x values at which to evalue the null distribution 
  max_null_patch_size <- laply(null_results[["nulldistr"]], 
                               function(o) max(o[["psd"]]))
  max_null_patch_size <- max(max_null_patch_size)
  xpsd <- exp( seq(log(1), to = log(max_null_patch_size), 
                   length.out = 512))
  xpsd <- unique(round(xpsd))
  
  # Extract null distributions and get quantiles of the inverse cumulative 
  # null distribution. 
  cumpsd_null_distr <- ldply(null_results[["nulldistr"]], function(o) { 
    cumpsd(o[["psd"]], x = xpsd)
  })
  cumpsd_null_distr <- ddply(cumpsd_null_distr, ~ patchsize, function(df) { 
    data.frame(qinf = quantile(df[ ,"y"], null_control[["qinf"]]), 
               qsup = quantile(df[ ,"y"], null_control[["qsup"]]), 
               median = median(df[ ,"y"]), 
               mean   = mean(df[ ,"y"]))
  })
  
  ans <- c(x, 
           cumpsd_null = list(cumpsd_null_distr), 
           plrange_pval = plrange_pval, 
           nulln = nulln, 
           null_method = null_method, 
           null_control = list(null_control))
  class(ans) <- c('patchdistr_sews_test_single', 
                  'patchdistr_sews_single', 
                  'sews_result_single', 
                  'sews_test')
  return(ans)
}

#'@export
indictest.patchdistr_sews_list <- function(x, 
                                           nulln = 999, 
                                           null_method = 'perm', 
                                           null_control = NULL, 
                                           ...) { 
  
  # Compute a distribution of null values for SDR
  results <- future_lapply_seed(x, indictest.patchdistr_sews_single, 
                                         nulln, null_method, null_control, ...)
  
  # Transfer names 
  names(results) <- names(x)
  
  # Format and return output
  class(results) <- c('patchdistr_sews_test_list', 
                      'patchdistr_sews_list', 
                      'sews_result_list', 
                      'sews_test')
  
  return(results)
  
}


# Print/Summary/etc. methods, which actually reuse most of the code from 
# the non-indictest version
#'@export
print.patchdistr_sews_test_list <- function(x, ...) { 
  summary.patchdistr_sews_test_list(x, ...)
}

#'@export
print.patchdistr_sews_test_single <- function(x, ...) { 
  summary.patchdistr_sews_test_single(x, ...)
}

#'@export
summary.patchdistr_sews_test_list <- function(object, ...) { 
  summary.patchdistr_sews_list(object, ...)
}

#'@export
summary.patchdistr_sews_test_single <- function(object, ...) { 
  summary.patchdistr_sews_single(object, ...)
}

# Convert each element to a data frame, and a column with the matrixn number
#'@export
as.data.frame.patchdistr_sews_test_list <- function(x, ...) { 
  newdat <- llply(x, as.data.frame.patchdistr_sews_test_single)
  newdat <- Map(function(n, o) data.frame(matrixn = n, o), 
                seq_along(newdat), newdat)
  do.call(rbind, newdat)
}
#'@export
as.data.frame.patchdistr_sews_test_single <- function(x, ...) { 
  # Do the same as non-test version, but add the pvalue for the plrange
  dat <- as.data.frame.patchdistr_sews_single(x)
  data.frame(dat, plrpval = x[["plrange_pval"]])
}

#'@export
plot_distr.patchdistr_sews_test_list <- function(x, 
                                                 along = NULL, 
                                                 best_only = TRUE, 
                                                 plrange = TRUE) { 
  
  gplot <- plot_distr.patchdistr_sews_list(x, along, best_only, plrange)
  
  # Add the null data to that plot
  nulldat <- Map(function(n, o) data.frame(matrixn = n, o[['cumpsd_null']]), 
                 seq_along(x), x)
  nulldat <- do.call(rbind, nulldat)
  
  # Create the `along` values in the table
  nulldat[ ,"along"] <- nulldat[ ,'matrixn']
  if ( ! is.null(along) ) { 
    nulldat[ ,'along'] <- along[nulldat[ ,'matrixn']]
  }
  
  # NOTE: we add layers this way to the ggplot object, so null values appear 
  # below the observed ones
  gplot$layers <- c(geom_ribbon(aes_q(x = ~patchsize, 
                                      ymin = ~qinf, 
                                      ymax = ~qsup, 
                                      group = ~matrixn), 
                                data = nulldat, alpha = .2), 
                    gplot$layers)
  
  return(gplot)
}

#'@export
plot_distr.patchdistr_sews_test_single <- function(x, 
                                                   along = NULL, 
                                                   best_only = TRUE, 
                                                   plrange = TRUE) { 
  
  gplot <- plot_distr.patchdistr_sews_single(x, along, best_only, plrange)
  
  # Add the null data to that plot. We modify the layers so that the ribbon 
  # appears underneath the observed values
  gplot$layers <- c(geom_ribbon(aes_q(x = ~patchsize, 
                                      ymin = ~qinf, 
                                      ymax = ~qsup), 
                                data = x[['cumpsd_null']], alpha = .2), 
                    gplot$layers)

  return(gplot)
}

# 
# Indictest is a method for testing the significance of spatial indicators 
# based on shuffling the original matrix. 
# 
# This file only defines the global method and maybe some helpers. For each 
#   method, please refer to the task_* files. 

#' @title Significance-assessment of spatial early-warning signals
#' 
#' @description Assess the significance of spatial early-warning indicators
#' 
#' @param x An object such as one produced by the \code{*_sews}
#'   functions, or \code{\link{compute_indicator}}
#' 
#' @param nulln The number of values to produce the null distribution 
#' 
#' @param null_method The method used to produce the null values (see Details)
#' 
#' @param null_control List of arguments used to control the generation of 
#'   null matrices. If NULL, then sensible defaults are chosen (see Details)
#' 
#' @param ... Additional arguments are ignored
#' 
#' @return An object with a class ending in \code{*_sews_test}, whose exact
#'   class depends on the input object. \code{plot}, \code{summary} methods are
#'   available to display the results of computations, and additional methods
#'   may be available depending on the input object (e.g. see
#'   \code{\link{patchdistr_sews_plot}}). 
#' 
#' @details 
#' 
#' \code{indictest} is used to test the significance of early-warning signals
#'   against 'null matrices', which represent the expected spatial structure 
#'   in the absence of the biological process of interest. 
#' 
#' For a given indicator, a null distribution is obtained by producing a set 
#'   of 'null' matrices, from which indicator values are recomputed. This
#'   produces a null distribution of \code{nulln} indicator values against
#'   which the observed value is tested. 
#' 
#' Several methods are available to produce the set of null matrices. If 
#'   \code{null_method} is set to "perm", the original matrix is reshuffled 
#'   to obtain a null matrix. If \code{null_method} is set to "intercept", then 
#'   a generalized linear model of the form `y ~ 1` (where y represents the 
#'   values of the matrix) is fitted, then values are drawn from this model. If 
#'   \code{null_method} is set to "smooth", then a smooth surface is fitted
#'   based on a generalized additive model (using \code{\link[mgcv]{gam}}) to
#'   the matrix, then values are drawn from this model. When using the
#'   "intercept" or "smooth" null models, it is important to make sure the
#'   model 'family' corresponds to the type of values present in the matrix. By
#'   default, if a matrix contains \code{TRUE}/\code{FALSE} values, a `binomial()` 
#'   family is used, otherwise a `gaussian()` family is used. More information about 
#'   null models is available in the \href{https://alex.lecairn.org/spatialwarnings-faq.html#Using_advanced_null_models}{spatialwarnings FAQ}. 
#' 
#' Please note that specific null methods may exists for some indicators, such as
#' \code{\link[=flowlength_sews]{flowlength}}. These are often based on 
#' analytical approximation and allow faster computations. 
#' 
#' If a matrix has attributes, then these are preserved and passed to the 
#'   function used to compute the indicator value, except when using the 
#'   null method 'perm', in which case matrix attributes are discarded. 
#' 
#' The list \code{null_control} can be used to adjust the computation of 
#'   null matrices. It can have the following components: 
#'   \describe{ 
#'     \item{\code{family}}{The family used in the model used to produce the null 
#'       matrices. Typically, it is one of \code{\link[=stats]{binomial}()}, 
#'       \code{\link[=stats]{gaussian}()}, etc. }
#' 
#'     \item{\code{qinf}}{The lower quantile to compute from the null distribution 
#'       and display in summaries/plots. A numeric value between 0 and 1.}
#' 
#'     \item{\code{qsup}}{The upper quantile to compute from the null distribution 
#'       and display in summaries/plots. A numeric value between 0 and 1.}
#'   }
#' 
#' @seealso \code{\link{generic_sews}}, \code{\link{spectral_sews}}, 
#'   \code{\link{kbdm_sews}},
#'   \code{\link{compute_indicator}}, \code{\link{flowlength_sews}}
#' 
#' @references 
#' 
#' Kefi, S., Guttal, V., Brock, W.A., Carpenter, S.R., Ellison, A.M., Livina, 
#' V.N., et al. (2014). Early Warning Signals of Ecological Transitions: 
#' Methods for Spatial Patterns. PLoS ONE, 9, e92097
#' 
#'@export
indictest <- function(x, 
                      nulln = 999, 
                      null_method = 'perm', 
                      null_control = NULL, 
                      ...) { 
  # If indictest has already run on the object, return itself
#   if ( any(grepl("_sews_test_", class(x))) ) { 
#     return(x)
#   }
  
  UseMethod('indictest')
}

# 
# Variogram-based indicators. 
# 
#' @title Early-Warning signals based on variograms (EXPERIMENTAL)
#' 
#' @description Compute Early-warning signals based on metrics derived 
#'   form semi-variograms. 
#' 
#' @param mat A matrix (TRUE/FALSE values) or a list of matrices 
#' 
#' @param model The variogram model to use, either "sph" (for a spherical
#'   model) or "exp" (for an exponential model)
#' 
#' @param nmax The maximum number of pairs of cells to use when computing the 
#'   variogram
#' 
#' @param nbins Number of distance bins to use to compute the variogram
#' 
#' @param cutoff Maximum distance to consider in the variogram. If NULL, then 
#'   a distance equal to one third of the diagonal of the matrix is used
#' 
#' @return A list object of class "variogram_sews", that can be displayed 
#'   using \code{summary()}, \code{plot()}, etc. Significance of values can 
#'   be tested using \code{\link{indictest}}. 
#' 
#' @details 
#' 
#' During ecosystem degradation and especially before a regime shift occurs in 
#'   some ecosystems, spatial autocorrelation is expected to increase in a
#'   landscape. This increase can be measured based on variograms, which
#'   represent how the difference (variance) between two points in a landscape
#'   varies as a function of distance. 
#' 
#' The approach used to derive variogram-based EWS is to compute the 
#'   empirical variogram of a landscape (represented passed as a matrix of 
#'   values), then fit a variogram model to it. Three 
#'   parameters are then extracted from the variogram model (see Nijp et al.
#'   2019 for a visual description of these parameters): 
#'   \enumerate{ 
#'     \item The \emph{nugget} (intercept)
#'     \item The \emph{partial sill}, i.e. the reduction in semivariance at 
#'       distance zero
#'     \item The \emph{correlation range}, i.e. the distance at which the 
#'       relationship between semivariance and distance flattens
#'   }
#' 
#' Additionally, the \emph{structural variance} is computed as 
#'   (partial sill)/(nugget + partial sill), wich quantifies whether the 
#'   data are spatially structured (structural variance of one), or completely 
#'   unstructured (value of zero). Theoretical work suggests that partial sill,
#'   correlation range and structural variance should increase before a regime
#'   shift occurs in an ecosystem (Nijp et al. 2019). 
#' 
#' This function offers to fit a spherical model or 
#'   an exponential model. The best-fitting model depends on your data, you 
#'   should try different options and review the fits using
#'   \code{\link{plot_variogram}}.
#' 
#' Please note that this part of the package is still experimental and deserves 
#'   more testing. 
#' 
#' @seealso \code{\link{raw_structvar}},
#'   \code{\link{plot_variogram}}, \code{\link{extract_variogram}},
#'   \code{\link[=predict.variogram_sews_list]{predict}} 
#' 
#' @seealso 
#'   \code{\link{indictest}}, to test the significance of indicator values. 
#' 
#' @references 
#' 
#' Nijp, Jelmer J., Arnaud J.A.M. Temme, George A.K. Voorn, Lammert Kooistra,
#'   Geerten M. Hengeveld, Merel B. Soons, Adriaan J. Teuling, and Jakob
#'   Wallinga. (2019) Spatial Early Warning Signals for Impending Regime Shifts:
#'   A Practical Framework for Application in Real-world Landscapes. Global
#'   Change Biology 25 (6): 1905-21. \doi{10.1111/gcb.14591}
#'
#' @examples 
#' 
#' \donttest{
#' serengeti_ews <- variogram_sews(serengeti, 
#'                                 model ="exp")
#' plot(serengeti_ews, along = serengeti.rain)
#' summary(serengeti_ews)
#' 
#' plot_variogram(serengeti_ews)
#' 
#' # nulln should be set to a higher values for meaningful results
#' serengeti_test <- indictest(serengeti_ews, nulln = 9)
#' plot(serengeti_test) # gray ribbons indicate the null indicator values 
#' summary(serengeti_test)
#' }
#' 
#'@export
variogram_sews <- function(mat, 
                           model = "sph", 
                           nmax = 1e5, 
                           nbins = 32, 
                           cutoff = NULL) { 
  
  if ( ! model %in% c("sph", "exp") ) { 
    stop(paste0("The variogram model must be one of 'sph' (spherical model) ", 
                "or 'exp' (exponential model)"))
  }
  
  # Handle list case
  if ( is.list(mat) ) { 
    results <- lapply(mat, variogram_sews, model, nmax, nbins, cutoff)
    names(results) <- names(mat)
    class(results) <- c('variogram_sews_list',  
                        'simple_sews_list', 
                        'sews_result_list')
    attr(results, 'indicname') <- "Variogram-based indicators"
    return(results)
  }
  
  # Returns a list with components 'vario' (the empirical variogram) and 
  # 'fit' (its fit)
  vario <- fit_variogram(mat, model = model, nmax = nmax, nbins = nbins, 
                         cutoff = cutoff)
  metrics <- compute_vario_metrics(vario[['pars']])
  
  # Return list containing both
  output <- list(variogram = vario[['vario']], 
                 fit = vario[['pars']], 
                 value = metrics, 
                 orig_data = mat, 
                 pars = list(model = model, 
                             nmax = nmax, 
                             nbins = nbins, 
                             cutoff = cutoff), 
                 taskname = "Variogram-based indicators")
  
  class(output) <- c('variogram_sews_single', 
                     'simple_sews_single', 
                     'sews_result_single')
  attr(output, 'indicname') <- "Variogram-based indicators"
  
  return(output)
}

# Extract the variogram as a data frame from a variogram_sews object
#' 
#' @title extract_variogram() method for variogram_sews objects
#' 
#' @description Extract the empirical variogram from a \code{variogram_sews} 
#'   object
#' 
#' @param x An object produced by \code{variogram_sews}, or
#'   \code{indictest}
#' 
#' @param ... Additional arguments (ignored)
#' 
#' @return A data.frame containing the variogram with the distances 
#'   (column \code{dist}), the empirical semivariance values (\code{gamma}), 
#'   and if object contains more than one matrix, a column named \code{matrixn}. 
#' 
#' @examples 
#' 
#' \donttest{ 
#' vario_indics <- variogram_sews(serengeti)
#' predict(vario_indics)
#' vario_test <- indictest(vario_indics, nulln = 19)
#' predict(vario_test) # same result
#' }
#' 
#' @seealso \code{\link{variogram_sews}}
#' 
#'@export 
extract_variogram <- function(x, ...) { 
  UseMethod("extract_variogram")
}
#'@method extract_variogram variogram_sews_list
#'@export
extract_variogram.variogram_sews_list <- function(x, ...) { 
  all_varios <- Map(function(n, o) { 
    data.frame(matrixn = n, o[["variogram"]]) 
  }, n = seq_along(x), o = x)
  do.call(rbind, all_varios)
}
#'@method extract_variogram variogram_sews_single
#'@export
extract_variogram.variogram_sews_single <- function(x, ...) { 
  x[["variogram"]]
}



# Predict methods 
# --------------------------------------------------
#' @rdname variogram_sews_predict
#' @name variogram_sews_predict
#' 
#' @title predict() method for variogram_sews objects
#' 
#' @description Export the fitted variogram(s)
#' 
#' @param object An object produced by variogram_sews 
#' 
#' @param newdist A vector of distances at which to return the variogram fit 
#'   values (defaults to 128 regularly-spaced values). 
#' 
#' @param ... Additional arguments (ignored)
#' 
#' @return A data.frame with the distances (column \code{dist}), the fitted 
#'   values (\code{gamma}), and if object contains more than one matrix, 
#'   a column \code{matrixn}. 
#' 
#' @examples 
#' 
#' \donttest{ 
#' vario_indics <- variogram_sews(serengeti)
#' predict(vario_indics)
#' vario_test <- indictest(vario_indics, nulln = 19)
#' predict(vario_test) # same result
#' }
#' 
#' @seealso \code{\link{variogram_sews}}
#' 
#'@export 
predict.variogram_sews_list <- function(object, newdist = NULL, ...) { 
  all_preds <- llply(object, predict.variogram_sews_single, newdist = newdist)
  all_preds <- Map(function(n, o) { 
    data.frame(matrixn = n, o)
  }, seq_along(all_preds), all_preds)
  do.call(rbind, all_preds)
}
#'@export 
predict.variogram_sews_single <- function(object, newdist = NULL, ...) { 
  if ( is.null(newdist) ) { 
   newdist <- with(object[["variogram"]], seq(0, max(dist), length.out = 128))
  }
  pars <- object[["value"]]
  if ( object[["pars"]][["model"]] == "sph" ) { 
    y <- spherical_model(newdist, pars[1], pars[2], pars[3])
  } else if ( object[["pars"]][["model"]] == "exp" ) { 
    y <- exponential_model(newdist, pars[1], pars[2], pars[3])
  } else { 
    stop("Unknown model")
  }
  data.frame(dist = newdist, gamma = y)
}

# Plot methods for variogram sews 
#' 
#' @rdname variogram_sews_plot
#' @name variogram_sews_plot
#' 
#' @title Early-warning signals based on variograms 
#' 
#' @description Plot trends of indicators based on variograms
#' 
#' @param x An object produced by \code{\link{variogram_sews}}, or the 
#'   result of applying \code{indictest} on such object.
#' 
#' @param along A vector providing values along which the indicator trends 
#'   will be plotted. If \code{NULL} then the indicator values are plotted
#'   sequentially in their original order. 
#' 
#' @details The \code{plot()} function will display how the estimated 
#'   variogram parameters change along a set of values (passed with argument
#'   \code{along}). If the object passed has been processed through 
#'   \code{indictest}, then the null values are also displayed. 
#'   \code{plot_variogram()} can be used to display the individual variograms
#'   that have been fit to the data. 
#' 
#' @seealso 
#' 
#' \code{\link{variogram_sews}}, \code{\link{indictest}}, 
#'   \code{\link{plot_variogram}}  
#' 
#' @examples 
#'   
#' \donttest{ 
#' serengeti_ews <- variogram_sews(serengeti, model ="exp")
#' 
#' # Display the change in variogram parameters 
#' plot(serengeti_ews, along = serengeti.rain) + 
#'   ggplot2::labs(x = "Rainfall (mm)")
#' 
#' # Visualize the fitted variograms
#' plot_variogram(serengeti_ews, along = serengeti.rain) 
#' 
#'   # Test the trends (nulln should be set to a higher value to obtain 
#'   # meaningful results
#'   serengeti_test <- indictest(serengeti_ews, nulln = 19)
#'   plot(serengeti_test, along = serengeti.rain)
#'   plot_variogram(serengeti_test, along = serengeti.rain)
#' }
#' 
#'@export
plot.variogram_sews <- function(x, along = NULL, ...) { 
  NextMethod("plot")
}
#'@export 
plot.variogram_sews_list <- function(x, along = NULL, ...) { 
  # Adjust x, then pass it on to the method for simple_sews plotting. Here 
  # we use a for loop so that the class attributes is preserved. 
  for (i in seq_along(x) ) { 
    x[["value"]] <- x[["metrics"]]
  }
  
  plot.simple_sews_list(x, along = along, ...)
}

#'@rdname variogram_sews_plot
#'@export
plot_variogram <- function(x, along = NULL, ...) { 
  UseMethod("plot_variogram")
}
#'@rdname variogram_sews_plot
#'@method plot_variogram variogram_sews
#'@export
plot_variogram.variogram_sews <- function(x, along = NULL, ...) { 
  UseMethod("plot_variogram")
}
#'@export
plot_variogram.variogram_sews_single <- function(x, along = NULL, ...) { 
  df <- extract_variogram(x)
  df_pred <- predict(x)
  
  ggobj <- ggplot(df, aes(x = dist, y = gamma)) + 
    geom_point() + 
    geom_line() + 
    geom_line(color = "red", data = df_pred) + 
    theme_spwarnings() + 
    linescale_spwarnings() + 
    labs(x = "Distance", 
         y = "gamma")
  
  return(ggobj)
}
#'@export
plot_variogram.variogram_sews_list <- function(x, along = NULL, ...) { 
  
  # Extract data and add along data
  df <- extract_variogram(x)
  df_pred <- predict(x)
  if ( is.null(along) ) { 
    df[ ,'along'] <- df[ ,"matrixn"]
    df_pred[ ,'along'] <- df_pred[ ,"matrixn"]
  } else { 
    df[ ,"along"] <- along[df[ ,"matrixn"]]
    df_pred[ ,"along"] <- along[df_pred[ ,"matrixn"]]
  }
  
  ggobj <- ggplot(df, aes_string(x = "dist", y = "gamma")) + 
    geom_point() + 
    geom_line() + 
    geom_line(color = "red", data = df_pred) + 
    theme_spwarnings() + 
    linescale_spwarnings() + 
    labs(x = "Distance", 
         y = "gamma") + 
    facet_wrap( ~ along, scales = "free_y")
  
  return(ggobj)
}


# Fit a variogram. We do it by hand and do not use gstat which seems to be 
# a bit unreliable. We initialize the varioram with a segmented regression 
# first, then refine that fit with bounded BFGS. 
fit_variogram <- function(mat, model, nmax, nbins, cutoff) { 
  
  # Handle missing arguments 
  if ( is.null(cutoff) ) { 
    cutoff <- (1/3) * sqrt(nrow(mat)^2 + ncol(mat)^2) 
  }
  
  # Make the model choice case-insensitive
  model <- tolower(model)
  
  # Check if mat is suitable
  check_mat(mat)
  
  # Compute the variogram 
  vario <- variogram_internal(mat, nmax, nbins, cutoff)
  
  # Check if sample has only a single type of values. In that case we return 
  # a default answer with missing values for parameters as the model will not 
  # fit.
  if ( length(unique(vario[ ,"gamma"])) < 2 ) { 
    default_answer <- 
      list(pars = list(nugget = NaN, 
                       psill  = NaN, 
                       range  = NaN), 
           vario = vario)
    return(default_answer)
  }
  
  # Fit variogram. We first use a segmented regression to find a breakpoint in 
  # the empirical variogram, then use that as starting values for correlation 
  # range. Note that the segmented regression can fail, especially when we start giving
  # null data to it, so we wrap it in a try() block.
  brk_model <- try(suppressWarnings({ 
    segmented::segmented(lm(gamma ~ dist, 
                            data = vario))
  }), silent = TRUE)
  
  range0 <- 1/3 * (max(vario[ ,"dist"]) - min(vario[ ,"dist"]))
  psill0 <- 0
  nugget0 <- 0
  if ( ! inherits(brk_model, "try-error") ) { 
    brk_coefs <- coef(brk_model)
    # Extract the break value, if there is one
    nugget0 <- max(0, as.vector(brk_coefs["(Intercept)"]))
    if ( ! is.null(brk_model[["psi"]][2]) ) { 
      range0 <- brk_model[["psi"]][2]
      psill0 <- abs(nugget0 - median(vario[vario$dist > range0, "gamma"]))
    } 
  } 
  
  pars0 <- c(nugget = nugget0, psill  = psill0, range  = range0)
  
  # Objective function
  ssq <- function(pars0) { 
    if ( model == "sph" ) { 
      pred <- spherical_model(vario[ ,"dist"], pars0[1], pars0[2], pars0[3]) 
    } else if ( model == "exp" ) { 
      pred <- exponential_model(vario[ ,"dist"], pars0[1], pars0[2], pars0[3]) 
    }
    sum( ( (vario[ ,"gamma"] - pred) )^2 )
  }
  
  fit <- optim_safe(ssq, pars0,
                    # sensible bounds for nugget, psill, range
                    lower = c(0, 0, 0),
                    upper = c(max(vario[ ,"gamma"]),
                              max(vario[ ,"gamma"]),
                              2*max(vario[ ,"dist"]))
                    )

  vario[ ,"pred"] <- with(as.list(fit$par), 
                          spherical_model(vario[ ,"dist"], 
                                          nugget, psill, range))
#   with(vario, plot(dist, gamma))
#   with(vario, lines(dist, pred, col = "blue"))
#   with(vario, lines(dist, 
#                     spherical_model(dist, pars0[1], pars0[2], pars0[3]), 
#                     col = "red"))
  
  list(pars = as.list(fit$par), 
       vario = vario)
}

# Function that wraps C++ code to compute the variogram
variogram_internal <- function(mat, nmax, nbins, cutoff) { 
  
  v <- variogram_internal_cpp(mat, nmax, nbins, cutoff)
  colnames(v) <- c("dist", "gamma", "np")
  v <- as.data.frame(v)
  
  # Discard all values at small distances which have zero gamma
  line <- 1
  while ( is.na(v[line, "gamma"]) || v[line, "gamma"] == 0 ) { 
    line <- line + 1 
  }
  v[ v[ ,"np"] > 0 & seq.int(nrow(v)) >= line, ]
}

spherical_model <- function(X, nugget, psill, range) { 
  ifelse(X < range, 
         nugget + psill * ( (3/2) * (X/range) - (1/2) * (X/range)^3 ), 
         nugget + psill)
}

exponential_model <- function(X, nugget, psill, range) { 
  nugget + psill * (1 - exp(-X/range))
}

compute_vario_metrics <- function(pars) { 
  structvar <- with(pars, psill / (nugget + psill))
  metrics <- with(pars, c(nugget, psill, range, structvar))
  names(metrics) <- c('nugget', 'psill', 'range', 'structvar')
  return(metrics)
}

# Raw version of the function
#' 
#' @title Variogram parameters
#' 
#' @description Compute the nugget, partial sill, correlation range and 
#'   structural variance on a matrix. 
#' 
#' @param mat A matrix 
#' 
#' @param model The variogram model to use, either "sph" (for a spherical
#'   model) or "exp" (for an exponential model)
#' 
#' @param nmax The maximum number of pairs of cells to use when computing the 
#'   variogram
#' 
#' @param nbins Number of distance bins to use to compute the variogram
#' 
#' @param cutoff Maximum distance to consider in the variogram. If NULL, then 
#'   a distance equal to one third of the diagonal of the matrix is used
#' 
#' @examples 
#' 
#' raw_variogram_metrics(serengeti[[5]])
#' 
#' @seealso variogram_sews, raw_structvar
#' 
#'@export
raw_variogram_metrics <- function(mat, 
                                  model = "sph", 
                                  nmax = 1e5L, 
                                  nbins = 32, 
                                  cutoff = NULL) { 
  if ( ! is.matrix(mat) ) { 
    stop("raw_variogram_metrics only accepts a single matrix as input.")
  }
  
  vario <- fit_variogram(mat, model, nmax, nbins, cutoff)
  compute_vario_metrics(vario[["pars"]])
}

#' @title Structural variance
#' 
#' @description Compute the structural variance on a matrix. 
#' 
#' @param mat A matrix 
#' 
#' @param model The variogram model to use, either "sph" (for a spherical
#'   model) or "exp" (for an exponential model)
#' 
#' @param nmax The maximum number of pairs of cells to use when computing the 
#'   variogram
#' 
#' @param nbins Number of distance bins to use to compute the variogram
#' 
#' @param cutoff Maximum distance to consider in the variogram. If NULL, then 
#'   a distance equal to one third of the diagonal of the matrix is used
#' 
#' @seealso raw_variogram_metrics, variogram_sews
#' 
#' @examples 
#' 
#' raw_structvar(serengeti[[5]])
#' 
#'@export
raw_structvar <- function(mat, 
                          model = "sph", 
                          nmax = 1e5L, 
                          nbins = 32, 
                          cutoff = NULL) { 
  raw_variogram_metrics(mat, model, nmax, nbins, cutoff)["structvar"]
}

# 
# 
# This file contains a unit tests that make sure the complete workflow is working
# 

context('Test workflows') 

data(forestgap)
data(serengeti)

datasets <- list(forestgap[[3]], 
                 serengeti[5:6])

test_methods <- function(teststring, datalength, obj, .test_df = TRUE) { 
  ok_print <- any(grepl(teststring, capture.output(print(obj))))
  expect_true(ok_print)
  
  ok_summary <- any(grepl(teststring, capture.output(summary(obj))))
  expect_true(ok_summary)
  
  
  if (.test_df) { 
    ok_as_df <- nrow(as.data.frame(obj)) == datalength
  } else { 
    ok_as_df <- is.data.frame(as.data.frame(obj)) 
  }
  expect_true(ok_as_df)
  
  return(TRUE)
}

test_that("The workflow functions work", { 
  skip_on_cran()
  
  for ( dataset in datasets ) { 
    
    # Length of data
    datal <- ifelse(is.matrix(dataset), 1, length(dataset))
    
    
    
    # Simple/generic indicators
    indics <- generic_sews(dataset) 
    test_methods("Generic indicators", 
                 datal*4, indics) # l(dataset) * 4 indics
    # test_methods("Generic Spatial Early-Warnings", 4, indics[[1]])
    
    indics.test <- indictest(indics, nulln = 3)
    test_methods("Generic indicators", 
                  datal*4, indics.test)
    
    if ( datal > 1 ) { # multiple values
      suppressWarnings( print( plot(indics.test) ) )
      suppressWarnings( print( plot(indics) ) )
    }
    
    # Warn when matrix is logical but no cg will be performed
    expect_warning({ 
      generic_sews(forestgap, subsize = 1)
    })

    
    
    # Spectral indicators
    indics <- spectral_sews(dataset, 
                            sdr_low_range  = c(0,  0.2), 
                            sdr_high_range = c(.8, 1)) 
    test_methods("Spectrum-based indicators", 
                 length(dataset), indics, .test_df = FALSE)
    expect_warning({ spectral_sews(dataset) }) # give a warning when no args are passed
    
    indics.test <- indictest(indics, nulln = 3)
    test_methods("Spectrum-based indicators", 
                  datal, indics.test, .test_df = FALSE)
    
    if ( datal > 1 ) { # multiple values
      suppressWarnings( print( plot(indics.test) ) )
      suppressWarnings( print( plot(indics) ) )
    }
    suppressWarnings( print( plot_spectrum(indics.test) ) ) 
    suppressWarnings( print( plot_spectrum(indics) ) ) 
    
    
    
    
    # PSD-based indicators. Suppress warnings that can be produced by optim() 
    # here. 
    indics <- suppressWarnings( patchdistr_sews(dataset, fit_lnorm = TRUE) )
    test_methods("Patch-based indicators", 
                 datal*4, indics) # l(dataset) * 4 psd types fitted
    # test_methods("Patch-based Early-Warnings results", 
    #              datal*4, indics[[1]])
    # Here, indictest may produce warnings on the testing datasets, so we 
    # quiet them down
    indics.test <- suppressWarnings( indictest(indics, nulln = 3) )
    test_methods("Patch-based indicators", 
                  datal*4, indics.test, .test_df = FALSE)
    
    # Test prediction of PSDs
    indics.pred <- predict(indics)
    
    if ( ! is.matrix(dataset) ) { 
      suppressWarnings( print( plot(indics) ) )
    }
    suppressWarnings( print( plot_distr(indics) ) )
    suppressWarnings( print( plot_distr(indics.test) ) )
    
    # We print a warning when there is no data to display because all fits 
    # failed. This can happen e.g. when there is a single patch in a matrix
    a <- matrix(FALSE, ncol = 10, nrow = 10) 
    a[1,1] <- TRUE
    a <- list(a, a)
    expect_warning({ 
      plot(patchdistr_sews(a))
    })
    
    
    
    # Variogram-based indicators 
    indics <- suppressWarnings( variogram_sews(dataset, model = "exp") )
    test_methods("Variogram-based indicators", 
                 datal*4, indics) # l(dataset) * 4 metrics reported 
    # Here, indictest may produce warnings on the testing datasets, so we 
    # quiet them down
    indics.test <- suppressWarnings( indictest(indics, nulln = 3) )
    test_methods("Variogram-based indicators", 
                 datal*4, indics.test, .test_df = FALSE)
    
    # Test prediction of PSDs
    indics.pred <- predict(indics)
    
    if ( ! is.matrix(dataset) ) { 
      suppressWarnings( print( plot(indics) ) )
    }
    suppressWarnings( print( plot_variogram(indics) ) )
    suppressWarnings( print( plot_variogram(indics.test) ) )
    
    
    # Flowlength-based indicator
    indics <- suppressWarnings( flowlength_sews(dataset, cell_size = 10) )
    test_methods("Flow length \\(uniform topography\\)", 
                 datal, indics) # l(dataset) * 4 metrics reported 
    indics.test <- indictest(indics, nulln = 3) 
    test_methods("Flow length \\(uniform topography\\)", 
                 datal, indics.test, .test_df = FALSE)
    
    # Make sure that args are passed through the workflow function 
    indics_standalone <- compute_indicator(dataset, raw_flowlength_uniform, 
                                           cell_size = 10, slope = 20)
    expect_equal(as.data.frame(indics)[ ,"value"], 
                 as.data.frame(indics_standalone)[ ,"value"])
    
    if ( ! is.matrix(dataset) ) { 
      suppressWarnings( print( plot(indics) ) )
    }
    
  }
  
})

# 
# This file tests the variogram-related low-level functions and indicators
# 

# Make sure it works
context("Computation of variograms")

test_that("Computation of variograms works", { 
  
  skip_on_cran()

  # Here so that testthat does not complain about empty tests when missing packages
  expect_true(TRUE)
  
  if ( requireNamespace("gstat", quietly = TRUE) && 
       requireNamespace("sp", quietly = TRUE) ) { 

    if ( exists("EXTENDED_TESTS") && EXTENDED_TESTS) { 
      mats <- list(arizona[[4]], forestgap[[8]], serengeti[[6]]) 
    } else { 
      mats <- list(forestgap[[8]])
      # Synthetic matrix
#       mats <- list(outer(seq.int(64), seq.int(64), function(x, y) x * y))
    }
    
    for ( example_mat in mats ) { 

      nbins <- 32
      nmax = 2e6L # 5e5L # prod(dim(example_mat)) ^2
      cutoff = sqrt(ncol(example_mat)^2 + nrow(example_mat)^2) / 10
      
      vario1 <- variogram_internal(example_mat, nmax, nbins, cutoff)
#       pairs(vario1)
      
      # Compare results to gstat
      locations <- expand.grid(seq.int(nrow(example_mat)),
                              seq.int(ncol(example_mat)))
      locations <- locations[sample.int(nrow(locations), 
                                        replace = FALSE, 
                                        size = min(nrow(locations), nmax/10)), ]
      values <- apply(locations, 1, function(X) example_mat[X[1], X[2]])
      locations.gstat <- sp::SpatialPointsDataFrame(locations, 
                                                    data.frame(z = values))
      vario2 <- gstat::variogram(z ~ 1, 
                                 data   = locations.gstat, 
                                 width  = cutoff / nbins, 
                                 cutoff = cutoff)
      plot(vario1[ ,"dist"], vario1[ ,"gamma"] )
      points(vario2[ ,"dist"], vario2[ ,"gamma"], col = "red", pch = 20)
      
      vario1_2 <- approx(vario1[ ,1], vario1[ ,2], vario2[ ,"dist"])$y
      
#       plot(vario1_2 - vario2[ ,"gamma"], vario2[ ,"gamma"])
#       abline(0,1)
      
      # The difference between the two variograms should be zero on average
      mod <- coef(lm(vario1_2 - vario2[ ,'gamma'] ~ 1))
      expect_true(abs(mod[1]) < 0.01) # Intercept ~= 0
      
      # Test if P-value is significant, we expect it not to be
      expect_true(kruskal.test(vario1_2, vario2[ ,"gamma"])$p.value > 0.1)
    }
    
  } # end of requireNamespace()
  
})

if ( exists("EXTENDED_TESTS") && EXTENDED_TESTS ) { 

context('Test the deprecation of functions')

  test_that("Deprecation works and produces warnings", { 
      expect_error(generic_spews(forestgap[[3]]))
      expect_error(spectral_spews(forestgap[[3]]))
      expect_error(patchdistr_spews(forestgap[[3]]))
      expect_error(indicator_sdr(forestgap[[2]]), nulln = 3)
      expect_error(indicator_variance(forestgap[[2]]), nulln = 3)
      expect_error(indicator_skewness(forestgap[[2]]), nulln = 3)
      expect_error(indicator_moran(forestgap[[2]]), nulln = 3)
  })
  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{display_size_info}
\alias{display_size_info}
\alias{display_size_info.sews_result_single}
\alias{display_size_info.sews_result_list}
\alias{display_size_info.list}
\title{Display size information of a sews_result object}
\usage{
display_size_info(x, ...)

\method{display_size_info}{sews_result_single}(x, ...)

\method{display_size_info}{sews_result_list}(x, ...)

\method{display_size_info}{list}(x, ...)
}
\arguments{
\item{x}{A \code{sews_result} object (as provided by **_sews functions, such
as \code{generic_sews()}}

\item{...}{Other arguments are ignored.}
}
\description{
Display size information of a sews_result object
}
\details{
This function is used to display information about the size of matrices
  used by various functions in spatialwarnings. It is mostly destined to internal
  use.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatialwarnings.R
\docType{package}
\name{spatialwarnings-package}
\alias{spatialwarnings}
\alias{spatialwarnings-package}
\title{Early Spatial-Warnings of Ecosystem Degradation}
\description{
Many dynamical systems such as ecosystems exhibit non-linear responses to 
changes in their external drivers, resulting in possible wide state shifts 
with strong ecological or economical consequences. This often happens when 
a system exhibit a change in its stability properties as a threshold is 
crossed, \emph{e.g.} going from multiple stable states to a single stable 
state. For a few decades, much research has been dedicated to finding a 
way to anticipate these tipping points in ecological systems. This has led 
to the development of indicators, which are metrics based on spatial
structure, that could reflect the proximity of an ecosystem to a tipping
point. 

This package implements many of these indicators, or 
\emph{early-warning signals} (EWS), on spatial raster data. High-level 
functions and methods provide familiar workflows to compute several related
indicators at once, and display their variations along environmental gradients 
or time-series. Lower-level functions are also available to integrate 
early-warning signals into different workflows. 

Main, "workflow" functions provided by this package: 

"Workflow" functions, which may compute several indicators at once:
\itemize{ 
  \item \code{\link{generic_sews}}: Generic spatial EWS
  \item \code{\link{spectral_sews}}: Spectrum-based EWS
  \item \code{\link{patchdistr_sews}}: EWS based on patch-size distributions
  \item \code{\link{kbdm_sews}}: Kolmogorov complexity
  \item \code{\link{flowlength_sews}}: Flow length 
  \item \code{\link{variogram_sews}}: Variogram-based indicators
  \item \code{\link{lsw_sews}}: Indicators based on the Lifshitz-Slyozov-Wagner distribution
}

Individual indicator functions, which may be used to compute raw indicator values 

\itemize{ 
  \item \code{\link{raw_cg_moran}}: lag-1 spatial autocorrelation 
  \item \code{\link{raw_cg_variance}}: Spatial variance
  \item \code{\link{raw_cg_skewness}}: Spatial skewness
  \item \code{\link{raw_sdr}}: Spectral density ratio (SDR)
  \item \code{\link{indicator_psdtype}}: Patch-size distribution shape 
  \item \code{\link{indicator_plrange}}, 
          \code{\link{raw_plrange}} : Power-law range
  \item \code{\link{raw_kbdm}}: Kolmogorov complexity
  \item \code{\link{raw_flowlength_uniform}}: Flow Length
  \item \code{\link{raw_structvar}}: Structural variance
  \item \code{\link{raw_lsw_aicw}}: Support for LSW distribution relative to lognormal
  \item \code{\link{raw_patch_radii_skewness}}: Skewness of patch radii
}

The package homepage is available at 
\href{https://github.com/spatial-ews/spatialwarnings}{Github}
and a  
\href{https://alex.lecairn.org/spatialwarnings-faq.html}{user guide} is 
also available. Do not hesitate to get in touch if you want to make changes to the 
package or add another indicator.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/spatial-ews/spatialwarnings}
  \item Report bugs at \url{https://github.com/spatial-ews/spatialwarnings/issues}
}

}
\author{
\strong{Maintainer}: Alexandre Genin \email{alexandre.genin@sete.cnrs.fr} (\href{https://orcid.org/0000-0002-3333-1338}{ORCID})

Authors:
\itemize{
  \item Alain Danet
  \item Vishwesha Guttal
  \item Sonia Kefi
  \item Sabiha Majumder
  \item Sumithra Sankaran
  \item Florian Schneider
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_kbdm.R
\name{kbdm_sews}
\alias{kbdm_sews}
\title{Indicator based on Kolmogorov Complexity}
\usage{
kbdm_sews(mat, subsize = 3)
}
\arguments{
\item{mat}{A logical matrix (\code{TRUE}/\code{FALSE} values) or a list of logical 
matrices}

\item{subsize}{A submatrix size to carry out the Block Decomposition Method
(must be between 1 and 3)}
}
\value{
\code{kbdm_sews} returns an object of class \code{simple_sews_single} 
    (a list) if mat is a single matrix, and an object of class 
    \code{simple_sews_list} if mat is a list of matrices. These objects can 
    be used with generic methods indictest (to test significance) or plot 
    (to display trends), see also the examples below.
}
\description{
Computes the Kolmogorov Complexity on a set of matrices, 
    using the Block Decomposition Method.
}
\details{
be a useful indicator to anticipate transitions in model ecological 
    systems (Dakos and Soler-Toscano, 2017). When close to the transition 
    critical point, the complexity is expected to decrease. 
  
  The Kolmogorov complexity cannot be computed directly for large strings 
    (i.e. matrices). However, the complexity of smaller submatrices can be 
    estimated, then combined to obtain an approximation of the complexity 
    of the whole matrix. This method, the Block Decomposition Method is 
    implemented in this indicator following Dakos and Soler-Toscano (2017).
}
\examples{

\donttest{ 

kbdm_result <- kbdm_sews(serengeti, subsize = 3)
plot(kbdm_result, along = serengeti.rain)

kbdm_test <- indictest(kbdm_result, nulln = 49)
plot(kbdm_test, along = serengeti.rain)

# Plot deviation to null expectation
plot(kbdm_test, along = serengeti.rain, what = "z_score") 

}


}
\references{
Dakos, V., and F. Soler-Toscano. 2017. Measuring complexity to infer 
  changes in the dynamics of ecological systems under stress. Ecological 
  Complexity 32:144-155.
}
\seealso{
\code{\link{raw_kbdm}}, \code{\link[acss]{acss}}, 
  \code{\link{indictest}}, to test the significance of indicator values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_generic_indic.R
\name{generic_sews}
\alias{generic_sews}
\title{Generic Spatial Early-Warning signals}
\usage{
generic_sews(
  mat,
  subsize = 4,
  abs_skewness = FALSE,
  moranI_coarse_grain = FALSE
)
}
\arguments{
\item{mat}{A matrix (quantitative data), a binary matrix (which contains 
\code{TRUE} or \code{FALSE} values) 
or a list of those}

\item{subsize}{The subsize used for the coarse-graining phase (see Details)}

\item{abs_skewness}{Should the absolute skewness be used instead of its 
raw values ?}

\item{moranI_coarse_grain}{Should the input matrix be coarse-grained before
computing the Moran's I indicator value ?}
}
\value{
\code{generic_sews} returns an object of class \code{simple_sews_single}
  (a list) if \code{mat} is a single matrix or an object of class 
  \code{simple_sews_list} if \code{mat} is a list. You probably want to use some  
  of the methods written for these complicated objects instead of extracting 
  values directly (they are displayed using \code{print(<object>)}).

\code{indictest} returns an object of class \code{generic_test} (a \code{data.frame}). 

\code{plot} methods return \code{ggplot} objects, usually immediately displayed 
  when \code{plot} is being called at the R prompt.
}
\description{
Computation, significance assessment and display of spatial 
  generic early warning signals (Moran's I, variance and skewness)
}
\details{
The Generic Early warning signal are based on the property of a 
  dynamical system to "slow down" when approaching a critical point, 
  that is, take more time to return to equilibrium after a perturbation. This 
  is expected to be reflected in several spatial characteristics: the 
  variance, the spatial autocorrelation (at lag-1) and the skewness. This 
  function provides a convenient workflow to compute these indicators, 
  assess their significance and display the results. 

Before computing the actual indicators, the matrix can be "coarse-grained". 
  This process reduces the matrix by averaging the nearby cells using 
  a square window defined by the \code{subsize} parameter. This makes spatial  
  variance and skewness reflect actual spatial patterns when working with 
  binary (\code{TRUE}/\code{FALSE} data), but is optional when using 
  continuous data. Keep in mind that it effectively reduces the size of 
  the matrix by approximately \code{subsize} on each dimension. 
  
The significance of generic early-warning signals can be estimated by 
  reshuffling the original matrix (function \code{indictest}). Indicators 
  are then recomputed on the shuffled matrices and the values obtained are 
  used as a null distribution. P-values are obtained based on the rank of 
  the observed value in the null distribution. A small P-value means 
  that the indicator is significantly above the null values, as expected 
  before a critical point. 

The \code{plot} method can displays the results graphically. A text summary 
  can be obtained using the \code{summary} method.
}
\examples{

data(serengeti)
gen_indic <- generic_sews(serengeti, subsize = 5, 
                           moranI_coarse_grain = TRUE)

# Display results
summary(gen_indic)

# Display trends along the varying model parameter
plot(gen_indic, along = serengeti.rain)

# Compute significance (long)
\donttest{
gen_test <- indictest(gen_indic, nulln = 199)

print(gen_test)

# Display the trend, now with a grey ribbon indicating the 5\%-95\% quantile
# range of the null distribution
plot(gen_test, along = serengeti.rain)

# Display the effect size compared to null distribution 
plot(gen_test, along = serengeti.rain, what = "z_score")

# Note that plot() method returns a ggplot object that can be modified
# for convenience
if ( require(ggplot2) ) { 
  plot(gen_test, along = serengeti.rain) + 
    geom_vline(xintercept = 733, color = "red", linetype = "dashed") +
    xlab('Annual rainfall') + 
    theme_minimal()
}

}

}
\references{
Kefi, S., Guttal, V., Brock, W.A., Carpenter, S.R., Ellison, A.M., 
  Livina, V.N., et al. (2014). Early Warning Signals of Ecological 
  Transitions: Methods for Spatial Patterns. PLoS ONE, 9, e92097.
  
  Dakos, V., van Nes, E. H., Donangelo, R., Fort, H., & 
  Scheffer, M. (2010). Spatial correlation as leading indicator of 
  catastrophic shifts. Theoretical Ecology, 3(3), 163-174.
  
  Guttal, V., & Jayaprakash, C. (2008). Spatial variance and spatial 
  skewness: leading indicators of regime shifts in spatial ecological 
  systems. Theoretical Ecology, 2(1), 3-12.
}
\seealso{
\code{\link{indictest}}, to test the significance of indicator values. 
  Individual indicators: \code{\link{raw_cg_moran}}
    \code{\link{raw_cg_variance}}, \code{\link{raw_cg_skewness}}, 
    \code{\link{simple_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{dda}
\alias{dda}
\alias{dda.pars}
\title{Density-dependent aggregation model}
\format{
An object of class \code{list} of length 4.

An object of class \code{data.frame} with 4 rows and 5 columns.
}
\source{
Kindly provided by Koen Siteur
}
\usage{
dda

dda.pars
}
\description{
\code{dda} is a list of matrices representing results from the 
  density-dependent aggregation model (Siteur et al. 2023) in \code{dda}.
  \code{dda.pars} is a data frame with the model parameters. Each row of
  \code{dda.pars} corresponds to a matrix in \code{dda}, in the same order. All
  parameters were maintained constant, except for tau (see model
  definition in Siteur et al. 2023).
}
\examples{

ddasews <- lsw_sews(dda)
plot(ddasews, along = dda.pars[ ,"tau"]) # tau is the changing parameter

display_matrix(dda[[4]]) 

}
\references{
Siteur, Koen, Quan-Xing Liu, Vivi Rottschäfer, Tjisse van der Heide, Max Rietkerk,
  Arjen Doelman, Christoffer Boström, and Johan van de Koppel. 2023. "Phase-Separation
   Physics Underlies New Theory for the Resilience of Patchy Ecosystems." Proceedings 
   of the National Academy of Sciences 120 (2): e2202683120. 
   https://doi.org/10.1073/pnas.2202683120.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/display_matrix.R
\name{display_matrix}
\alias{display_matrix}
\title{Plot a matrix}
\usage{
display_matrix(object, palette = "RdYlBu", along = NULL, ...)
}
\arguments{
\item{object}{A matrix, a list of matrices, an object produced by 
\code{*_sews} functions or \code{indictest()}}

\item{palette}{A color palette to use in the plot. It can be any color 
palette understood by \link[ggplot2]{scale_fill_brewer}.}

\item{along}{A vector of values used in facet headers. If \code{object} is 
a matrix, this is ignored.}

\item{...}{Other arguments are ignored.}
}
\value{
A ggplot2 object, which is printed when this function is used 
  interactively.
}
\description{
Display a matrix or a list of matrices in a plot
}
\details{
This function will plot a matrix using ggplot2, using the provided 
  palette. Its use-case is very much like \code{image()}, but its produces 
  nicer plots by default (\code{image()} is much faster, however).
}
\examples{

# display_matrix works with single matrices or lists of matrices 
display_matrix(serengeti[2:3])

# display_matrix is compatible with "*_sews" objects
indics <- compute_indicator(serengeti[2:3], raw_moran) 
display_matrix(indics)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{arizona}
\alias{arizona}
\title{Aerial views of vegetation from Arizona, USA}
\format{
A list of logical matrices which were obtained through the 
  classification of aerial images of vegetation taken in Arizona (USA).
}
\source{
Derived from the images provided in the Supplementary Material of 
  Rodriguez et al. (2017).
}
\usage{
arizona
}
\description{
Aerial views of vegetation from Arizona, USA
}
\references{
Rodriguez, F., A. G. Mayor, M. Rietkerk, and S. Bautista. 2017. A null model 
  for assessing the cover-independent role of bare soil connectivity as 
  indicator of dryland functioning and dynamics. Ecological Indicators.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coarse_graining.R
\name{coarse_grain}
\alias{coarse_grain}
\title{Matrix coarse-graining}
\usage{
coarse_grain(mat, subsize)
}
\arguments{
\item{mat}{A matrix}

\item{subsize}{Dimension of the submatrix. This has to be a positive 
integer smaller than the dimension of input matrix.}
}
\value{
A matrix of reduced dimension.
}
\description{
This function averages the spatial data locally. It divides 
  the input matrix into submatrices of dimension \code{subsize} and 
  averages the spatial data in these submatrices. By doing this, the 
  dimension of resultant matrix is reduced by a factor of 
  \code{subsize}.
}
\details{
If the data is classified into discrete units, the calculation of 
variance and skewness can give spurious results irrelevant to the proximity 
to transition. Therefore, discrete data should be 'coarse-grained' before 
calculating the spatial early warning signals. However, this can also be 
applied to continuous state data.
}
\examples{
rmat <- matrix(runif(20*10) > .5, 
               ncol = 20, nrow = 10)
rmat.cg <- coarse_grain(rmat, subsize = 2)

par(mfrow = c(1, 2))
image(rmat)
title('Raw matrix') 
image(rmat.cg) 
title('Coarse-grained matrix')

}
\references{
Sankaran, S., Majumder, S., Kefi, S. and Guttal, V. (2017). Implications 
  of being discrete and spatial for detecting early warning signals of 
  regime shifts. Ecological Indicators.
}
\seealso{
\code{\link{generic_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_generic_indic.R
\name{raw_cg_variance}
\alias{raw_cg_variance}
\title{Spatial variance indicator}
\usage{
raw_cg_variance(mat, subsize = 5)
}
\arguments{
\item{mat}{A matrix. Its values can be logical, with \code{FALSE} (empty) 
or \code{TRUE} (occupied) values. The entries can also be continuous 
(like NDVI or EVI data).}

\item{subsize}{Dimension of the submatrix used to coarse-grain the 
original matrix. This must be an integer less than size of the full 
matrix. Coarse-graining reduces the size of the matrix by a factor 
\code{subsize} in each dimension of the matrix. Variance is calculated 
on the coarse-grained matrix.}
}
\value{
The variance of the coarse-grained matrix as a named vector
}
\description{
This functions computes the spatial variance of a matrix.
}
\details{
Spatial variance is a measure of fluctuations in space. Based on the theory 
of critical slowing down, when systems approach critical points
they are expected to show increased fluctuations in space. Thus, increasing 
spatial variance is proposed as an early warning signal of impending 
critical transitions. 

Many high resolution spatial data are classified as FALSE (empty) 
or TRUE (occupied). In such cases, spatial variance captures just 
the variance in data, but not that of spatial structure. 
To resolve the issue, this function employs a method called coarse-graining, 
proposed in Kefi et al (2014), and described in detail in 
Sankaran et al. (2017). One must specify a subsize above one for 
binary valued data sets to obtain meaningful values. 

\code{subsize} has to be an integer. It has to be less than or equal to 
half of matrix size (N). \code{subsize} must also be preferably a 
divisor of N. If it is not a divisor of N, the remainder rows and columns 
are discarded when computing coarse-graining matrices. 

Null model evaluations are also done on coarse-grained matrices.
}
\examples{

\donttest{
data(serengeti)
raw_cg_variance(serengeti[[1]])
compute_indicator(serengeti, fun = raw_cg_variance, subsize = 5)
}

}
\references{
Guttal, V., and Jayaprakash, C. (2009). Spatial variance and 
spatial skewness: leading indicators of regime shifts in spatial 
ecological systems. Theoretical Ecology, 2(1), 3-12.

Kefi, S., Guttal, V., Brock, W.A., Carpenter, S.R., Ellison, A.M., 
Livina, V.N., et al. (2014). Early Warning Signals of Ecological 
Transitions: Methods for Spatial Patterns. PLoS ONE, 9, e92097.

Sankaran, S., Majumder, S., Kefi, S., and Guttal, V. (2017). Implication 
of being discrete and spatial in detecting early warning signals
of regime shifts. Ecological Indicators.
}
\seealso{
\code{\link{generic_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_spectral_indic_methods.R
\name{extract_spectrum}
\alias{extract_spectrum}
\title{Extract the r-spectrum from objects}
\usage{
extract_spectrum(x, ...)
}
\arguments{
\item{x}{An object produced by \code{spectral_sews} or the result of the 
\code{indictest} function called on such object}

\item{...}{Other arguments are ignored}
}
\value{
The empirical r-spectrum as a \code{data.frame}
}
\description{
Extract the r-spectrum from objects produced by
  \code{spectral_sews}.
}
\examples{

# Extract the r-spectrum after computing indicators
indics <- spectral_sews(serengeti[2:3])
extract_spectrum(indics) 

}
\seealso{
\code{\link{spectral_sews}}, \code{\link{rspectrum}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{raw_moran}
\alias{raw_moran}
\title{Spatial correlation at lag 1}
\usage{
raw_moran(mat)
}
\arguments{
\item{mat}{A matrix containing logical, numeric or integer values}
}
\value{
The Moran's I numeric value as a numeric number.
}
\description{
This function computes the Moran's I index of spatial 
  correlation at lag 1.
}
\details{
This function returns the spatial correlation as measured by 
  the Moran's I index. If the variance of the matrix is zero, then 
  \code{NaN} is returned. This function assumes a 4-way neighborhood (a.k.a. 
  von-Neumann neighborhood), and does not wrap around at the sides of the matrix.
}
\examples{

# Spatial correlation of white noise is close to zero
rmat <- matrix(runif(1000) > .5, ncol = 100)
raw_moran(rmat) 

# Spatial correlation of a half-ones / half-zeros matrix is close to one. 
# This would produce close but inaccurate results in version <3.0.2
m <- cbind(matrix(1, nrow = 100, ncol = 50), 
           matrix(0, nrow = 100, ncol = 50))

raw_moran(m)

}
\seealso{
\code{\link{generic_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_patch_indic_methods.R
\name{patchdistr_sews_predict}
\alias{patchdistr_sews_predict}
\alias{predict.patchdistr_sews_single}
\title{predict method for patchdistr_sews objects}
\usage{
\method{predict}{patchdistr_sews_single}(object, ..., newdata = NULL, best_only = FALSE, xmin_rescale = FALSE)
}
\arguments{
\item{object}{An \code{\link{patchdistr_sews}} object}

\item{...}{Additional arguments (ignored)}

\item{newdata}{A vector of patch sizes at which the fit is returned (default 
to 200 regularly-spaced values).}

\item{best_only}{Return values for only the best fit of each element (matrix)
in \code{object}, or return the values for all fitted distribution.}

\item{xmin_rescale}{If the xmin value used for fits is above one, then setting this
to \code{TRUE} will rescale the predicted probabilities so that they align on 
the cumulative distribution of the observed patch sizes}
}
\value{
A list with component \code{obs}, a \code{data.frame} containing the observed 
  distribution values and \code{pred}, a \code{data.frame} containing the fitted 
  values. Both \code{data.frame}s have columns \code{matrixn}, the number of 
  the matrix for which values are given, \code{type}, the fitted type of distribution, 
  as well as \code{patchsize} and \code{y}, the patch size and value of the
  inverse cumulative distribution function (i.e. \eqn{1 - P(x<k)}).
}
\description{
Export the observed and fitted patch size distributions
}
\details{
The function \code{\link{patchdistr_sews}} fits competing 
  distribution models to the observed patch size distributions. This 
  functions is able to export the observed values and the fitted values 
  altogether.
}
\examples{

\donttest{ 
patch_indics <- patchdistr_sews(forestgap)

predict(patch_indics)

}

}
\seealso{
\code{\link{patchdistr_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_matrix.R
\name{convert_to_matrix}
\alias{convert_to_matrix}
\title{Convert an object to a matrix}
\usage{
convert_to_matrix(object, ...)
}
\arguments{
\item{object}{An object (typically, a matrix or a list of matrices)}

\item{...}{Additional arguments (currently ignored)}
}
\description{
This function is mainly for internal use by the 
  \code{\link{spatialwarnings}} package to convert objects before they are 
  processed by \code{*_sews} functions.
}
\details{
This generic function is here so that other packages can extend it. 
For example, \href{https://github.com/spatial-ews/spatialwarningsGis}{spatialwarningsGis} will provide methods so that GIS objects can be handled 
(e.g. \code{RasterLayer} from package \code{raster}).
}
\examples{

# this does nothing
convert_to_matrix(serengeti[2:3]) 

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_generic_indic.R
\name{raw_cg_moran}
\alias{raw_cg_moran}
\title{Moran's Index at lag of 1}
\usage{
raw_cg_moran(mat, subsize = 1)
}
\arguments{
\item{mat}{A matrix}

\item{subsize}{logical. Dimension of the submatrix used to coarse-grain the 
original matrix (set to 1 for no coarse-graining).}
}
\value{
The Moran's I index measuring autocorrelation at lag 1 as a 
  named vector
}
\description{
This functions computes the Moran's spatial correlation index 
  (with lag one) on a matrix.
}
\examples{

\donttest{
data(serengeti)
raw_cg_moran(serengeti[[1]], subsize = 1)
}

}
\references{
Dakos, V., van Nes, E. H., Donangelo, R., Fort, H., & 
Scheffer, M. (2010). Spatial correlation as leading indicator of 
catastrophic shifts. Theoretical Ecology, 3(3), 163-174.

Legendre, P., & Legendre, L. F. J. (2012). Numerical Ecology.
Elsevier Science.
}
\seealso{
\code{\link{generic_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_variogram_indic.R,
%   R/task_variogram_indic_indictest.R
\name{variogram_sews_plot}
\alias{variogram_sews_plot}
\alias{plot.variogram_sews}
\alias{plot_variogram}
\alias{plot_variogram.variogram_sews}
\alias{plot.variogram_sews_test}
\alias{plot_variogram.variogram_sews_test}
\title{Early-warning signals based on variograms}
\usage{
\method{plot}{variogram_sews}(x, along = NULL, ...)

plot_variogram(x, along = NULL, ...)

\method{plot_variogram}{variogram_sews}(x, along = NULL, ...)

\method{plot}{variogram_sews_test}(x, along = NULL, what = "value", display_null = TRUE, ...)

\method{plot_variogram}{variogram_sews_test}(x, along = NULL, what = "value", display_null = TRUE, ...)
}
\arguments{
\item{x}{An object produced by \code{\link{variogram_sews}}, or the 
result of applying \code{indictest} on such object.}

\item{along}{A vector providing values along which the indicator trends 
will be plotted. If \code{NULL} then the indicator values are plotted
sequentially in their original order.}

\item{...}{Other arguments are ignored.}

\item{what}{The trendline to be displayed. Defaults to the indicator's 
values ("value") but other metrics can be displayed. Accepted values are 
"value", "pval", "difference" (obs - null mean), or "z_score" 
( (obs - null mean) / (null sd) ).}

\item{display_null}{Chooses whether a grey ribbon should be added to reflect
the null distribution. Note that it can not be displayed when the trend 
line reflects something else than the indicator values (when \code{what} 
is not set to "value").}
}
\description{
Plot trends of indicators based on variograms
}
\details{
The \code{plot()} function will display how the estimated 
  variogram parameters change along a set of values (passed with argument
  \code{along}). If the object passed has been processed through 
  \code{indictest}, then the null values are also displayed. 
  \code{plot_variogram()} can be used to display the individual variograms
  that have been fit to the data.
}
\examples{
  
\donttest{ 
serengeti_ews <- variogram_sews(serengeti, model ="exp")

# Display the change in variogram parameters 
plot(serengeti_ews, along = serengeti.rain) + 
  ggplot2::labs(x = "Rainfall (mm)")

# Visualize the fitted variograms
plot_variogram(serengeti_ews, along = serengeti.rain) 

  # Test the trends (nulln should be set to a higher value to obtain 
  # meaningful results
  serengeti_test <- indictest(serengeti_ews, nulln = 19)
  plot(serengeti_test, along = serengeti.rain)
  plot_variogram(serengeti_test, along = serengeti.rain)
}

}
\seealso{
\code{\link{variogram_sews}}, \code{\link{indictest}}, 
  \code{\link{plot_variogram}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indictest.R
\name{indictest}
\alias{indictest}
\title{Significance-assessment of spatial early-warning signals}
\usage{
indictest(x, nulln = 999, null_method = "perm", null_control = NULL, ...)
}
\arguments{
\item{x}{An object such as one produced by the \code{*_sews}
functions, or \code{\link{compute_indicator}}}

\item{nulln}{The number of values to produce the null distribution}

\item{null_method}{The method used to produce the null values (see Details)}

\item{null_control}{List of arguments used to control the generation of 
null matrices. If NULL, then sensible defaults are chosen (see Details)}

\item{...}{Additional arguments are ignored}
}
\value{
An object with a class ending in \code{*_sews_test}, whose exact
  class depends on the input object. \code{plot}, \code{summary} methods are
  available to display the results of computations, and additional methods
  may be available depending on the input object (e.g. see
  \code{\link{patchdistr_sews_plot}}).
}
\description{
Assess the significance of spatial early-warning indicators
}
\details{
\code{indictest} is used to test the significance of early-warning signals
  against 'null matrices', which represent the expected spatial structure 
  in the absence of the biological process of interest. 

For a given indicator, a null distribution is obtained by producing a set 
  of 'null' matrices, from which indicator values are recomputed. This
  produces a null distribution of \code{nulln} indicator values against
  which the observed value is tested. 

Several methods are available to produce the set of null matrices. If 
  \code{null_method} is set to "perm", the original matrix is reshuffled 
  to obtain a null matrix. If \code{null_method} is set to "intercept", then 
  a generalized linear model of the form `y ~ 1` (where y represents the 
  values of the matrix) is fitted, then values are drawn from this model. If 
  \code{null_method} is set to "smooth", then a smooth surface is fitted
  based on a generalized additive model (using \code{\link[mgcv]{gam}}) to
  the matrix, then values are drawn from this model. When using the
  "intercept" or "smooth" null models, it is important to make sure the
  model 'family' corresponds to the type of values present in the matrix. By
  default, if a matrix contains \code{TRUE}/\code{FALSE} values, a `binomial()` 
  family is used, otherwise a `gaussian()` family is used. More information about 
  null models is available in the \href{https://alex.lecairn.org/spatialwarnings-faq.html#Using_advanced_null_models}{spatialwarnings FAQ}. 

Please note that specific null methods may exists for some indicators, such as
\code{\link[=flowlength_sews]{flowlength}}. These are often based on 
analytical approximation and allow faster computations. 

If a matrix has attributes, then these are preserved and passed to the 
  function used to compute the indicator value, except when using the 
  null method 'perm', in which case matrix attributes are discarded. 

The list \code{null_control} can be used to adjust the computation of 
  null matrices. It can have the following components: 
  \describe{ 
    \item{\code{family}}{The family used in the model used to produce the null 
      matrices. Typically, it is one of \code{\link[=stats]{binomial}()}, 
      \code{\link[=stats]{gaussian}()}, etc. }

    \item{\code{qinf}}{The lower quantile to compute from the null distribution 
      and display in summaries/plots. A numeric value between 0 and 1.}

    \item{\code{qsup}}{The upper quantile to compute from the null distribution 
      and display in summaries/plots. A numeric value between 0 and 1.}
  }
}
\references{
Kefi, S., Guttal, V., Brock, W.A., Carpenter, S.R., Ellison, A.M., Livina, 
V.N., et al. (2014). Early Warning Signals of Ecological Transitions: 
Methods for Spatial Patterns. PLoS ONE, 9, e92097
}
\seealso{
\code{\link{generic_sews}}, \code{\link{spectral_sews}}, 
  \code{\link{kbdm_sews}},
  \code{\link{compute_indicator}}, \code{\link{flowlength_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_variogram_indic.R
\name{extract_variogram}
\alias{extract_variogram}
\title{extract_variogram() method for variogram_sews objects}
\usage{
extract_variogram(x, ...)
}
\arguments{
\item{x}{An object produced by \code{variogram_sews}, or
\code{indictest}}

\item{...}{Additional arguments (ignored)}
}
\value{
A data.frame containing the variogram with the distances 
  (column \code{dist}), the empirical semivariance values (\code{gamma}), 
  and if object contains more than one matrix, a column named \code{matrixn}.
}
\description{
Extract the empirical variogram from a \code{variogram_sews} 
  object
}
\examples{

\donttest{ 
vario_indics <- variogram_sews(serengeti)
predict(vario_indics)
vario_test <- indictest(vario_indics, nulln = 19)
predict(vario_test) # same result
}

}
\seealso{
\code{\link{variogram_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_variogram_indic.R
\name{raw_variogram_metrics}
\alias{raw_variogram_metrics}
\title{Variogram parameters}
\usage{
raw_variogram_metrics(
  mat,
  model = "sph",
  nmax = 100000L,
  nbins = 32,
  cutoff = NULL
)
}
\arguments{
\item{mat}{A matrix}

\item{model}{The variogram model to use, either "sph" (for a spherical
model) or "exp" (for an exponential model)}

\item{nmax}{The maximum number of pairs of cells to use when computing the 
variogram}

\item{nbins}{Number of distance bins to use to compute the variogram}

\item{cutoff}{Maximum distance to consider in the variogram. If NULL, then 
a distance equal to one third of the diagonal of the matrix is used}
}
\description{
Compute the nugget, partial sill, correlation range and 
  structural variance on a matrix.
}
\examples{

raw_variogram_metrics(serengeti[[5]])

}
\seealso{
variogram_sews, raw_structvar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lsw_sews.R
\name{lsw_sews}
\alias{lsw_sews}
\alias{raw_patch_radii_skewness}
\alias{raw_lsw_aicw}
\title{Indicators based on the LSW distribution}
\usage{
lsw_sews(mat, wrap = FALSE)

raw_patch_radii_skewness(mat, wrap = FALSE)

raw_lsw_aicw(mat, wrap = FALSE)
}
\arguments{
\item{mat}{A logical matrix (\code{TRUE}/\code{FALSE} values) or a list of such
matrices}

\item{wrap}{Determines whether patches are considered to wrap around the 
matrix when reaching on of its edges}
}
\value{
\code{lsw_sews} returns an object of class \code{simple_sews_single}
  (a list) if \code{mat} is a single matrix or an object of class 
  \code{simple_sews_list} if \code{mat} is a list. You probably want to use some  
  of the methods written for these complicated objects instead of extracting 
  values directly (they are displayed using \code{print(<object>)}).
}
\description{
LSW indicators for systems with density-dependent aggregation
}
\details{
In systems where a mobile resource or consumer can be fixed in space by a sessile 
species, specific patterns are expected to appear. Such systems can include situations 
where nutrients available in the environmeent are fixed by a sessile species (e.g. 
seagrasses or corals), or where the behavior of herbivores is altered to restrict 
them to certain areas (see full theoretical background in Siteur et al. 2023). 

In those systems, as environmental conditions change and the global density of 
the sessile species decreases, its spatial structure is expected to change. The area 
of patches of the sessile species (as measured by their radii, which assumes circular
patches), is expected to go from a log-normal to a  Lifshitz–Slyozov–Wagner (LSW)
distribution. Thus, measuring how close the observed distribution of radii are to 
those two candidate distributions can constitute an indicator of ecosystem 
degradation. 

This function measures this through the relative support based on AIC for the two 
distributions (equal to 1 when the empirical distribution is best-approximated by an 
LSW, and 0 when it is a log-normal distribution (\code{\link{dlnorm}}), and the 
skewness of the observed patch radii, which should approach a value around -0.92 as
conditions worsen.
}
\examples{

data(dda)
data(dda.pars)

# Compute all indicators at once (skewness and relative AIC support)
indics <- lsw_sews(dda)
plot(indics, along = dda.pars[ ,"tau"]) 

# Compute individual indicators 

# Skewness of the distribution of patch radii
radii_skewness <- compute_indicator(dda, raw_patch_radii_skewness)
plot(radii_skewness, along = dda.pars[ ,"tau"])

# Aic weight of LSW distribution relative to a lognormal distribution. tau here 
# represents the density at equilibrium in Siteur et al's model (2023)
lsw_aicw <- compute_indicator(dda, raw_lsw_aicw)
plot(lsw_aicw, along = 1 - dda.pars[ ,"tau"])

}
\references{
Siteur, Koen, Quan-Xing Liu, Vivi Rottschäfer, Tjisse van der Heide, Max Rietkerk, 
Arjen Doelman, Christoffer Boström, and Johan van de Koppel. 2023. 
"Phase-Separation Physics Underlies New Theory for the Resilience of Patchy 
Ecosystems." Proceedings of the National Academy of Sciences 120 (2): e2202683120.
https://doi.org/10.1073/pnas.2202683120.
}
\seealso{
\code{\link{dLSW}}, \code{\link{dda}},
  \code{\link{raw_patch_radii_skewness}}, \code{\link{raw_lsw_aicw}}
}
\author{
This code has received contributions from Koen Siteur
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitpsd.R
\name{pl_fit}
\alias{pl_fit}
\alias{exp_fit}
\alias{lnorm_fit}
\alias{tpl_fit}
\title{Distribution-fitting functions}
\usage{
pl_fit(dat, xmin = 1)

exp_fit(dat, xmin = 1)

lnorm_fit(dat, xmin = 1)

tpl_fit(dat, xmin = 1)
}
\arguments{
\item{dat}{The set of values to which the distribution are fit}

\item{xmin}{The minimum possible value to consider when fitting the
distribution}
}
\value{
A list containing at list the following components: 

\describe{ 
   \item{\code{type}}{The type of distribution fitted (as a character string)}
   \item{\code{method}}{The method used for the fit - here, maximum likelihood, 'll'}
   \item{\code{ll}}{The log likelihood at the estimated parameter values}
   \item{\code{xmin}}{The value of xmin used for the fit}
   \item{\code{npars}}{The number of parameters of the distribution}
 }

Additionally, this list may have one or more of the following elements depending on 
  the type of distribution that has been fitted: 
  \describe{ 
    \item{\code{plexpo}}{The exponent of the power-law}
    \item{\code{cutoff}}{The rate of truncation, for truncated power law and 
       exponential fits}
    \item{\code{meanlog}}{The mean of the lognormal distribution}
    \item{\code{sdlog}}{The s.d. of the lognormal distribution}
  }
}
\description{
These functions fit parametric distributions to a set of
  discrete values.
}
\details{
These functions will fit distributions to a set of values using 
  maximum-likelihood estimation. In the context of the 'spatialwarnings' 
  package, they are most-often used to fit parametric distributions on patch
  size distributions. As a result, these functions assume that the data 
  contains only integer, strictly positive values. The type of distribution
  depends on the prefix of the function: 'pl' for power-law, 'tpl' for
  truncated power-law, 'lnorm' for lognormal and 'exp' for an exponential
  distribution. 

In the context of distribution-fitting, 'xmin' represents the minimum value 
  that a distribution can take. It is often used to represent the minimum 
  scale at which a power-law model is appropriate (Clauset et al. 2009), and 
  can be estimated on an empirical distribution using
  \code{\link{xmin_estim}}. Again, please note that the fitting procedure 
  assumes here that xmin is equal or grater than one.

Please note that a best effort is made to have the fit converge, but 
  it may sometimes fail when the parameters are far from their usual 
  range, and numerical issues may occur. It is good practice to make 
  sure the fits are sensible when convergence warnings are reported.

For reference, the shape of the distributions is as follow: 

\describe{
  \item{power-law}{\eqn{x^{-a}}{x^(-a)} where a is the power-law exponent}
  \item{exponential}{\eqn{exp(-bx)}{exp(-bx)} where b is the truncation rate
          of the exponential}
  \item{truncated power-law}{\eqn{x^{-a}exp(-bx)}{x^(-a)exp(-bx)} where a
    and b are the exponent of the power law and the rate of truncation}
}

The lognormal form follows the \link[=dlnorm]{standard definition}.

The following global options can be used to change the behavior of fitting functions 
  and/or produce more verbose output: 
\describe{ 
  \item{spatialwarnings.constants.reltol}{the relative tolerance to use to compute 
    the power-law normalizing constant
    \deqn{sum_{k=1}^{\infty} x^{ak}e^{-bk}}{sum( x^(ak)exp(-bk)) for k in 1:Inf}. 
    Increase to increase the precision of this constant, which can be useful in some 
    cases, typically with large sample sizes. Default is 1e-8.}
  \item{spatialwarnings.constants.maxit}{the maximum number of iterations to compute 
    the normalizing constant of a truncated power-law. Increase if you get a warning 
    that the relative tolerance level (defined above) was not reached. Default is 1e8}
  \item{spatialwarnings.debug.fit_warn_on_bound}{logical value. Warn if the fit is 
    at the boundary of the valid range for distribution parameter}
  \item{spatialwarnings.debug.fit_warn_on_NA}{logical value. Warn if the returned fit 
    has \code{NA}/\code{NaN} parameters}
}
}
\examples{

# Fit an exponential model to patch size distribution 
exp_fit(patchsizes(forestgap[[8]]))
 
# Use the estimated parameters as an indicator function
\donttest{

get_truncation <- function(mat) { 
   c(exp_cutoff = exp_fit(patchsizes(mat))$cutoff)
}
trunc_indic <- compute_indicator(forestgap, get_truncation)
plot(trunc_indic)
plot(indictest(trunc_indic, nulln = 19))

}

}
\references{
Clauset, Aaron, Cosma Rohilla Shalizi, and M. E. J. Newman. 2009. “Power-Law
Distributions in Empirical Data.” SIAM Review 51 (4): 661–703. 
https://doi.org/10.1137/070710111.
}
\seealso{
\code{\link{patchdistr_sews}}, \code{\link{xmin_estim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_spectral_indic.R
\name{raw_sdr}
\alias{raw_sdr}
\title{Spectral Density Ratio (SDR) indicator}
\usage{
raw_sdr(mat, sdr_low_range = NULL, sdr_high_range = NULL)
}
\arguments{
\item{mat}{A matrix with continuous values, or a logical matrix 
(\code{TRUE}/\code{FALSE} values).}

\item{sdr_low_range}{The range of values (in proportion) to 
use for the computation of the spectral density ratio.
For example, for the lowest 20\% (default value), set \code{sdr_low_range}
to \code{c(0, .2)}.}

\item{sdr_high_range}{The range of values (in proportion) to 
use for the computation of the spectral density ratio. For example, for 
the highest 20\% (default value), set \code{sdr_high_range} to 
\code{c(.8, 1)}.}
}
\value{
The SDR values computed on the matrix as a named vector
}
\description{
Compute the ratio of low frequencies over high frequencies
  of the r-spectrum.
}
\details{
SDR measures the increase in long-range correlations before a critical point. 
  It is the ratio of the average low frequency value over high frequency 
  values. In this implementation, an increase in SDR implies a "reddening" 
  of the \link[=rspectrum]{r-spectrum}. See also \code{\link{spectral_sews}} for 
  a more complete description. 

Low and high frequencies are averaged in order to compute the SDR. The 
  parameters \code{sdr_low_range} and \code{sdr_high_range} control which 
  frequencies are selected for averaging. For example 
  \code{sdr_low_range = c(0, .2)} (default) uses the lower 20% to compute 
  the average of low frequencies. \code{sdr_high_range = c(.8, 1)} uses the 
  higher 20% for the average of high frequencies.
}
\examples{

\donttest{ 
data(serengeti)
serengeti.sdr <- raw_sdr(serengeti[[1]], 
                         sdr_low_range = c(0, 0.2), 
                         sdr_high_range = c(0.8, 1))
compute_indicator(serengeti, raw_sdr)
}

}
\references{
Carpenter, S.R. & Brock, W.A. (2010). Early warnings of regime shifts in 
  spatial dynamics using the discrete Fourier transform. Ecosphere
}
\seealso{
\code{\link{indictest}}, 
  \code{\link{rspectrum}}, \code{\link{plot_spectrum}}, 
  \code{\link{spectral_sews}}, \code{\link{extract_spectrum}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_flowlength.R
\name{raw_flowlength_uniform}
\alias{raw_flowlength_uniform}
\title{Flow length (uniform slope)}
\usage{
raw_flowlength_uniform(mat, slope, cell_size)
}
\arguments{
\item{mat}{The input matrix (must be a logical matrix)}

\item{slope}{The slope of the area documented by the matrix (in degrees).}

\item{cell_size}{The horizontal size of a cell in the matrix (as viewed 
from above).}
}
\value{
A named vector of length 1 containing the flow length numerical 
  value
}
\description{
Compute a simple approximation of the flow length assuming a 
  constant slope
}
\details{
This function computes the Flowlength of a given matrix, using a 
    uniform approximation (the slope is constant across the whole matrix, 
    with maximum slope being from the top of the matrix to its bottom), 
    as per Rodriguez et al. (2017). See \code{\link{flowlength_sews}} for 
    more details.
}
\examples{

\donttest{ 
raw_flowlength_uniform(arizona[[1]], slope = 20, cell_size = 1)
}


}
\references{
Rodriguez, F., A. G. Mayor, M. Rietkerk, and S. Bautista. 2017. A null model 
  for assessing the cover-independent role of bare soil connectivity as 
  indicator of dryland functioning and dynamics. Ecological Indicators.
}
\seealso{
\code{\link{flowlength_sews}}

\code{\link{indictest}}, to test the significance of indicator values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{rspectrum}
\alias{rspectrum}
\title{r-spectrum}
\usage{
rspectrum(mat)
}
\arguments{
\item{mat}{A matrix with logical or numeric values}
}
\value{
A data.frame with two columns: \code{dist}, the wave number and 
  \code{rspec}, the normalized value of the r-spectrum
}
\description{
Compute the r-spectrum of a matrix
}
\details{
This functions returns a \code{data.frame} with \code{NA}s in the
  \code{rspec} column if the input matrix has zero variance. Note that if the matrix 
  is not square, then only the largest square matrix fitting in the upper left
  corner is used.
}
\examples{

# Spectrum of white noise
rmat <- matrix(runif(100*100) > .5, ncol = 100)
spec <- rspectrum(rmat) 
plot(spec, type = "l")

# Add some spatial correlation and compare the two spectra
rmat.cor <- rmat
for (i in seq(1, nrow(rmat)-1)) { 
  for (j in seq(1, nrow(rmat)-1)) { 
    rmat.cor[i,j] <- mean(rmat[(i-1):(i+1), (j-1):(j+1)])
  }
}
spec.cor <- rspectrum(rmat.cor)
plot(spec.cor, type = "n")
lines(spec, col = "black")
lines(spec.cor, col = "blue")

}
\seealso{
\code{\link{spectral_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{forestgap}
\alias{forestgap}
\alias{forestgap.pars}
\title{A list of binary matrices and their associated parameters}
\format{
A list of logical matrices which are the end results of simulations 
  from Kubo's Forest Gap model along a gradient of increasing values of 
  stress (see references).

The parameters used for the simulations, as a data frame.
}
\source{
Generated using the implementation of Kubo's model in caspr 0.2.0 
  \url{https://github.com/fdschneider/caspr}.
}
\usage{
forestgap

forestgap.pars
}
\description{
A list of binary matrices and their associated parameters
}
\details{
Kubo's forest gap model has three parameters, \eqn{\alpha}{alpha} 
  that controls the reproductive rate of trees, \eqn{d}{d} controls the 
  non-spatialized mortality and \eqn{\delta}{delta} the increased mortality 
  due to the presence of a neighboring gap.
}
\references{
Kubo, T., Iwasa, Y., & Furumoto, N. (1996). Forest spatial dynamics with gap
  expansion: Total gap area and gap size distribution. Journal of Theoretical
  Biology, 180(3), 229-246. \doi{10.1006/jtbi.1996.0099}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustering.R
\name{raw_clustering}
\alias{raw_clustering}
\alias{pair_counts}
\title{Clustering of pairs}
\usage{
raw_clustering(mat, wrap = TRUE, use_8_nb = FALSE)

pair_counts(mat, wrap = TRUE, use_8_nb = FALSE, prop = TRUE)
}
\arguments{
\item{mat}{A matrix, usually with discrete values (logical, integers, etc.)}

\item{wrap}{Whether space should be considered to wrap at the edges}

\item{use_8_nb}{Set to \code{TRUE} to use an 8-way neighborhood. The default is set
to FALSE, which uses a 4-way neighborhood}

\item{prop}{Return the counts for all pairs in the matrix (FALSE), or
the proportions for each pair (out of all possible pairs)}
}
\value{
A vector with the requested clutering values for \code{raw_clustering},
    whose names are equal to each state (unique value) found in the
    original matrix, preceded by 'clust_' (to make sure names are
    compatible with other functions).

 \code{pair_counts} returns the counts of pairs of states in the matrix,
    or the proportion of each pair, depending on the value of \code{prop}
}
\description{
Compute the number of pairs of neighbor cells in a landscape and derive 
  from it clustering indices
}
\details{
The clustering of pairs is defined as the density of pairs, i.e.
    the proportion of all neighboring pairs of cells that share the same state, 
    divided by the null expectation given a random, homogeneous spatial structure.

  For example, let's consider a matrix with two states, 'a' and 'b'.
    \code{raw_clustering} will count all pairs of cells 'a-a' or 'b-b', and
    divide this by the total number of pairs. This proportion is then again
    divided by the probability of obtaining these proportion of pairs under
    the assumption of no spatial structure (random mixing of cells in the
    matrix). 

  Clustering is equal to one when there is no spatial structure. It is
    above one when two states are found next to each other (i.e. cluster)
    more than expected by chance. Values below one means that those two
    states tend to be neighbors less frequently than expected by chance. 

  If you are only interested in the proportion of pairs for each combination
    of states, you can use the function \code{pair_counts}, which returns
    a matrix with as many rows and columns as there are states in the matrix, and 
    contains the counts for all possible pairs of cells found in the matrix 
    (or their relative proportions).
}
\examples{

# The clustering of a random matrix is close to one
ls <- 100 # lattice size
mm <- matrix(sample(c("sp1", "sp2", "sp3", "sp4"), size = ls^2, replace = TRUE),
             nrow = ls, ncol = ls)
clust <- raw_clustering(mm, wrap = TRUE, use_8_nb = TRUE)
print(clust)

# Compute clustering along the gradient for the serengeti dataset
\donttest{

data(forestgap)
clust_indic <- compute_indicator(serengeti, raw_clustering,
                                 wrap = TRUE, use_8_nb = FALSE)
# The interesting one is the clustering of state 0 (FALSE in the original matrix),
# which corresponds to grassland pixels, which get more and more clustered with 
# increasing rainfall (see also ?generic_sews for how that compares with generic 
# indicators)
plot(clust_indic, along = serengeti.rain)

# Add null trend
clust_test <- indictest(clust_indic, nulln = 19)
plot(clust_test, along = serengeti.rain)

# Show the proportion of each pairs of states in the matrix... 
pair_counts(serengeti[[5]])

# ... or the total count
pair_counts(serengeti[[5]], prop = FALSE)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_variogram_indic.R
\name{variogram_sews_predict}
\alias{variogram_sews_predict}
\alias{predict.variogram_sews_list}
\title{predict() method for variogram_sews objects}
\usage{
\method{predict}{variogram_sews_list}(object, newdist = NULL, ...)
}
\arguments{
\item{object}{An object produced by variogram_sews}

\item{newdist}{A vector of distances at which to return the variogram fit 
values (defaults to 128 regularly-spaced values).}

\item{...}{Additional arguments (ignored)}
}
\value{
A data.frame with the distances (column \code{dist}), the fitted 
  values (\code{gamma}), and if object contains more than one matrix, 
  a column \code{matrixn}.
}
\description{
Export the fitted variogram(s)
}
\examples{

\donttest{ 
vario_indics <- variogram_sews(serengeti)
predict(vario_indics)
vario_test <- indictest(vario_indics, nulln = 19)
predict(vario_test) # same result
}

}
\seealso{
\code{\link{variogram_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label.R
\name{patchsizes}
\alias{patchsizes}
\title{Get patch sizes.}
\usage{
patchsizes(mat, merge = FALSE, nbmask = "von_neumann", wrap = FALSE)
}
\arguments{
\item{mat}{A logical matrix or a list of such matrices.}

\item{merge}{Controls whether the obtained patch size distributions are to 
be pooled together if \code{mat} is a list of matrices.}

\item{nbmask}{Either "moore" for 8-way neighborhood, "von_neumann" for four-way 
neighborhood (default), or a 3x3 matrix describing which neighbors to 
consider around a cell. See \code{\link{patchsizes}} for details on how to specify 
such neighborhoods.}

\item{wrap}{Whether to wrap around lattice boundaries (`TRUE`/`FALSE`), 
effectively using periodic boundaries.}
}
\value{
If mat is a logical matrix, then the function returns a vector of 
  patch sizes. If mat is a list of logical matrices, then it returns 
  a list of vectors of patch sizes: this list is flattened if merge is TRUE.
}
\description{
Get the distribution of patch sizes from a logical matrix
}
\examples{

data(forestgap)
patchsizes(forestgap[[5]]) # Use a single matrix

# Compute the average patch size of each matrix
list_patches <- patchsizes(forestgap) # get the patch size for each matrix
print( sapply(list_patches, mean)) # print the average patch size 

# Example with 8-way neighborhood
patchsizes(forestgap[[5]], nbmask = "moore")

# Same neighborhood as above, but specified in matrix form 
moore_nb <- matrix(c(1, 1, 1, 
                     1, 0, 1, 
                     1, 1, 1), 
                   nrow = 3, ncol = 3, byrow = TRUE)
patchsizes(forestgap[[5]], nbmask = moore_nb) 

}
\seealso{
\code{\link{label}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simple_indic_indictest.R,
%   R/simple_indic_methods.R
\name{plot.simple_sews_test}
\alias{plot.simple_sews_test}
\alias{plot.simple_sews_list}
\title{Spatial early-warning signals: display of trends}
\usage{
\method{plot}{simple_sews_test}(x, along = NULL, what = "value", display_null = TRUE, ...)

\method{plot}{simple_sews_list}(x, along = NULL, ...)
}
\arguments{
\item{x}{A \code{simple_sews_test} object (as provided by **_sews functions, such 
as \code{generic_sews()}}

\item{along}{A vector providing values over which the indicator trend 
will be plotted. If \code{NULL} then the values are plotted sequentially 
in their original order.}

\item{what}{The trendline to be displayed. Defaults to the indicator's 
values ("value") but other metrics can be displayed. Accepted values are 
"value", "pval", "difference" (obs - null mean), or "z_score" 
( (obs - null mean) / (null sd) ).}

\item{display_null}{Chooses whether a grey ribbon should be added to reflect
the null distribution. Note that it can not be displayed when the trend 
line reflects something else than the indicator values (when \code{what} 
is not set to "value").}

\item{...}{Other arguments are ignored.}
}
\description{
Spatial early-warning signals: display of trends
}
\details{
Note that the produced plot is adjusted depending on whether 
  \code{along} is numeric or not.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_patch_indic_methods.R
\name{patchdistr_sews_plot}
\alias{patchdistr_sews_plot}
\alias{plot.patchdistr_sews}
\alias{plot_distr}
\title{Early-warning signals based on patch size distributions}
\usage{
\method{plot}{patchdistr_sews}(x, along = NULL, ...)

plot_distr(x, along = NULL, best_only = TRUE, plrange = TRUE)
}
\arguments{
\item{x}{An object as produced by \code{\link{patchdistr_sews}}, or 
the result of \code{\link{indictest}} called on such object}

\item{along}{A vector providing values along which the indicator trends 
will be plotted. If \code{NULL} then the values are plotted sequentially 
in their original order.}

\item{...}{Ignored}

\item{best_only}{Plot only the best fit the empirical (inverse cumulative) 
patch-size distribution with an overlay of the estimated fits.}

\item{plrange}{Plot the power-law range}
}
\description{
Plot early-warning signals based on patch size distributions
}
\details{
The \code{plot} function will produce a figure summarizing the changes
  in patch size distributions along a set of values. The figure has two 
  panels: 
  \itemize{ 
     \item the upper panel shows the percolation status of empty 
       (\code{FALSE}) and occupied cells (\code{TRUE}), and shows the mean 
       value (proportion of \code{TRUE} values). The background shows 
       the proportion of each type of distribution for each unique values 
       of the \code{along} vector. 
       
     \item the bottom panel displays the power-law range
  }
 
 The \code{plot_distr} function displays each distribution in an 
   individual facet, with an overlay of the best distribution fit and a blue 
   bar showing the power-law range. If appropriate, a grey ribbon is used to
   display the expected distribution given the null expectation (i.e. when 
   \code{plot_distr} is called on the results of \code{indictest()}. This 
   function can produce quite crowded graphs, but it displays in full the
   shape of the distributions, and can be useful e.g. to assess the quality 
   of the fits.
}
\examples{

\donttest{ 
data(forestgap)
psd_indic <- patchdistr_sews(forestgap)

plot(psd_indic, along = forestgap.pars[ ,"d"]) 

# When along is non-numeric, bars are used for display
plot(psd_indic, along = as.factor(forestgap.pars[ ,"d"]))

# Display individual distributions
plot_distr(psd_indic, along = forestgap.pars[ ,"d"])

# We can display the distributions along with the null expectation after 
# indictest() is run
psd_test <- indictest(psd_indic, nulln = 19)
plot_distr(psd_test, along = forestgap.pars[ ,"d"])
}

}
\seealso{
\code{\link{patchdistr_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extra_doc.R
\name{simple_sews}
\alias{simple_sews}
\alias{simple_sews_object}
\title{\code{simple_sews} objects}
\description{
This help page describes the structure of \code{simple_sews_*} 
  objects, such as those defined by the classes \code{simple_sews_single},
  \code{simple_sews_list}
}
\details{
The \code{\link{spatialwarnings}} uses S3 objects (lists) internally to store 
  indicator values, along with the necessary data to plot and display
  results. It is not recommended to extract data directly from these objects,
  as they are subject to change with different releases of the package. The 
  preferred method is to use dedicated generic functions such as 
  \code{plot()} or \code{as.data.frame()} to display or export the results.
  Nonetheless, we document the structure of these objects here for reference. 

\code{simple_sews} objects are returned by all indicator functions that 
  return numeric values. This includes for example \code{\link{generic_sews}}, 
  \code{\link{flowlength_sews}}, \code{\link{compute_indicator}} but *not*
  \code{\link{patchdistr_sews}} or \code{\link{spectral_sews}}, which provide
  indicators that depend on non-numeric values (e.g. patch-size distribution
  type), or need to store more information than just a single numerical value
  (e.g. the spectrum of the input matrix). 

\code{simple_sews} objects come in multiple variants:
  \code{simple_sews_single} is the result of an indicator function applied to 
  a single matrix, and \code{simple_sews_test_single} is the result of 
  \code{\link{indictest}} applied to a \code{simple_sews_single} object. Both 
  these objects have list equivalents, \code{simple_sews_list} and 
  \code{simple_sews_test_list} which are simply a collection of their 
  'single' equivalent. These 'list' objects are used to store the results 
  of computations when working with multiple matrices. 

A \code{simple_sews_single} object is a list with the following components
  \describe{
    \item{\code{itemvalue}}{the indicator values. A vector of length one if there is 
      only one numeric value returned by the indicator function (e.g.
      \code{\link{flowlength_sews}}, or with a length above one otherwise}
    \item{\code{orig_data}}{the original matrix on which the indicator was computed}
    \item{\code{fun.args}}{the argument used in the call to the indicator function 
      (the function that given a matrix, returns the spatial metrics of
      interest)}
    \item{\code{taskname}}{a character string describing the current indicator(s) 
      being computed}
    \item{\code{indicf}}{the indicator function, which given the matrix, returns the
      spatial metric(s) of interest}
  }

  \code{simple_sews_test_single} have all of the above components, plus 
the following: 

\describe{ 
  \item{\code{nulldistr}}{the null distribution of values, with nulln rows and 
    as many columns as the number of values returned by the indicator 
    function}
  \item{\code{null_mean}}{the mean indicator values in the null distribution}
  \item{\code{null_sd}}{the standard deviation of the null distribution}
  \item{\code{null_qsup}}{the upper quantile of the null distribution, by default 
    the 95% upper quantile, but see \code{\link{indictest}} for a way to 
    adjust this}
  \item{\code{null_qinf}}{the upper quantile of the null distribution, by default 
    the 05% upper quantile, but see \code{\link{indictest}} for a way to 
    adjust this}
  \item{\code{z_score}}{the z_score of the observed value relative to the null 
    distribution, i.e. (value - null_mean) / null_sd}
  \item{\code{pval}}{the p-value of the indicator, i.e. the proporation of values 
    of the null distribution that fall below the observed indicator value}
  \item{\code{null_method}}{the method used to produce the null matrices. See 
    \code{\link{indictest}} for details}
  \item{\code{nulln}}{the number of null matrices used}
  \item{\code{get_nullmat}}{a function that can be called to obtain a randomized 
    matrix}
  \item{\code{matrixn}}{the number of the matrix, can be above one if the
    computations have been run on a list of matrices, or non-existent if 
    only one matrix was used}
}
}
\seealso{
\code{\link{custom_indicator}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_generic_indic.R
\name{raw_cg_skewness}
\alias{raw_cg_skewness}
\title{Skewness indicator}
\usage{
raw_cg_skewness(mat, subsize = 5, absolute = TRUE)
}
\arguments{
\item{mat}{A matrix. The matrix 
values can be logical, with \code{FALSE} (empty) or \code{TRUE} (occupied) 
values. The entries can also be continuous (like NDVI or EVI data).}

\item{subsize}{Dimension of the submatrix used to coarse-grain the 
original matrix. This must be an integer less than size of the full 
matrix. Coarse-graining reduces the size of the matrix by a factor 
\code{subsize} in each dimension of the matrix. Skewness is calculated 
on the coarse-grained matrix.}

\item{absolute}{Should the function return the absolute value or raw value 
of skewness ?}
}
\value{
The spatial skewness of the matrix as a named vector
}
\description{
Compute the spatial skewness of spatial data (a matrix).
}
\details{
Spatial skewness is a measure of fluctuations in space; specifically, it 
measures if fluctuations are getting biased (skewed) in one direction. Based 
on the theory of critical slowing down, when systems approach critical 
points they are expected to show increased fluctuations in space. Thus, 
increasing spatial skewness is proposed as an early warning signal of 
impending critical transitions. 

Computing spatial skewness is straightforward. However, detecting trends of 
skewness that correspond to critical slowing down can be tricky, especially 
if data come from discrete classification of state variable.

Many high resolution spatial data are classified as FALSE (empty) 
or TRUE (occupied by plant). In such cases, spatial skewness captures just 
the skewness in data, but not that of spatial structure. 
To resolve the issue, this function employs a method called coarse-graining, 
proposed in Kefi et al (2014), and described in detail in 
Sankaran et al. (2017). One must specify a subsize above one for 
binary valued data sets to obtain meaningful values. 

\code{subsize} has to be an integer. It has to be less than or equal to 
half of matrix size (N). \code{subsize} must also be preferably a 
divisor of N. If it is not a divisor of N, the remainder rows and columns 
are discarded when computing coarse-graining matrices. 

Null model evaluations are also done on coarse-grained matrices.
}
\examples{

data(serengeti)
\donttest{
raw_cg_skewness(serengeti[[1]])
compute_indicator(serengeti, fun = raw_cg_skewness, subsize = 5)
}

}
\references{
Guttal, V., and Jayaprakash, C. (2009). Spatial variance and 
spatial skewness: leading indicators of regime shifts in spatial 
ecological systems. Theoretical Ecology, 2(1), 3-12.

Kefi, S., Guttal, V., Brock, W.A., Carpenter, S.R., Ellison, A.M., 
Livina, V.N., et al. (2014). Early Warning Signals of Ecological 
Transitions: Methods for Spatial Patterns. PLoS ONE, 9, e92097.

Sankaran, S., Majumder, S., Kefi, S., and Guttal, V. (2017). Implication of 
being discrete and spatial in detecting early warning signals of regime 
shifts. Ecological indicators.
}
\seealso{
\code{\link{generic_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_spectral_indic_methods.R
\name{plot_spectrum}
\alias{plot_spectrum}
\title{Display the r-spectrum of a \code{spectral_sews} object}
\usage{
plot_spectrum(x, along = NULL, log = TRUE, display_null = TRUE, ...)
}
\arguments{
\item{x}{An object produced by \code{\link{spectral_sews}} or the result 
returned by \code{\link{indictest}} applied on such object}

\item{along}{A vector providing numerical or categorical values along 
which the indicator trends will be plotted. If \code{NULL}, then the
indicator values are plotted sequentially in their original order.}

\item{log}{Whether to use a log scale or a linear scale on the y axis}

\item{display_null}{Whether to display null information. This argument is 
ignored if \code{x} has not been produced through \code{\link{indictest}} 
(and thus does not contain data regarding the null model)}

\item{...}{Other arguments are ignored}
}
\description{
Display the r-spectrum (or multiple spectra) that are contained
  in an object returned by \code{\link{spectral_sews}} object (or the result 
  of \code{\link{indictest}} applied on such object).
}
\seealso{
\code{\link{rspectrum}}, \code{\link{spectral_sews}}, 
  \code{\link{extract_spectrum}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_spectral_indic.R
\name{spectral_sews}
\alias{spectral_sews}
\title{Spectrum-based spatial early-warning signals.}
\usage{
spectral_sews(mat, sdr_low_range = NULL, sdr_high_range = NULL, quiet = FALSE)
}
\arguments{
\item{mat}{The input matrix or a list of matrices.}

\item{sdr_low_range}{The range of values (in proportion) to 
use for the computation of the spectral density ratio.
For example, for the lowest 20\% (default value), set \code{sdr_low_range} to 
\code{c(0, .2)}. See also the Details section.}

\item{sdr_high_range}{The range of values (in proportion) to 
use for the computation of the spectral density ratio. For example, for 
the higher 20\% (default value), set \code{sdr_high_range} to 
\code{c(.8, 1)}. See also the Details section.}

\item{quiet}{Do not display some warnings}
}
\value{
Function \code{spectral_sews} object of class \code{spectral_sews_list} or 
  \code{spectral_sews_single} depending on whether the input was a list of 
  matrices or a single matrix. 

Function \code{indictest} 

The \code{plot} methods returns a ggplot object (usually displayed 
immediately when called interactively).
}
\description{
Computation of spatial early warning signals based on spectral
  properties.
}
\details{
Spectral early warning signals are based on the fact that some 
  dynamical systems can exhibit an change in specific characteristics of their 
  spatial structure when approaching a transition. In particular, long-range 
  correlations are expected to have an increased importance. This is expected 
  to be reflected in the spectrum of the spatial structure as an increase of the
  relative importance of lower frequencies over higher frequencies ("reddening" 
  of the spatial spectrum).
  
This task allows computing the radial-spectrum which gives the relative 
  importance of each space scale as a function of distance, from 1 to 
  \code{N/2} (\code{N} being the minimum between the number of rows and columns). 
  If the matrix is not square, then it is cropped to biggest square that 
  fits within the left side of the matrix. 

Additionally, it summarizes this spectrum into a Spectral 
  Density Ratio (SDR), which is the ratio of low frequencies over 
  high frequencies of the r-spectrum. The SDR value is expected to increase
  before a transition point.

The significance of spectral early-warning signals can be estimated by 
  reshuffling the original matrix (function \code{indictest}). Indicators 
  are then recomputed on the shuffled matrices and the values obtained are 
  used as a null distribution. P-values are obtained based on the rank of 
  the observed value in the null distribution. 

The trend of SDR values can be plotted using the \code{plot()} method. 
  Alternatively, the spectrum itself can be plotted (with facets 
  if multiple input matrices were used) using the \code{plot_spectrum} 
  method.
}
\examples{
\donttest{

data(serengeti) 
data(serengeti.rain) 


spec_indic <- spectral_sews(serengeti, 
                            sdr_low_range  = c(0, .2), 
                            sdr_high_range = c(.8, 1))

summary(spec_indic)

# Display trends along the varying model parameter
plot(spec_indic, along = serengeti.rain)

# Computing spectra many times is expensive, consider setting parallel 
# computing using: options(mc.cores = n)

# Assess significance
spec_test <- indictest(spec_indic, nulln = 199)
summary(spec_test)

# Display the SDR trend, now with a grey ribbon representing 5\%-95\% 
# quantiles of the null distribution
plot(spec_test, along = serengeti.rain)

# Add a line highlighting the shift 
if (require(ggplot2)) {
  plot(spec_test, along = serengeti.rain) + 
    geom_vline(xintercept = 590, color = "red", linetype = "dashed")
}


# Display radial-spectra
plot_spectrum(spec_indic, along = serengeti.rain)

# Graphics can be modified using ggplot2 functions
if (require(ggplot2)) { 
  plot_spectrum(spec_indic, along = serengeti.rain) + 
    scale_y_log10()
}

}
}
\references{
Kefi, S., Guttal, V., Brock, W.A., Carpenter, S.R., Ellison, A.M., 
  Livina, V.N., et al. (2014). Early Warning Signals of Ecological 
  Transitions: Methods for Spatial Patterns. PLoS ONE, 9, e92097.
}
\seealso{
\code{\link{rspectrum}}, \code{\link{plot_spectrum}}, 
  \code{\link{raw_sdr}}, \code{\link{extract_spectrum}}

\code{\link{indictest}}, to test the significance of indicator values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicator_plrange.R
\name{indicator_plrange}
\alias{indicator_plrange}
\title{Power-law range indicator}
\usage{
indicator_plrange(mat, merge = FALSE, xmin_bounds = NULL)
}
\arguments{
\item{mat}{A logical matrix, or a list of logical matrices}

\item{merge}{Controls whether the patch-size distributions of the input 
matrices are merged together before computing the power-law range. Setting 
this value to \code{TRUE} makes the function return a single value even 
if multiple matrices are given as input.}

\item{xmin_bounds}{A vector of two integer values, defining a range in which 
to search for the best xmin (see Details).}
}
\value{
A \code{data.frame} with columns \code{minsize}, \code{maxsize} which are the
  observed minimum and maximum patch sizes, the estimated \eqn{x_{min}} as column
  \code{xmin} and the value of the power-law range as \code{plrange}. If multiple
  matrices were provided, then a list of data.frames is returned
}
\description{
Compute the power-law range of a matrix
}
\details{
Some ecosystems show typical changes in their patch-size 
distribution as they become more and more degraded. In particular, an 
increase in the truncation of the patch-size distribution (PSD) is expected 
to occur. The power-law range (PLR) measures the truncation of the PSD 
in a single value (see also \code{\link{patchdistr_sews}} for more details). 

To compute the PLR, power-laws are fitted with a variable 
minimum patch size (\eqn{x_{min}}) and the one with the lowest Kolmogorov-Smirnov
distance to the empirical distribution is retained. PLR is then computed 
using this best-fitting \eqn{x_{min}} as: 

\deqn{\frac{log(x_{max}) - log(x_{min})}{log(x_{max}) - log(x_{smallest})}}{ (log(xmax) - log(xmin))/(log(xmax) - log(xsmallest))}

where \eqn{x_{max}}{x_max} is the maximum observed patch size, and 
\eqn{x_{smallest}}{x_smallest} is the minimum observed patch size.
}
\examples{
\donttest{
forestgap.plr <- indicator_plrange(forestgap) 
do.call(rbind, forestgap.plr) # convert results to data.frame

# Restrict to small xmins 
forestgap.plr2 <- indicator_plrange(forestgap, xmin_bounds = c(1, 10)) 
do.call(rbind, forestgap.plr2) 
}
}
\references{
Clauset, A., Shalizi, C. R., & Newman, M. E. (2009). 
  Power-law distributions in empirical data. SIAM review, 51(4), 661-703.

Berdugo, M., Kefi, S., Soliveres, S. & Maestre, F.T. (2017). Plant spatial 
patterns identify alternative ecosystem multifunctionality states in 
global drylands. Nature in Ecology and Evolution.
}
\seealso{
\code{\link{patchdistr_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_kbdm.R
\name{raw_kbdm}
\alias{raw_kbdm}
\title{Kolmogorov complexity of a matrix}
\usage{
raw_kbdm(mat, subsize)
}
\arguments{
\item{mat}{A logical matrix (with TRUE/FALSE values)}

\item{subsize}{A submatrix size to carry out the Block Decomposition Method
(must be between 1 and 3)}
}
\value{
The KBDM numeric value as a named vector
}
\description{
Compute the Kolmogorov complexity of a matrix using the 
  Block Decomposition Method (requires the \code{acss} package).
}
\details{
The Kolmogorov complexity cannot be computed directly for large strings 
    (i.e. matrices). However, the complexity of smaller submatrices can be 
    estimated, then combined to obtain an approximation of the complexity 
    of the whole matrix. This method, the Block Decomposition Method is 
    implemented in this function. See also \code{\link{kbdm_sews}} 
    for more details.
}
\examples{

\donttest{ 
raw_kbdm(forestgap[[1]], subsize = 3)
}

}
\seealso{
\code{\link{kbdm_sews}}, \code{\link[acss]{acss}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicator_psdtype.R
\name{indicator_psdtype}
\alias{indicator_psdtype}
\title{Change in patch-size distributions types}
\usage{
indicator_psdtype(
  x,
  xmin = 1,
  merge = FALSE,
  fit_lnorm = FALSE,
  xmin_bounds = NULL,
  best_by = "AIC",
  wrap = FALSE,
  nbmask = "von_neumann"
)
}
\arguments{
\item{x}{A logical (\code{TRUE}/\code{FALSE} values) matrix or a list of these.}

\item{xmin}{The xmin to be used to fit the patch size distributions. Use 
the special values "estimate" to use an estimated xmin for each fit}

\item{merge}{The default behavior is to produce indicators values for each 
matrix. If this parameter is set to TRUE then the patch size distributions 
are pooled together for fitting.}

\item{fit_lnorm}{Fit also a log-normal distribution}

\item{xmin_bounds}{Restrict the possible xmins in this range (defaults to 
the whole range of observed patch sizes)}

\item{best_by}{The criterion used to select the best distribution type 
(one of \code{"AIC"}, \code{"BIC"} or \code{"AICc"}).}

\item{wrap}{Determines whether patches are considered to wrap around the 
matrix when reaching the side}

\item{nbmask}{Either "moore" for 8-way neighborhood, "von_neumann" for four-way 
neighborhood (default), or a 3x3 matrix describing which neighbors to 
consider around a cell. See \code{\link{patchsizes}} for details on how to specify 
such neighborhoods.}
}
\value{
A data.frame (or a list of these if x is a list) with the 
  following columns:
    \itemize{
      \item \code{method} the method used for fitting (currently: only 
         log-likelihood is implemented, "ll")
      \item \code{type} the type of distribution
      \item npars the number of parameters of the distribution type
      \item AIC, `AICc` and `BIC` the values for Akaike Information 
        Criterion (or the corrected for small samples equivalent AICc), 
        and Bayesion Information Criterion (BIC)
      \item \code{best} A logical vector indicating which distribution is the 
        best fit 
      \item \code{plexpo}, \code{cutoff}, \code{meanlog}, \code{sdlog} the estimates
         for distribution parameters (see \code{\link{pl_fit}})
      \item \code{percolation} A logical value indicating whether there is 
        \code{\link{percolation}} in the system. 
    }
}
\description{
This functions fits different patch size distributions 
 types (power-law, log-normal, exponential and truncated power-law) to 
 the patches contained in a matrix. The distributions are returned with 
 their corresponding AIC, BIC and AICc to select the best fit.
}
\details{
Patterned ecosystems can exhibit a change in their spatial structure as they 
become more and more stressed. It has been suggested that this should be 
reflected in changes in the observed patch size distributions (PSD). 
The following sequence is expected to occur (Kefi et al. 2011) as patterned 
ecosystems become more and more degraded:

  - Percolation of vegetation patches occurs (a patch has a width or height 
  equal to the size of the system)
  
  - The patch-size distribution follows a power-law
  
  - The patch-size distribution deviates from a power-law as larger patches 
  break down
  
  - The patch-size distribution is closer to an exponential 
  distribution

This indicator fits the observed patch size distribution based on 
maximum-likelihood (following Clauset et al. 2009 recommendations), then 
select the best model using AIC, BIC (default) or AICc.
}
\examples{

data(forestgap)

# One logical matrix only
indicator_psdtype(forestgap[[1]])

# A list of these matrices
\donttest{ 
indicator_psdtype(forestgap)
}

}
\references{
Kefi, S., Rietkerk, M., Roy, M., Franc, A., de Ruiter, P.C. & Pascual, M.
(2011). Robust scaling in ecosystems and the meltdown of patch size
distributions before extinction: Patch size distributions towards 
extinction. Ecology Letters, 14, 29-35.

Kefi, S., Rietkerk, M., Alados, C.L., Pueyo, Y., Papanastasis, V.P., ElAich,
A., et al. (2007). Spatial vegetation patterns and imminent desertification
in Mediterranean arid ecosystems. Nature, 449, 213-217.

Clauset, A., Shalizi, C. R., & Newman, M. E. (2009). 
  Power-law distributions in empirical data. SIAM review, 51(4), 661-703.
}
\seealso{
\code{\link{patchdistr_sews}}

\code{\link{patchdistr_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simple_indic.R
\name{create_indicator}
\alias{create_indicator}
\alias{custom_indicator}
\alias{compute_indicator}
\title{Custom Spatial Early-Warning signals}
\usage{
create_indicator(fun, taskname = as.character(substitute(fun)))

compute_indicator(mat, fun, taskname = as.character(substitute(fun)), ...)
}
\arguments{
\item{fun}{A function that takes a matrix as input and returns 
a vector of numerical values. If the function returns a named vector, 
then the names will be used in plots and summaries. The function may also
accept extra arguments.}

\item{taskname}{The task name. A character string used used for plots and
textual summaries that describes the indicator (or set of indicators) 
being computed. If a task name cannot be derived from \code{fun}, then 
a default name is used.}

\item{mat}{A matrix or a list of matrices.}

\item{...}{Additional arguments to pass to the function \code{fun}}
}
\value{
\code{create_indicator} returns a function that can be used in the same way 
  than the other \code{*_sews} functions (e.g. \code{generic_sews}). This 
  function as well as \code{compute_indicator} will return
  \code{\link[=simple_sews]{simple_sews_*}} objects.
}
\description{
Computation, significance assessment and display of trends 
  of a custom, user-defined indicator.
}
\details{
\code{spatialwarnings} provides "workflow functions", named \code{*_sews}, 
  that assist the user in computing, displaying and assessing the 
  significance of indicator values. The functions \code{create_indicator} and 
  \code{compute_indicator} provides such workflow for any arbitrary function. 
  
\code{create_indicator} takes a function `fun` and returns another function 
  that can be used as an indicator similar to the \code{*_sews} functions. The 
  results of this function can be assessed for significance using 
  \code{indictest} and trends can be displayed using 
  \code{plot}, \code{summary}, etc. (see Examples). \code{compute_indicator} 
  does the same but without needing an intermediate indicator function.
}
\examples{

# Use the maximum patch size as indicator of degradation
maxpatchsize <- function(mat) { 
  max(patchsizes(mat))
}

# Create the indicator function
maxpatch_sews <- create_indicator(maxpatchsize)

# Then work with this function as if it were a function from the *_sews 
# family. 
mp_indic <- maxpatch_sews(forestgap)
summary(mp_indic)

\donttest{ 
# Assess significance and display trends
mp_test <- indictest(mp_indic, nulln = 49)
plot(mp_test)
}



# Try spatial coefficient of variation as a spatial EWS. This function can 
# have arguments. 
spatial_cv <- function(mat, subsize) { 
  matc <- coarse_grain(mat, subsize)
  return( sd(matc) / mean(matc) )
}

# Create indicator function
cv_sews <- create_indicator(spatial_cv)

# Compute and display trends
cv_indic <- cv_sews(serengeti, subsize = 3)
plot(cv_indic, along = serengeti.rain)

# We can do the same work in one go using compute_indicator
cv_indic2 <- compute_indicator(serengeti, spatial_cv, subsize = 3)
plot(cv_indic2, along = serengeti.rain)

\donttest{ 
indictest(cv_indic, nulln = 99)
}
}
\seealso{
\code{\link{simple_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_flowlength.R
\name{flowlength_sews}
\alias{flowlength_sews}
\title{Flowlength connectivity indicator (uniform topography)}
\usage{
flowlength_sews(mat, slope = 20, cell_size = 1)
}
\arguments{
\item{mat}{The input matrix (must be a logical matrix)}

\item{slope}{The slope of the area documented by the matrix (in degree).}

\item{cell_size}{The horizontal size of a cell in the matrix (as viewed 
from above).}
}
\value{
A `simple_sews` object containing the flow length value, among 
  other things, see \code{\link{simple_sews_object}} for more information.
}
\description{
Measures the connectivity of runoff-source areas as determined 
  by vegetation patterns and (uniform) topography
}
\details{
This function computes Flowlength, a simple metric that measures the 
  potential hydrological connectivity of runoff-source areas 
  (e.g., bare soil) considering vegetation cover, vegetation patterns and 
  topography. Flowlength is defined as the average length of all the 
  potential runoff pathways in the target area. Thus, a higher value of 
  the index indicates a higher hydrologic connectivity of runoff source 
  areas. This function is designed for an idealized uniform hillslope 
  (e.g., with constant slope angle, the direction of maximum slope being 
  from the top to the bottom of the input matrices). 

The deviations of Flowlength from its expected values under random or 
  aggregated-pattern null models can be used as an indicator of imminent 
  transition to a degraded state (Rodriguez et al. 2017) in the context 
  of arid drylands. An increased deviation of flowlength compared to its 
  null values is expected as a possible transition gets closer. This deviation
  can be computed using any null model provided by \code{spatialwarnings} (see 
  \code{\link{indictest}} for more details), but a specific null model is 
  provided for Flowlength based on a much-faster analytical approximation, 
  using the argument \code{null_method = "approx_rand"} when calling 
  \code{indictest} (see examples below). 

In general, Flowlength can be used as indicator of dryland functional status 
  by assessing potential water and soil losses in patchy landscapes 
  (Mayor et al. 2008, Moreno-de las Heras et al. 2012, Mayor et al. 2013, 
  Okin et al. 2015). Finally, the combination of observed and expected 
  Flowlength under null models for random or aggregated vegetation cover 
  can be used for assessing the cover-independent role of bare-
  soil connectivity (Rodriguez et al. 2018).
}
\examples{

\donttest{ 
fl_result <- flowlength_sews(arizona, slope = 20, cell_size = 1)

# Compute the Z-score (standardized deviation to null distribution) and plot 
#   its variations along the gradient. This Z-score is suggested by 
#   Rodriguez et al. (2017) as an indicator of degradation. 
fl_test <- indictest(fl_result, nulln = 19)
plot(fl_test, what = "z_score")

# Use the analytical approximation suggested in Rodriguez et al. (2017), 
# instead of permuting the original values in the matrix (much faster)
fl_test <- indictest(fl_result, null_method = "approx_rand")
plot(fl_test, what = "z_score")

}

}
\references{
Rodriguez, F., A. G. Mayor, M. Rietkerk, and S. Bautista. 2017. A null model 
  for assessing the cover-independent role of bare soil connectivity as 
  indicator of dryland functioning and dynamics. Ecological Indicators.

Mayor, A.G., Bautista, S., Small, E.E., Dixon, M., Bellot, J., 2008. 
  Measurement of the connectivity of runoff source areas as determined by 
  vegetation pattern and topography: a tool for assessing potential water 
  and soil losses in drylands. Water Resour. Res. 44, W10423.

Mayor, A.G., Kefi, S., Bautista, S., Rodriguez, F., Carteni, F., Rietkerk, 
  M., 2013. Feedbacks between vegetation pattern and resource loss 
  dramatically decrease ecosystem resilience and restoration potential in 
  a simple dryland model. Landsc. Ecol. 28, 931-942.

Moreno-de las Heras, M., Saco, P.M., Willgoose, G.R., Tongway, D.J., 2012. 
  Variations in hydrological connectivity of Australian semiarid landscapes 
  indicate abrupt changes in rainfall-use efficiency of vegetation. 
  J. Geophys. Res. 117, G03009.

Okin, G.S., Moreno-de las Heras, M., Saco, P.M., Throop, H.L., Vivoni, E.R., 
  Parsons, A.J., Wainwright, J., Peters, D.P.C., 2015. Connectivity in 
  dryland landscapes: shifting concepts of spatial interactions. 
  Front. Ecol. Environ. 13 (1), 20-27.
}
\seealso{
\code{\link{raw_flowlength_uniform}}, 
  \code{\link{indictest}} to test the significance of indicator values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{serengeti}
\alias{serengeti}
\alias{serengeti.rain}
\title{Serengeti dataset}
\format{
A list of logical matrices

The annual rainfall corresponding to the matrices in the serengeti 
  dataset, in the corresponding order.
}
\source{
Extracted from Eby's et al (2017) supplementary material 
  \url{https://github.com/tee-lab/spacetime-csd/}
}
\usage{
serengeti

serengeti.rain
}
\description{
Vegetation data along a rainfall gradient in Serengeti national 
  park.
}
\details{
The data-set consists of a rectangular area of size 7.5 km x 90 km. 
 These data are represented as a list of matrices. Each matrix is a moving 
 window of 7.5 km x 7.5 km which moves my 2.5 km along the length of the 
 rectangular data-set. 
  
 Each entry in the matrix is vegetation data at a resolution of 30m as 
 classified into binary units with FALSE (grass) and TRUE (forest).
 The rainfall data provided here is the average rainfall (mm/yr) of a 
 moving window of dimension 7.5km which moves my 2.5 km along the length 
 of the rectangular data-set.
}
\references{
Eby, S., Agrawal, A., Majumder, S., Dobson, A.P. & Guttal, V. (2017). 
Alternative stable states and spatial indicators of critical slowing down 
along a spatial gradient in a savanna ecosystem: Global Ecology 
and Biogeography, 26, 638-649

Reed, D. N., Anderson, T. M., Dempewolf, J., Metzger, K., & Serneels, S. (2009). 
The spatial distribution of vegetation types in the Serengeti ecosystem: 
the influence of rainfall and topographic relief on vegetation patch 
characteristics. Journal of Biogeography, 36(4), 770-782.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_variogram_indic.R
\name{variogram_sews}
\alias{variogram_sews}
\title{Early-Warning signals based on variograms (EXPERIMENTAL)}
\usage{
variogram_sews(mat, model = "sph", nmax = 1e+05, nbins = 32, cutoff = NULL)
}
\arguments{
\item{mat}{A matrix (TRUE/FALSE values) or a list of matrices}

\item{model}{The variogram model to use, either "sph" (for a spherical
model) or "exp" (for an exponential model)}

\item{nmax}{The maximum number of pairs of cells to use when computing the 
variogram}

\item{nbins}{Number of distance bins to use to compute the variogram}

\item{cutoff}{Maximum distance to consider in the variogram. If NULL, then 
a distance equal to one third of the diagonal of the matrix is used}
}
\value{
A list object of class "variogram_sews", that can be displayed 
  using \code{summary()}, \code{plot()}, etc. Significance of values can 
  be tested using \code{\link{indictest}}.
}
\description{
Compute Early-warning signals based on metrics derived 
  form semi-variograms.
}
\details{
During ecosystem degradation and especially before a regime shift occurs in 
  some ecosystems, spatial autocorrelation is expected to increase in a
  landscape. This increase can be measured based on variograms, which
  represent how the difference (variance) between two points in a landscape
  varies as a function of distance. 

The approach used to derive variogram-based EWS is to compute the 
  empirical variogram of a landscape (represented passed as a matrix of 
  values), then fit a variogram model to it. Three 
  parameters are then extracted from the variogram model (see Nijp et al.
  2019 for a visual description of these parameters): 
  \enumerate{ 
    \item The \emph{nugget} (intercept)
    \item The \emph{partial sill}, i.e. the reduction in semivariance at 
      distance zero
    \item The \emph{correlation range}, i.e. the distance at which the 
      relationship between semivariance and distance flattens
  }

Additionally, the \emph{structural variance} is computed as 
  (partial sill)/(nugget + partial sill), wich quantifies whether the 
  data are spatially structured (structural variance of one), or completely 
  unstructured (value of zero). Theoretical work suggests that partial sill,
  correlation range and structural variance should increase before a regime
  shift occurs in an ecosystem (Nijp et al. 2019). 

This function offers to fit a spherical model or 
  an exponential model. The best-fitting model depends on your data, you 
  should try different options and review the fits using
  \code{\link{plot_variogram}}.

Please note that this part of the package is still experimental and deserves 
  more testing.
}
\examples{

\donttest{
serengeti_ews <- variogram_sews(serengeti, 
                                model ="exp")
plot(serengeti_ews, along = serengeti.rain)
summary(serengeti_ews)

plot_variogram(serengeti_ews)

# nulln should be set to a higher values for meaningful results
serengeti_test <- indictest(serengeti_ews, nulln = 9)
plot(serengeti_test) # gray ribbons indicate the null indicator values 
summary(serengeti_test)
}

}
\references{
Nijp, Jelmer J., Arnaud J.A.M. Temme, George A.K. Voorn, Lammert Kooistra,
  Geerten M. Hengeveld, Merel B. Soons, Adriaan J. Teuling, and Jakob
  Wallinga. (2019) Spatial Early Warning Signals for Impending Regime Shifts:
  A Practical Framework for Application in Real-world Landscapes. Global
  Change Biology 25 (6): 1905-21. \doi{10.1111/gcb.14591}
}
\seealso{
\code{\link{raw_structvar}},
  \code{\link{plot_variogram}}, \code{\link{extract_variogram}},
  \code{\link[=predict.variogram_sews_list]{predict}}

\code{\link{indictest}}, to test the significance of indicator values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/task_patch_indic.R
\name{patchdistr_sews}
\alias{patchdistr_sews}
\title{Early-warning signals based on patch size distributions}
\usage{
patchdistr_sews(
  mat,
  merge = FALSE,
  fit_lnorm = FALSE,
  best_by = "BIC",
  xmin = 1,
  xmin_bounds = NULL,
  wrap = FALSE,
  nbmask = "von_neumann"
)
}
\arguments{
\item{mat}{A logical matrix (\code{TRUE}/\code{FALSE} values) or a list of these}

\item{merge}{The default behavior is to produce indicators values for each 
matrix. If this parameter is set to TRUE then the patch size distributions 
are pooled together before fitting, yielding only one final indicator 
value for the set of input matrices (argument \code{mat}).}

\item{fit_lnorm}{When patch size distributions are compared, should we 
consider lognormal type ? (see details)}

\item{best_by}{The criterion to use to select the best fit (one of "AIC", 
"BIC" or "AICc")}

\item{xmin}{The \eqn{x_{min}} to be used to fit the patch size distributions. Use 
the special value "estimate" to compute first the \eqn{x_{min}} that produces 
the best power-law fit, then use this estimated value to fit all 
distributions.}

\item{xmin_bounds}{Bounds when estimating \eqn{x_{min}} for power-law distributions}

\item{wrap}{Determines whether patches are considered to wrap around the 
matrix when reaching the side}

\item{nbmask}{Either "moore" for 8-way neighborhood, "von_neumann" for four-way 
neighborhood (default), or a 3x3 matrix describing which neighbors to 
consider around a cell. See \code{\link{patchsizes}} for details on how to specify 
such neighborhoods.}
}
\value{
A list object of class 'psdfit' containing among other things 
  - the observed patch size distribution data
  - the model outputs for the candidate distribution fits
  - the power-law range values 
  - the percolation values (if several matrices were provided and 
  `merge` was TRUE, then the average percolation value is returned)
}
\description{
Compute early-warning signals based on patch size distributions
}
\details{
Patterned ecosystems can exhibit a change in their spatial structure as they 
become more and more stressed. It has been suggested that this should be 
reflected in changes in the observed patch size distributions (PSD). 
The following sequence is expected to occur (Kefi et al. 2011) as patterned 
ecosystems become more and more degraded:

  - Percolation of vegetation patches occurs (a patch has a width or height 
  equal to the size of the system)
  
  - The patch-size distribution follows a power-law
  
  - The patch-size distribution deviates from a power-law as larger patches 
  break down
  
  - The patch-size distribution is closer to an exponential 
  distribution

Additionally, it has been suggested that these changes in patch size 
distribution shape should be reflected in the power-law range (PLR). This 
function carries out all the required computations and helps display 
the results in a convenient form. 

The fitting of PSDs is based on maximum-likelihood following Clauset et al.'s 
procedure. The best discrete distribution is estimated among these 
candidates: a power-law \eqn{x^\lambda}, an exponential 
\eqn{exp(\alpha x)}, a truncated power-law and \eqn{x^\lambda exp(\alpha x)},
and optionally, a log-normal. Each distribution parameter is estimated 
using maximum-likelihood, with a minimum patch size (xmin) fixed to one. 
The best distribution is selected based on BIC by default. In raw results, 
\code{plexpo} refers to the power-law exponent (\eqn{\lambda} in the previous 
equations) and \code{cutoff} referes to the exponential decay rate
\eqn{\alpha}. 

To compute the Power-law range (PLR), power-laws are fitted with a variable 
minimum patch size (xmin) and the one with the lowest Kolmogorov-Smirnov
distance to the empirical distribution is retained. PLR is then computed 
using this best-fitting xmin: 

\deqn{\frac{log(x_{max}) - log(x_{min})}{log(x_{max}) - log(x_{smallest})}}{ (log(xmax) - log(xmin))/(log(xmax) - log(xsmallest))}

Results can be displayed using the text-based \code{summary} and \code{print}, 
but graphical options are also available to plot the trends (\code{plot}) and 
the fitted distributions (\code{\link{plot_distr}}). Plotting functions are 
documented in a \link[=patchdistr_sews_plot]{separate page}. Observed and 
fitted distributions can be produced using the \code{predict} function, 
as documented on \link[=patchdistr_sews_predict]{this page}.
}
\examples{

data(forestgap)

\donttest{
psd_indic <- patchdistr_sews(forestgap)

summary(psd_indic)
plot(psd_indic)

# Plots can be modified using ggplot2 directives 
if ( require(ggplot2) ) { 
  plot(psd_indic) + 
    theme_minimal()
}

# Export results to a data.frame
psd_indic_export <- as.data.frame(psd_indic) 
head(psd_indic_export)

}
}
\references{
Kefi, S., Rietkerk, M., Alados, C. L., Pueyo, Y., Papanastasis, 
  V. P., ElAich, A., & De Ruiter, P. C. (2007). Spatial vegetation patterns 
  and imminent desertification in Mediterranean arid ecosystems. 
  Nature, 449(7159), 213-217.

Kefi, S., Rietkerk, M., Roy, M., Franc, A., de Ruiter, P.C. & Pascual, M. 
  (2011). Robust scaling in ecosystems and the meltdown of patch size 
  distributions before extinction: Patch size distributions towards 
  extinction. Ecology Letters, 14, 29-35.

Berdugo, M, Sonia Kefi, Santiago Soliveres, and Fernando T. Maestre. (2017) 
  Plant Spatial Patterns Identify Alternative Ecosystem Multifunctionality 
  States in Global Drylands. Nature in Ecology and Evolution, no. 1.

Clauset, A., Shalizi, C. R., & Newman, M. E. (2009). 
  Power-law distributions in empirical data. SIAM review, 51(4), 661-703.
}
\seealso{
\code{\link{patchsizes}}, \code{\link{plot_distr}}, 
  \code{\link[=predict.patchdistr_sews_single]{predict}}, 
  \code{\link[=plot.patchdistr_sews]{plot}},

\code{\link{indictest}}, to test the significance of indicator values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label.R
\name{label}
\alias{label}
\alias{percolation}
\title{Labelling of unique patches and detection of percolation.}
\usage{
label(mat, nbmask = "von_neumann", wrap = FALSE)

percolation(mat, nbmask = "von_neumann")
}
\arguments{
\item{mat}{A binary matrix}

\item{nbmask}{Either "moore" for 8-way neighborhood, "von_neumann" for four-way 
neighborhood (default), or a 3x3 matrix describing which neighbors to 
consider around a cell. See \code{\link{patchsizes}} for details on how to specify 
such neighborhoods.}

\item{wrap}{Whether to wrap around lattice boundaries (`TRUE`/`FALSE`), 
effectively using periodic boundaries.}
}
\value{
A matrix containing ID numbers for each connected patch. Default 
  parameters assume 4-cell neighborhood and periodic boundaries. The 
  distribution of patch sizes is returned as the attribute "psd" and the 
  percolation status as "percolation" (whether a TRUE patch has a width 
  or height equal to the size of the matrix).
}
\description{
Label each patch with a number in a binary matrix

\code{percolation()} detects whether percolation occurs in the
  matrix (i.e. a patch has a width or a height equal to the size of the 
  matrix)
}
\details{
The \code{label} function "labels" the patches of a binary 
  (\code{TRUE}/\code{FALSE}) matrix. It returns a matrix of similar height and width, 
  with integer values representing the ID of each unique patch (contiguous
   cells). Empty cells are labelled as \code{NA}.
}
\examples{

data(forestgap)

rmat <- matrix(rnorm(100) > .1, ncol = 10)
display_matrix(label(rmat))

# With 8-way neighborhood mask and no wrapping around borders
display_matrix(label(rmat, "moore", wrap = FALSE))

# On real data: 
display_matrix(label(forestgap[[5]], "moore", wrap = FALSE))

}
\seealso{
\code{\link{patchsizes}}, \code{\link{patchdistr_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitpsd.R
\name{xmin_estim}
\alias{xmin_estim}
\title{Estimate the minimum patch size of a power-law distribution}
\usage{
xmin_estim(dat, bounds = range(dat))
}
\arguments{
\item{dat}{A vector of integer values}

\item{bounds}{A vector of two values representing the bounds in which 
the best xmin is searched}
}
\value{
The estimated xmin as an integer value
}
\description{
When fitting a power-law to a discrete distribution, it might 
  be worth discarding points below a certain threshold (xmin) to improve 
  the fit. This function estimates the optimal xmin based on the 
  Kolmogorov-Smirnoff distance between the fit and the empirical 
  distribution, as suggested by Clauset et al. (2009).
}
\details{
The function returns NA if \code{dat} has only three unique values 
  or if the power-law fit failed.
}
\examples{

\donttest{ 
psd <- patchsizes(forestgap[[5]])
xmin_estim(psd)
}
}
\references{
Clauset, A., Shalizi, C. R., & Newman, M. E. (2009). 
  Power-law distributions in empirical data. SIAM review, 51(4), 661-703.
}
\seealso{
\code{\link{patchdistr_sews}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lsw_sews.R
\name{dLSW}
\alias{dLSW}
\alias{pLSW}
\alias{LSW_fit}
\title{The Lifshitz-Slyozov-Wagner distribution}
\usage{
dLSW(x, mu, log = FALSE)

pLSW(x, mu, lower.tail = TRUE, log.p = FALSE)

LSW_fit(x)
}
\arguments{
\item{x}{vector of quantiles}

\item{mu}{the mean of the distribution}

\item{log, log.p}{logical; if \code{TRUE}, probabilities p are given as log(p)}

\item{lower.tail}{logical; if TRUE (default), probabilities are
\eqn{P[X \le x]} otherwise, \eqn{P[X > x]}}
}
\value{
dLSW gives the density, pLSW gives the distribution function, both as numerical 
  vectors determined by the length of \code{x}.
}
\description{
Density and distribution function for the Lifshitz-Slyozov-Wagner (LSW)
  distribution with mean mu.
}
\details{
The LSW distribution is a continuous distribution with density

\deqn{ 
  f(x) = \frac{4x^2}{9\mu^3} ( \frac{3\mu}{3\mu + x} )^{7/3} 
           ( \frac{3\mu}{3\mu - 2x} )^{11/3} e^{\frac{2x}{2x - 3\mu}} 
}

where \eqn{\mu} is the mean of the distribution. 

The functions \code{dLSW} gives the probability density, \code{pLSW} gives the 
distribution function. \code{qLSW} and \code{rLSW} are not implemented. You 
can use \code{LSW_fit} to fit an LSW distribution to a set of observations. 

The length of the results is determined by the length of \code{x}, and \code{mu} can 
only be a single value. 

Please note that this distribution has support only on the interval \code{[0,mu*3/2)}. 
  Probabilities outside this interval are returned as 0.
}
\examples{

# Plot the density 
x <- seq(0, 10, l = 128) 
plot(x, dLSW(x, mu = 3), type = "l", col = "black")
lines(x, dLSW(x, mu = 5), type = "l", col = "red")
lines(x, dLSW(x, mu = 7), type = "l", col = "blue")
legend(x = 0, y = max(dLSW(x, mu = 3)), lty = 1, col = c("black", "red", "blue"), 
       legend = paste("mu =", c(3, 5, 7)))

}
\references{
Siteur, Koen, Quan-Xing Liu, Vivi Rottschäfer, Tjisse van der Heide, Max Rietkerk, 
Arjen Doelman, Christoffer Boström, and Johan van de Koppel. 2023. 
"Phase-Separation Physics Underlies New Theory for the Resilience of Patchy 
Ecosystems." Proceedings of the National Academy of Sciences 120 (2): e2202683120.
https://doi.org/10.1073/pnas.2202683120.
}
\seealso{
\code{\link{lsw_sews}}
}
